// Alarm.cpp: implementation of the CAlarm class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Dics.h"
#include "Alarm.h"
#ifdef _MOTOR_LOAD_FACTOR
#include "DefineAlarmMotorLoadFactor.h"
#else
#include "DefineAlarm_MJ.h"
#endif


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAlarm::CAlarm()
{
	m_nLineNo	= 0;
	m_nAlarmNo  = 0;
	
#ifdef _MY_DEBUG
	m_bBuzzerOff	= FALSE;
	m_bAlarmReset	= FALSE;
#endif

	m_bBtnAlarmReset = FALSE;
}

CAlarm::~CAlarm()
{
	m_arAlarmListSet.RemoveAll(); 
	m_arAlarmListReset.RemoveAll();
	m_arAlarmListSave.RemoveAll();
}



//--------------------------------------------------------------------
// OpenAlarmItems
//--------------------------------------------------------------------
void CAlarm::OpenAlarmItems()
{
	CAlarmItem* pItem;

	int nId		= 0;
	int nCode	= 0;
	char cLevel	= ' ';
	CString sText	= "Not Defined";
	CString sTime	= "00000000000000";

	for( int i=0; i<MAX_ALARM_NO; i++ )
	{
		pItem = new CAlarmItem( nId, nCode, cLevel, sText, sTime );
		m_arrayAlarmItem.Add( pItem );
	}

//#ifdef _NO_RB

#ifdef _MOTOR_LOAD_FACTOR
	ParseAlarm( "DefineAlarmMotorLoadFactor.h");
#else
	ParseAlarm( "DefineAlarm_MJ.h");
#endif

// #else
// 	ParseAlarm( "DefineAlarm.h" );
// #endif
}

//--------------------------------------------------------------------
// ParseAlarm
//--------------------------------------------------------------------
BOOL CAlarm::ParseAlarm(LPCTSTR lpszFileName)
{
	FILE* pFile;
	CString sData;

	if( ( pFile = fopen( lpszFileName, "rt" ) ) == NULL ) 
	{
		AfxMessageBox( "Alarm file is not loaded" );
		return FALSE;
	}

	while( !feof( pFile ) )
	{
		char szLine[200];

		// Read one line
		CString sData;
		fgets( szLine, 200, pFile );
		sData.Format("%s", szLine );
		m_nLineNo += 1;

		// Parse the first token
		char szSeps[] = " \t\n";
		char* ptoken;
		CString sToken;

		ptoken = strtok( szLine, szSeps );
		sToken.Format( "%s", ptoken );

		// Check the slash begin & end
		static BOOL bSkip = FALSE;
		if( !sToken.Compare( "//BEGIN" ) ) bSkip = TRUE;
		if( bSkip )
		{
			if( !sToken.Compare( "//END" ) ) { bSkip = FALSE; }
			continue;
		}
		
		// Parse the useful line again.
		if( sToken.Find( "////" ) )	// Check the skip one line
		{
			ParseLine( sData.GetBuffer( sData.GetLength() ) );
			sData.ReleaseBuffer();
		}
	} 

	fclose( pFile ); 

	return TRUE;
}

//--------------------------------------------------------------------
// ParseLine()
// Format : #define ALM_EMO1				1	// NOTE : 2	S	EMO Alarm1
//			//#define Spare 3
//--------------------------------------------------------------------
BOOL CAlarm::ParseLine( char* szLine )
{
	// Parse the first token
	char szSeps[] = " \t\n";
	char* ptoken;
	CString sToken;
	CString sData( szLine );

	ptoken = strtok( szLine, szSeps );		// 1. "define" or "//define"
	sToken.Format("%s",ptoken);	

	// Check the first token
	BOOL bSpare;
	if( sToken.Compare("#define") && sToken.Compare("//#define") ) 
	{
		// DisplayErrMsg( "Missing #define" );	// Skip 
		return FALSE;
	}

	if( sToken.Find("//") != -1 ) bSpare = TRUE;	// find the spare mark
	else 
	{
		bSpare = FALSE;
		m_nAlarmNo++;
	}

	// Get the second token
	ptoken = strtok( NULL, szSeps );		// 2. ALM_EMO1 => Skip

	// Get the 3rd token
	ptoken = strtok( NULL, szSeps );		// 3. Alarm ID
	sToken.Format( "%s", ptoken );
	if( ptoken == NULL ) { DisplayErrMsg( "Missing Alarm ID" ); return FALSE; }
	int nAlarmId = atoi( sToken );

	// Get the other information
	int		nAlarmCode = 0;
	CString sText;
	char	cLevel = ' ';
	int		nFind;

	if( ( nFind = sData.Find( "NOTE" ) ) == -1 ) sText = "Not defined";
	else 
	{
		sText = sData.Mid( nFind + 4 );
		sText.TrimLeft(": ");
		sText.TrimLeft(" :");
											// 4. Alarm Code
		ptoken = strtok( sText.GetBuffer( sText.GetLength() ), " \t\n" );
		sText.ReleaseBuffer();
		sToken.Format( "%s", ptoken );
		if( ptoken == NULL ) { DisplayErrMsg( "Missing Alarm Code" ); return FALSE; }
		nAlarmCode = atoi( sToken );

		ptoken = strtok( NULL, " \t\n" );	// 5. Alarm level
		sToken.Format( "%s", ptoken );
		if( ptoken == NULL ) { DisplayErrMsg( "Missing Alarm Level" ); return FALSE; }
		if( sToken == "S" ) cLevel = 'S';
		else if( sToken == "H" ) cLevel = 'H';
		else if( sToken == "L" ) cLevel = 'L';

		ptoken = strtok( NULL, "\t\n" );	// 6. Alarm Text
		sToken.Format( "%s", ptoken );
		if( ptoken == NULL ) { DisplayErrMsg( "Missing Alarm Text" ); return FALSE; }
		sText = sToken;
	}

	// Set the Alarm information to array pointer
	SetItem( nAlarmId, nAlarmCode, cLevel, sText );
	return TRUE;
}

//--------------------------------------------------------------------
// AddIOItem
//--------------------------------------------------------------------
BOOL CAlarm::SetItem(int nId, int nCode, char cLevel, CString sText)
{
    CAlarmItem* pItem;

	pItem = m_arrayAlarmItem.GetAt( nId - 1 );

	pItem->m_nId	= nId;
	pItem->m_nCode	= nCode;
	pItem->m_cLevel = cLevel;
	pItem->m_sText	= sText;

    return TRUE;
}

CAlarmItem* CAlarm::GetItem(int nId)
{
	if( nId > m_arrayAlarmItem.GetSize() || nId < 1 ) return NULL;	

	CAlarmItem* pItem;
	pItem = m_arrayAlarmItem.GetAt(nId - 1);
	ASSERT( pItem );
	return pItem;
}

//--------------------------------------------------------------------
// DisplayErrMsg
//--------------------------------------------------------------------
void CAlarm::DisplayErrMsg( char* szComment )
{
	CString sMsg;
	sMsg.Format( "%s : %d line(s)", szComment, m_nLineNo );
	AfxMessageBox( sMsg );
}

//--------------------------------------------------------------------
// CloseAlarmItems
//--------------------------------------------------------------------
void CAlarm::CloseAlarmItems()
{
   // Delete all of the allocated CItem objects.
    CAlarmItem* pItem;
    for ( int index=0; index<m_arrayAlarmItem.GetSize(); index++ )
    {
        pItem = m_arrayAlarmItem.GetAt( index );
        if( pItem != NULL ) delete pItem;
        pItem = NULL;
    }
    m_arrayAlarmItem.RemoveAll();
}

//---------------------------------------------------------------------------
//
// Common Functions
//
//---------------------------------------------------------------------------
// ClearAlarmListAll()
//---------------------------------------------------------------------------
void CAlarm::ClearAlarmList()
{
	// Turn off the TOWER_R
	SetLampState( lampTOWER_LAMP_R, lampOFF );

	m_arAlarmListSet.RemoveAll(); 
	m_arAlarmListReset.RemoveAll();
	m_arAlarmListSave.RemoveAll();
}

//---------------------------------------------------------------------------
// SetAlarm()
//---------------------------------------------------------------------------
void CAlarm::SetAlarm(int nAlarmId)
{
	if( nAlarmId > MAX_ALARM_NO || nAlarmId < 1 )	return;	//ipj051212 modify

	int nSize = m_arAlarmListSave.GetSize();
	for( int i=0; i<nSize; i++ )
	{
		int nId = m_arAlarmListSave.GetAt( i );
		if( nId == nAlarmId )	return; 
	}
	// Buzzer Off switch toggle & Melody ON
	SetLampState( lampBUZZER_OFF, lampTOGGLE );
	SetLampState( lampALARM_RESET, lampTOGGLE );
	

	CSingleLock sLock(&m_mutex );
	sLock.Lock();

	// Add alarm
	m_arAlarmListSet.Add( nAlarmId );
	m_arAlarmListSave.Add( nAlarmId );

	SYSTEMTIME SystemTime;
	GetLocalTime(&SystemTime);
	CString sDateTime;
	sDateTime.Format("%04d%02d%02d%02d%02d%02d.%02d",
		SystemTime.wYear, 
		SystemTime.wMonth, 
		SystemTime.wDay, 
		SystemTime.wHour, 
		SystemTime.wMinute, 
		SystemTime.wSecond, 
		SystemTime.wMilliseconds );

	CAlarmItem * pItem = GetItem( nAlarmId );
	pItem->m_sDateTime = sDateTime; 

	sLock.Unlock();
}

//---------------------------------------------------------------------------
// ResetAlarm()
//---------------------------------------------------------------------------
void CAlarm::ResetAlarm(int nAlarmId)
{
	if( nAlarmId <= 0 ) return;

	//---------- 040519 KDS : ׳ AlarmReset Ҷ ٷ Return ϱ Ͽ 
	int nSize = m_arAlarmListSave.GetSize();
	if( nSize == 0 ) //ipj050830 Ǽ, alarm ϳ  Lamp OFF  
	{
		// Turn off the TOWER_R
		SetLampState( lampTOWER_LAMP_R, lampOFF );
		// Buzzer Off switch and Alarm Reset Lamp Off
		SetLampState( lampBUZZER_OFF, lampOFF );
		SetLampState( lampALARM_RESET, lampOFF );
		SetLampState( lampMELODY1, 		lampOFF );
		SetLampState( lampMELODY2, 		lampOFF );

		//׳ AlarmReset Ҷ ٷ Return ϱ Ͽ 
		return;
	}

	CSingleLock sLock(&m_mutex );
	sLock.Lock();

	BOOL bAlarmExist = FALSE;
	for( int i=0; i<nSize; i++ )
	{
		int nId = m_arAlarmListSave.GetAt( (nSize-1) -i ); //LYS20070213
		if( nId == nAlarmId ) 
		{ 
			m_arAlarmListSave.RemoveAt( (nSize-1) -i ); 
			bAlarmExist = TRUE;
		}
	}

	//Alarm ߻   ׳ return (ipj051019 ߰)
	if( !bAlarmExist )	return; 

	//ipj050830 ߰, alarm ϳ  Lamp OFF 
	nSize = m_arAlarmListSave.GetSize();
	if( nSize == 0 )  
	{
		SetLampState( lampTOWER_LAMP_R, lampOFF );
		SetLampState( lampBUZZER_OFF, lampOFF );
		SetLampState( lampALARM_RESET, lampOFF );
	}

	m_arAlarmListReset.Add( nAlarmId );

	sLock.Unlock();
}

//---------------------------------------------------------------------------
// ResetAlarm()
//---------------------------------------------------------------------------
void CAlarm::ResetAlarmAll()
{
	// Turn off the TOWER_R
	SetLampState( lampTOWER_LAMP_R, lampOFF );
	// Buzzer Off switch and Alarm Reset Lamp Off
	SetLampState( lampBUZZER_OFF, lampOFF );
	SetLampState( lampALARM_RESET, lampOFF );
	SetLampState( lampMELODY1, lampOFF );
	SetLampState( lampMELODY2, lampOFF );

	//---------- 040519 KDS : ׳ AlarmReset Ҷ ٷ Return ϱ Ͽ 
	int nSize = m_arAlarmListSave.GetSize();
	if( nSize == 0 ) return;
	
	for( int i=0; i<nSize; i++ )
	{
		int nId = m_arAlarmListSave.GetAt( 0 );
		m_arAlarmListReset.Add( nId );
		m_arAlarmListSave.RemoveAt( 0 );
	}
}

//---------------------------------------------------------------------------
// Is pushed Alarm Reset Button ?
// Call this function at sequence when you want to off the buzzer or reset the alarm
//---------------------------------------------------------------------------
BOOL CAlarm::IsPushedAlarmReset()
{
	return FALSE;
}

//---------------------------------------------------------------------------
// Toggle lamp
// bOn = TRUE : nSwNo switch lamp is flashed.
//---------------------------------------------------------------------------
void CAlarm::SetLampState(int i, int state)
{
	m_nLampState[i] = state;
}

BOOL CAlarm::IsAlarm( int nAlarmId )
{
	if( nAlarmId <= 0 ) return FALSE;

	int nSize = m_arAlarmListSave.GetSize();
	if( nSize == 0 ) return FALSE;
	
	for( int i=0; i<nSize; i++ )
	{
		int nId = m_arAlarmListSave.GetAt( i );
		if( nId == nAlarmId ) return TRUE;
	}

	return FALSE;
}

CString CAlarm::GetAlarmMsg(int nAlarmId)
{
	CAlarmItem* pItem = GetItem( nAlarmId );
	if( pItem == NULL ) return "";

	return pItem->m_sText;
}
