// EqpAssy.cpp: implementation of the CEqpAssy class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "EqpAssy.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEqpAssy::CEqpAssy()
{
	m_pChannelPackage	= NULL;
	m_pDeviceArray		= NULL;
}

CEqpAssy::CEqpAssy(CDigitalInputs * pDi, CDigitalOutputs * pDo, CAnalogInputs * pAi, CAnalogOutputs * pAo)
{
	m_pDi = pDi;
	m_pDo = pDo;
	m_pAi = pAi;
	m_pAo = pAo;

	m_pChannelPackage = new CChannelPackage( pDi, pDo, pAi, pAo );
	m_pDeviceArray = new CPtrArray();
}

CEqpAssy::~CEqpAssy()
{
	if( m_pChannelPackage ) delete m_pChannelPackage;
	if( m_pDeviceArray ) delete m_pDeviceArray;
}

void CEqpAssy::SetDevice(XCtlDevice *pDevice)
{
	m_pDevice = pDevice;
}

BOOL CEqpAssy::Initialize()
{
	BOOL bRv = TRUE;
		
/////Common Device/////////////////////////////////////////////////////////////////////
	AddDevice( &m_PowerConfirm,				"POWER CONFIRM" );
	AddDevice( &m_PowerConfirm2,			"POWER CONFIRM2" );
	AddDevice( &m_SwitchEmo,				"SWITCH EMO" );
	AddDevice( &m_SwitchEms,				"SWITCH EMS" );
	AddDevice( &m_SwitchBuzzerOff,			"SWITCH BUZZER OFF" );
	AddDevice( &m_SwitchAlarmReset,			"SWITCH ALARM RESET" );
	AddDevice( &m_TowerLamp,				"TOWER LAMP" );
	AddDevice( &m_Melody,					"MELODY" );
	AddDevice( &m_SensorDoor,				"DOOR SENSOR" );
	AddDevice( &m_LdSensorDoor,				"LD DOOR SENSOR" );
	AddDevice( &m_UlSensorDoor,				"UL DOOR SENSOR" );
	AddDevice( &m_SensorCover,				"BATH COVER SENSOR" );
	AddDevice( &m_SensorLeak,				"LEAK SENSOR" );
#ifdef _BUFTANK_USE
	AddDevice( &m_SensorLeakBufTank,		"LEAK SENSOR BUFTANK" );
#endif
	AddDevice( &m_SensorArea,				"AREA SENSOR" );
	AddDevice( &m_SignalFfu,				"FFU ALARM" );
	AddDevice( &m_SignalIonizer,			"IONIZER ALARM" );
	AddDevice( &m_OutputIonizer,			"IONIZER OUTPUT" );
	AddDevice( &m_SensorSafty,				"SAFTY SENSOR" );
	AddDevice( &m_IntegrateFlowmeter,		"INTEGRATE FLOWMETER" );
	AddDevice( &m_RbServoCp,				"RB SERVO CP" );
	AddDevice( &m_SignalESD,				"ESD ALARM");	// hskwak 0502
//////////////////////////////////////////////////////////////////////////

/////TR Device///////////////////////////////////////////////////////
	AddDevice( &m_TrJointConfirm,			"TR JOINT CONFIRM" );
	AddDevice( &m_CylinderTrLdAlign,		"CYLINDER TR LD ALIGN" );
	AddDevice( &m_CylinderTrWaitAlign,		"CYLINDER TR WAIT ALIGN" );
	AddDevice( &m_CylinderTrJointUpDn,		"CYLINDER TR JOINT UP DN" );
	AddDevice( &m_CylinderTrJointLock,		"CYLINDER TR JOINT LOCK" );
	AddDevice( &m_CylinderTrTurn,			"CYLINDER TR TURN" );
	AddDevice( &m_GlassExistTr,				"GLASS EXIST TR UNIT" );

	AddDevice( &m_CylinderUlAlign,			"CYLINDER UL ALIGN");	// hskwak 2019.05.09
	AddDevice( &m_CylinderUlAlignUD,	"CYLINDER UL ALIGN UPDOWN");	// hskwak 2019.05.09
	AddDevice( &m_CylinderUlPinUD,		"CYLINDER UL PIN UPDOWN");	// hskwak 2019.05.09
	m_CylinderUlPinUD.m_nActionTime[UP] = 0;
	m_CylinderUlPinUD.m_nActionTime[DN] = 0;
//////////////////////////////////////////////////////////////////////////

/////Cassette Port Device/////////////////////////////////////////////////
	AddDevice( &m_SwitchCstInputOK,			"SWITCH CST INPUT OK" );
	AddDevice( &m_SwitchCstOutputOK,		"SWITCH CST OUTPUT OK" );
	AddDevice( &m_CylinderCstClamp[0],		"CYLINDER CST CLAMP LO" );
	AddDevice( &m_CylinderCstClamp[1],		"CYLINDER CST CLAMP UP" );
	AddDevice( &m_CylinderCstAlign[0],		"CYLINDER CST ALIGN LO" );
	AddDevice( &m_CylinderCstAlign[1],		"CYLINDER CST ALIGN UP" );
	AddDevice( &m_MaskDetectSensor,			"MASK DETECT SENSOR GROUP" );
	AddDevice( &m_CstDetectSensor,			"BUF CST DETECT SENSOR GROUP" );
//////////////////////////////////////////////////////////////////////////

/////DC Motor Device////////////////////////////////////////////////////////
	AddDevice( &m_CvMotor,					"CONVEYOR MOTOR" );
	AddDevice( &m_CvMotorAll,				"CONVEYOR MOTOR ALL" );
	AddDevice( &m_RbMotor,					"ROLL BRUSH MOTOR" );
//////////////////////////////////////////////////////////////////////////

/////Interrupt Device////////////////////////////////////////////////////////
	AddDevice( &m_SensorLdCvHandInterlock,	"LD CV UNIT HAND INTERLOCK" );
	AddDevice( &m_SensorUlCvHandInterlock,	"UL CV UNIT HAND INTERLOCK" );
	AddDevice( &m_SensorRobotHandInterlock,	"ROBOT HAND INTERLOCK" );
	AddDevice( &m_SensorBuf2Interlock,		"BUF2 UNIT STOP INTERLOCK" );
//	AddDevice( &m_SensorBuf3Interlock,		"BUF3 UNIT STOP INTERLOCK" );
	AddDevice( &m_SensorLdUnitInterlock,	"LD UNIT UP DOWN INTERLOCK" );
	AddDevice( &m_SensorAPDStatus,			"APD STATUS");
	AddDevice( &m_SensorDIAlarm,			"DI ALARM");	
//////////////////////////////////////////////////////////////////////////

/////Glass Exist Sensor Device////////////////////////////////////////////
	AddDevice( &m_GlassExistUnit,			"GLASS EXIST UNIT IN OUT" );
	AddDevice( &m_GlassExistUnitIn,			"GLASS EXIST UNIT IN" );
	AddDevice( &m_GlassExistUnitOut,		"GLASS EXIST UNIT OUT" );
	AddDevice( &m_GlassExistServo,			"GLASS EXIST SERVO UNIT" );
	AddDevice( &m_FixedBrokenSensorUpLd,	"FIXED TYPE BROKEN SENSOR UP LD" );
	AddDevice( &m_FixedBrokenSensorLd,		"FIXED TYPE BROKEN SENSOR LD" );
	AddDevice( &m_FixedBrokenSensorUl,		"FIXED TYPE BROKEN SENSOR UL" );
	AddDevice( &m_ScanBrokenSensorUpLd,		"SCAN TYPE BROKEN SENSOR UP LD" );
	AddDevice( &m_ScanBrokenSensorLd,		"SCAN TYPE BROKEN SENSOR LD" );
	AddDevice( &m_ScanBrokenSensorUl,		"SCAN TYPE BROKEN SENSOR UL" );
//	AddDevice( &m_ScanBrokenSensorBuf3,		"SCAN TYPE BROKEN SENSOR BUF3" );	// 110412 halesu
//	AddDevice( &m_ScanBrokenSensorCcd,		"SCAN TYPE BROKEN SENSOR CCD" );	// 120111 halesu
//	AddDevice( &m_GlassExistDecelSensorLd,	"GLASS EXIST LD SPEED DEC" );	// SanghoCho 2015.03.17
	AddDevice( &m_GlassExistDecelSensorUl,	"GLASS EXIST UL SPEED DEC" );	// SanghoCho 2015.03.17	
	AddDevice( &m_GlassExistAccelSensorUl,	"GLASS EXIST UL SPEED ACC" );	// hskwak 2019.05.09
//////////////////////////////////////////////////////////////////////////

/////Servo Position Sensor Device/////////////////////////////////////////
	AddDevice( &m_PosSensorHandMainView,	"POSITION SENSOR MAIN VIEW" );
	AddDevice( &m_PosSensorTr,				"POSITION SENSOR TR" );
	AddDevice( &m_PosSensorLdUp,			"POSITION SENSOR LD UP HAND" );
	AddDevice( &m_PosSensorLdLo,			"POSITION SENSOR LD LO HAND" );
	AddDevice( &m_PosSensorUlLo,			"POSITION SENSOR UL LO HAND" );
	AddDevice( &m_PosSensorUlUp,			"POSITION SENSOR UL UP HAND" );
	AddDevice( &m_PosSensorLdUnit,			"POSITION SENSOR LD CV UNIT" );
	AddDevice( &m_PosSensorUlHand,			"POSITION SENSOR UL HAND" );
//////////////////////////////////////////////////////////////////////////

/////Cylinder Device//////////////////////////////////////////////////////
	AddDevice( &m_CylinderLdAlign,			"CYLINDER LD ALIGN" );
	AddDevice( &m_CylinderUlAlign,			"CYLINDER UL ALIGN" );
	AddDevice( &m_CylinderLdFishBone,		"CYLINDER LD FISH BONE" );
	AddDevice( &m_CylinderUlFishBone,		"CYLINDER UL FISH BONE" );
	AddDevice( &m_CylinderChemicalShutter,	"CYLINDER CHEMICAL SHUTTER" );
	AddDevice( &m_CylinderLdTilt,			"CYLINDER LD TILT" );
	AddDevice( &m_CylinderUlTilt,			"CYLINDER UL TILT" );
	AddDevice( &m_CylinderUlHand,			"CYLINDER UL HAND" );
//////////////////////////////////////////////////////////////////////////

/////IF Signal Device//////////////////////////////////////////////////////
	AddDevice( &m_IfUpStream,				"INTERFACE SIGNAL UP STREAM" );
	AddDevice( &m_IfDnStream,				"INTERFACE SIGNAL DN STREAM" );
	AddDevice( &m_IfLoader,					"INTERFACE SIGNAL LOADER" );
//////////////////////////////////////////////////////////////////////////

/////Analog Input Device//////////////////////////////////////////////////////
#ifdef _BUFTANK_USE
	AddDevice( &m_AnalogInput,				"ANALOG INPUT MOTOR LOAD FACTOR BUFTANK" );
#else
#ifdef _MOTOR_LOAD_FACTOR
	AddDevice( &m_AnalogInput,				"ANALOG INPUT MOTOR LOAD FACTOR ADD" );
#else
	AddDevice( &m_AnalogInput,				"ANALOG INPUT" );
#endif
#endif
	AddDevice( &m_AnalogInputAk,			"ANALOG INPUT AK INTR" );
	AddDevice( &m_AnalogInputTank,			"ANALOG INPUT TANK" );
//////////////////////////////////////////////////////////////////////////

/////Auto Valve Device//////////////////////////////////////////////////////
#ifdef _BUFTANK_USE
	AddDevice( &m_AutoValvePiping,			"AUTO VALVE PIPING VIEW USE BUFTANK" );
#else
	AddDevice( &m_AutoValvePiping,			"AUTO VALVE PIPING VIEW USE" );
#endif
	AddDevice( &m_AutoValveDet,				"AUTO VALVE DET VIEW USE" );	
	AddDevice( &m_AutoValveAk,				"AUTO VALVE AK CDA" );
	AddDevice( &m_AutoValveAkUp,			"AUTO VALVE AK UPPER" );
	AddDevice( &m_AutoValveAkLo,			"AUTO VALVE AK LOWER" );
	AddDevice( &m_AutoValveSbjDi,			"AUTO VALVE SBJ DI" );
	AddDevice( &m_AutoValveSbjCda,			"AUTO VALVE SBJ CDA" );
	AddDevice( &m_AutoValveAcCda,			"AUTO VALVE AIR CURTAIN CDA" );
	AddDevice( &m_AutoValveFrShower,		"AUTO VALVE FR SHOWER" );
	AddDevice( &m_AutoValveSwr1Shower,		"AUTO VALVE SWR1 SHOWER" );

#ifdef _BUFTANK_USE
	AddDevice( &m_AutoValvePumpOut,			"AUTO VALVE PUMP OUT BUFTANK" );
#else
	AddDevice( &m_AutoValvePumpOut,			"AUTO VALVE PUMP OUT" );
#endif
	AddDevice( &m_AutoValveDirect,			"AUTO VALVE DIRECT" );

	AddDevice( &m_AutoValveDrvAir,			"AUTO VALVE DRIVING AIR" );
	AddDevice( &m_AutoValveRbShower,		"AUTO VALVE RB SHOWER" );
	AddDevice( &m_AutoValveGearSupply,		"AUTO VALVE GEAR SUPPLY" );
	AddDevice( &m_AutoValveMistNozzle,		"AUTO VALVE MIST NOZZLE" );
	AddDevice( &m_AutoValveHotDiBathDrain,	"AUTO VALVE HOT DI BATH DRAIN" );
	AddDevice( &m_AutoValueSWR2BathDrain,		"AUTO VALVE BATH DRAIN");		// hskwak 0502
	AddDevice( &m_AutoValueSWR2TankReturn,		"AUTO VALVE TANK RETURN");		// hskwak 0502
//////////////////////////////////////////////////////////////////////////

/////TANK Device//////////////////////////////////////////////////////
	AddDevice( &m_DiTank[0],				"TANK DI[1]" );
	AddDevice( &m_DiTank[1],				"TANK DI[2]" );	// hskwak 0502
#ifdef _BUFTANK_USE
	AddDevice( &m_DiTank[2],				"TANK DI[3]" );
#endif
	AddDevice( &m_HotDiTank[0],				"TANK HOT DI[1]" );
	AddDevice( &m_DetTank[0],				"TANK DETERGENT[1]" );
	AddDevice( &m_DetTank[1],				"TANK DETERGENT[2]" );
//////////////////////////////////////////////////////////////////////////
	
/////EUV Device//////////////////////////////////////////////////////
	AddDevice( &m_UshioEuv,					"USHIO EUV" );
	AddDevice( &m_GsEuv,					"GS EUV" );
	AddDevice( &m_AutoValveEuv,				"AUTO VALVE EUV" );
	AddDevice( &m_AutoValveEuvN2,			"AUTO VALVE EUV N2" );
	AddDevice( &m_AutoValveEuvHumN2,		"AUTO VALVE EUV HUMIDIFIER N2" );
	AddDevice( &m_AutoValveEuvPcw,			"AUTO VALVE EUV PCW" );
	AddDevice( &m_AutoValveEuvCda,			"AUTO VALVE EUV CDA" );	
	AddDevice( &m_SensorEuvHouseClose,		"EUV HOUSE CLOSE" );
	AddDevice( &m_CylinderEuvUpDn,			"CYLINDER EUV UP DN" );
//////////////////////////////////////////////////////////////////////////

/////HPMJ Device//////////////////////////////////////////////////////////
	AddDevice( &m_Hpmj,						"HPMJ UNIT" );
//////////////////////////////////////////////////////////////////////////

/////Plasma Device//////////////////////////////////////////////////////////
	AddDevice( &m_Plasma,					"PLASMA UNIT" );
//////////////////////////////////////////////////////////////////////////

/////Timer Device////////////////////////////////////////////////////////// halesu
	AddDevice( &m_TimerStart,				"TIMER START" );
	AddDevice( &m_TimerReset,				"TIMER RESET" );   
//////////////////////////////////////////////////////////////////////////
	
////Loader EMO//////////////////////////////////////////////////////////// 101108 halesu
	AddDevice( &m_LoaderEmo,				"LOADER EMO" );
	AddDevice( &m_IFLoaderEmo,				"INTERFACE LOADER EMO" );
//////////////////////////////////////////////////////////////////////////

	return bRv;
}

void CEqpAssy::AddDevice(void *newDevice, CString sName)
{
	BOOL bUsed = FALSE;

	( ( CDevice* )newDevice )->SetMmc( m_pMmc );
	( ( CDevice* )newDevice )->SetDevice( m_pDevice );
	( ( CDevice* )newDevice )->SetDicsData( m_pDicsData );
	( ( CDevice* )newDevice )->SetChannelPackage( m_pChannelPackage );
	( ( CDevice* )newDevice )->SetName( sName );
	bUsed = ( ( CDevice* )newDevice )->ReadConfiguration();
	if( bUsed ) 
	{
		( ( CDevice* )newDevice )->NewSectionKeyList();
		AddDeviceToArray( newDevice );
	}
}

void CEqpAssy::AddDeviceToArray(void *newElement)
{
	m_pDeviceArray->Add( newElement );
}

// it returns CDevice type class pointer array 
CPtrArray* CEqpAssy::GetDeviceArray()
{
	return m_pDeviceArray;
}

CDevice* CEqpAssy::GetDeviceFromDeviceArray(CString sDeviceName)
{
	for( int i = 0 ; i < m_pDeviceArray->GetSize() ; i ++ )
	{
		if( ( (CDevice*)m_pDeviceArray->GetAt( i ) )->GetName() == sDeviceName )
		{
			return (CDevice*)m_pDeviceArray->GetAt( i );
		}
	}
	return NULL;
}
