// FaAnalogInput.cpp: implementation of the CFaAnalogInput class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaAnalogInput.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaAnalogInput::CFaAnalogInput()
{
	m_nAiCount = 0;
	m_bInitialized = FALSE;
	m_pAi = NULL;

	m_nMaxApdAiNo = 0;
	m_nMaxEdcAiNo = 0;
	m_nMaxIntrAiNo = 0;
	m_aiSet = NULL;
}

CFaAnalogInput::~CFaAnalogInput()
{
	if( m_bInitialized )
		delete []m_pAi;
	delete []m_aiSet;
}

BOOL CFaAnalogInput::ReadConfiguration()
{
	CString sSetupFile;
	int i;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf;
	CString sName;
	// Read Channel information & Initialize Sections and Keys
	CStringList *pKeyList;
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bAiInit = TRUE;
	pKeyList = GetKeyList( "AI" );
	m_nAiCount = pKeyList->GetCount();
	if( m_nAiCount == 0 )	m_pAi = new XAnalogInput*[1];
	else m_pAi = new XAnalogInput*[m_nAiCount];

	if( m_nAiCount == 0 )
	{
		sName.Format( "INPUT[%d]",0);
		m_IniConfiguration.SetStringValue( "AI", sName, "ai_SPARE", TRUE );
		pKeyList = GetKeyList( "AI" );
		m_nAiCount = pKeyList->GetCount();
	}
	for( i=0; i<m_nAiCount; i++)
	{
		sName.Format( "INPUT[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "AI", sName, "ai_SPARE" );
		m_pAi[i] = m_pChannelPackage->GetAIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "AI", sName, sBuf, TRUE );

 		if( m_pAi[i] != NULL )
		{
			m_pAi[i]->AddConnectingObjectName( m_sName );
			bAiInit &= bAiInit;
		}
		else bAiInit = FALSE;
	}

	m_bInitialized = bAiInit;

	m_aiSet = new AI_SET[m_nAiCount];
	for( i=0; i<m_nAiCount; i++ )
	{
		m_aiSet[i].nReal = GetAddress( i );
		m_aiSet[i].uRESID = -1;
		m_aiSet[i].nOwnerViewNo = -1;
		m_aiSet[i].nOwnerUnitNo = -1;
		m_aiSet[i].nAiIntrNo = -1;
		m_aiSet[i].nApdReportNo = -1;
		m_aiSet[i].nEdcReportNo = -1;
		m_aiSet[i].sName = "";
		m_aiSet[i].sKind = "";
		m_aiSet[i].sUnit = "";
	}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////

	if( !m_bInitialized ) 
	{
		sBuf.Format( "FA Analog Input [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}
	return TRUE;
}

WORD CFaAnalogInput::GetAdcVal(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nAiCount ) ||
		( !m_pAi[nNo]->IsDeviceLink() ) ) return 0;

	return m_pAi[nNo]->Get();
}

int CFaAnalogInput::GetAddress(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nAiCount ) ||
		( !m_pAi[nNo]->IsDeviceLink() ) ) return 0;

	return m_pAi[nNo]->GetAddress();
}

float CFaAnalogInput::GetRealVal(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nAiCount ) ||
		( !m_pAi[nNo]->IsDeviceLink() ) ) return 0.0;

	return m_pAi[nNo]->GetCalAi();
}
