// FaCylinder.cpp: implementation of the CFaCylinder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaCylinder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaCylinder::CFaCylinder()
{
	m_bInitialized = FALSE;
	m_nDiFwCount = 0;
	m_nDiBwCount = 0;
	m_nDoFwCount = 0;
	m_nDoBwCount = 0;
	m_nTimeoutFwCount = 0;
	m_nTimeoutBwCount = 0;

	m_pDiFw = NULL;
	m_pDiBw = NULL;
	m_pDoFw = NULL;
	m_pDoBw = NULL;
	m_nTimeoutFw = NULL;
	m_nTimeoutBw = NULL;
}

CFaCylinder::~CFaCylinder()
{
	if( m_bInitialized )
	{
		delete []m_pDiFw;
		delete []m_pDiBw;
		delete []m_pDoFw;
		delete []m_pDoBw;
		delete []m_nTimeoutFw;
		delete []m_nTimeoutBw;
	}
}

BOOL CFaCylinder::ReadConfiguration()
{
	CString sSetupFile;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf, sName;
	// Read Channel information & Initialize Sections and Keys
	CStringList *pKeyList;

//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDiFwInit = TRUE;
	pKeyList = GetKeyList( "DI_FW" );
	m_nDiFwCount = pKeyList->GetCount();
	if( m_nDiFwCount == 0 )	m_pDiFw = new XDigitalInput*[1];
	else m_pDiFw = new XDigitalInput*[m_nDiFwCount];

	if( m_nDiFwCount == 0 )
	{
		sName.Format( "FW[%d]",0);
		m_IniConfiguration.SetStringValue( "DI_FW", sName, "di_SPARE", TRUE );
		pKeyList = GetKeyList( "DI_FW" );
		m_nDiFwCount = pKeyList->GetCount();
	}
	for( int i=0; i<m_nDiFwCount; i++)
	{
		sName.Format( "FW[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DI_FW", sName, "di_SPARE" );
		m_pDiFw[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DI_FW", sName, sBuf, TRUE );

		if( m_pDiFw[i] != NULL )
		{
			m_pDiFw[i]->AddConnectingObjectName( m_sName );
			bDiFwInit &= bDiFwInit;
		}
		else bDiFwInit = FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDiBwInit = TRUE;
	pKeyList = GetKeyList( "DI_BW" );
	m_nDiBwCount = pKeyList->GetCount();
	if( m_nDiBwCount == 0 ) m_pDiBw = new XDigitalInput*[1];
	else m_pDiBw = new XDigitalInput*[m_nDiBwCount];

	if( m_nDiBwCount == 0 )
	{
		sName.Format( "BW[%d]",0);
		m_IniConfiguration.SetStringValue( "DI_BW", sName, "di_SPARE", TRUE );
		pKeyList = GetKeyList( "DI_BW" );
		m_nDiBwCount = pKeyList->GetCount();
	}
	for( int i = 0; i< m_nDiBwCount; i++)
	{
		sName.Format( "BW[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DI_BW", sName, "di_SPARE" );
		m_pDiBw[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DI_BW", sName, sBuf, TRUE );

		if( m_pDiBw[i] != NULL )
		{
			m_pDiBw[i]->AddConnectingObjectName( m_sName );
			bDiBwInit &= bDiBwInit;
		}
		else bDiBwInit = FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDoFwInit = TRUE;
	pKeyList = GetKeyList( "DO_FW" );
	m_nDoFwCount = pKeyList->GetCount();
	if( m_nDoFwCount == 0 ) m_pDoFw = new XDigitalOutput*[1];
	else m_pDoFw = new XDigitalOutput*[m_nDoFwCount];

	if( m_nDoFwCount == 0 )
	{
		sName.Format( "FW[%d]",0);
		m_IniConfiguration.SetStringValue( "DO_FW", sName, "do_SPARE", TRUE );
		pKeyList = GetKeyList( "DO_FW" );
		m_nDoFwCount = pKeyList->GetCount();
	}
	for( int i = 0; i<m_nDoFwCount; i++)
	{
		sName.Format( "FW[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DO_FW", sName, "do_SPARE" );
		m_pDoFw[i] = m_pChannelPackage->GetDOChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DO_FW", sName, sBuf, TRUE );

		if( m_pDoFw[i] != NULL )
		{
			m_pDoFw[i]->AddConnectingObjectName( m_sName );
			bDoFwInit &= bDoFwInit;
		}
		else bDoFwInit = FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDoBwInit = TRUE;
	pKeyList = GetKeyList( "DO_BW" );
	m_nDoBwCount = pKeyList->GetCount();
	if( m_nDoBwCount == 0 ) m_pDoBw = new XDigitalOutput*[1];
	else m_pDoBw = new XDigitalOutput*[m_nDoBwCount];

	if( m_nDoBwCount == 0 )
	{
		sName.Format( "BW[%d]",0);
		m_IniConfiguration.SetStringValue( "DO_BW", sName, "do_SPARE", TRUE );
		pKeyList = GetKeyList( "DO_BW" );
		m_nDoBwCount = pKeyList->GetCount();
	}
	for( int i = 0; i<m_nDoBwCount; i++)
	{
		sName.Format( "BW[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DO_BW", sName, "do_SPARE" );
		m_pDoBw[i] = m_pChannelPackage->GetDOChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DO_BW", sName, sBuf, TRUE );

		if( m_pDoBw[i] != NULL )
		{
			m_pDoBw[i]->AddConnectingObjectName( m_sName );
			bDoBwInit &= bDoBwInit;
		}
		else bDoBwInit = FALSE;
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	pKeyList = GetKeyList( "TIMEOUT_FW" );
	m_nTimeoutFwCount = pKeyList->GetCount();
	if( m_nTimeoutFwCount == 0 ) m_nTimeoutFw = new INT[1];
	else m_nTimeoutFw = new INT[m_nTimeoutFwCount];

	if( m_nTimeoutFwCount == 0 )
	{
		sName.Format( "PARA_TIME[%d]",0);
		m_IniConfiguration.SetStringValue( "TIMEOUT_FW", sName, "10", TRUE );
		pKeyList = GetKeyList( "TIMEOUT_FW" );
		m_nTimeoutFwCount = pKeyList->GetCount();
	}
	for( int i = 0; i<m_nTimeoutFwCount; i++)
	{
		sName.Format( "PARA_TIME[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "TIMEOUT_FW", sName, "10" );
		m_nTimeoutFw[i] = atoi( sBuf );
		m_IniConfiguration.SetStringValue( "TIMEOUT_FW", sName, sBuf, TRUE );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	pKeyList = GetKeyList( "TIMEOUT_BW" );
	m_nTimeoutBwCount = pKeyList->GetCount();
	if( m_nTimeoutBwCount == 0 ) m_nTimeoutBw = new INT[1];
	else m_nTimeoutBw = new INT[m_nTimeoutBwCount];

	if( m_nTimeoutBwCount == 0 )
	{
		sName.Format( "PARA_TIME[%d]",0);
		m_IniConfiguration.SetStringValue( "TIMEOUT_BW", sName, "10", TRUE );
		pKeyList = GetKeyList( "TIMEOUT_BW" );
		m_nTimeoutBwCount = pKeyList->GetCount();
	}
	for( int i = 0; i<m_nTimeoutBwCount; i++)
	{
		sName.Format( "PARA_TIME[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "TIMEOUT_BW", sName, "10" );
		m_nTimeoutBw[i] = atoi( sBuf );
		m_IniConfiguration.SetStringValue( "TIMEOUT_BW", sName, sBuf, TRUE );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////
	m_bInitialized = ( bDiFwInit && bDiBwInit && bDoFwInit && bDoBwInit );

	if( !m_bInitialized ) 
	{
		sBuf.Format( "FA Cylinder [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}

	return TRUE;
}

void CFaCylinder::SetFw()
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nDoBwCount; i++ ) SetBw( i, OFF );
	for( int i=0; i<m_nDoFwCount; i++ ) SetFw( i, ON );
}

void CFaCylinder::SetBw()
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nDoFwCount; i++ ) SetFw( i, OFF );
	for( int i=0; i<m_nDoBwCount; i++ ) SetBw( i, ON );	
}

void CFaCylinder::SetStop()
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nDoFwCount; i++ ) SetFw( i, OFF );
	for( int i=0; i<m_nDoBwCount; i++ ) SetBw( i, OFF );	
}

BOOL CFaCylinder::GetFw()
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = TRUE;
	for( int i=0; i<m_nDiFwCount; i++ ) bRv &= GetFw( i );
	return bRv;
}

BOOL CFaCylinder::GetBw()
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = TRUE;
	for( int i=0; i<m_nDiBwCount; i++ ) bRv &= GetBw( i );
	return bRv;
}

void CFaCylinder::SetFw(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDoFwCount ) || 
	    !m_pDoFw[nNo]->IsDeviceLink() ) return;

	m_pDoFw[nNo]->Set( bVal );
}

void CFaCylinder::SetBw(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDoBwCount ) ||
		!m_pDoBw[nNo]->IsDeviceLink() ) return;

	m_pDoBw[nNo]->Set( bVal );
}

BOOL CFaCylinder::GetFw(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiFwCount ) || 
	    !m_pDiFw[nNo]->IsDeviceLink() ) return FALSE;

	return m_pDiFw[nNo]->Get();
}

BOOL CFaCylinder::GetBw(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiBwCount ) || 
	    !m_pDiBw[nNo]->IsDeviceLink() ) return FALSE;

	return m_pDiBw[nNo]->Get();
}

INT CFaCylinder::GetFwTime(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nTimeoutFwCount ) ) return 0;

	return m_nTimeoutFw[nNo];
}

INT CFaCylinder::GetBwTime(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nTimeoutBwCount ) ) return 0;

	return m_nTimeoutBw[nNo];
}

void CFaCylinder::SetFwState()
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nDoBwCount; i++ ) SetBwState( i, OFF );
	for( int i=0; i<m_nDoFwCount; i++ ) SetFwState( i, ON );
}

void CFaCylinder::SetBwState()
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nDoFwCount; i++ ) SetFwState( i, OFF );
	for( int i=0; i<m_nDoBwCount; i++ ) SetBwState( i, ON );
}

BOOL CFaCylinder::GetFwState()
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = TRUE;
	for( int i=0; i<m_nDoFwCount; i++ ) bRv &= GetFwState( i );
	return bRv;
}

BOOL CFaCylinder::GetBwState()
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = TRUE;
	for( int i=0; i<m_nDoBwCount; i++ ) bRv &= GetBwState( i );
	return bRv;
}

void CFaCylinder::SetFwState( int nNo=0, BOOL bVal )
{
	if( !IsInitialized() ||
		( nNo >= m_nDiFwCount ) || 
	    !m_pDiFw[nNo]->IsDeviceLink() ) return;

	m_pDiFw[nNo]->SetState( bVal );
}

void CFaCylinder::SetBwState( int nNo=0, BOOL bVal )
{
	if( !IsInitialized() ||
		( nNo >= m_nDiBwCount ) || 
	    !m_pDiBw[nNo]->IsDeviceLink() ) return;

	m_pDiBw[nNo]->SetState( bVal );
}

BOOL CFaCylinder::GetFwState( int nNo=0 )
{
	if( !IsInitialized() ||
		( nNo >= m_nDoFwCount ) || 
	    !m_pDoFw[nNo]->IsDeviceLink() ) return FALSE;

	return m_pDoFw[nNo]->Get();
}

BOOL CFaCylinder::GetBwState( int nNo=0 )
{
	if( !IsInitialized() ||
		( nNo >= m_nDoBwCount ) || 
	    !m_pDoBw[nNo]->IsDeviceLink() ) return FALSE;

	return m_pDoBw[nNo]->Get();
}
