// FaDcMotor.cpp: implementation of the CFaDcMotor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaDcMotor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaDcMotor::CFaDcMotor()
{
	m_bInitialized = FALSE;
	m_nMaxSectionNo = 0;
	m_nMaxMotorRpm = 3000;
	
	m_pDiCpAlarm = NULL;
	m_pDiMotorAlarm = NULL;
	m_pDoMotorFw = NULL;
	m_pDoMotorBw = NULL;
	m_pAoMotorSpeed = NULL;

	m_dRollerDia = NULL;
	m_dGearRatio = NULL;
	m_dSpeedOffset = NULL;
	m_dSpeedScale = NULL;
	m_wMotorSpeed = NULL;
	m_wMotorRpm = NULL;
	
	m_nUnitMotorCount = NULL;
}

CFaDcMotor::~CFaDcMotor()
{
	if( m_bInitialized )
	{
		for( int i=0; i<m_nMaxSectionNo; i++ )
		{
			delete []m_pDiCpAlarm[i];
			delete []m_pDiMotorAlarm[i];
			delete []m_pDoMotorFw[i];
			delete []m_pDoMotorBw[i];
			delete []m_pAoMotorSpeed[i];

			delete []m_dRollerDia[i];
			delete []m_dGearRatio[i];
			delete []m_dSpeedOffset[i];
			delete []m_dSpeedScale[i];
			delete []m_wMotorSpeed[i];
			delete []m_wMotorRpm[i];
		}
		
		delete []m_pDiCpAlarm;
		delete []m_pDiMotorAlarm;
		delete []m_pDoMotorFw;
		delete []m_pDoMotorBw;
		delete []m_pAoMotorSpeed;

		delete []m_dRollerDia;
		delete []m_dGearRatio;
		delete []m_dSpeedOffset;
		delete []m_dSpeedScale;
		delete []m_wMotorSpeed;
		delete []m_wMotorRpm;
		
		delete []m_nUnitMotorCount;
	}
}

BOOL CFaDcMotor::ReadConfiguration()
{
////SETUP READ//////////////////////////////////////////////////////////////////////////////////
	CString sSetupFile;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}
	int nMaxSectionNo = m_IniConfigurationSetup.GetIntValue( "MAX SECTION NO", "VALUE", 0 );
	if( nMaxSectionNo == 0 )
	{
		SetMaxSectionNo( 1 );
		m_IniConfigurationSetup.SetIntValue( "MAX SECTION NO", "VALUE", 1, TRUE );
	}
	else SetMaxSectionNo( nMaxSectionNo );
////////////////////////////////////////////////////////////////////////////////////////////////

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf, sSectionName, sKeyName;
	// Read Channel information & Initialize Sections and Keys
	BOOL bDiInit = TRUE;
	CStringList *pKeyList;

	m_nUnitMotorCount = new int[m_nMaxSectionNo];

	m_pDiCpAlarm = new XDigitalInput**[m_nMaxSectionNo];
	m_pDiMotorAlarm = new XDigitalInput**[m_nMaxSectionNo];
	m_pDoMotorFw = new XDigitalOutput**[m_nMaxSectionNo];
	m_pDoMotorBw = new XDigitalOutput**[m_nMaxSectionNo];
	m_pAoMotorSpeed = new XAnalogOutput**[m_nMaxSectionNo];

	m_dRollerDia = new double*[m_nMaxSectionNo];
	m_dGearRatio = new double*[m_nMaxSectionNo];
	m_dSpeedOffset = new double*[m_nMaxSectionNo];
	m_dSpeedScale = new double*[m_nMaxSectionNo];
	m_wMotorSpeed = new WORD*[m_nMaxSectionNo];
	m_wMotorRpm = new WORD*[m_nMaxSectionNo];

	for( int n=0; n<m_nMaxSectionNo; n++ )
	{
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
		sSectionName.Format("UNIT%d", n);
		pKeyList = GetKeyList( sSectionName );
		m_nUnitMotorCount[n] = (int)(pKeyList->GetCount()/9);	// Motor  9 Key Ƿ...
		
		if( m_nUnitMotorCount[n] == 0 )
		{
			m_pDiCpAlarm[n] = new XDigitalInput*[1];
			m_pDiMotorAlarm[n] = new XDigitalInput*[1];
			m_pDoMotorFw[n] = new XDigitalOutput*[1];
			m_pDoMotorBw[n] = new XDigitalOutput*[1];
			m_pAoMotorSpeed[n] = new XAnalogOutput*[1];

			m_dRollerDia[n] = new double[1];
			m_dGearRatio[n] = new double[1];
			m_dSpeedScale[n] = new double[1];
			m_dSpeedOffset[n] = new double[1];
			m_wMotorSpeed[n] = new WORD[1];
			m_wMotorRpm[n] = new WORD[1];
		}
		else
		{
			m_pDiCpAlarm[n] = new XDigitalInput*[m_nUnitMotorCount[n]];
			m_pDiMotorAlarm[n] = new XDigitalInput*[m_nUnitMotorCount[n]];
			m_pDoMotorFw[n] = new XDigitalOutput*[m_nUnitMotorCount[n]];
			m_pDoMotorBw[n] = new XDigitalOutput*[m_nUnitMotorCount[n]];
			m_pAoMotorSpeed[n] = new XAnalogOutput*[m_nUnitMotorCount[n]];

			m_dRollerDia[n] = new double[m_nUnitMotorCount[n]];
			m_dGearRatio[n] = new double[m_nUnitMotorCount[n]];
			m_dSpeedScale[n] = new double[m_nUnitMotorCount[n]];
			m_dSpeedOffset[n] = new double[m_nUnitMotorCount[n]];
			m_wMotorSpeed[n] = new WORD[m_nUnitMotorCount[n]];
			m_wMotorRpm[n] = new WORD[m_nUnitMotorCount[n]];
		}

		if( m_nUnitMotorCount[n] == 0 )
		{
			sKeyName.Format( "DI_CP_ALARM[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "di_SPARE", TRUE );

			sKeyName.Format( "DI_MOTOR_ALARM[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "di_SPARE", TRUE );

			sKeyName.Format( "DO_MOTOR_FW[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "do_SPARE", TRUE );

			sKeyName.Format( "DO_MOTOR_BW[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "do_SPARE", TRUE );

			sKeyName.Format( "AO_MOTOR_SPEED[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "ao_SPARE", TRUE );

			sKeyName.Format( "PARA_ROLLER_DIA[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "1.0", TRUE );

			sKeyName.Format( "PARA_GEAR_RATIO[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "1.0", TRUE );

			sKeyName.Format( "PARA_SPEED_SCALE[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "1.0", TRUE );

			sKeyName.Format( "PARA_SPEED_OFFSET[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "0.0", TRUE );

			pKeyList = GetKeyList( sSectionName );
			m_nUnitMotorCount[n] = (int)(pKeyList->GetCount()/9);	// Motor  9 Key Ƿ...
		}
		for( int i=0; i<m_nUnitMotorCount[n]; i++)
		{
			sKeyName.Format( "DI_CP_ALARM[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "di_SPARE" );
			m_pDiCpAlarm[n][i] = m_pChannelPackage->GetDIChannel( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			sKeyName.Format( "DI_MOTOR_ALARM[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "di_SPARE" );
			m_pDiMotorAlarm[n][i] = m_pChannelPackage->GetDIChannel( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			sKeyName.Format( "DO_MOTOR_FW[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "do_SPARE" );
			m_pDoMotorFw[n][i] = m_pChannelPackage->GetDOChannel( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			sKeyName.Format( "DO_MOTOR_BW[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "do_SPARE" );
			m_pDoMotorBw[n][i] = m_pChannelPackage->GetDOChannel( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			sKeyName.Format( "AO_MOTOR_SPEED[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "ao_SPARE" );
			m_pAoMotorSpeed[n][i] = m_pChannelPackage->GetAOChannel( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			m_dRollerDia[n][i] = 0.0;
			sKeyName.Format( "PARA_ROLLER_DIA[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "1.0" );
			m_dRollerDia[n][i] = atof( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			m_dGearRatio[n][i] = 0.0;
			sKeyName.Format( "PARA_GEAR_RATIO[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "1.0" );
			m_dGearRatio[n][i] = atof( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			m_dSpeedScale[n][i] = 0.0;
			sKeyName.Format( "PARA_SPEED_SCALE[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "1.0" );
			m_dSpeedScale[n][i] = atof( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			m_dSpeedOffset[n][i] = 0.0;
			sKeyName.Format( "PARA_SPEED_OFFSET[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "0.0" );
			m_dSpeedOffset[n][i] = atof( sBuf );
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );

			if( ( m_pDiCpAlarm[n][i] != NULL ) &&
				( m_pDiMotorAlarm[n][i] != NULL ) &&
				( m_pDoMotorFw[n][i] != NULL ) &&
				( m_pDoMotorBw[n][i] != NULL ) &&
				( m_pAoMotorSpeed[n][i] != NULL ) )
			{
				m_pDiCpAlarm[n][i]->AddConnectingObjectName( m_sName );
				m_pDiMotorAlarm[n][i]->AddConnectingObjectName( m_sName );
				m_pDoMotorFw[n][i]->AddConnectingObjectName( m_sName );
				m_pDoMotorBw[n][i]->AddConnectingObjectName( m_sName );
				m_pAoMotorSpeed[n][i]->AddConnectingObjectName( m_sName );

				bDiInit &= bDiInit;
			}
			else bDiInit = FALSE;
		}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	}

	m_bInitialized = bDiInit;

	if( !m_bInitialized ) 
	{
		sBuf.Format( "FA DC Motor [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}

	return TRUE;
}

void CFaDcMotor::SetMaxSectionNo(int nNo)
{
	m_nMaxSectionNo = nNo;
}

int CFaDcMotor::GetMaxSectionNo()
{
	return m_nMaxSectionNo;
}

//  [7/1/2008][Ѱ CP OFF Ǹ Alarm]
BOOL CFaDcMotor::GetMotorCpAlarm(int nUnit)
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = FALSE;
	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) bRv |= GetMotorCpAlarm(nUnit, i);
	return bRv;
}

//  [7/1/2008][Ѱ Motor Alarm ߻ Ǹ Alarm]
BOOL CFaDcMotor::GetMotorAlarm(int nUnit)
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = FALSE;
	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) bRv |= GetMotorAlarm(nUnit, i);
	return bRv;
}

void CFaDcMotor::SetMotorFw(int nUnit, BOOL bVal)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetMotorFw(nUnit, i, bVal);
}

void CFaDcMotor::SetMotorBw(int nUnit, BOOL bVal)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetMotorBw(nUnit, i, bVal);
}

void CFaDcMotor::SetMotorSpeed(int nUnit, int nMmSec)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetMotorSpeed(nUnit, i, nMmSec);
}

void CFaDcMotor::SetMotorRpm(int nUnit, int nRpm)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetMotorRpm(nUnit, i, nRpm);
}

void CFaDcMotor::SetRollerDiameter(int nUnit, double dDiameter)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetRollerDiameter(nUnit, i, dDiameter);
}

void CFaDcMotor::SetGearRatio(int nUnit, double dRatio)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetGearRatio(nUnit, i, dRatio);
}

void CFaDcMotor::SetSpeedOffset(int nUnit, double dSpeedOffset)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetSpeedOffset(nUnit, i, dSpeedOffset);
}

void CFaDcMotor::SetSpeedScale(int nUnit, double dSpeedScale)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetSpeedScale(nUnit, i, dSpeedScale);
}

//  [7/1/2008]

BOOL CFaDcMotor::GetMotorCpAlarm(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDiCpAlarm[nUnit][nNo]->IsDeviceLink() ) ) return FALSE;

	return m_pDiCpAlarm[nUnit][nNo]->Get();
}

BOOL CFaDcMotor::GetMotorAlarm(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDiMotorAlarm[nUnit][nNo]->IsDeviceLink() ) ) return FALSE;

	return m_pDiMotorAlarm[nUnit][nNo]->Get();
}

void CFaDcMotor::SetMotorFw(int nUnit, int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDoMotorFw[nUnit][nNo]->IsDeviceLink() ) ) return;

	m_pDoMotorFw[nUnit][nNo]->Set(bVal);
}

void CFaDcMotor::SetMotorBw(int nUnit, int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDoMotorBw[nUnit][nNo]->IsDeviceLink() ) ) return;

	m_pDoMotorBw[nUnit][nNo]->Set(bVal);
}

void CFaDcMotor::SetMotorSpeed(int nUnit, int nNo, int nMmSec)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pAoMotorSpeed[nUnit][nNo]->IsDeviceLink() ) ) return;

	double dVal;
	int nMaxVoltage = 0x4000;
	if( m_dRollerDia == 0 ) 
	{
		AfxMessageBox( "Roller Diameter is zero. zero value can't devide anything." );
		return;
	}
	dVal = ( (double)nMmSec * (double)nMaxVoltage ) / ( m_Pi * m_dRollerDia[nUnit][nNo] * (double)m_nMaxMotorRpm / m_dGearRatio[nUnit][nNo] );
	m_wMotorSpeed[nUnit][nNo] = (WORD)(dVal*m_dSpeedScale[nUnit][nNo]);
	m_pAoMotorSpeed[nUnit][nNo]->Set( m_wMotorSpeed[nUnit][nNo] );
}

void CFaDcMotor::SetMotorRpm(int nUnit, int nNo, int nRpm)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pAoMotorSpeed[nUnit][nNo]->IsDeviceLink() ) ) return;

	double dVal;
	int nMaxVoltage = 0x4000;

	dVal = ( (double)nRpm * (double)nMaxVoltage ) / ( (double)m_nMaxMotorRpm / m_dGearRatio[nUnit][nNo] );
	//m_wMotorRpm[nUnit][nNo] = (WORD)dVal;//lantest change temp
    m_wMotorRpm[nUnit][nNo] = (WORD)(dVal*m_dSpeedScale[nUnit][nNo]);
	m_pAoMotorSpeed[nUnit][nNo]->Set( m_wMotorRpm[nUnit][nNo] );
}

void CFaDcMotor::SetRollerDiameter(int nUnit, int nNo, double dDiameter)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return;

	m_dRollerDia[nUnit][nNo] = dDiameter;	
}

void CFaDcMotor::SetGearRatio(int nUnit, int nNo, double dRatio)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return;

	m_dGearRatio[nUnit][nNo] = dRatio;
}

void CFaDcMotor::SetSpeedOffset(int nUnit, int nNo, double dSpeedOffset)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return;

	m_dSpeedOffset[nUnit][nNo] = dSpeedOffset;
}

void CFaDcMotor::SetSpeedScale(int nUnit, int nNo, double dSpeedScale)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return;

	m_dSpeedScale[nUnit][nNo] = dSpeedScale;
}


// Motor State Set/Get
void CFaDcMotor::SetMotorCpAlarmState(int nUnit, BOOL bVal)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetMotorCpAlarmState(nUnit, i, bVal);
}

void CFaDcMotor::SetMotorAlarmState(int nUnit, BOOL bVal)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nUnitMotorCount[nUnit]; i++ ) SetMotorAlarmState(nUnit, i, bVal);
}

void CFaDcMotor::SetMotorCpAlarmState(int nUnit, int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDiCpAlarm[nUnit][nNo]->IsDeviceLink() ) ) return;

	m_pDiCpAlarm[nUnit][nNo]->SetState(bVal);
}

void CFaDcMotor::SetMotorAlarmState(int nUnit, int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDiMotorAlarm[nUnit][nNo]->IsDeviceLink() ) ) return;

	m_pDiMotorAlarm[nUnit][nNo]->SetState(bVal);
}

BOOL CFaDcMotor::GetMotorFwState(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDoMotorFw[nUnit][nNo]->IsDeviceLink() ) ) return FALSE;

	return m_pDoMotorFw[nUnit][nNo]->Get();
}

BOOL CFaDcMotor::GetMotorBwState(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ||
		( !m_pDoMotorBw[nUnit][nNo]->IsDeviceLink() ) ) return FALSE;

	return m_pDoMotorBw[nUnit][nNo]->Get();
}

WORD CFaDcMotor::GetMotorSpeed(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return 0;

	return m_wMotorSpeed[nUnit][nNo];
}

WORD CFaDcMotor::GetMotorRpm(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return 0;

	return m_wMotorRpm[nUnit][nNo];
}

double CFaDcMotor::GetRollerDiameter(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return 0.0;

	return m_dRollerDia[nUnit][nNo];
}

double CFaDcMotor::GetGearRatio(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return 0.0;

	return m_dGearRatio[nUnit][nNo];
}

double CFaDcMotor::GetSpeedOffset(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return 0.0;

	return m_dSpeedOffset[nUnit][nNo];
}

double CFaDcMotor::GetSpeedScale(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nUnitMotorCount[nUnit] ) ) return 0.0;

	return m_dSpeedScale[nUnit][nNo];
}
