// FaPlasma.cpp: implementation of the CFaPlasma class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaPlasma.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaPlasma::CFaPlasma()
{
	m_bInitialized = FALSE;
	m_pAiGaugeGroup = NULL;
	m_pAoGaugeGroup = NULL;
	m_pDiPlasmStatus = NULL;
}

CFaPlasma::~CFaPlasma()
{
	if( m_bInitialized )
	{
		delete []m_pAiGaugeGroup;
		delete []m_pAoGaugeGroup;
		delete []m_pDiPlasmStatus;
	}
}

BOOL CFaPlasma::ReadConfiguration()
{
	CString sSetupFile;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf;
	CString sName;
	// Read Channel information & Initialize Sections and Keys
	CStringList *pKeyList;
	pKeyList = GetKeyList( "AI" );
	m_nAiCount = pKeyList->GetCount();
	m_pAiGaugeGroup = new XAnalogInput*[m_nAiCount];
	if( pKeyList->GetCount() == 0 )
	{
		m_pAiGaugeGroup = new XAnalogInput*[1];
		sName.Format( "aigPLASMA_GAUGE[%d]",0 );
		m_IniConfiguration.SetStringValue( "AI", sName, "aiSPARE", TRUE );
		pKeyList = GetKeyList( "AI" );
	}

	for( int i = 0; i< pKeyList->GetCount(); i++)
	{
		sName.Format( "aigPLASMA_GAUGE[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "AI", sName, "aiSPARE" );
		m_pAiGaugeGroup[i] = m_pChannelPackage->GetAIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "AI", sName, sBuf, TRUE );
	}
	pKeyList = GetKeyList( "AO" );
	m_nAoCount = pKeyList->GetCount();
	m_pAoGaugeGroup = new XAnalogOutput*[m_nAoCount];
	if( pKeyList->GetCount() == 0 )
	{
		m_pAoGaugeGroup = new XAnalogOutput*[1];
		sName.Format( "aogPLASMA_GAUGE[%d]",0 );
		m_IniConfiguration.SetStringValue( "AO", sName, "aoSPARE", TRUE );
		pKeyList = GetKeyList( "AO" );
	}
	for( int i = 0; i< pKeyList->GetCount(); i++)
	{		
		sName.Format( "aogPLASMA_GAUGE[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "AO", sName, "aoSPARE" );
		m_pAoGaugeGroup[i] = m_pChannelPackage->GetAOChannel( sBuf );
		m_IniConfiguration.SetStringValue( "AO", sName, sBuf, TRUE );
	}
	pKeyList = GetKeyList( "DICP" );
	m_nDiCount = pKeyList->GetCount();
	m_pDiPlasmStatus = new XDigitalInput*[m_nDiCount];
	if( pKeyList->GetCount() == 0 )
	{
		m_pDiPlasmStatus = new XDigitalInput*[1];
		sName.Format( "digPLASMA_STATUS[%d]",0);
		m_IniConfiguration.SetStringValue( "DICP", sName, "diSPARE", TRUE );
		pKeyList = GetKeyList( "DICP" );

	}
	for( int i = 0; i< pKeyList->GetCount(); i++)
	{
		sName.Format( "digPLASMA_STATUS[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DICP", sName, "diSPARE" );
		m_pDiPlasmStatus[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DICP", sName, sBuf, TRUE );
	}
	
	// Read Channel information & Initialize Sections and Keys
	sBuf = m_IniConfiguration.GetStringValue( "DO", "PLASMA_POWER", "doSPARE" );
	m_pDoPower = m_pChannelPackage->GetDOChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DO", "PLASMA_POWER", sBuf, TRUE );

	// Read Channel information & Initialize Sections and Keys
	sBuf = m_IniConfiguration.GetStringValue( "DO", "PLASMA_HOUSE_FW", "doSPARE" );
	m_pDoHouseCylFW = m_pChannelPackage->GetDOChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DO", "PLASMA_HOUSE_FW", sBuf, TRUE );
	// Read Channel information & Initialize Sections and Keys
	sBuf = m_IniConfiguration.GetStringValue( "DO", "PLASMA_HOUSE_BW", "doSPARE" );
	m_pDoHouseCylBW = m_pChannelPackage->GetDOChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DO", "PLASMA_HOUSE_BW", sBuf, TRUE );

	sBuf = m_IniConfiguration.GetStringValue( "DO", "PLASMA_INTERLOCK", "doSPARE" );
	m_pDoInterlock = m_pChannelPackage->GetDOChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DO", "PLASMA_INTERLOCK", sBuf, TRUE );
	
	sBuf = m_IniConfiguration.GetStringValue( "DI", "PLASMA_HOUSE_FW", "diSPARE" );
	m_pDiHouseCylFW = m_pChannelPackage->GetDIChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DI", "PLASMA_HOUSE_FW", sBuf, TRUE );

	sBuf = m_IniConfiguration.GetStringValue( "DI", "PLASMA_HOUSE_BW", "diSPARE" );
	m_pDiHouseCylBW = m_pChannelPackage->GetDIChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DI", "PLASMA_HOUSE_BW", sBuf, TRUE );

	sBuf = m_IniConfiguration.GetStringValue( "DI", "PLASMA_HOUSE_CLOSE", "diSPARE" );
	m_pDiHouseClose = m_pChannelPackage->GetDIChannel( sBuf );
	m_IniConfiguration.SetStringValue( "DI", "PLASMA_HOUSE_CLOSE", sBuf, TRUE );


	BOOL bInitialized = TRUE;

	for( int i = 0; i <m_nAiCount; i++ )
	{

		if( m_pAiGaugeGroup[i] != NULL ) 
		bInitialized &= bInitialized;
		else if( m_pAiGaugeGroup[i] == NULL )
		bInitialized = FALSE;
	}

	for( int i = 0; i <m_nAoCount; i++ )
	{

		if( m_pAoGaugeGroup[i] != NULL ) 
		bInitialized &= bInitialized;
		else if( m_pAoGaugeGroup[i] == NULL )
		bInitialized = FALSE;
	}
	for( int i = 0; i <m_nDiCount; i++ )
	{

		if( m_pDiPlasmStatus[i] != NULL ) 
		bInitialized &= bInitialized;
		else if( m_pDiPlasmStatus[i] == NULL )
		bInitialized = FALSE;
	}
			
	bInitialized =  
		m_pDoPower != NULL &&
		m_pDoInterlock != NULL &&
		m_pDoHouseCylFW != NULL &&
		m_pDoHouseCylBW != NULL &&
		m_pDiHouseCylFW != NULL && 
		m_pDiHouseCylBW != NULL && 
		m_pDiHouseClose != NULL;
	
	m_bInitialized = bInitialized;
	
	for( int i = 0; i <m_nAiCount; i++ ) 
	{
		if( NULL != m_pAiGaugeGroup[i] )m_pAiGaugeGroup[i]->AddConnectingObjectName( m_sName );
	}
	for( int i = 0; i <m_nAoCount; i++ ) 
	{
		if( NULL != m_pAoGaugeGroup[i] )m_pAoGaugeGroup[i]->AddConnectingObjectName( m_sName );
	}
	for( int i = 0; i <m_nDiCount; i++ )
	{
		if( NULL != m_pDiPlasmStatus[i] )m_pDiPlasmStatus[i]->AddConnectingObjectName( m_sName );
	}
	if( NULL != m_pDoPower )		m_pDoPower->AddConnectingObjectName( m_sName );
	if( NULL != m_pDoHouseCylFW )	m_pDoHouseCylFW->AddConnectingObjectName( m_sName );
	if( NULL != m_pDoHouseCylBW )	m_pDoHouseCylBW->AddConnectingObjectName( m_sName );
	if( NULL != m_pDiHouseCylFW )	m_pDiHouseCylFW->AddConnectingObjectName( m_sName );
	if( NULL != m_pDiHouseCylBW )	m_pDiHouseCylBW->AddConnectingObjectName( m_sName );
	if( NULL != m_pDiHouseClose )	m_pDiHouseCylBW->AddConnectingObjectName( m_sName );
	
	if( !bInitialized ) 
	{
		sBuf.Format( "FA Plasma [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}

	return TRUE;
}

void CFaPlasma::SetPlasmaValue(int nNo, WORD nValue)
{
	if( !IsInitialized() ) return;
	if( nNo >= m_nAoCount ) return;
	if( !m_pAoGaugeGroup[nNo]->IsDeviceLink() ) return;

	m_pAoGaugeGroup[nNo]->Set( nValue );
}

float CFaPlasma::GetPalsmGaugeState(int nNo)
{
	if( !IsInitialized() ) return 0.0;
	if( nNo >= m_nAoCount ) return 0.0;
	if( !m_pAiGaugeGroup[nNo]->IsDeviceLink() ) return 0.0;

	int nAddress = m_pAoGaugeGroup[nNo]->GetAddress();
	return (float)( exp( (double)m_pAiGaugeGroup[nNo]->GetCalAi() ) );
}

BOOL CFaPlasma::GetPlasmaStatus( int nNo )
{
	if( !IsInitialized() ) return FALSE;
	if( nNo >= m_nDiCount ) return FALSE;
	if( !m_pDiPlasmStatus[nNo]->IsDeviceLink() ) return FALSE;

	return m_pDiPlasmStatus[nNo]->Get();
}

void CFaPlasma::SetPowerOn(BOOL bPowerStat)
{
	if( !IsInitialized() ) return;
	if( !m_pDoPower->IsDeviceLink() ) return;
	m_pDoPower->Set( bPowerStat );
}

BOOL CFaPlasma::GetPowerStatus()
{
	if( !IsInitialized() ) return FALSE;
	//int nSize = sizeof( m_pDoPower ) / sizeof( XDigitalOutput );
	//if( nSize <= 0 ) return FALSE;
	if( !m_pDoPower->IsDeviceLink() ) return FALSE;

	return m_pDoPower->Get();
}

void CFaPlasma::SetPlasmaHouseAct(int nAct )
{
	if( !IsInitialized() ) return;
	int nSize = sizeof( m_pDoHouseCylFW ) / sizeof( XDigitalOutput );
	int nSize1 = sizeof( m_pDoHouseCylBW ) / sizeof( XDigitalOutput );
	if( nSize <= 0 || nSize1 <= 0 ) return;
	if( !m_pDoHouseCylFW->IsDeviceLink() || !m_pDoHouseCylBW->IsDeviceLink() ) return;

	switch( nAct )
	{
	case actUP :
		m_pDoHouseCylFW->Set( ON );
		m_pDoHouseCylBW->Set( OFF );
		break;

	case actDN :
		m_pDoHouseCylFW->Set( OFF );
		m_pDoHouseCylBW->Set( ON );
		break;

	case actSTOP:
		m_pDoHouseCylFW->Set( OFF );
		m_pDoHouseCylBW->Set( OFF );
		break;		
	}
}

int CFaPlasma::GetPlasmaHouseActState()
{
	if( !IsInitialized() ) return actSTOP;
	int nSize = sizeof( m_pDoHouseCylFW ) / sizeof( XDigitalOutput );
	int nSize1 = sizeof( m_pDoHouseCylBW ) / sizeof( XDigitalOutput );
	if( nSize <= 0 || nSize1 <= 0 ) return actSTOP;
	if( !m_pDoHouseCylFW->IsDeviceLink() || !m_pDoHouseCylBW->IsDeviceLink() ) return actSTOP;

	int nState = actSTOP;
	if( m_pDoHouseCylFW->Get() && !m_pDoHouseCylBW->Get() )
	{
		nState = actUP;
	}
	else if( !m_pDoHouseCylFW->Get() && m_pDoHouseCylBW->Get() )
	{
		nState = actDN;
	}
	return nState;
}

int CFaPlasma::GetHouseSensorState( )
{
	if( !IsInitialized() ) return actSTOP;
	if( !m_pDiHouseCylFW->IsDeviceLink() || !m_pDiHouseCylBW->IsDeviceLink() ) return actSTOP;

	int  nState = actSTOP;

	if( m_pDiHouseCylFW->Get() && !m_pDiHouseCylBW->Get())
	{
		nState = actUP;
	}
	else if( !m_pDiHouseCylFW->Get() && m_pDiHouseCylBW->Get() )
	{
		nState = actDN;
	}
	return nState;
}

void CFaPlasma::SetPlasmaInterlock(BOOL bAct)
{
	if( !IsInitialized() ) return;
	int nSize = sizeof( m_pDoInterlock ) / sizeof( XDigitalOutput );
	if( nSize <= 0 ) return;
	if( !m_pDoInterlock->IsDeviceLink() ) return;

	m_pDoInterlock->Set( bAct );
}

BOOL CFaPlasma::GetPlasmaInterlockActState()
{
	if( !IsInitialized() ) return FALSE;
	int nSize = sizeof( m_pDoInterlock ) / sizeof( XDigitalOutput );
	if( nSize <= 0 ) return FALSE;
	if( !m_pDoInterlock->IsDeviceLink() ) return FALSE;

	return m_pDoInterlock->Get();
}

BOOL CFaPlasma::GetHouseCloseState()
{
	if( !IsInitialized() ) return FALSE;
	int nSize = sizeof( m_pDiHouseClose ) / sizeof( XDigitalInput );
	if( nSize <= 0 ) return FALSE;
	if( !m_pDiHouseClose->IsDeviceLink() ) return FALSE;

	return m_pDiHouseClose->Get();
}
