// FaScanBrokenSensor.cpp: implementation of the CFaScanBrokenSensor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaScanBrokenSensor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaScanBrokenSensor::CFaScanBrokenSensor()
{
	m_bInitialized = FALSE;
	m_nDiCount = 0;
	m_nDoCount = 0;
	m_pDi = NULL;
	m_pDiScanStart = NULL;
	m_pDiScanEnd = NULL;
	m_pDo = NULL;
}

CFaScanBrokenSensor::~CFaScanBrokenSensor()
{
	if( m_bInitialized )
	{
		delete []m_pDi;
		delete []m_pDiScanStart;
		delete []m_pDiScanEnd;
		delete []m_pDo;
	}
}

BOOL CFaScanBrokenSensor::ReadConfiguration()
{
	CString sSetupFile;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf, sName;
	// Read Channel information & Initialize Sections and Keys
	CStringList *pKeyList;
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDiInit = TRUE;
	pKeyList = GetKeyList( "DI" );
	m_nDiCount = (int)(pKeyList->GetCount()/3);	// DI 3 Key ִ.
	if( m_nDiCount == 0 )
	{
		m_pDi = new XDigitalInput*[1];
		m_pDiScanStart = new XDigitalInput*[1];
		m_pDiScanEnd = new XDigitalInput*[1];
	}
	else
	{
		m_pDi = new XDigitalInput*[m_nDiCount];
		m_pDiScanStart = new XDigitalInput*[m_nDiCount];
		m_pDiScanEnd = new XDigitalInput*[m_nDiCount];
	}

	if( m_nDiCount == 0 )
	{
		sName.Format( "ALARM[%d]",0);
		m_IniConfiguration.SetStringValue( "DI", sName, "di_SPARE", TRUE );

		sName.Format( "SCAN_START[%d]",0);
		m_IniConfiguration.SetStringValue( "DI", sName, "di_SPARE", TRUE );

		sName.Format( "SCAN_END[%d]",0);
		m_IniConfiguration.SetStringValue( "DI", sName, "di_SPARE", TRUE );

		pKeyList = GetKeyList( "DI" );
		m_nDiCount = (int)(pKeyList->GetCount()/3);
	}
	for( int i=0; i<m_nDiCount; i++)
	{
		sName.Format( "ALARM[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DI", sName, "di_SPARE" );
		m_pDi[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DI", sName, sBuf, TRUE );

		sName.Format( "SCAN_START[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DI", sName, "di_SPARE" );
		m_pDiScanStart[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DI", sName, sBuf, TRUE );

		sName.Format( "SCAN_END[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DI", sName, "di_SPARE" );
		m_pDiScanEnd[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DI", sName, sBuf, TRUE );

		if( ( m_pDi[i] != NULL ) &&
			( m_pDiScanStart[i] != NULL ) &&
			( m_pDiScanEnd[i] != NULL ) )
		{
			m_pDi[i]->AddConnectingObjectName( m_sName );
			m_pDiScanStart[i]->AddConnectingObjectName( m_sName );
			m_pDiScanEnd[i]->AddConnectingObjectName( m_sName );
			bDiInit &= bDiInit;
		}
		else bDiInit = FALSE;
	}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDoInit = TRUE;
	pKeyList = GetKeyList( "DO" );
	m_nDoCount = pKeyList->GetCount();
	if( m_nDoCount == 0 )	m_pDo = new XDigitalOutput*[1];
	else m_pDo = new XDigitalOutput*[m_nDoCount];

	if( m_nDoCount == 0 )
	{
		m_pDo[0] = NULL;
		sName.Format( "RESET[%d]",0);
		m_IniConfiguration.SetStringValue( "DO", sName, "do_SPARE", TRUE );
		pKeyList = GetKeyList( "DO" );
		m_nDoCount = pKeyList->GetCount();
	}
	for( int i=0; i<m_nDoCount; i++)
	{
		m_pDo[i] = NULL;
		sName.Format( "RESET[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DO", sName, "do_SPARE" );
		m_pDo[i] = m_pChannelPackage->GetDOChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DO", sName, sBuf, TRUE );

		if( m_pDo[i] != NULL )
		{
			m_pDo[i]->AddConnectingObjectName( m_sName );
			bDoInit &= bDoInit;
		}
		else bDoInit = FALSE;
	}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////

	m_bInitialized = bDiInit && bDoInit;

	if( !m_bInitialized ) 
	{
		sBuf.Format( "FA Scan Type Broken [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}

	return TRUE;
}

void CFaScanBrokenSensor::Reset(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDoCount ) ||
		( !m_pDo[nNo]->IsDeviceLink() ) ) return;

	m_pDo[nNo]->Set( bVal );
}

BOOL CFaScanBrokenSensor::GetAlarm(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDi[nNo]->IsDeviceLink() ) ) return FALSE;

	return ( m_pDi[nNo]->Get() );
}

void CFaScanBrokenSensor::SetAlarmState(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDi[nNo]->IsDeviceLink() ) ) return;

	m_pDi[nNo]->SetState(bVal);
}

BOOL CFaScanBrokenSensor::GetScanStart(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDiScanStart[nNo]->IsDeviceLink() ) ) return FALSE;

	return ( m_pDiScanStart[nNo]->Get() );
}

void CFaScanBrokenSensor::SetScanStartState(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDiScanStart[nNo]->IsDeviceLink() ) ) return;

	m_pDiScanStart[nNo]->SetState(bVal);
}

BOOL CFaScanBrokenSensor::GetScanEnd(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDiScanEnd[nNo]->IsDeviceLink() ) ) return FALSE;

	return ( m_pDiScanEnd[nNo]->Get() );
}

void CFaScanBrokenSensor::SetScanEndState(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDiScanEnd[nNo]->IsDeviceLink() ) ) return;

	m_pDiScanEnd[nNo]->SetState(bVal);
}

BOOL CFaScanBrokenSensor::GetResetState( int nNo )
{
	if( !IsInitialized() ||
		( nNo >= m_nDoCount ) ||
		( !m_pDo[nNo]->IsDeviceLink() ) ) return FALSE;

	return m_pDo[nNo]->Get();
}