// FaServoPosSensor.cpp: implementation of the CFaServoPosSensor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaServoPosSensor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaServoPosSensor::CFaServoPosSensor()
{
	m_bInitialized = FALSE;
	m_nMaxSectionNo = 0;
	m_pDi = NULL;
	m_nDiCount = NULL;
	m_pDiMmc = NULL;
}

CFaServoPosSensor::~CFaServoPosSensor()
{
	if( m_bInitialized )
	{
		for( int i=0; i<m_nMaxSectionNo; i++ )
		{
			delete []m_pDi[i];
			delete []m_pDiMmc[i];
		}
		delete []m_pDi;
		delete []m_nDiCount;
		delete []m_pDiMmc;
	}
}

BOOL CFaServoPosSensor::ReadConfiguration()
{
////SETUP READ//////////////////////////////////////////////////////////////////////////////////
	CString sSetupFile;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}
	int nMaxSectionNo = m_IniConfigurationSetup.GetIntValue( "MAX SECTION NO", "VALUE", 0 );
	if( nMaxSectionNo == 0 )
	{
		SetMaxSectionNo( 1 );
		m_IniConfigurationSetup.SetIntValue( "MAX SECTION NO", "VALUE", 1, TRUE );
	}
	else SetMaxSectionNo( nMaxSectionNo );
////////////////////////////////////////////////////////////////////////////////////////////////

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf, sSectionName, sKeyName;
	// Read Channel information & Initialize Sections and Keys
	BOOL bDiInit = TRUE;
	CStringList *pKeyList;

	m_nDiCount = new int[m_nMaxSectionNo];
	m_pDi = new XDigitalInput**[m_nMaxSectionNo];
	m_pDiMmc = new XDigitalInput*[m_nMaxSectionNo];
	for( int n=0; n<m_nMaxSectionNo; n++ )
	{
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
		sSectionName.Format("POS%d", n);
		pKeyList = GetKeyList( sSectionName );
		m_nDiCount[n] = pKeyList->GetCount();
		if( m_nDiCount[n] == 0 )
		{
			m_pDi[n] = new XDigitalInput*[1];
			m_pDiMmc[n] = new XDigitalInput[1];
		}
		else
		{
			m_pDi[n] = new XDigitalInput*[m_nDiCount[n]];
			m_pDiMmc[n] = new XDigitalInput[m_nDiCount[n]];
		}

		if( m_nDiCount[n] == 0 )
		{
			sKeyName.Format( "DI_SENSOR[%d]",0);
			m_IniConfiguration.SetStringValue( sSectionName, sKeyName, "di_SPARE", TRUE );
			pKeyList = GetKeyList( sSectionName );
			m_nDiCount[n] = pKeyList->GetCount();
		}
		for( int i=0; i<m_nDiCount[n]; i++)
		{
			sKeyName.Format( "DI_SENSOR[%d]",i);
			sBuf = m_IniConfiguration.GetStringValue( sSectionName, sKeyName, "di_SPARE" );

			int nIndex = sBuf.Find( "MMC_HOME_AXIS" );
			if( nIndex < 0 )
			{
				m_pDi[n][i] = m_pChannelPackage->GetDIChannel( sBuf );
				m_IniConfiguration.SetStringValue( sSectionName, sKeyName, sBuf, TRUE );
			}
			else
			{
				CString sAxis = sBuf.Mid( nIndex+13 );
				int nAxis = atoi( sAxis );
				m_pDiMmc[n][i].Create( m_pDevice, 0, 0, 0, "MMC_HOME_AXIS" );
				m_pDi[n][i] = &m_pDiMmc[n][i];
				m_pDi[n][i]->SetMmcLink( m_pMmc, nAxis, ST_HOME_SWITCH );
			}

			if( m_pDi[n][i] != NULL )
			{
				m_pDi[n][i]->AddConnectingObjectName( m_sName );
				bDiInit &= bDiInit;
			}
			else bDiInit = FALSE;
		}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	}

	m_bInitialized = bDiInit;

	if( !m_bInitialized ) 
	{
		sBuf.Format( "FA Servo Position [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}

	return TRUE;
}

void CFaServoPosSensor::SetMaxSectionNo( int nNo )
{
	m_nMaxSectionNo = nNo;
}

int CFaServoPosSensor::GetMaxSectionNo()
{
	return m_nMaxSectionNo;
}

BOOL CFaServoPosSensor::GetAnd(int nUnit)
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = TRUE;

	for( int i=0; i<m_nDiCount[nUnit]; i++ )
	{
		bRv &= Get(nUnit, i);
	}

	return bRv;
}

BOOL CFaServoPosSensor::GetOr(int nUnit)
{
	if( !IsInitialized() ) return FALSE;

	BOOL bRv = FALSE;

	for( int i=0; i<m_nDiCount[nUnit]; i++ )
	{
		bRv |= Get(nUnit, i);
	}

	return bRv;
}

void CFaServoPosSensor::Set(int nUnit, BOOL bVal)
{
	if( !IsInitialized() ) return;

	for( int i=0; i<m_nDiCount[nUnit]; i++ )
	{
		Set( nUnit, i, bVal );
	}
}

BOOL CFaServoPosSensor::Get(int nUnit, int nNo)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nDiCount[nUnit] ) ||
		( !m_pDi[nUnit][nNo]->IsDeviceLink() ) ) return FALSE;

	return ( m_pDi[nUnit][nNo]->Get() );
}

void CFaServoPosSensor::Set(int nUnit, int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nUnit >= m_nMaxSectionNo ) ||
		( nNo >= m_nDiCount[nUnit] ) ||
		( !m_pDi[nUnit][nNo]->IsDeviceLink() ) ) return;

	m_pDi[nUnit][nNo]->SetState( bVal );
}
