// FaSwitch.cpp: implementation of the CFaSwitch class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "FaSwitch.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFaSwitch::CFaSwitch()
{
	m_bInitialized = FALSE;
	m_nDiCount = 0;
	m_nDoCount = 0;
	m_pDi = NULL;
	m_pDo = NULL;
}

CFaSwitch::~CFaSwitch()
{
	if( m_bInitialized )
	{
		delete []m_pDi;
		delete []m_pDo;
	}
}

BOOL CFaSwitch::ReadConfiguration()
{
	CString sSetupFile;
	sSetupFile.Format("%s SETUP", m_sName);
	m_IniConfigurationSetup.Open( sSetupFile, "Configuration" );
	CString sDeviceUse = m_IniConfigurationSetup.GetStringValue( "DEVICE USE", "VALUE", "NO" );
	if( sDeviceUse == "NO" )
	{
		m_IniConfigurationSetup.SetStringValue( "DEVICE USE", "VALUE", "NO", TRUE );
		return FALSE;
	}

	m_IniConfiguration.Open( m_sName, "Configuration" );

	CString sBuf, sName;
	// Read Channel information & Initialize Sections and Keys
	CStringList *pKeyList;
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDiInit = TRUE;
	pKeyList = GetKeyList( "DI" );
	m_nDiCount = pKeyList->GetCount();
	if( m_nDiCount == 0 )	m_pDi = new XDigitalInput*[1];
	else m_pDi = new XDigitalInput*[m_nDiCount];

	if( m_nDiCount == 0 )
	{
		sName.Format( "SWITCH[%d]",0);
		m_IniConfiguration.SetStringValue( "DI", sName, "di_SPARE", TRUE );
		pKeyList = GetKeyList( "DI" );
		m_nDiCount = pKeyList->GetCount();
	}
	for( int i=0; i<m_nDiCount; i++)
	{
		sName.Format( "SWITCH[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DI", sName, "di_SPARE" );
		m_pDi[i] = m_pChannelPackage->GetDIChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DI", sName, sBuf, TRUE );

		if( m_pDi[i] != NULL )
		{
			m_pDi[i]->AddConnectingObjectName( m_sName );
			bDiInit &= bDiInit;
		}
		else bDiInit = FALSE;
	}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////
	BOOL bDoInit = TRUE;
	pKeyList = GetKeyList( "DO" );
	m_nDoCount = pKeyList->GetCount();
	if( m_nDoCount == 0 )	m_pDo = new XDigitalOutput*[1];
	else m_pDo = new XDigitalOutput*[m_nDoCount];

	if( m_nDoCount == 0 )
	{
		m_pDo[0] = NULL;
		sName.Format( "SWITCH[%d]",0);
		m_IniConfiguration.SetStringValue( "DO", sName, "do_SPARE", TRUE );
		pKeyList = GetKeyList( "DO" );
		m_nDoCount = pKeyList->GetCount();
	}
	for( int i=0; i<m_nDoCount; i++)
	{
		m_pDo[i] = NULL;
		sName.Format( "SWITCH[%d]",i);
		sBuf = m_IniConfiguration.GetStringValue( "DO", sName, "do_SPARE" );
		m_pDo[i] = m_pChannelPackage->GetDOChannel( sBuf );
		m_IniConfiguration.SetStringValue( "DO", sName, sBuf, TRUE );

		if( m_pDo[i] != NULL )
		{
			m_pDo[i]->AddConnectingObjectName( m_sName );
			bDoInit &= bDoInit;
		}
		else bDoInit = FALSE;
	}
//	///////////////////////////////////////////////////////////////////////////////////////////
//	///////////////////////////////////////////////////////////////////////////////////////////

	m_bInitialized = bDiInit && bDoInit;

	if( !m_bInitialized ) 
	{
		sBuf.Format( "FA Switch [%s] ReadConfiguration Failed.", m_sName );
		AfxMessageBox( sBuf );
	}

	return TRUE;
}

BOOL CFaSwitch::Get(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDi[nNo]->IsDeviceLink() ) ) return FALSE;

	return ( m_pDi[nNo]->Get() );
}

void CFaSwitch::SetState(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDiCount ) ||
		( !m_pDi[nNo]->IsDeviceLink() ) ) return;
	
	m_pDi[nNo]->SetState( bVal );
}

void CFaSwitch::Set(int nNo, BOOL bVal)
{
	if( !IsInitialized() ||
		( nNo >= m_nDoCount ) ||
		( !m_pDo[nNo]->IsDeviceLink() ) ) return;

	m_pDo[nNo]->Set( bVal );
}

BOOL CFaSwitch::GetState(int nNo)
{
	if( !IsInitialized() ||
		( nNo >= m_nDoCount ) ||
		( !m_pDo[nNo]->IsDeviceLink() ) ) return FALSE;

	return m_pDo[nNo]->Get();
}
