// FileMap.cpp: implementation of the CFileMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileMap.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CFileMap::CFileMap()
{
	m_bMap = FALSE;
	m_hMapFile = NULL;
}

CFileMap::~CFileMap()
{
}

BOOL CFileMap::CreateFileMap()
{
//-----------------------------------------------------------------------------
    BOOL fInit; 
 
	// Create a named file mapping object.
    m_hMapFile = CreateFileMapping(	INVALID_HANDLE_VALUE, // use paging file
									NULL,                 // no security attributes
									PAGE_READWRITE,       // read/write access
									0,                    // size: high 32-bits
									SHMEMSIZE,            // size: low 32-bits
									"dllmemfilemap");     // name of map object
	if (m_hMapFile == NULL)
	{
		m_bMap = FALSE;
		return FALSE; 
	}

    // The first process to attach initializes memory.
	fInit = (GetLastError() != ERROR_ALREADY_EXISTS); 
	
	// Get a pointer to the file-mapped shared memory.
    m_lpMapAddress = MapViewOfFile(	m_hMapFile,				// object to map view of
									FILE_MAP_ALL_ACCESS,	// read/write access
									0,						// high offset:  map from
									0,						// low offset:   beginning
									0);						// default: map entire file
	if (m_lpMapAddress == NULL)
	{
		m_bMap = FALSE;
		return FALSE; 
	}
 
    // Initialize memory if this is the first process.
    if (fInit) memset(m_lpMapAddress, '1', SHMEMSIZE);
	
//-----------------------------------------------------------------------------
	m_bMap = TRUE;
	return TRUE;
}

void CFileMap::CloseFileMap()
{
	if (!UnmapViewOfFile(m_lpMapAddress)) 
	{ 
		AfxMessageBox("Could not unmap view of file."); 
	}
	if( m_hMapFile != NULL )
		CloseHandle(m_hMapFile); 

	m_bMap = FALSE;
}

int CFileMap::OpenFileMap()
{
	m_hMapFile = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, "dllmemfilemap"); 
	if(m_hMapFile == NULL) 
	{
		m_bMap = FALSE;
		ErrorHandler("Could not open file-mapping object.");
		return -1;
	} 
 
	m_lpMapAddress = MapViewOfFile(m_hMapFile, FILE_MAP_ALL_ACCESS, 0, 0, 0);
	if(m_lpMapAddress == NULL) 
	{
		m_bMap = FALSE;
		ErrorHandler("Could not map view of file.");
		return -1;
	} 

	m_bMap = TRUE;
	return 0;
}

void CFileMap::ErrorHandler(LPCTSTR lpszText)
{
	AfxMessageBox(lpszText);
}

unsigned short CFileMap::ReceiveBit(short nDevNo)
{
	if( !m_bMap ) return FALSE;

	unsigned short nRv=0;
	if( *( (LPDWORD) m_lpMapAddress + nDevNo ) == 1 ) nRv = TRUE;
	else nRv = FALSE;
	return nRv;
}

BOOL CFileMap::SendBit(short nDevNo, short nData)
{
	if( !m_bMap ) return FALSE;

	if( nDevNo < 0 ) return FALSE;
	if( nData > 0 )	*( (LPDWORD) m_lpMapAddress + nDevNo ) = TRUE;
	else *( (LPDWORD) m_lpMapAddress + nDevNo ) = FALSE;
	return TRUE;
}

unsigned short CFileMap::ReceiveWord(short nDevNo)
{
	if( !m_bMap ) return FALSE;

	unsigned short nRv = (unsigned short)*( (LPDWORD) m_lpMapAddress + nDevNo );
	return nRv;
}

BOOL CFileMap::SendWord(short nDevNo, unsigned short nData)
{
	if( !m_bMap ) return FALSE;

	if( nDevNo < 0 ) return FALSE;
	*( (LPDWORD) m_lpMapAddress + nDevNo ) = nData;
	return TRUE;
}

//////////////////////////////////////////////////////////////////
// ReadString(D007000, lpszBuf, 8)
// sStr    = 32 31 34 33 62 61 64 63
// lpszBuf = "1234abcd"
short CFileMap::ReadString(short nDevNo, LPTSTR lpszBuf, int nLen)
{
	if( !m_bMap ) return FALSE;

	// Convert to string
	int nWord = nLen / 2 + nLen % 2;
	CString sStr;

	for( int i=0; i<nWord; i++ )
	{
		unsigned short nData = (unsigned short)*( (LPDWORD) m_lpMapAddress + nDevNo + i );
		char ch;
		ch = nData & 0x00FF;
		if( ch == 0 ) ch = ' ';
		sStr += ch;
		ch = ( nData >> 8 ) & 0x00FF;
		if( ch == 0 ) ch = ' ';
		sStr += ch;
	}

	strncpy( lpszBuf, sStr.GetBuffer(nLen), nLen );
	sStr.ReleaseBuffer();
	lpszBuf[nLen] = '\x0';
	return 1;
}

//////////////////////////////////////////////////////////////////
// WriteString(D007000, lpszBuf, 8)
// sStr	 = "1234abcd"
// nData = 32 31 34 33 62 61 64 63
short CFileMap::WriteString(short nDevNo, CString sStr, int nLen)
{
	if( !m_bMap ) return FALSE;

	// Find the address
	int nRv;
	int nWord = nLen / 2 + nLen % 2;

	for( int i=0; i < nWord; i++ )
	{
		short nSize = 2;
		unsigned short nData;
		char ch1, ch2;

		if( sStr.GetLength() <= i*2 ) ch1 = 0;
		else ch1 = sStr.GetAt( i*2 );

		if( sStr.GetLength() <= i*2 + 1 ) ch2 = 0;
		else ch2 = sStr.GetAt( i*2 + 1 );

		if( ch1 == ' ' ) ch1 = 0x20;	// Space ״ ...........
		if( ch2 == ' ' ) ch2 = 0x20;
		if( ch1 == '\r' ) ch1 = 0x00;	// Space ״ ...........
		if( ch2 == '\r' ) ch2 = 0x00;

		nData = ( ch2 << 8 ) | ch1;

		if( ( nRv = SendWord( nDevNo+i, nData ) ) == 0 )
		{
			CString sError;
			sError.Format("Device Number[%d] is not registered !");
			return 0;
		}
	}
	return 1;		
}
