// DicsCommonInterlockSeq.cpp: implementation of the CDicsCommonInterlockSeq class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "DicsCommonInterlockSeq.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDicsCommonInterlockSeq::CDicsCommonInterlockSeq()
{

}

CDicsCommonInterlockSeq::~CDicsCommonInterlockSeq()
{

}


void CDicsCommonInterlockSeq::SetOwnerPointer(CWnd* pWnd)
{
	SetDicsView( (CDicsView *) pWnd );
	SetDicsData( m_pDicsView->m_pDicsData );
	SetDevice( m_pDicsView->m_pDevice );
	SetSetupItems( m_pDicsView->GetDocument()->m_pSetupItems );
	SetEqpStateSequence( m_pDicsView->m_pEqpStateSeq );
	SetSeqTimer( m_pDicsView->m_pTimer );
	SetMmc( m_pDicsView->m_pMmc );
	SetEqpAssy( m_pDicsView->m_pEqpAssy );
}

void CDicsCommonInterlockSeq::SetLog( CString sSeqName, int nPortNo, int nSlotNo, CString sMessage )
{
	CString sLog("");
	CString sPortNo;
	if( nPortNo <= 0 ) sPortNo = "";
	else sPortNo.Format( "%X", nPortNo );

	CString sSlotNo;
	if( nSlotNo <= 0 ) sSlotNo = "";
	else sSlotNo.Format( "%X", nSlotNo );

	sLog.Format( "INTR    \t%s\t%s\t%s\t%s", sSeqName, sPortNo, sSlotNo, sMessage );
	Log.TextOut( sLog );

	if( m_pDicsData->m_GenInfo.bEnableEqpLog )
	{
		sLog.Format( "%s : %s", sSeqName, sMessage );
		Msg( WM_EQP_LOG, 0, (LPARAM)(LPCTSTR) sLog );
	}
}

BOOL CDicsCommonInterlockSeq::Initialize()
{	
	InitParameter();

	return TRUE;
}

void CDicsCommonInterlockSeq::InitParameter()
{
	m_nSeqEmoNo			= 0;
	m_nSeqLeakSensorNo	= 0;
	m_nSeqCoverSensorNo	= 0;
	m_nSeqDoorSensorNo	= 0;
	m_nSeqIonizerCondNo = 0;
	m_nSeqFfuCondNo		= 0;

	for( int i=0; i<MAX_PARTS_NO; i++ )
	{
		m_nSeqPartsCondNo[i]	= 0;
		m_nSeqPartsUseTimeNo[i] = 0;
	}

	m_pDicsData->m_GenInfo.bEmo				= FALSE;
	m_pDicsData->m_GenInfo.bAreaSensor		= FALSE;
	m_pDicsData->m_GenInfo.bCoverOpen		= FALSE;
	m_pDicsData->m_GenInfo.bDoorOpen		= FALSE;
	m_pDicsData->m_GenInfo.bLeakSensor		= FALSE;
	m_pDicsData->m_GenInfo.bIonizerAlarm	= FALSE;
	m_pDicsData->m_GenInfo.bFfuAlarm		= FALSE;	

	for( int i = 0; i < MAX_CV_NO; i++  )
	{
		m_Interlock.bCvMotorCond[i]	= TRUE;
	}

	CDicsApp* pApp = (CDicsApp*)AfxGetApp();	
	CString sEntryName;
	for( int i = 0; i < MAX_PARTS_NO; i++ )
	{
		m_PartsData[i].bPartsCond = TRUE;
		sEntryName.Format( "%s GLS NO", m_pSetupItems->m_infoPartsLifeTimePara[i].sName );
		m_PartsData[i].nGlsCount = pApp->GetProfileInt( "DICS SET", sEntryName, 0 );
		sEntryName.Format( "%s USED TIME", m_pSetupItems->m_infoPartsLifeTimePara[i].sName );
		m_PartsData[i].nUsedTime = pApp->GetProfileInt( "DICS SET", sEntryName, 0 );
	}
}

BOOL CDicsCommonInterlockSeq::Sequence()
{
	Sleep( 10 );

	SeqEMO();
	SeqLeakSensor();
	SeqCoverSensor();
	SeqDoorSensor();
	SeqCvMotorCondition();
	SeqIonizerCondition();
	SeqFfuCondition();

	for( int i=0; i<MAX_PARTS_NO; i++ )
	{
		SeqPartsUseTime( i );
		SeqPartsCondition( i );
	}

	return TRUE;
}


BOOL CDicsCommonInterlockSeq::GetCvCond( int nUnitNo )
{
	return m_Interlock.bCvMotorCond[nUnitNo];
}

//---------------------------------------------------------------------------
// Description :
//---------------------------------------------------------------------------
void CDicsCommonInterlockSeq::SeqEMO()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
 
	int nSeqNo = m_nSeqEmoNo;

	static BOOL bAuto;
	static BOOL bServoUse[MAX_RBT_NO] = { FALSE, };
	static int nServoNo = 0;
	static int nAlarmId = 0;

	m_sSeqName.Format( "%s", "SEQ EMO " );

/////////////// Loader EMO 101108 halesu //////////////////////
#ifndef _MY_DEBUG
	BOOL 	bLoaderEmo = FALSE;
	static DWORD dwStartTick = GetTickCount();
	static BOOL bInit = FALSE;
	
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
	for( int i=0; i<m_pEqpAssy->m_SwitchEmo.GetDiSize(); i++ )
	{
		if( m_pEqpAssy->m_SwitchEmo.Get(i) )
		{
			bLoaderEmo = TRUE;
			break;
		}	
	}
#ifndef _TYPE7
	if( bLoaderEmo )
	{
		if( !m_pEqpAssy->m_LoaderEmo.GetState() )
		{ 
			m_pEqpAssy->m_LoaderEmo.Set( 0, ON );
			SetLog( m_sSeqName, 0, 0, "Loader EMO ON" );
		}	
	}
	else
	{
		if( m_pEqpAssy->m_LoaderEmo.GetState() ) 
		{
			m_pEqpAssy->m_LoaderEmo.Set( 0, OFF );
			SetLog( m_sSeqName, 0, 0, "Loader EMO OFF" );
		}
	}
#endif
#endif
////////////////////////////////////////////////////////////////

	switch( nSeqNo )
	{
	case 0:
		TIMER_ON( TM_SEQ_EMO, 10 ); // 1 sec Delay...
		nSeqNo = 1;
		break;

	case 1:
		if( TIMER( TM_SEQ_EMO ) ) nSeqNo = 10;
		break;

	case 10:
		{
			BOOL 	bEmo = FALSE;
			CString sTmp;

#ifdef _MY_DEBUG
			bEmo 	 = FALSE;
			nAlarmId = ALM_EMO1;
			if( bEmo ) m_pEqpStateSeq->SetAlarm( nAlarmId );
#else			
			for( int i=0; i<m_pEqpAssy->m_SwitchEmo.GetDiSize(); i++ )
			{
				if( m_pEqpAssy->m_SwitchEmo.Get(i) )
				{
					nAlarmId = ALM_EMO1 + i;
					m_pEqpStateSeq->SetAlarm( nAlarmId );

					bEmo = TRUE;
					sTmp.Format( "EMO %d Pressed", i+1 );
					SetLog( m_sSeqName, 0, 0, sTmp );	
				}
			}
			for( int i=0; i<m_pEqpAssy->m_SwitchEms.GetDiSize(); i++ )
			{
				if( m_pEqpAssy->m_SwitchEms.Get(i) )
				{
					nAlarmId = ALM_EMS1 + i;
					m_pEqpStateSeq->SetAlarm( nAlarmId );

					bEmo = TRUE;
					sTmp.Format( "EMS %d Pressed", i+1 );
					SetLog( m_sSeqName, 0, 0, sTmp );						
				}
			}
#endif			
			if( bEmo ) 
			{
				nSeqNo = 20;
			}
#ifndef _MY_DEBUG
			else if( !m_pEqpAssy->m_PowerConfirm.GetOr() )
#else
			else if( FALSE )
#endif
			{
				nAlarmId = ALM_POWER_OFF;
				m_pEqpStateSeq->SetAlarm( nAlarmId );
				SetLog( m_sSeqName, 0, 0, "Power On Switch OFF" );	
				nSeqNo = 20;
			}
			else 
			{
				m_pDicsData->m_GenInfo.bEmo = FALSE;
			}
		}	
		break;

	case 20 :
		{
			m_pDicsData->m_GenInfo.bEmo = TRUE;
			SetLog( m_sSeqName, 0, 0, "C/V All Stop" );
			
			nSeqNo = 30;
		}
		break;

	case 30 :
		if( m_pMmc->EStopRbt( nServoNo ) )
		{
			// Set Log
			m_sMessage.Format( "%2d Servo motor off", nServoNo + 1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );

			// Go to the next servo motor
			nServoNo++;
			if( nServoNo >= MAX_RBT_NO )
			{
				nServoNo = 0;
				nSeqNo = 1000;
			}
		}
		break;

	case 1000 :
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			BOOL 	bEmo = FALSE;

#ifdef _MY_DEBUG
			bEmo 	 = FALSE;
			nAlarmId = ALM_EMO1;
#else			
			if( !m_pEqpAssy->m_PowerConfirm.Get() )
			{
				bEmo = TRUE;
			}
			for( int i=0; i<m_pEqpAssy->m_SwitchEmo.GetDiSize(); i++ )
			{
				if( m_pEqpAssy->m_SwitchEmo.Get(i) )	bEmo = TRUE;
			}
			for( int i=0; i<m_pEqpAssy->m_SwitchEms.GetDiSize(); i++ )
			{
				if( m_pEqpAssy->m_SwitchEms.Get( i ) )	bEmo = TRUE;
			}
#endif			
			if( !bEmo )
			{
				if( m_pEqpStateSeq->IsAlarm( ALM_POWER_OFF ) )
				{
					m_pEqpStateSeq->ResetAlarm( ALM_POWER_OFF );
					SetLog( m_sSeqName, 0, 0, "Power On Switch ON" );		
				}
				for( int i=0; i<m_pEqpAssy->m_SwitchEmo.GetDiSize(); i++ )
				{
					if( m_pEqpStateSeq->IsAlarm( ALM_EMO1+i ) )
					{
						m_pEqpStateSeq->ResetAlarm( ALM_EMO1+i );

						CString sTmp;
						sTmp.Format( "EMO %d Released", i+1 );
						SetLog( m_sSeqName, 0, 0, sTmp );
					}
				}
				for( int i=0; i<m_pEqpAssy->m_SwitchEms.GetDiSize(); i++ )
				{
					if( m_pEqpStateSeq->IsAlarm( ALM_EMS1+i ) )
					{
						m_pEqpStateSeq->ResetAlarm( ALM_EMS1+i );

						CString sTmp;
						sTmp.Format( "EMS %d Released", i+1 );
						SetLog( m_sSeqName, 0, 0, sTmp );
					}
				}

//				for( i=0; i<MAX_RBT_NO; i++)
//				{
//					bServoUse[i]  = m_pDicsView->m_pMainSeq->m_SeqFlag.bInitComp;
//					bServoUse[i] &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDB_SERVO1_USE+i] ? 1 : 0;
//					bServoUse[i] &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] ? 1 : 0;
//				}
//				
//				nSeqNo = 110;

				TIMER_ON( TM_SEQ_EMO, 30 ); //3 sec Delay...
				nSeqNo = 120;
			}
		}
		break;
//	case 110 :
//		if( !bServoUse[nServoNo] || 
//		  ( m_pMmc->ResetRbt( nServoNo ) && 
//		    m_pMmc->m_bServoReady[nServoNo] ) )
//		{
//			if( bServoUse[nServoNo] ) m_sMessage.Format( "%2d servo motor on", nServoNo + 1 );
//			else	m_sMessage.Format( "%2d servo motor Reset Skip", nServoNo + 1 );
//
//			SetLog( m_sSeqName, 0, 0, m_sMessage );
//
//			// Go to the next servo motor
//			nServoNo++;
//			if( nServoNo >= MAX_RBT_NO )
//			{
//				TIMER_ON( TM_SEQ_EMO, 30 ); //3 sec Delay...
//
//				nServoNo = 0;
//				nSeqNo = 120;
//			}
//		}
//		break;
	case 120 :
		if( TIMER(TM_SEQ_EMO) ) 
		{
			nServoNo = 0;
			nSeqNo = 130;
		}
		break;

	case 130 :
		if( m_pMmc->ResetRbt( nServoNo ) )
		{
			// Set Log
			m_sMessage.Format( "%2d Servo motor Reset", nServoNo + 1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );

			// Go to the next servo motor
			nServoNo++;
			if( nServoNo >= MAX_RBT_NO )
			{
				nServoNo = 0;
				nSeqNo = 140;
			}
		}
		break;
		
	case 140 :
		{
			m_pDicsData->m_GenInfo.bEmo = FALSE;
			nSeqNo = 0;
		}		
		break;		
	}

	m_nSeqEmoNo = nSeqNo;
}

void CDicsCommonInterlockSeq::SeqLeakSensor()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

///////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
///////////////////////////////////////////////////////////////////////////////
	m_sSeqName.Format( "SEQ LEAK" );
	
	BOOL bNg = FALSE;
	BOOL bInterlock = FALSE;
 	static int nAlarmId[MAX_LEAK_SENSOR_NO]	= { 0, };

///////////////////////////////////////////////////////////////////////////////
	if( !bInterlock && !m_pEqpAssy->m_SensorLeak.GetOr() ) return;
///////////////////////////////////////////////////////////////////////////////

	for( int i=0; i<m_pEqpAssy->m_SensorLeak.GetDiSize(); i++ )
	{
		int nIndex = CSetupItems::intrLEAK_SENSOR1+i;
#ifndef _MY_DEBUG
		bNg = m_pSetupItems->m_nValueLeakDetectIntr[nIndex] & m_pEqpAssy->m_SensorLeak.Get(i);
#else
		bNg = 0;
#endif

		if( bNg && ( nAlarmId[i] == 0 ) )
		{
			nAlarmId[i] = ALM_LEAK1 + i;
			m_pEqpStateSeq->SetAlarm( nAlarmId[i] );

			m_sMessage.Format( "%s alarm set", m_pSetupItems->m_sItemLeakDetectIntr[nIndex] );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
		}
		else if( ( nAlarmId[i] != 0 ) && m_pEqpStateSeq->IsPushedAlarmReset() && !bNg )
		{
			m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
			nAlarmId[i] = 0;
			m_sMessage.Format( "%s alarm reset", m_pSetupItems->m_sItemLeakDetectIntr[nIndex] );
			SetLog( m_sSeqName, 0, 0, m_sMessage );					
		}

		bInterlock |= ( nAlarmId[i] !=0 ? TRUE : FALSE );
	}

	if( bInterlock && !m_pDicsData->m_GenInfo.bLeakSensor )
	{
		m_pDicsData->m_GenInfo.bLeakSensor = TRUE;
	}
	else if( !bInterlock && m_pDicsData->m_GenInfo.bLeakSensor )
	{
		m_pDicsData->m_GenInfo.bLeakSensor = FALSE;
	}
}

//---------------------------------------------------------------------------
// Description :
//---------------------------------------------------------------------------
void CDicsCommonInterlockSeq::SeqDoorSensor()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

///////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
///////////////////////////////////////////////////////////////////////////////
	m_sSeqName.Format( "SEQ DOOR" );

	BOOL bNg = FALSE;
	BOOL bInterlock = FALSE;
	static int nAlarmId[MAX_DOOR_OPEN_NO] = { 0, };

///////////////////////////////////////////////////////////////////////////////
	if( !bInterlock && !m_pEqpAssy->m_SensorDoor.GetOr() ) return;
///////////////////////////////////////////////////////////////////////////////

	for( int i=0; i<m_pEqpAssy->m_SensorDoor.GetDiSize(); i++ )
	{
		int nIndex = CSetupItems::intrDOOR_OPEN1+i;
#ifndef _MY_DEBUG
		bNg = m_pSetupItems->m_nValueDoorIntr[nIndex] & m_pEqpAssy->m_SensorDoor.Get( i );
#else
		bNg = 0;
#endif

		if( bNg && ( nAlarmId[i] == 0 ) )
		{
			nAlarmId[i] = ALM_DOOR_OPEN1 + i;
			m_pEqpStateSeq->SetAlarm( nAlarmId[i] );

			m_sMessage.Format( "%s alarm set", m_pSetupItems->m_sItemDoorIntr[nIndex] );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
		}
		else if( ( nAlarmId[i] != 0 ) && m_pEqpStateSeq->IsPushedAlarmReset() && !bNg )
		{
			m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
			nAlarmId[i] = 0;
			m_sMessage.Format( "%s alarm reset", m_pSetupItems->m_sItemDoorIntr[nIndex] );
			SetLog( m_sSeqName, 0, 0, m_sMessage );					
		}

		bInterlock |= ( nAlarmId[i] !=0 ? TRUE : FALSE );
	}

	if( bInterlock && !m_pDicsData->m_GenInfo.bDoorOpen )
	{
		m_pDicsData->m_GenInfo.bDoorOpen = TRUE;
	}
	else if( !bInterlock && m_pDicsData->m_GenInfo.bDoorOpen )
	{
		m_pDicsData->m_GenInfo.bDoorOpen = FALSE;	
	}
}

//---------------------------------------------------------------------------
// Description :
//---------------------------------------------------------------------------
void CDicsCommonInterlockSeq::SeqCoverSensor()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

///////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
///////////////////////////////////////////////////////////////////////////////
	m_sSeqName.Format( "SEQCOVER" );

	BOOL bNg = FALSE;
	BOOL bInterlock = FALSE;
	static int nAlarmId[MAX_COVER_OPEN_NO] = { 0, };

///////////////////////////////////////////////////////////////////////////////
	if( !bInterlock && !m_pEqpAssy->m_SensorCover.GetOr() ) return;
///////////////////////////////////////////////////////////////////////////////

	for( int i=0; i<m_pEqpAssy->m_SensorCover.GetDiSize(); i++ )
	{
		int nIndex = CSetupItems::intrCOVER_OPEN1+i;
#ifndef _MY_DEBUG
		bNg = m_pSetupItems->m_nValueBathCoverIntr[nIndex] & m_pEqpAssy->m_SensorCover.Get(i);
#else
		bNg = 0;
#endif

		if( bNg && ( nAlarmId[i] == 0 ) )
		{
			nAlarmId[i] = ALM_COVER_OPEN1 + i;
			m_pEqpStateSeq->SetAlarm( nAlarmId[i] );

			m_sMessage.Format( "%s alarm set", m_pSetupItems->m_sItemBathCoverIntr[nIndex] );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
		}
		else if( ( nAlarmId[i] != 0 ) && m_pEqpStateSeq->IsPushedAlarmReset() && !bNg )
		{
			m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
			nAlarmId[i] = 0;
			m_sMessage.Format( "%s alarm reset", m_pSetupItems->m_sItemBathCoverIntr[nIndex] );
			SetLog( m_sSeqName, 0, 0, m_sMessage );					
		}

		bInterlock |= ( nAlarmId[i] !=0 ? TRUE : FALSE );
	}

	if( bInterlock && !m_pDicsData->m_GenInfo.bCoverOpen )
	{
		m_pDicsData->m_GenInfo.bCoverOpen = TRUE;
	}
	else if( !bInterlock && m_pDicsData->m_GenInfo.bCoverOpen )
	{
		m_pDicsData->m_GenInfo.bCoverOpen = FALSE;	
	}
}

void CDicsCommonInterlockSeq::SeqCvMotorCondition()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

///////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();

	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 3000 )	bInit = TRUE;
		if( !bInit ) return;
	}
///////////////////////////////////////////////////////////////////////////////

	BOOL bNg[2] = { FALSE, };
	static int nAlarmId[MAX_CV_MOTOR_NO*2] = { 0, };

	for( int i=0; i<m_pEqpAssy->m_CvMotorAll.GetMaxSectionNo(); i++ )
	{
#ifndef _MY_DEBUG
		bNg[0] = m_pEqpAssy->m_CvMotorAll.GetMotorCpAlarm( i );
		bNg[1] = m_pEqpAssy->m_CvMotorAll.GetMotorAlarm( i );
#endif

		if( bNg[0] && ( nAlarmId[i*2 + 0] == 0 ) )
		{
			m_Interlock.bCvMotorCond[i] = FALSE;
			nAlarmId[i*2 + 0] = ALM_CV_MOTOR_DRIVER_CP_OFF1 + i;
			m_pEqpStateSeq->SetAlarm( ALM_CV_MOTOR_DRIVER_CP_OFF1 + i );

#ifndef _MY_DEBUG
			m_pDicsData->m_GenInfo.bCycleStop = TRUE;
#endif
		}
		else if( bNg[1] && ( nAlarmId[i*2 + 1] == 0 ) )
		{
			m_Interlock.bCvMotorCond[i] = FALSE;
			nAlarmId[i*2 + 1] = ALM_CV_MOTOR_DRIVER_ALARM1 + i;

			m_pEqpStateSeq->SetAlarm( ALM_CV_MOTOR_DRIVER_ALARM1 + i );
#ifndef _MY_DEBUG
			m_pDicsData->m_GenInfo.bCycleStop = TRUE;
#endif
		}
		else if( !m_Interlock.bCvMotorCond[i] && 
				m_pEqpStateSeq->IsPushedAlarmReset() &&
				!bNg[0] && !bNg[1] )
		{
			m_Interlock.bCvMotorCond[i] = TRUE;
			if( nAlarmId[i*2 + 0] != 0 )
			{
				m_pEqpStateSeq->ResetAlarm( nAlarmId[i*2 + 0] );
				nAlarmId[i*2 + 0] = 0;
			}
			if( nAlarmId[i*2 + 1] !=0 )
			{
				m_pEqpStateSeq->ResetAlarm( nAlarmId[i*2 + 1] );
				nAlarmId[i*2 + 1] = 0;
			}
		}
	}	
}


//---------------------------------------------------------------------------
// Description :
//---------------------------------------------------------------------------
void CDicsCommonInterlockSeq::SeqIonizerCondition()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

///////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
///////////////////////////////////////////////////////////////////////////////
	m_sSeqName.Format( "SEQ ION " );
	
	BOOL bNg = FALSE;
	BOOL bInterlock = FALSE;
 	static int nAlarmId[MAX_IONIZER_NO][3]	= { 0, };

///////////////////////////////////////////////////////////////////////////////
	if( !bInterlock && !m_pEqpAssy->m_SignalIonizer.GetOr() ) return;
///////////////////////////////////////////////////////////////////////////////

	for( int i=0; i<m_pEqpAssy->m_SignalIonizer.GetDiSize(); i++ )
	{
		int nIndex = CSetupItems::intrIONIZER1 + i;

		for( int k=0; k<3; k++ )
		{
			bNg = m_pSetupItems->m_nValueETCIntr[nIndex] && m_pEqpAssy->m_SignalIonizer.Get( ( i*3 + k ) );

			if( bNg && ( nAlarmId[i][k] == 0 ) )
			{
				nAlarmId[i][k] = ALM_IONIZER_ALARM1 + i*3 + k;
				m_pEqpStateSeq->SetAlarm( nAlarmId[i][k] );

				m_sMessage.Format( "%s alarm set", m_pSetupItems->m_sItemETCIntr[nIndex] );
				SetLog( m_sSeqName, 0, 0, m_sMessage );
				m_pDicsData->m_EqpInfo.cIonizerState = 'D';
				m_pDicsData->m_IfCim.bIonizerStatusChangeReq = TRUE;
			}
#ifndef _MY_DEBUG
			else if( ( nAlarmId[i][k] != 0 ) && m_pEqpStateSeq->IsPushedAlarmReset() && !bNg )
#else
			else if ( ( nAlarmId[i][k] != 0 ) && m_pEqpStateSeq->IsPushedAlarmReset() )
#endif
			{
				m_pEqpStateSeq->ResetAlarm( nAlarmId[i][k] );
				nAlarmId[i][k] = 0;
				m_sMessage.Format( "%s alarm reset", m_pSetupItems->m_sItemETCIntr[nIndex] );
				SetLog( m_sSeqName, 0, 0, m_sMessage );	
				m_pDicsData->m_EqpInfo.cIonizerState = 'R';
				m_pDicsData->m_IfCim.bIonizerStatusChangeReq = TRUE;
			}

			bInterlock |= ( nAlarmId[i][k] !=0 ? TRUE : FALSE );
		}
	}

	if( bInterlock && !m_pDicsData->m_GenInfo.bIonizerAlarm )
	{
		m_pDicsData->m_GenInfo.bIonizerAlarm = TRUE;
#ifndef _MY_DEBUG
		m_pDicsData->m_GenInfo.bCycleStop = TRUE;
#endif		
	}
	else if( !bInterlock && m_pDicsData->m_GenInfo.bIonizerAlarm )
	{
		m_pDicsData->m_GenInfo.bIonizerAlarm = FALSE;	
	}
}

//---------------------------------------------------------------------------
// Description :
//---------------------------------------------------------------------------
void CDicsCommonInterlockSeq::SeqFfuCondition()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

///////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
///////////////////////////////////////////////////////////////////////////////
	m_sSeqName.Format( "SEQ FFU " );
	
	BOOL bNg = FALSE;
	BOOL bInterlock = FALSE;
 	static int nAlarmId[MAX_FFU_NO][2]	= { 0, };

///////////////////////////////////////////////////////////////////////////////
	if( !bInterlock && !m_pEqpAssy->m_SignalFfu.GetOr() ) return;
///////////////////////////////////////////////////////////////////////////////

	for( int i=0; i<m_pEqpAssy->m_SignalFfu.GetDiSize(); i++ )
	{
		int nIndex = CSetupItems::intrFFU1 + i;

		for( int k=0; k<2; k++ )
		{
			bNg = m_pSetupItems->m_nValueFFUAlarmIntr[nIndex] && m_pEqpAssy->m_SignalFfu.Get( ( i*2 + k ) );

			if( bNg && ( nAlarmId[i][k] == 0 ) )
			{
				nAlarmId[i][k] = ALM_FFU_CP_TURN_OFF1 + i*2 + k;
				m_pEqpStateSeq->SetAlarm( nAlarmId[i][k] );

				m_sMessage.Format( "%s alarm set", m_pSetupItems->m_sItemFFUAlarmIntr[nIndex] );
				SetLog( m_sSeqName, 0, 0, m_sMessage );
			}
			else if( ( nAlarmId[i][k] != 0 ) && m_pEqpStateSeq->IsPushedAlarmReset() && !bNg )
			{
				m_pEqpStateSeq->ResetAlarm( nAlarmId[i][k] );
				nAlarmId[i][k] = 0;
				m_sMessage.Format( "%s alarm reset", m_pSetupItems->m_sItemFFUAlarmIntr[nIndex] );
				SetLog( m_sSeqName, 0, 0, m_sMessage );					
			}

			bInterlock |= ( nAlarmId[i][k] !=0 ? TRUE : FALSE );
		}
	}

	if( bInterlock && !m_pDicsData->m_GenInfo.bFfuAlarm )
	{
		m_pDicsData->m_GenInfo.bFfuAlarm = TRUE;
#ifndef _MY_DEBUG
		m_pDicsData->m_GenInfo.bCycleStop = TRUE;
#endif		
	}
	else if( !bInterlock && m_pDicsData->m_GenInfo.bFfuAlarm )
	{
		m_pDicsData->m_GenInfo.bFfuAlarm = FALSE;	
	}
}

void CDicsCommonInterlockSeq::SeqPartsUseTime( int nPartNo )
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	int i = nPartNo;
	int nSeqNo = m_nSeqPartsUseTimeNo[i];

	DWORD dwPeriod = 1*60*1000;	// 1 min

	static DWORD dwTickStart[MAX_PARTS_NO] = { 0, };
	static int	 nTime[MAX_PARTS_NO]	= { 0, };

	m_sSeqName.Format( "%s", "SEQPARTS" );

	switch( nSeqNo )
	{
	case 0:
		dwTickStart[i] = GetTickCount(); 
		nSeqNo = 1;
		break;

	case 1:
		if( GetTickCount() - dwTickStart[i] > dwPeriod ) nSeqNo = 10;	
		break;

	case 10:
		{
			if( GetPartsCheckCond(i) )
			{
				nTime[i]++;
#ifndef _MY_DEBUG
				if( nTime[i] * dwPeriod >= 60*60*1000 )
#else
				if( nTime[i] * dwPeriod >= 60*1000 )
#endif
				{ 	// 1Hour  
					PartsUsedTimeIncrease( i );
					nTime[i] = 0;
				}
			}

			nSeqNo = 0;
		}
		break;
	}

	m_nSeqPartsUseTimeNo[i] = nSeqNo;
}

void CDicsCommonInterlockSeq::SeqPartsCondition( int nPartNo )
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	int i = nPartNo;
	int nSeqNo = m_nSeqPartsCondNo[i];

	static DWORD dwTickStart[MAX_PARTS_NO] = { 0, };
	static int 	 nAlarmId[MAX_PARTS_NO] = { 0, };
	m_sSeqName.Format( "%s", "SEQPARTS" );

	switch( nSeqNo )
	{
	case 0:
		dwTickStart[i] = GetTickCount(); 
		nSeqNo = 1;
		break;

	case 1:
		if( GetTickCount() - dwTickStart[i] > 1000 ) nSeqNo = 10;	
		break;

	case 10:
		{
			if( GetPartsCond(i) )
			{
				nSeqNo = 0;
			}
			else
			{
				nAlarmId[i] = ALM_PARTS_LIFE_TIME_OVER1 + i;
				m_pEqpStateSeq->SetAlarm( nAlarmId[i] );
				m_sMessage.Format( "%s Life Time Over Warning", m_pSetupItems->m_infoPartsLifeTimePara[i].sName );
				SetLog( m_sSeqName, 0, 0, m_sMessage );

#ifndef _MY_DEBUG
				m_pDicsData->m_GenInfo.bCycleStop = TRUE;
#endif
				nSeqNo = 1000;
			}
		}
		break;

	case 1000 :
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			SetLog( m_sSeqName, 0, 0, "Pushed AlarmReset" );					

			if( !GetPartsCond(i) )
			{
				SetLog( m_sSeqName, 0, 0, "Warning Not Cleared" );
				dwTickStart[i] = GetTickCount();
#ifndef _MY_DEBUG
				m_pDicsData->m_GenInfo.bCycleStop = TRUE;
#endif
				nSeqNo = 1100;
			}
			else
			{
				m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
				m_sMessage.Format( "%s Life Time Over Reset", m_pSetupItems->m_infoPartsLifeTimePara[i].sName );
				SetLog( m_sSeqName, 0, 0, m_sMessage );
			
				nSeqNo = 0;
			}
		}
		break;

	case 1100:
		if( ( GetTickCount() - dwTickStart[i] ) >= (DWORD)( 2 * 1000 ) )
		{
			nSeqNo = 1000;
		}
		break;
	}

	m_nSeqPartsCondNo[i] = nSeqNo;
}

int CDicsCommonInterlockSeq::GetPartsUsedTime( int nPartsNo )
{
	return m_PartsData[nPartsNo].nUsedTime;
}

int CDicsCommonInterlockSeq::GetPartsGlsCount( int nPartsNo )
{
	return m_PartsData[nPartsNo].nGlsCount;
}

void CDicsCommonInterlockSeq::ResetPartsData( int nPartsNo )
{
	m_IniSetUpInfo.Open("DicsSet", "GenInfo");
	m_PartsData[nPartsNo].nGlsCount = 0;
	m_PartsData[nPartsNo].nUsedTime = 0;

	CDicsApp* pApp = (CDicsApp*)AfxGetApp();	
	CString sEntryName;
	sEntryName.Format( "%s GLS NO", m_pSetupItems->m_infoPartsLifeTimePara[nPartsNo].sName );
	m_IniSetUpInfo.SetIntValue( "DICS SET", sEntryName, m_PartsData[nPartsNo].nGlsCount, TRUE );

	sEntryName.Format( "%s USED TIME", m_pSetupItems->m_infoPartsLifeTimePara[nPartsNo].sName );
	m_IniSetUpInfo.SetIntValue( "DICS SET", sEntryName, m_PartsData[nPartsNo].nUsedTime, TRUE );
}

void CDicsCommonInterlockSeq::PartsGlsCountIncrease( int nPartsNo )
{
	m_IniSetUpInfo.Open("DicsSet", "GenInfo");
	if( GetPartsCheckCond(nPartsNo) )
	{
		m_PartsData[nPartsNo].nGlsCount++;
	
		CDicsApp* pApp = (CDicsApp*)AfxGetApp();	
		CString sEntryName;
		sEntryName.Format( "%s GLS NO", m_pSetupItems->m_infoPartsLifeTimePara[nPartsNo].sName );
		m_IniSetUpInfo.SetIntValue( "DICS SET", sEntryName, m_PartsData[nPartsNo].nGlsCount, TRUE );
	}
}

void CDicsCommonInterlockSeq::PartsUsedTimeIncrease( int nPartsNo )
{
	m_IniSetUpInfo.Open("DicsSet", "GenInfo");
	if( GetPartsCheckCond(nPartsNo) )
	{
		m_PartsData[nPartsNo].nUsedTime++;

		CDicsApp* pApp = (CDicsApp*)AfxGetApp();	
		CString sEntryName;
		sEntryName.Format( "%s USED TIME", m_pSetupItems->m_infoPartsLifeTimePara[nPartsNo].sName );
		m_IniSetUpInfo.SetIntValue( "DICS SET", sEntryName, m_PartsData[nPartsNo].nUsedTime, TRUE );
	}
}

BOOL CDicsCommonInterlockSeq::GetPartsCond( int nPartsNo )
{
	int nVal[4];
	nVal[0]	= GetPartsGlsCount( nPartsNo );
	nVal[1]	= m_pSetupItems->m_nValuePartsLifeTimePara[0][nPartsNo];
	nVal[2]	= GetPartsUsedTime( nPartsNo );
	nVal[3]	= m_pSetupItems->m_nValuePartsLifeTimePara[1][nPartsNo];
			
	if( nVal[0] > nVal[1] ||
		nVal[2] > nVal[3] )
	{
		m_PartsData[nPartsNo].bPartsCond = FALSE;
	}
	else
	{
		m_PartsData[nPartsNo].bPartsCond = TRUE;
	}

	return m_PartsData[nPartsNo].bPartsCond;
}

BOOL CDicsCommonInterlockSeq::GetPartsCheckCond( int nPartsNo )
{
	BOOL bCheck = TRUE;

	switch( nPartsNo ) 
	{
	case CSetupItems::partsSWR2RB_1:
		bCheck &= m_pDicsView->m_pRbSeq->IsRbMotorRun( 0 );
		break;
	default:
		break;
	}

	return bCheck;
}
