// DicsEuvSeq.h: interface for the CDicsEuvSeq class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DicsEuvSeq_H__D5D7DED8_7726_47C1_A569_2EC5F0FA4BC0__INCLUDED_)
#define AFX_DicsEuvSeq_H__D5D7DED8_7726_47C1_A569_2EC5F0FA4BC0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//////////////////////////////////////////////////////////////////////
// Header File  Sequence->Data->X File ...
//   BOOL -> int -> float -> double -> WORD -> DWORD -> Char -> CString -> enum
// ٸ      ִ.. 
//////////////////////////////////////////////////////////////////////
#include "..\Sequence\EqpStateSequence.h"

#include "..\DicsData.h"
#include "..\DicsDoc.h"
#include "..\View\SetupItems.h"

#include "..\X\xsequence.h"
#include "..\X\XCtlDevice.h"
#include "..\X\XTimer.h"
#include "..\FactoryLib\EqpAssy.h"
#include "..\FactoryLib\IniData.h"

///////////////////////////////////////////////////////////////////
// property
// ----------------------------------------------------------------
// 1. Lamp State ( ON / OFF )
// 2. EUV Mode ( Auto / Manual )
// 3. EUV Alarm ( Alarm )  
// 4. UT State
// 5. House State
// 6. Lamp Intensity Value
// ?. Lamp Life Time

///////////////////////////////////////////////////////////////////
// Methode
// ----------------------------------------------------------------
// 1. Lamp On Select ( for Lamp On )
// 2. Lamp Control ( Lamp On / Off )
// 3. Lamp Monitor Select ( for Intensity Check )
// 4. Lamp Intensity Check
// 5. EUV UT Control ( Valve On / Off ) 
// 6. GetLampOnState
// 7. IsRemoteMode
// 8. GetEuvAlarm
// 10. GetHouseState
// 11. GetIntensityValue
// ?. Get Lamp Life Time

class CDicsEuvSeq : public XSequence
{
public:
	void LampMonitorSelect(int nPos);
	CDicsView*			m_pDicsView;
	XCtlDevice*			m_pDevice;
	CEqpStateSequence*	m_pEqpStateSeq;
	CSetupItems*		m_pSetupItems;
	CDicsData*			m_pDicsData;
	XTimer*				m_pTimer;
	CEqpAssy*			m_pEqpAssy;
	CIniData			m_IniConfiguration;

	int m_nSeqInitNo;
	int m_nSeqEuvNo;
	int m_nSeqAlarmNo;
	int m_nSeqEuvProgressNo;
	int m_nSeqLampOnTimeNo[MAX_LAMP_NO];
	int m_nSeqTimeOverNo[MAX_LAMP_NO];
	int m_nSeqIntensityCheckNo[MAX_LAMP_INTENSITY_NO];
	
	CString				m_sMessage;

	// Define the Struct variables
	struct tagSeqFlag
	{
		BOOL bInitReq;					// if initialized, it's TRUE 
		BOOL bInitComp;					// if initialized, it's TRUE 
		BOOL bInitFail;
	} m_SeqFlag;

	struct tagIfEuv
	{
//		BOOL    bLampAlarm;
//		BOOL    bLampPause;
//		BOOL	bLampOnComp;
//		BOOL	bLampOnFail;
//		BOOL    bLampIntensityCheckComp[MAX_LAMP_INTENSITY_NO];
//		BOOL    bLampIntensityCheckFail[MAX_LAMP_INTENSITY_NO];
		int		nLampControl;
		BOOL    bLampIntensityCheckReq[MAX_LAMP_INTENSITY_NO];
		float	fLampIntensity[MAX_LAMP_INTENSITY_NO];
		UINT	nLampOnTime[MAX_LAMP_NO];	
		BOOL	bLampOnTimer;
	} m_IfEuv;

	enum euvLAMP_CONTROL
	{
		euvLAMP_MANUAL,
		euvLAMP_OFF,
		euvLAMP_PAUSE,
		euvLAMP_ON_KEEP,
		euvLAMP_ON,
	};

	CDicsEuvSeq();
	virtual ~CDicsEuvSeq();

	// Set pointer
	void SetOwnerPointer(CWnd* pWnd);
	void SetDicsView( CDicsView* pView )						{ m_pDicsView = pView; }
	void SetEqpStateSequence(CEqpStateSequence* pEqpStateSeq)	{ m_pEqpStateSeq = pEqpStateSeq; }
	void SetSetupItems( CSetupItems* pSetupItems )				{ m_pSetupItems = pSetupItems; }
	void SetData( CDicsData* pData )							{ m_pDicsData = pData; }
	void SetDevice( XCtlDevice* pDevice )						{ m_pDevice = pDevice; }
	void SetSeqTimer(XTimer* pTimer)							{ m_pTimer = pTimer; }
	void SetEqpAssy( CEqpAssy* pEqpAssy )						{ m_pEqpAssy = pEqpAssy; }

	BOOL Initialize();
	BOOL Sequence();
	void InitParameter();
	void SeqInit();
	void SetLog( CString sSeqName, int nPortNo, int nSlotNo, CString sMessage );
	
	void SeqEuv();
	void SeqLampOnTime( int i );
	void SeqTimeOver( int i );
	void SeqEuvProgress();
	void SeqUtControl();

	BOOL IsEuvUse();
	BOOL GetEuvCond();
	BOOL GetLampControl();
	UINT GetLampUsedTime( int nLampNo );
	void SetLampUsedTime( int nLampNo, int nType );
	virtual BOOL GetLampOnState();//2008.08.26 kimgun virtual .
	float GetLampIntensity( int nPos );
	BOOL GetLampOnSelected( int nLampNo );

	virtual void SeqLampIntensityCheck();
	virtual void SeqEuvAlarm();
	
	virtual void LampOnSelect( );
	virtual void LampControl( BOOL bOn );
	virtual void UtControl( BOOL bOn );

	virtual int	SetLampControl();
	virtual int	GetEuvAlarm();
//	virtual BOOL GetLampOnSelected( int nLampNo );
	BOOL IsRemoteMode();
	virtual BOOL GetHouseCloseState();
	virtual BOOL IsGlassExist();
	virtual BOOL GetUtState();
};

#endif // !defined(AFX_DicsEuvSeq_H__D5D7DED8_7726_47C1_A569_2EC5F0FA4BC0__INCLUDED_)
