// DicsGaugeInterlockSeq.cpp: implementation of the CDicsGaugeInterlockSeq class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "DicsGaugeInterlockSeq.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDicsGaugeInterlockSeq::CDicsGaugeInterlockSeq()
{
	m_nMaxIntroNo = 0;
}

CDicsGaugeInterlockSeq::~CDicsGaugeInterlockSeq()
{
	delete []m_Interlock.bGaugeAlarmSet;
	delete []m_Interlock.bGaugeWarningSet;
	delete []m_Interlock.bNg;
	delete []m_nSeqCheckInterlockNo;
}

void CDicsGaugeInterlockSeq::SetOwnerPointer(CWnd* pWnd)
{
	SetDicsView( (CDicsView *) pWnd );
	SetDicsData( m_pDicsView->m_pDicsData );
	SetDevice( m_pDicsView->m_pDevice );
	SetSetupItems( m_pDicsView->GetDocument()->m_pSetupItems );
	SetEqpStateSequence( m_pDicsView->m_pEqpStateSeq );
	SetSeqTimer( m_pDicsView->m_pTimer );
	SetEqpAssy( m_pDicsView->m_pEqpAssy );
}

void CDicsGaugeInterlockSeq::SetLog( CString sSeqName, int nPortNo, int nSlotNo, CString sMessage )
{
	CString sLog("");
	CString sPortNo;
	if( nPortNo <= 0 ) sPortNo = "";
	else sPortNo.Format( "%X", nPortNo );

	CString sSlotNo;
	if( nSlotNo <= 0 ) sSlotNo = "";
	else sSlotNo.Format( "%X", nSlotNo );

	sLog.Format( "GAUGE\t%s\t%s\t%s\t%s", sSeqName, sPortNo, sSlotNo, sMessage );
	Log.TextOut( sLog );

	if( m_pDicsData->m_GenInfo.bEnableEqpLog )
	{
		sLog.Format( "%s : %s", sSeqName, sMessage );
		Msg( WM_EQP_LOG, 0, (LPARAM)(LPCTSTR) sLog );
	}
}


BOOL CDicsGaugeInterlockSeq::Initialize()
{
	//InitParameter(); Piping View?? ??? ?? 

	return TRUE;
}

void CDicsGaugeInterlockSeq::InitParameter()
{
	m_nMaxIntroNo = m_pEqpAssy->m_AnalogInput.m_nMaxIntrAiNo;
	
	m_Interlock.bGaugeAlarmSet = new BOOL[2*m_nMaxIntroNo];
	m_Interlock.bGaugeWarningSet = new BOOL[2*m_nMaxIntroNo];
	m_Interlock.bNg = new BOOL[m_nMaxIntroNo];
	m_nSeqCheckInterlockNo = new int[m_nMaxIntroNo];

	for( int i = 0 ; i <m_nMaxIntroNo ; i++ )
	{
		m_Interlock.bGaugeAlarmSet[i*2 + 0]		= FALSE;
		m_Interlock.bGaugeAlarmSet[i*2 + 1]		= FALSE;
		m_Interlock.bGaugeWarningSet[i*2 + 0]	= FALSE;
		m_Interlock.bGaugeWarningSet[i*2 + 1]	= FALSE;
		m_Interlock.bNg[i]						= FALSE;
		m_nSeqCheckInterlockNo[i] = 0;
	}	
}

BOOL CDicsGaugeInterlockSeq::Sequence()
{
	Sleep( 300 );

	for( int i = 0 ; i < m_pEqpAssy->m_AnalogInput.GetAiSize(); i ++ )
	{
#ifndef _MY_DEBUG
		SeqGaugeInterlock( i );
#endif
	}

	return TRUE;
}

BOOL CDicsGaugeInterlockSeq::GetGaugeCond( int nUnitNo )
{
	BOOL bNg = FALSE;

	for( int i = 0 ; i < m_pEqpAssy->m_AnalogInput.GetAiSize(); i ++ )
	{
		int nIntrNo = m_pEqpAssy->m_AnalogInput.m_aiSet[i].nAiIntrNo;
		if( nIntrNo < 0 ) continue;

		int nCvNo = m_pEqpAssy->m_AnalogInput.m_aiSet[i].nOwnerUnitNo - IF_UNIT_LD;
		if( nCvNo == nUnitNo )
		{
			bNg |= m_Interlock.bNg[nIntrNo];
		}
	}

	return !bNg;
}


BOOL CDicsGaugeInterlockSeq::GetCheckCond( int nIntrNo )
{
	BOOL bCheckCond = TRUE;
	bCheckCond &= m_pDicsData->m_GenInfo.bAuto &&
				 !m_pDicsData->m_GenInfo.bEmo &&
				 !m_pDicsData->m_GenInfo.bLeakSensor &&
				 !m_pDicsData->m_GenInfo.bCoverOpen &&
				 !m_pDicsData->m_GenInfo.bDoorOpen &&
//				 !m_pDicsData->m_GenInfo.bPause &&
				  m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] &&	
			      m_pSetupItems->m_nValueGaugeLimit_Option[nIntrNo];

// Must be init : every 'bCheckCond' flag
// For Example
//	case CSetupItems::gaugeMIX_TANK1_TEMP1:			
//	case CSetupItems::gaugeMIX_TANK1_TEMP2:		
//		bCheckCond &= m_pDicsData->m_IfPump.bChemicalStart[0];
//		bCheckCond &= ( ( (CDicsView*) m_pOwnerWnd )->m_pPumpSeq->m_nMixTankNo[0] == 0 );
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDET_SHW_MODE];
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK1_USE];
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK1_HEATER_USE];
//		break;
//	case CSetupItems::gaugeMIX_TANK2_TEMP1:			
//	case CSetupItems::gaugeMIX_TANK2_TEMP2:		
//		bCheckCond &= m_pDicsData->m_IfPump.bChemicalStart[0];
//		bCheckCond &= ( ( (CDicsView*) m_pOwnerWnd )->m_pPumpSeq->m_nMixTankNo[0] == 1 );
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDET_SHW_MODE];
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK2_USE];
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK2_HEATER_USE];
//		break;
//	case CSetupItems::gaugeMIX_TANK1_CONDUCTIVITY:
//		bCheckCond &= m_pDicsData->m_IfPump.bChemicalStart[0];
//		bCheckCond &= ( ( (CDicsView*) m_pOwnerWnd )->m_pPumpSeq->m_nMixTankNo[0] == 0 );
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK1_USE];
//		break;
//	case CSetupItems::gaugeMIX_TANK2_CONDUCTIVITY:	
//		bCheckCond &= m_pDicsData->m_IfPump.bChemicalStart[0];
//		bCheckCond &= ( ( (CDicsView*) m_pOwnerWnd )->m_pPumpSeq->m_nMixTankNo[0] == 1 );
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK2_USE];
//		break;
//	case CSetupItems::gaugeDB1_UNIT_UP_SHW_DI_FLOW:	
//	case CSetupItems::gaugeDB1_UNIT_LO_SHW_DI_FLOW:	
//	case CSetupItems::gaugeDB2_UNIT_UP_SHW_DI_FLOW:	
//	case CSetupItems::gaugeDB2_UNIT_LO_SHW_DI_FLOW:	
//	case CSetupItems::gaugeDB1_UNIT_EXT_PRESS:			
//	case CSetupItems::gaugeDB1_UNIT_UP_AC_CDA_PRESS:	
//	case CSetupItems::gaugeDB1_UNIT_LO_AC_CDA_PRESS:	
//	case CSetupItems::gaugeDB2_UNIT_EXT_PRESS:			
//	case CSetupItems::gaugeDB2_UNIT_UP_AC_CDA_PRESS:	
//	case CSetupItems::gaugeDB2_UNIT_LO_AC_CDA_PRESS:
//		bCheckCond &= m_pDicsData->m_IfPump.bChemicalStart[0];
//		bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK1_USE] ||
//					  m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupMIX_TANK2_USE];
//		break;		
//// RB Unit
//	case CSetupItems::gaugeRB_UNIT_SHW2_DI_FLOW:
//	case CSetupItems::gaugeRB_UNIT_SHW3_DI_FLOW:
//	case CSetupItems::gaugeSBJ2_UNIT_SHW1_DI_FLOW:		
//	case CSetupItems::gaugeRB_UNIT_EXT_PRESS:
//	case CSetupItems::gaugeRB_UNIT_AC_CDA_PRESS:		
//		bCheckCond &= m_pDicsData->m_IfPump.bDiStart &&
//					  m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP2_USE];
//		break;
//// SBJ#2 Unit
//	case CSetupItems::gaugeSBJ2_UNIT_SHW2_DI_FLOW:		
//	case CSetupItems::gaugeSBJ2_UNIT_SBJ1_DI_FLOW:		
//	case CSetupItems::gaugeSBJ2_UNIT_SBJ2_DI_FLOW:		
//	case CSetupItems::gaugeSBJ3_UNIT_SHW1_DI_FLOW:		
//	case CSetupItems::gaugeSBJ2_UNIT_EXT_PRESS:		
//	case CSetupItems::gaugeSBJ2_UNIT_SBJ1_CDA_PRESS:	
//	case CSetupItems::gaugeSBJ2_UNIT_SBJ2_CDA_PRESS:	
//		bCheckCond &= m_pDicsData->m_IfPump.bDiStart &&
//					  m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP3_USE];
//		break;
//// SBJ#3 Unit
//	case CSetupItems::gaugeSBJ3_UNIT_SHW2_UP_DI_FLOW:	
//	case CSetupItems::gaugeSBJ3_UNIT_SHW2_DN_DI_FLOW:	
//	case CSetupItems::gaugeSBJ3_UNIT_SBJ_DI_FLOW:		
//	case CSetupItems::gaugeFR_UNIT_SBJ_DI_FLOW:		
//	case CSetupItems::gaugeSBJ3_UNIT_EXT_PRESS:		
//	case CSetupItems::gaugeSBJ3_UNIT_SBJ_CDA_PRESS:	
//		bCheckCond &= m_pDicsData->m_IfPump.bDiStart &&
//					  m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP4_USE];
//		break;
//// FR Unit
//	case CSetupItems::gaugeFR_UNIT_EXT_PRESS:			
//	case CSetupItems::gaugeFR_UNIT_UP_SBJ_CDA_PRESS:	
//	case CSetupItems::gaugeFR_UNIT_LO_SBJ_CDA_PRESS:	
//	case CSetupItems::gaugeFR_UNIT_SHW_DI_FLOW:		
//	case CSetupItems::gaugeFR_UNIT_AQK_DI_FLOW:		
//	case CSetupItems::gaugeFR_UNIT_WATER_RESIST:	
//		bCheckCond &= m_pDicsData->m_IfPump.bDiStart;
//		break;
//// AK Unit
//	case CSetupItems::gaugeAK_UNIT_UP_EXT_PRESS:		
//	case CSetupItems::gaugeAK_UNIT_LO_EXT_PRESS:		
//	case CSetupItems::gaugeAK_UNIT_UP_AK1_CDA_FLOW:	
//	case CSetupItems::gaugeAK_UNIT_LO_AK1_CDA_FLOW:	
//	case CSetupItems::gaugeAK_UNIT_UP_AK2_CDA_FLOW:	
//	case CSetupItems::gaugeAK_UNIT_UP_AK1_CDA_PRESS:	
//	case CSetupItems::gaugeAK_UNIT_LO_AK1_CDA_PRESS:	
//	case CSetupItems::gaugeAK_UNIT_UP_AK2_CDA_PRESS:	
//		break;
//// Common Unit
//	case CSetupItems::gaugeDRIVING_AIR_PRESS:			
//		break;
//	}	

	return bCheckCond;
}

void CDicsGaugeInterlockSeq::SeqGaugeInterlock( int nNo )
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
	
	// return if not initialized
	if( !m_pDicsData->m_GenInfo.bEqpInitComp ) return;

	int nIntrNo = m_pEqpAssy->m_AnalogInput.m_aiSet[nNo].nAiIntrNo;
	if( nIntrNo < 0 ) return;

	static int nReturnSeqNo[MAX_AI_INTR_NO];
	static DWORD dwTickStart[MAX_AI_INTR_NO];
	static DWORD dwRecheckDelayTime[MAX_AI_INTR_NO]	= { 0, };
	static DWORD dwFirstDelayTime[MAX_AI_INTR_NO] = { 0, };	// 170721 halesu

	// 170721 halesu
	if( nIntrNo == CSetupItems::gaugeSWR1_UNIT_DI_SPL_WR ||
		nIntrNo == CSetupItems::gaugeSWR2_UNIT_DI_SPL_WR ||
		nIntrNo == CSetupItems::gaugeSWR3_UNIT_DI_SPL_WR ||
		nIntrNo == CSetupItems::gaugeAK_UNIT_DI_SPL_WR  )
	{
		dwFirstDelayTime[nIntrNo] = 1000 * atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupWR_CHECK_DELAY] );
	}
	else
		dwFirstDelayTime[nIntrNo] = 1000 * atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupGAUGE_CHECK_DELAY] );

	int nPortNo = 0;
	int nSlotNo = 0;
	int nSeqNo	= m_nSeqCheckInterlockNo[nIntrNo];

	BOOL bCheckCond = TRUE;
	bCheckCond = GetCheckCond( nNo );

	m_sSeqName.Format( "%s", "INTRLOCK" );

	double dCurVal	= (double) m_pEqpAssy->m_AnalogInput.GetRealVal( nNo );

	switch( nSeqNo )
	{
	case 0 : // ?? Check On?? Time Delay? ??.
		if( bCheckCond )
		{
			dwTickStart[nIntrNo] = GetTickCount();			
			nSeqNo = 5;
		}
		else
		{
			CString sName = m_pEqpAssy->m_AnalogInput.m_aiSet[nNo].sName;

			if( m_Interlock.bGaugeAlarmSet[nIntrNo*2+0] )
			{
				m_Interlock.bGaugeAlarmSet[nIntrNo*2+0] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_LOWER_ALARM1 + nIntrNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", sName );
				SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
			}
			if( m_Interlock.bGaugeAlarmSet[nIntrNo*2+1] )
			{
				m_Interlock.bGaugeAlarmSet[nIntrNo*2+1] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_UPPER_ALARM1 + nIntrNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", sName );
				SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
			}
			if( m_Interlock.bGaugeWarningSet[nIntrNo*2+0] )
			{
				m_Interlock.bGaugeWarningSet[nIntrNo*2+0] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_LOWER_WARNING1 + nIntrNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", sName );
				SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
			}
			if( m_Interlock.bGaugeWarningSet[nIntrNo*2+1] )
			{
				m_Interlock.bGaugeWarningSet[nIntrNo*2+1] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_UPPER_WARNING1 + nIntrNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", sName );
				SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
			}

			if( m_Interlock.bNg[nIntrNo] )
			{
				m_Interlock.bNg[nIntrNo] = FALSE;
			}
		}
		break;

	case 5 :
		if( ( GetTickCount() - dwTickStart[nIntrNo] ) > dwFirstDelayTime[nIntrNo] ) nSeqNo = 10;
		break;
		
	case 10 :	 
		if( !bCheckCond ) nSeqNo = 0;
		else
		{
			BOOL bAlarm[2]		= { FALSE, FALSE };
			BOOL bWarning[2]	= { FALSE, FALSE };

			// interlock ??? Check??.
								
			int	 nLowLimit	= m_pSetupItems->m_nValueGaugeLimit[0][nNo];
			int	 nLoWarning	= m_pSetupItems->m_nValueGaugeLimit[1][nNo];
			int	 nUpWarning	= m_pSetupItems->m_nValueGaugeLimit[2][nNo];
			int	 nUpLimit	= m_pSetupItems->m_nValueGaugeLimit[3][nNo];
			CString sName	= m_pEqpAssy->m_AnalogInput.m_aiSet[nNo].sName;

			if( dCurVal < nLowLimit )
			{
				bAlarm[0] = TRUE;	bAlarm[1] = FALSE;	bWarning[0] = FALSE;	bWarning[1] = FALSE;
			}
			else if( dCurVal > nUpLimit )
			{
				bAlarm[0] = FALSE;	bAlarm[1] = TRUE;	bWarning[0] = FALSE;	bWarning[1] = FALSE;
			}
			else if( dCurVal < nLoWarning  )
			{
				bAlarm[0] = FALSE;	bAlarm[1] = FALSE;	bWarning[0] = TRUE;		bWarning[1] = FALSE;
			}		
			else if( dCurVal > nUpWarning  )
			{
				bAlarm[0] = FALSE;	bAlarm[1] = FALSE;	bWarning[0] = FALSE;	bWarning[1] = TRUE;
			}

			if( bAlarm[0] || bAlarm[1] || bWarning[0] || bWarning[1] )
			{
				dwRecheckDelayTime[nIntrNo]	= 2000;
				dwTickStart[nIntrNo]	= GetTickCount();
				nSeqNo = 20;			
			}
		}
		break;
	case 20:
		if( !bCheckCond ) nSeqNo = 0;
		else if( ( GetTickCount() - dwTickStart[nIntrNo] ) > dwRecheckDelayTime[nIntrNo] )
		{
			BOOL bAlarm[2]		= { FALSE, FALSE };
			BOOL bWarning[2]	= { FALSE, FALSE };

			// interlock ??? Check??.
			
			int	 nLowLimit	= m_pSetupItems->m_nValueGaugeLimit[0][nNo];
			int	 nLoWarning	= m_pSetupItems->m_nValueGaugeLimit[1][nNo];
			int	 nUpWarning	= m_pSetupItems->m_nValueGaugeLimit[2][nNo];
			int	 nUpLimit	= m_pSetupItems->m_nValueGaugeLimit[3][nNo];
			CString sName	= m_pEqpAssy->m_AnalogInput.m_aiSet[nNo].sName;
	

			if( dCurVal < nLowLimit )
			{
				bAlarm[0] = TRUE;	bAlarm[1] = FALSE;	bWarning[0] = FALSE;	bWarning[1] = FALSE;
			}
			else if( dCurVal > nUpLimit )
			{
				bAlarm[0] = FALSE;	bAlarm[1] = TRUE;	bWarning[0] = FALSE;	bWarning[1] = FALSE;
			}
			else if( dCurVal < nLoWarning  )
			{
				bAlarm[0] = FALSE;	bAlarm[1] = FALSE;	bWarning[0] = TRUE;		bWarning[1] = FALSE;
			}		
			else if( dCurVal > nUpWarning  )
			{
				bAlarm[0] = FALSE;	bAlarm[1] = FALSE;	bWarning[0] = FALSE;	bWarning[1] = TRUE;
			}


			if( bAlarm[0] )
			{
				if( !m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 0] )
				{
					m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 0] = TRUE;
					m_pEqpStateSeq->SetAlarm( ALM_GAUGE_LOWER_ALARM1 + nIntrNo );
					m_sMessage.Format( "Alarm : %s lower Alarm", sName );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
					m_sMessage.Format( "%s = %.2f ( %d, %d, %d, %d)", sName, 
						dCurVal, nLowLimit, nLoWarning, nUpWarning, nUpLimit );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
				}
				if( m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] )
				{
					m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_LOWER_WARNING1 + nIntrNo );
				}
			}
			else
			{
				if( m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 0] )
				{
					m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 0] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_LOWER_ALARM1 + nIntrNo );
					m_sMessage.Format( "Recovery : %s lower Alarm", sName );
					SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
				}
			}

			if( bAlarm[1] )
			{
				if( !m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 1] )
				{
					m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 1] = TRUE;
					m_pEqpStateSeq->SetAlarm( ALM_GAUGE_UPPER_ALARM1 + nIntrNo );
					m_sMessage.Format( "Alarm : %s upper Alarm", sName );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
					m_sMessage.Format( "%s = %.2f ( %d, %d, %d, %d)", sName, 
						dCurVal, nLowLimit, nLoWarning, nUpWarning, nUpLimit );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
				}
				if( m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] )
				{
					m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_UPPER_WARNING1 + nIntrNo );
				}
			}
			else
			{
				if( m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 1] )
				{
					m_Interlock.bGaugeAlarmSet[nIntrNo*2 + 1] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_UPPER_ALARM1 + nIntrNo );
					m_sMessage.Format( "Recovery : %s upper Alarm", sName );
				}
			}
			
			if( bWarning[0] )
			{
				if( !m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] )
				{
					m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] = TRUE;
					m_pEqpStateSeq->SetAlarm( ALM_GAUGE_LOWER_WARNING1 + nIntrNo );
					m_sMessage.Format( "Warning : %s lower Warning", sName );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
					m_sMessage.Format( "%s = %.2f ( %d, %d, %d, %d)", sName, 
						dCurVal, nLowLimit, nLoWarning, nUpWarning, nUpLimit );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
				}
			}
			else
			{
				if( m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] && m_pEqpStateSeq->IsPushedAlarmReset() )
				{
					m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_LOWER_WARNING1 + nIntrNo );
					m_sMessage.Format( "Recovery : %s lower Warning", sName );
				}
			}	
			
			if( bWarning[1] )
			{
				if( !m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] )
				{
					m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] = TRUE;
					m_pEqpStateSeq->SetAlarm( ALM_GAUGE_UPPER_WARNING1 + nIntrNo );
					m_sMessage.Format( "Warning : %s upper Warning", sName );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
					m_sMessage.Format( "%s = %.2f ( %d, %d, %d, %d)", sName, 
						dCurVal, nLowLimit, nLoWarning, nUpWarning, nUpLimit );
					SetLog( m_sSeqName, 0, 0, m_sMessage );
				}
			}
			else
			{
				if( m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] && m_pEqpStateSeq->IsPushedAlarmReset() )
				{
					m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_GAUGE_UPPER_WARNING1 + nIntrNo );
					m_sMessage.Format( "Recovery : %s upper Warning", sName );
				}
			}
			
			if( !bAlarm[0] && !bAlarm[1] )
			{
				if( m_Interlock.bNg[nIntrNo] )
				{
					m_Interlock.bNg[nIntrNo] = FALSE;
					m_sMessage.Format( "OK : Recovery[%s]", sName );
					SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
				}

				if( !m_Interlock.bGaugeWarningSet[nIntrNo*2 + 0] && 
					!m_Interlock.bGaugeWarningSet[nIntrNo*2 + 1] )
				{
					nSeqNo = 10;
				}
			}
			else
			{	
 				if( !m_Interlock.bNg[nIntrNo] )
				{
					m_Interlock.bNg[nIntrNo] = TRUE;
				}
				else
				{
					m_sMessage.Format( "NG : Recovery[%s]", sName );
					SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
				}
				
				nReturnSeqNo[nIntrNo] = nSeqNo;
				nSeqNo = 1000;
				break;
			}			
		}
		break;
	// Error ??
	case 1000 :
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			CString sName = m_pSetupItems->m_infoGaugeLimit[nIntrNo].sName;
			m_sMessage.Format( "Check Recovery[%s]", sName );
			SetLog( m_sSeqName, nPortNo, nSlotNo, m_sMessage );
			
			dwRecheckDelayTime[nIntrNo] = dwFirstDelayTime[nIntrNo];
			dwTickStart[nIntrNo] = GetTickCount();
			nSeqNo = nReturnSeqNo[nIntrNo];
		}
		break;
	}
	m_nSeqCheckInterlockNo[nIntrNo] = nSeqNo;	
}

