// DicsPumpSeq.cpp: implementation of the CDicsPumpSeq class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "DicsPumpSeq.h"
#include "..\DefineConstant.h"
#include "..\MainFrm.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define _NO_USE			0
#define _USE			1

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDicsPumpSeq::CDicsPumpSeq()
{
	m_SeqFlag.bInitReq	= FALSE;
	m_SeqFlag.bInitComp	= FALSE;
	m_SeqFlag.bInitFail	= FALSE;
}

CDicsPumpSeq::~CDicsPumpSeq()
{

}

void CDicsPumpSeq::SetOwnerPointer(CWnd* pWnd)
{
	SetDicsView( ( CDicsView* )pWnd );
	SetDicsData( m_pDicsView->m_pDicsData );
	SetDevice( m_pDicsView->m_pDevice );
	SetSetupItems( m_pDicsView->GetDocument()->m_pSetupItems );
	SetEqpStateSequence( m_pDicsView->m_pEqpStateSeq );
	SetSeqTimer( m_pDicsView->m_pTimer );
	SetEqpAssy( m_pDicsView->m_pEqpAssy );
}

BOOL CDicsPumpSeq::Initialize()
{
	InitParameter();

////////////////////////////////////////////////////////////////////
// Must be Init Address : for pump Interlock flow 
// for Example
// 	m_nInterlockFlowMeterID[0] = noSBJ1_UNIT_SHW_DI_FLOW;
////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////
// Must be Init Address : for Tank Level Simulation
// for Example
//#ifdef _MY_DEBUG
//	m_nTankLevelStartAddress[0] = DI_NODE3+11;
//#endif
////////////////////////////////////////////////////////////////////
	return TRUE;
}

void CDicsPumpSeq::InitParameter()
{
	m_nSeqInitNo			= 0;
	m_nSeqModeChangeNo		= 0;
	m_nSeqGlassExistNo		= 0;		
	m_nSeqAirKnifeNo		= 0;
	m_nSeqDiwIdleRunningNo	= 0;

	m_pEqpAssy		= ( (CDicsView *) m_pOwnerWnd )->m_pEqpAssy;
	
	for( int i=0; i<MAX_DI_TANK_NO; i++ )
	{
		m_nSeqDiTankNo[i]			= 0;
		m_nSeqDiPumpNo[i]			= 0;
		m_nSeqDiPumpInterlockNo[i]	= 0;

		m_SeqFlag.bDiTankReady[i]				= FALSE;
		m_SeqFlag.bInsufficientDiFlowrate[i]	= FALSE;
		

		for( int j=0; j<lvMAX; j++ )
		{
			m_nSeqLevelSensorNo[i][j]		= 0;
			m_SeqFlag.bDiTankLevel[i][j]	= FALSE;
			m_SeqFlag.bDiTankLevelSensorFault[i][j] = FALSE;
		}
	}
	// 231220 halesu OverRead Fix
	for( int i=0; i<MAX_HOT_DI_TANK_NO; i++ )
	{
		m_SeqFlag.bHotDiTankReady[i]			= FALSE;
		m_SeqFlag.bInsufficientHotDiFlowrate[i]	= FALSE;

		for( int j=0; j<lvMAX; j++ )
		{
			m_SeqFlag.bHotDiTankLevel[i][j]	= FALSE;
			m_SeqFlag.bHotDiTankLevelSensorFault[i][j] = FALSE;
		}
	}

	for( int i = 0; i < MAX_SBJ_NO; i++ )
	{
		m_nSeqSbjNo[i] = 0;	
	}
}

BOOL CDicsPumpSeq::Sequence()
{
	Sleep( 100 );

	SeqInit();
	SeqModeChange();
	SeqGlassExist();
	SeqAirKnife();			

	for( int i=0; i<MAX_DI_TANK_NO; i++ )
	{
		SeqDiPumpInterlock( i );
		SeqDiTank( i );
		SeqDiPump( i );
		for( int j=0; j<lvMAX; j++ )
		{
			SeqLevelSensor( i, j );
		}
	}

	for ( int i=0; i<MAX_SBJ_NO; i++ )
	{
		SeqSbj( i );
	}

	SeqDiwIdleRunning();

	return TRUE;
}

//----------------------------------------------------------------------------------------------
// Description :
//----------------------------------------------------------------------------------------------
void CDicsPumpSeq::SetLog(CString sSeqName, int nPortNo, int nSlotNo, CString sMessage)
{
	CString sLog("");
	CString sBuf;
	COleDateTime time = COleDateTime::GetCurrentTime();

	if( nPortNo != -1 && nSlotNo != -1 )
	{
		sLog.Format( "PUMP    \t%s\t%d\t%02d\t%s", sSeqName, nPortNo + 1, nSlotNo + 1, sMessage );
		Log.TextOut( sLog, XLog::stampDateTime );
	}
	else if( nSlotNo != -1 )
	{
		sLog.Format( "PUMP    \t%s\t%d\t\t%s", sSeqName, nPortNo + 1, sMessage );
		Log.TextOut( sLog, XLog::stampDateTime );
	}
	else
	{ 
		sLog.Format( "PUMP    \t%s\t\t\t%s", sSeqName, sMessage );
		Log.TextOut( sLog, XLog::stampDateTime );
		if( m_pDicsData->m_GenInfo.bEnableEqpLog )
		{
			sLog.Format( "PUMP : [%s] %s", sSeqName, sMessage );
			Msg( WM_EQP_LOG, 0, (LPARAM)(LPCTSTR) sLog );
		}
	}
}

//---------------------------------------------------------------------------
// SeqInit
//---------------------------------------------------------------------------
void CDicsPumpSeq::SeqInit()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	int nSeqNo = m_nSeqInitNo;

	m_sSeqName.Format( "INIT    " );

	static BOOL bAlarm = FALSE;
	static DWORD dwStartTick;
	static int nAlarmId[MAX_DI_PUMP_NO];

	switch( nSeqNo )
	{
	case 0:
		if( m_SeqFlag.bInitReq )
		{
			m_SeqFlag.bInitReq	= FALSE;
			m_SeqFlag.bInitComp	= FALSE;
			m_SeqFlag.bInitFail = FALSE;

			int nPumpNo = MAX_DI_PUMP_NO;
			for( int i=0; i<nPumpNo; i++ )
			{
				m_sMessage.Format( "%2d%d", CDicsData::initPUMP1_CHECK + i, MSG_STATUS_START );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_PUMP1+i, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
			}

			SetLog( m_sSeqName, -1, -1, "Initialize : Start" );

			dwStartTick = GetTickCount();
			nSeqNo = 10;
		}
		break;
	case 10:
		if( GetTickCount() - dwStartTick >= 1000 )
		{
			if( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] == 1  )
			{
				for( int i=0; i<MAX_DI_TANK_NO; i++ )	// 231220 halesu OverRead Fix
				{
					bAlarm |= m_pEqpAssy->m_DiTank[i].GetPumpAlarm();
				}
			}
			nSeqNo = 20;
		}
		break;
	case 20:
		if( !bAlarm )
		{
			for( int i=0; i<MAX_DI_PUMP_NO; i++ )
			{
				m_sMessage.Format( "%2d%d", CDicsData::initPUMP1_CHECK + i, MSG_STATUS_OK );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_PUMP1+i, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

				m_sMessage.Format( "DI Pump %d Check OK", i+1 );
				SetLog( m_sSeqName, -1, -1, m_sMessage );
			}

			dwStartTick = GetTickCount();

			nSeqNo = 100;
		}
		else
		{
			for( int i=0; i<MAX_DI_PUMP_NO; i++ )
			{
				if( i < NO_BUFTANK )
				{
					if( m_pEqpAssy->m_DiTank[i].GetPumpAlarm() )
					{
						m_sMessage.Format( "%2d%d", CDicsData::initPUMP1_CHECK + i, MSG_STATUS_NG );
						Msg( WM_INIT_STATUS, MSG_STATUS_SET_PUMP1+i, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
					
						m_sMessage.Format( "DI PUMP %d Alarm", i+1 );
						SetLog( m_sSeqName, -1, -1, m_sMessage );

						nAlarmId[i] = ALM_DI_PUMP_ALARM1 + i;
						m_pEqpStateSeq->SetAlarm( nAlarmId[i] );
					}
				}
				else	// 231220 halesu OverRead Fix
				{
					if( m_pEqpAssy->m_DiTank[NO_BUFTANK].GetPumpAlarm(i-NO_BUFTANK) )
					{
						m_sMessage.Format( "%2d%d", CDicsData::initPUMP1_CHECK + i, MSG_STATUS_NG );
						Msg( WM_INIT_STATUS, MSG_STATUS_SET_PUMP1+i, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
					
						m_sMessage.Format( "DI PUMP %d Alarm", i+1 );
						SetLog( m_sSeqName, -1, -1, m_sMessage );

						nAlarmId[i] = ALM_DI_PUMP_ALARM1 + i;
						m_pEqpStateSeq->SetAlarm( nAlarmId[i] );
					}
				}
			}

			m_SeqFlag.bInitFail = TRUE;
			
			nSeqNo = 1000;
		}
		break;

	case 100:
		{ // Initialize Complete
			m_SeqFlag.bInitComp = TRUE;
			m_SeqFlag.bInitReq	= FALSE;

			SetLog( m_sSeqName, -1, -1, "Initialize : End" );

			Msg( WM_EQP_MESSAGE, MSG_EQP_MESSAGE, (LPARAM)(LPCTSTR) "Initialize End" );
			nSeqNo = 0;
		}
		break;
	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			for( int i=0; i<MAX_DI_PUMP_NO; i++ )
			{
				m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
			}
			SetLog( m_sSeqName, 0, 0, "Alarm reset" );
			bAlarm = FALSE;//2007.09.10 kimgun
			nSeqNo = 0;
		}
		break;
	
	}

	m_nSeqInitNo = nSeqNo;
}

//---------------------------------------------------------------------------
// Description :
//---------------------------------------------------------------------------
void CDicsPumpSeq::SeqModeChange()
{
	static BOOL bDiStart;
	
	int nSeqNo = m_nSeqModeChangeNo;

	m_sSeqName.Format( "%s", "MODE    " );

	switch( nSeqNo ) 
	{
	case 0 :
		if( m_pDicsData->m_GenInfo.bAuto )
		{
			SetLog( m_sSeqName, -1, -1, "AUTO MODE" );
			nSeqNo = 10;
		}
		break;
	case 10 :
		if( !m_pDicsData->m_GenInfo.bAuto )
		{	// manual ȯɶ : Idle run, DI Stop

			bDiStart = m_pDicsData->m_IfPump.bDiStart;
			SetLog( m_sSeqName, -1, -1, "MANUAL MODE : SAVE CUR STATE" );			
			m_sMessage.Format( "SAVE : bDiStart : %s", bDiStart ? "ON":"OFF" );
			SetLog( m_sSeqName, -1, -1, m_sMessage );
			m_pDicsData->m_IfPump.bDiStart = FALSE;
			
			nSeqNo = 20;
		}
		break;
	case 20 :
		if( m_pDicsData->m_GenInfo.bAuto )
		{	// ٽ auto ȯɶ 
			//   ·  ȵȴ. 
			{
				// Set Valve Status
				SetLog( m_sSeqName, -1, -1, "AUTO MODE : SET ORIGINAL STATE" );

				bDiStart &= ( m_pDicsData->m_GenInfo.nGlassNo > 0 ? TRUE : FALSE );
				m_pDicsData->m_IfPump.bDiStart = bDiStart;
				m_sMessage.Format( "SET : bDiStart : %s", m_pDicsData->m_IfPump.bDiStart ? "ON":"OFF" );
				SetLog( m_sSeqName, -1, -1, m_sMessage );
			}

//			m_nSeqGlassExistNo = 0;

			nSeqNo = 10;
		}
		break;
	}
	m_nSeqModeChangeNo = nSeqNo;
}


//---------------------------------------------------------------------------
// - SeqGlassExist
//---------------------------------------------------------------------------
void CDicsPumpSeq::SeqGlassExist()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	if( !m_SeqFlag.bInitComp ) return;
	if( !m_pDicsData->m_GenInfo.bAuto ) return; 
//	if( !m_pDicsData->m_GenInfo.bAuto || 
//		 m_pDicsData->m_GenInfo.bPause ||
//		 m_pDicsData->m_GenInfo.bEmo ||
//		 m_pDicsData->m_GenInfo.bLeakSensor ||
//		 m_pDicsData->m_GenInfo.bCoverOpen )
//	{	
////		m_nSeqGlassExistNo = 0;
////		m_pDicsData->m_GenInfo.bIdleRunning = FALSE;
//		return;
//	}

	static DWORD dwStartTick = 0;
	DWORD nIdleWaitTime = 60 * 1000 * atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupIDLE_WAIT_TIME] );

	int nSeqNo = m_nSeqGlassExistNo;

	switch( nSeqNo )
	{
	case 0:
		// case 0  m_pDicsData->m_GenInfo.nGlassNo = 0  
	  	// case 10  m_pDicsData->m_GenInfo.nGlassNo > 0  Ǵ 
		if( !m_pDicsData->m_GenInfo.nGlassNo ) 
		{
//			m_pDicsData->m_GenInfo.bIdleRunning = TRUE;
//			m_nSeqDiwIdleRunningNo = 0;

			nSeqNo = 20;
		}
		else nSeqNo = 20;
		break;

	case 10:
		if( !m_pDicsData->m_GenInfo.nGlassNo )
		{
			m_pDicsData->m_IfPump.bDiStart = FALSE;

			dwStartTick = GetTickCount();
			
			nSeqNo = 20;
		}
		else if( m_pDicsData->m_GenInfo.nGlassNo &&
				 m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] == 1 )
		{
			BOOL bProcessRequired = FALSE;
			for( int i=IF_UNIT_LD*2; i<=IF_UNIT_UL*2; i++ )
			{
				bProcessRequired |= m_pDicsData->m_GlsInfo[i].bIsValid;
			}
			
			if( !bProcessRequired && m_pDicsData->m_IfPump.bDiStart )
			{
				m_pDicsData->m_IfPump.bDiStart = FALSE;
			}
			else if( bProcessRequired && !m_pDicsData->m_IfPump.bDiStart )
			{
				m_pDicsData->m_IfPump.bDiStart = TRUE;
			}
		}
		break;

	case 20:
		if( m_pDicsData->m_GenInfo.nGlassNo ) 
		{
			m_pDicsData->m_GenInfo.bIdleRunning = FALSE;
//			if( m_pDicsData->m_GenInfo.bIdleRunning )
//			{
//				m_pDicsData->m_GenInfo.bIdleRunning = FALSE;
//				m_pDicsData->m_IfPump.bDiStart = FALSE;
//				Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_END, 0L, FALSE );
//				m_nSeqDiwIdleRunningNo = 0;				
//			}
			nSeqNo = 10;
		}
		else if( !m_pDicsData->m_GenInfo.bEqpInitComp )
		{
			dwStartTick = GetTickCount();
		}
		else if( !m_pDicsData->m_GenInfo.bIdleRunning &&
				 ( GetTickCount()-dwStartTick ) > nIdleWaitTime )
		{
			m_pDicsData->m_GenInfo.bIdleRunning = TRUE;
//			m_nSeqDiwIdleRunningNo = 0;
		}
		break;
	}

	m_nSeqGlassExistNo = nSeqNo;
}

//----------------------------------------------------------------------------------------------
// Description :
//----------------------------------------------------------------------------------------------
void CDicsPumpSeq::SeqDiTank(int i)
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	m_sSeqName.Format( "DI TANK%d", i + 1 );


	BOOL bRunCond = TRUE;
	bRunCond &= !m_pDicsData->m_GenInfo.bLeakSensor;
	bRunCond &= !m_pDicsData->m_GenInfo.bEmo;
	bRunCond &= !m_pDicsData->m_GenInfo.bCoverOpen;
	bRunCond &= !m_pDicsData->m_GenInfo.bDoorOpen;
	bRunCond &= ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] == 1 );
	bRunCond &= ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP1_USE+i] == 1 );	
	bRunCond &= !GetLevelSensorFault( i );


	if( !bRunCond || m_SeqFlag.bDiTankLevel[i][lvH] )
	{
		if( m_pEqpAssy->m_DiTank[i].GetTankSupplyInValveState() )
		{
			m_pEqpAssy->m_DiTank[i].SetTankSupplyInValve( 0, OFF );
			if( !bRunCond )						SetLog( m_sSeqName, -1, -1, "DI in Valve Close : RunCond(FALSE)" );
			if( m_SeqFlag.bDiTankLevel[i][lvH] )SetLog( m_sSeqName, -1, -1, "DI in Valve Close : M Level Exist" );
			
			SetLog( m_sSeqName, -1, -1, "In Valve Close" );
		}
	}

	BOOL bNotReady = FALSE;
	bNotReady |= !m_SeqFlag.bDiTankLevel[i][lvM];
	bNotReady |= !m_SeqFlag.bDiTankLevel[i][lvL];
	bNotReady |= GetLevelSensorFault( i );

	if( bNotReady && m_SeqFlag.bDiTankReady[i] )
	{
		m_SeqFlag.bDiTankReady[i] = FALSE;
		SetLog( m_sSeqName, -1, -1, "Not Ready Condition" );
	}

//	if( !m_SeqFlag.bInitComp ) return;

	//--------------------------------------------------------------------
	if( !bRunCond ||
	  ( m_pDicsData->m_GenInfo.bAuto && !m_SeqFlag.bInitComp ) ||
	  ( !m_pDicsData->m_GenInfo.bAuto && ( !m_pDicsData->m_IfPump.bDiStart && !m_pEqpAssy->m_DiTank[i].GetPumpOnState() ) ) ) return; 
	//--------------------------------------------------------------------
	//--------------------------------------------------------------------
//	if( !m_pDicsData->m_GenInfo.bAuto && !bRunCond && !m_pDicsData->m_IfPump.bDiStart ) return; 
	//--------------------------------------------------------------------

#ifdef _MY_DEBUG
	static int nTime[MAX_DI_TANK_NO] = { 0, };
#endif

	int nSeqNo	= m_nSeqDiTankNo[i];
	int tm		= TM_SEQ_DI_TANK( i );
	static int nReturnSeqNo[MAX_DI_TANK_NO];
	static DWORD dwStartTick[MAX_DI_TANK_NO] = { 0, };

	switch( nSeqNo )
	{
	case 0 :
		if( bRunCond && 
			!m_SeqFlag.bDiTankLevel[i][lvHH] &&
		  (
			!m_SeqFlag.bDiTankLevel[i][lvH] ||
		    !m_SeqFlag.bDiTankLevel[i][lvM] ||
			!m_SeqFlag.bDiTankLevel[i][lvL] ) )
		{
			if( m_pEqpAssy->m_DiTank[i].GetTankSupplyInValveState() == OFF )
			{
				SetLog( m_sSeqName, -1, -1, "DI Supply" );
				m_pEqpAssy->m_DiTank[i].SetTankSupplyInValve( 0, ON );

#ifdef _MY_DEBUG
				TIMER_ON( tm, 40 );
				nTime[i] = 0;
#endif
			}
		}
		else if( !bRunCond || m_SeqFlag.bDiTankLevel[i][lvH] )
		{
			if( m_pEqpAssy->m_DiTank[i].GetTankSupplyInValveState() == ON )
			{
				SetLog( m_sSeqName, -1, -1, "DI Supply Stop" );
				m_pEqpAssy->m_DiTank[i].SetTankSupplyInValve( 0, OFF );
			}
		}		
		
		nSeqNo = 10;
		break;

	case 10 :
		if( ( !m_SeqFlag.bDiTankLevel[i][lvL] /*||
			  !m_SeqFlag.bDiTankLevel[i][lvLL] */) &&
			  m_SeqFlag.bDiTankReady[i] )
		{
			m_SeqFlag.bDiTankReady[i] = FALSE;
		}
		nSeqNo = 20;
		break;
		
	case 20:
#ifdef _MY_DEBUG
		if( TIMER( tm ) && 
			m_pEqpAssy->m_DiTank[i].GetTankSupplyInValveState()  == ON  /*&& !m_pEqpAssy->m_DiTank[i].GetTankLevel( lvMH ) */ )  
		{
			TIMER_ON( tm, 40 );
			m_pEqpAssy->m_DiTank[i].SetTankLevelState( nTime[i], ON );
//			m_pDevice->SetDiState( m_nTankLevelStartAddress[i] - nTime[i], ON );
			nTime[i] ++;
		}
#endif
		if(/* m_SeqFlag.bDiTankLevel[i][lvMH] && */
			m_SeqFlag.bDiTankLevel[i][lvM] &&
			m_SeqFlag.bDiTankLevel[i][lvL] &&
			//m_SeqFlag.bDiTankLevel[i][lvLL] &&
			!m_SeqFlag.bDiTankReady[i] )
		{
			m_SeqFlag.bDiTankReady[i] = TRUE;
		}
		nSeqNo = 0;
		break;
	}
	m_nSeqDiTankNo[i] = nSeqNo;
}


//----------------------------------------------------------------------------------------------
// Description :
//----------------------------------------------------------------------------------------------
void CDicsPumpSeq::SeqDiPump( int i )
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();

	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 1000 )	bInit = TRUE;
		if( !bInit ) return;
	}

	m_sSeqName.Format( "DI PUMP%d", i + 1 );

	int nSeqNo = m_nSeqDiPumpNo[i];
	static int nReturnSeqNo[MAX_DI_TANK_NO];
	static int nAlarmId[MAX_DI_TANK_NO] = { 0, };

#ifdef _MY_DEBUG
	static float fCount = 0.0;
#endif

////////////////////////////////////////////////////////////////////////////////
// manual ¿ Pump, Valve Control ϱ ؼ Ʒ Flag ʿϴ.
	static BOOL bDiStop[MAX_DI_TANK_NO] = { TRUE, };
	static BOOL bAqkStop = TRUE;
////////////////////////////////////////////////////////////////////////////////

	BOOL bRunCond = TRUE;
	bRunCond &= !m_pDicsData->m_GenInfo.bEmo;
	bRunCond &= !m_pDicsData->m_GenInfo.bLeakSensor;
	bRunCond &= !m_pDicsData->m_GenInfo.bCoverOpen;
	bRunCond &= !m_pDicsData->m_GenInfo.bDoorOpen;
//	bRunCond &= ( !m_pDicsData->m_GenInfo.bPause || !m_pDicsData->m_GenInfo.bAuto );
	bRunCond &= ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] == 1 );
	bRunCond &= !m_SeqFlag.bInsufficientDiFlowrate[i];
	bRunCond &= m_SeqFlag.bDiTankLevel[i][lvL];
	bRunCond &= m_SeqFlag.bDiTankLevel[i][lvM];

	BOOL bPumpRun = bRunCond; 
	bPumpRun &=	m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP1_USE+i] == 1;
	bPumpRun &=	!m_pEqpAssy->m_DiTank[i].GetPumpAlarm(); 
	bPumpRun &=	m_SeqFlag.bDiTankReady[i];
	bPumpRun &=	m_pDicsData->m_IfPump.bDiStart;

	switch( nSeqNo )
	{ 
	case 0:
		if( bPumpRun )
		{
			bDiStop[i] = FALSE;
		
			if( !m_pEqpAssy->m_DiTank[i].GetPumpOnState() )
			{ // Turn on DI Pump Motor
				m_pEqpAssy->m_DiTank[i].SetPumpOn( 0, ON );
				SetLog( m_sSeqName, -1, -1, "DI Pump Motor On" );
			}
		}
		nSeqNo = 10;
		break;

	case 10:
		if( !bPumpRun )
		{
			if( !bRunCond || m_pDicsData->m_GenInfo.bAuto || !bDiStop[i] )
			{
				bDiStop[i] = TRUE;
			
				if( m_pEqpAssy->m_DiTank[i].GetPumpOnState() )
				{ // DI Pump Stop
					m_pEqpAssy->m_DiTank[i].SetPumpOn(0, OFF);
					SetLog( m_sSeqName, -1, -1, "DI Pump motor Off" );
				}
			}

			// Pump Motor Alarm
			if( m_pEqpAssy->m_DiTank[i].GetPumpAlarm(0) )
			{
				if( m_pEqpAssy->m_DiTank[i].GetPumpOnState(0) ) 
				{
					m_pEqpAssy->m_DiTank[i].SetPumpOn(0, OFF);
					SetLog( m_sSeqName, -1, -1, "DI Pump Motor Stop" );
				}
				
				nAlarmId[i] = ALM_DI_PUMP_ALARM1 + i;
				m_pEqpStateSeq->SetAlarm( nAlarmId[i] );
				SetLog( m_sSeqName, -1, -1, "DI Pump Motor Alarm" );

				nReturnSeqNo[i] = nSeqNo;
				nSeqNo = 1000;
				break;
			}
		}
		nSeqNo = 0;
		break;
	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			SetLog( m_sSeqName, -1, -1, "Alarm reset" );

			m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
			nAlarmId[i] = 0;
			nSeqNo = nReturnSeqNo[i];
		}
		break;
	}
	m_nSeqDiPumpNo[i] = nSeqNo;
}

//----------------------------------------------------------------------------------------------
// Description :
//----------------------------------------------------------------------------------------------
void CDicsPumpSeq::SeqDiPumpInterlock(int i)
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
	static int nReturnSeqNo[MAX_DI_TANK_NO];
	static int nAlarmId[MAX_DI_TANK_NO];
	static DWORD dwStartTickCount[MAX_DI_TANK_NO];
	static DWORD dwMarginTime;

	float fFlowRate[MAX_DI_TANK_NO];
	fFlowRate[i] = m_pEqpAssy->m_DiTank[0].GetShowerFlowInterlock(0);//2008.08.28 kimgun //m_pDevice->GetCalAI( m_nInterlockFlowMeterID[i] );
	
	int nSeqNo = m_nSeqDiPumpInterlockNo[i];

	m_sSeqName.Format( "DI_INTR%d", i + 1 );

	switch( nSeqNo )
	{
	case 0 :
		if(	m_pEqpAssy->m_DiTank[i].GetPumpOnState() )
		{
			dwStartTickCount[i] = GetTickCount();
			dwMarginTime = (DWORD)( PUMP_FLOW_LIMIT_CHECK_TIME * 1000 );
			nSeqNo = 10;
		}
		break;

	case 10 :
		if( !m_pEqpAssy->m_DiTank[i].GetPumpOnState() ||
		  ( fFlowRate[i] >= (float)DI_PUMP_INTERLOCK_MIN_FLOWRATE ) )
		{
			m_SeqFlag.bInsufficientDiFlowrate[i] = FALSE;
			nSeqNo = 0;
		}
		else if( GetTickCount() - dwStartTickCount[i] > dwMarginTime ) // 3 Sec.
		{
			m_pEqpAssy->m_DiTank[i].SetPumpOn(0, OFF);
	
			m_SeqFlag.bInsufficientDiFlowrate[i] = TRUE;

			SetLog( m_sSeqName, -1, -1, "Insufficient Flow-rate" );

			nAlarmId[i] = ALM_DI_PUMP1_INTERLOCK_ALARM + i;
			m_pEqpStateSeq->SetAlarm( nAlarmId[i] );
			nReturnSeqNo[i] = 0;
			nSeqNo = 1000;
		}
		break;

	case 1000 :
		if( m_pEqpAssy->m_DiTank[i].GetPumpOnState() ) m_pEqpAssy->m_DiTank[i].SetPumpOn(0, OFF);
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			SetLog( m_sSeqName, -1, -1, "Error Recovery" );

			m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );

			m_SeqFlag.bInsufficientDiFlowrate[i] = FALSE;
			nSeqNo = nReturnSeqNo[i];
		}
		break;
	}
	m_nSeqDiPumpInterlockNo[i] = nSeqNo;
}


//----------------------------------------------------------------------------------------------
// - SeqIdleRunning
//----------------------------------------------------------------------------------------------
void CDicsPumpSeq::SeqDiwIdleRunning()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
	if( !m_SeqFlag.bInitComp ) return;

	int nRunTime  = m_pSetupItems->m_nValueIdleRun[CSetupItems::idleDIW_IDLE_RUN_TIME];
	int nStopTime = m_pSetupItems->m_nValueIdleRun[CSetupItems::idleDIW_IDLE_STOP_TIME] * 60;

	BOOL bIdleCond = TRUE;
	bIdleCond &= m_SeqFlag.bInitComp;
//	bIdleCond &= !m_pDicsData->m_GenInfo.bPause;
	bIdleCond &= m_pDicsData->m_GenInfo.bAuto;
	bIdleCond &= !m_pDicsData->m_GenInfo.bEmo;
	bIdleCond &= !m_pDicsData->m_GenInfo.bLeakSensor;
	bIdleCond &= !m_pDicsData->m_GenInfo.bCoverOpen;
	bIdleCond &= !m_pDicsData->m_GenInfo.bDoorOpen;
 	bIdleCond &= !m_pDicsData->m_EqpInfo.nEqpState[CDicsData::stateDOWN];
 	
	bIdleCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE];
	bIdleCond &= m_pSetupItems->m_nValueIdleRun[CSetupItems::idleDIW_ZONE_USE];
	bIdleCond &= m_pDicsData->m_GenInfo.bIdleRunning;
	bIdleCond &= ( nRunTime > 0 );

#ifndef _MY_DEBUG
	bIdleCond &= !m_pDicsData->m_GenInfo.bCycleStop;
#endif

	static int nReturnSeqNo = 0;
	static int nOldTime = 0;

	int nTime = 0;
	int tm = TM_SEQ_DIW_IDLE;

	int nSeqNo = m_nSeqDiwIdleRunningNo;
					
	switch( nSeqNo )
	{
	case 0 :
		if( bIdleCond )
		{
			m_pDicsData->m_IfPump.bDiStart = !nStopTime;
			nSeqNo = 10;
		}
		break;
	case 10 :
		if( !bIdleCond )
		{			
			m_pDicsData->m_IfPump.bDiStart = FALSE;
			nSeqNo = 0;
		}
		else
		{
			nOldTime = 0;
			TIMER_ON( tm, nStopTime*10 );
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_START, nStopTime, FALSE );
			nSeqNo = 20;
		}
		break;
	case 20 : // Stop Time
		nTime = (int)( m_pTimer->m_pTimer[tm].dwNow / 100 );
		if( nTime - nOldTime > 2 )
		{
			nOldTime = nTime;
			
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_SET, nTime, FALSE );
		}
		if( !bIdleCond )
		{
			m_pDicsData->m_IfPump.bDiStart = FALSE;
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_END, 0, FALSE );
			nSeqNo = 0;
		}
		else if( TIMER( tm ) ) 
		{
			m_pDicsData->m_IfPump.bDiStart = TRUE;

			TIMER_ON( tm, nRunTime*10 );
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_START, nRunTime, FALSE );
			nOldTime = 0;

			nSeqNo = 30;
		}
		break;
	case 30 : // Run Time
		nTime = (int)( m_pTimer->m_pTimer[tm].dwNow / 100 );
		if( nTime - nOldTime > 2 ) 
		{
			nOldTime = nTime;
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_SET, nTime, FALSE );
		}
		if( !bIdleCond )
		{
			m_pDicsData->m_IfPump.bDiStart = FALSE;
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_END, 0, FALSE );
			nSeqNo = 0;
		}
		else if( TIMER( tm ) )
		{
			m_pDicsData->m_IfPump.bDiStart = !nStopTime;
			Msg( WM_DIW_IDLE_RUNNING_PROGRESS, PROGRESS_END, 0, FALSE );
			nSeqNo = 10;
		}
		break;
	}

	m_nSeqDiwIdleRunningNo = nSeqNo;
}

void CDicsPumpSeq::SeqAirKnife()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	static DWORD dwTickStart;
	int nSeqNo = m_nSeqAirKnifeNo;
	switch( nSeqNo )
	{
	case 0 :
		{
			dwTickStart = GetTickCount();
			nSeqNo = 10;
		}
		break;

	case 10 :
		if( GetTickCount() - dwTickStart > 1000 )
		{
			nSeqNo = 20;
		}
		break;

	case 20 :
		if( m_pDicsData->m_GenInfo.bAuto )
		{
			if( !m_pEqpAssy->m_AutoValveAkUp.GetState() ) m_pEqpAssy->m_AutoValveAkUp.Set(0, ON);
			if( !m_pEqpAssy->m_AutoValveAkLo.GetState() ) m_pEqpAssy->m_AutoValveAkLo.Set(0, ON);
		}
		else
		{
			if( ( m_pEqpAssy->m_CvMotorAll.GetMotorFwState( CV_MOTOR_NO_AK1 ) || m_pEqpAssy->m_CvMotorAll.GetMotorFwState( CV_MOTOR_NO_AK2 ) ) &&
				( !m_pEqpAssy->m_AutoValveAkUp.GetState() || !m_pEqpAssy->m_AutoValveAkLo.GetState() ) )
			{
				if( !m_pEqpAssy->m_AutoValveAkUp.GetState() ) m_pEqpAssy->m_AutoValveAkUp.Set(0, ON);
				if( !m_pEqpAssy->m_AutoValveAkLo.GetState() ) m_pEqpAssy->m_AutoValveAkLo.Set(0, ON);
			}
		}
		break;
	}
	m_nSeqAirKnifeNo = nSeqNo;
}

//----------------------------------------------------------------------------------------------
// Description :
//----------------------------------------------------------------------------------------------
void CDicsPumpSeq::SeqSbj( int i )
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	m_sSeqName.Format( "SBJ %d   ", i + 1 );

	int nSeqNo = m_nSeqSbjNo[i];
	static int nReturnSeqNo;

////////////////////////////////////////////////////////////////////////////////
// manual ¿ Pump, Valve Control ϱ ؼ Ʒ Flag ʿϴ.
	static BOOL bStop[MAX_SBJ_NO] = { TRUE, };
////////////////////////////////////////////////////////////////////////////////

	BOOL bRunCond = TRUE;
	bRunCond &= !m_pDicsData->m_GenInfo.bEmo &&
				!m_pDicsData->m_GenInfo.bLeakSensor &&
				!m_pDicsData->m_GenInfo.bCoverOpen &&
				!m_pDicsData->m_GenInfo.bDoorOpen &&
				( !m_pDicsData->m_GenInfo.bPause || !m_pDicsData->m_GenInfo.bAuto ) &&
				( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] == 1 );
				
	BOOL bSbjRun = bRunCond; 
	bSbjRun &=	m_pDicsData->m_IfPump.bDiStart;

	switch( nSeqNo )
	{ 
	case 0:
		if( bSbjRun )
		{
			bStop[i] = FALSE;		
			
			// Open SBJ CDA Valve
			if( !m_pEqpAssy->m_AutoValveSbjCda.GetState( i ) )
			{ 
				m_pEqpAssy->m_AutoValveSbjCda.Set( i, ON );
				SetLog( m_sSeqName, -1, -1, "SBJ CDA ON" );
			}
			// Open SBJ DI Valve
			if( !m_pEqpAssy->m_AutoValveSbjDi.GetState( i ) )
			{ 
				m_pEqpAssy->m_AutoValveSbjDi.Set( i, ON );
				SetLog( m_sSeqName, -1, -1, "SBJ DI ON" );
			}
		}
		nSeqNo = 10;
		break;

	case 10:
		if( !bSbjRun )
		{
			if( !bRunCond || m_pDicsData->m_GenInfo.bAuto || !bStop[i] )
			{
				bStop[i] = TRUE;
			
				// Close SBJ CDA Valve
				if( m_pEqpAssy->m_AutoValveSbjCda.GetState(i) )
				{ 
					m_pEqpAssy->m_AutoValveSbjCda.Set( i, OFF );
					SetLog( m_sSeqName, -1, -1, "SBJ CDA Stop" );
				}
				// Close SBJ DI Valve
				if( m_pEqpAssy->m_AutoValveSbjDi.GetState(i) )
				{ 
					m_pEqpAssy->m_AutoValveSbjDi.Set( i, OFF );
					SetLog( m_sSeqName, -1, -1, "SBJ DI Stop" );
				}
			}
		}
		nSeqNo = 0;
		break;
	}
	m_nSeqSbjNo[i] = nSeqNo;
}


void CDicsPumpSeq::SeqLevelSensor( int nTankNo, int nSensorNo )
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	static BOOL bInit = FALSE;
	static DWORD dwInitTick = GetTickCount();

	if( !bInit )
	{
		if( GetTickCount() - dwInitTick > 1000 )	bInit = TRUE;
		if( !bInit ) return;
	}

	m_sSeqName.Format( "LEVEL%d-%d", nTankNo + 1 , nSensorNo + 1);


	int nSeqNo	= m_nSeqLevelSensorNo[nTankNo][nSensorNo];
	DWORD nConfirmTime = atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupLEVEL_SENSOR_CONFIRM_TIME] );
	static int nAlarmId[MAX_DI_TANK_NO][lvMAX]		= { 0, };
	static DWORD dwStartTick[MAX_DI_TANK_NO][lvMAX] = { 0, };


	switch( nSeqNo )
	{
	case 0 :
		{
			if( m_pEqpAssy->m_DiTank[nTankNo].GetTankLevel( nSensorNo) &&
				!m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo] )
			{
				dwStartTick[nTankNo][nSensorNo] = GetTickCount();
			}
	
			nSeqNo = 10;
		}
		break;
	case 10 :
		if( ( GetTickCount()-dwStartTick[nTankNo][nSensorNo] > nConfirmTime*1000 ) &&
			 m_pEqpAssy->m_DiTank[nTankNo].GetTankLevel( nSensorNo) )
		{
			if( ( nSensorNo == lvL ) || m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo-1] )
			{
				if( !m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo] )	
				{
					m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo] = TRUE;
					SetLog( m_sSeqName, -1, -1, "Level ON : Confirm" );
				}
				nSeqNo = 0;
			}
			else
			{	// alarm
				m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo] = FALSE;
				m_SeqFlag.bDiTankLevelSensorFault[nTankNo][nSensorNo] = TRUE;
				SetLog( m_sSeqName, -1, -1, "Level Sensor Fault" );

				nAlarmId[nTankNo][nSensorNo] = ALM_DI_TANK1_LV_SENSOR_ALARM1 + ( nTankNo*lvMAX + nSensorNo );
				m_pEqpStateSeq->SetAlarm( nAlarmId[nTankNo][nSensorNo] );
				nSeqNo = 1000;			
			}
		}
		else if( !m_pEqpAssy->m_DiTank[nTankNo].GetTankLevel( nSensorNo) )
		{
			if( m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo] )
			{
				m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo] = FALSE;
				SetLog( m_sSeqName, -1, -1, "Level OFF : Confirm" );
			}

			if( ( nSensorNo == lvMAX-1 ) || !m_SeqFlag.bDiTankLevel[nTankNo][nSensorNo+1] )
			{
				nSeqNo = 0;
			}
			else
			{	// alarm
				m_SeqFlag.bDiTankLevelSensorFault[nTankNo][nSensorNo] = TRUE;
				SetLog( m_sSeqName, -1, -1, "Level Sensor Fault" );
				
				nAlarmId[nTankNo][nSensorNo] = ALM_DI_TANK1_LV_SENSOR_ALARM1 + ( nTankNo*lvMAX + nSensorNo );
				m_pEqpStateSeq->SetAlarm( nAlarmId[nTankNo][nSensorNo] );
				nSeqNo = 1000;			
			}
		}
		break;
		
	case 1000 :
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			m_SeqFlag.bDiTankLevelSensorFault[nTankNo][nSensorNo] = FALSE;
			SetLog( m_sSeqName, -1, -1, "Alarm reset" );

			m_pEqpStateSeq->ResetAlarm( nAlarmId[nTankNo][nSensorNo] );
			nAlarmId[nTankNo][nSensorNo] = 0;
			dwStartTick[nTankNo][nSensorNo] = 0;
			nSeqNo = 1100;
		}
		break;		
	case 1100 :
		if( GetTickCount()-dwStartTick[nTankNo][nSensorNo] > 2000 )
		{
			nSeqNo = 0;
		}
		break;
	}

	m_nSeqLevelSensorNo[nTankNo][nSensorNo] = nSeqNo;
}


BOOL CDicsPumpSeq::GetLevelSensorFault( int nTankNo )
{
	BOOL bFault = FALSE;

	for( int i = 0 ; i < lvMAX ; i++ )
	{
		bFault |= m_SeqFlag.bDiTankLevelSensorFault[nTankNo][i];
	}

	return bFault;
}

BOOL CDicsPumpSeq::GetHotTankLevelSensorFault( int nTankNo )
{
	BOOL bFault = FALSE;

	for( int i = 0 ; i < lvMAX ; i++ )
	{
		bFault |= m_SeqFlag.bHotDiTankLevelSensorFault[nTankNo][i];
	}

	return bFault;
}
