// CDicsRbSeq.cpp: implementation of the DicsRbSeq class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "DicsRbSeq.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDicsRbSeq::CDicsRbSeq()
{
}

CDicsRbSeq::~CDicsRbSeq()
{

}

void CDicsRbSeq::SetOwnerPointer(CWnd* pWnd)
{
	SetDicsView( (CDicsView *) pWnd );
	SetData( m_pDicsView->m_pDicsData );
	SetDevice( m_pDicsView->m_pDevice );
	SetSetupItems( m_pDicsView->GetDocument()->m_pSetupItems );
	SetEqpStateSequence( m_pDicsView->m_pEqpStateSeq );
	SetMmc( m_pDicsView->m_pMmc );
	SetSeqTimer( m_pDicsView->m_pTimer );
	SetEqpAssy( m_pDicsView->m_pEqpAssy );
}

BOOL CDicsRbSeq::Initialize()
{
	for( int i=0; i<m_pEqpAssy->m_RbMotor.GetMaxSectionNo(); i++ ) 
	{
		m_RbMotor.dErrRbVelocity[i] = 0;
		m_RbMotor.dCurRbVelocity[i] = 0;
		m_RbMotor.nRefRbVelocity[i] = 0;
	}
	
	InitParameter();

	if( MAX_RB_NO == MAX_RB_GAP_SERVO_NO )
		m_nServoMode = servoBOTH;
	else
		m_nServoMode = servoUP_ONLY;

	return TRUE;
}

void CDicsRbSeq::InitParameter()
{

	for( int i=0; i<m_pEqpAssy->m_RbMotor.GetMaxSectionNo(); i++  )
	{
		m_RbMotor.bRbMotorCond[i]	= TRUE;
		m_nSeqInitNo[i] = 0;
	}

	for( int i=0; i<MAX_RB_GAP_SERVO_NO; i ++ )
	{
		m_nSeqRbGapControlNo[i]		= 0;

		m_RbGap.nRefRbPosition[i]	= rbNOOP;
		m_RbGap.bGapSettingComp[i]	= FALSE;

		m_SeqFlag.bInitReq[i]	= FALSE;
		m_SeqFlag.bInitComp[i]	= FALSE;
		m_SeqFlag.bInitFail[i]	= FALSE;
	}
}

BOOL CDicsRbSeq::Sequence()
{
	Sleep( 10 );

#ifdef _RB_USE_SERVO
	SeqInit();
#endif

	for( int i=0; i<m_pEqpAssy->m_RbMotor.GetMaxSectionNo(); i++ ) 
	{
		SeqRbMotorControl( i );
		SeqRbMotorCond( i );
	}

#ifdef _RB_USE_SERVO
	for( i = 0 ; i <MAX_RB_GAP_SERVO_NO ; i ++ )
	{
		SeqRbGapControl( i );
	}
#endif

	
	return TRUE;
}

//---------------------------------------------------------------------------
// Set the Log
//---------------------------------------------------------------------------
void CDicsRbSeq::SetLog( CString sSeqName, int nPortNo, int nSlotNo, CString sMessage )
{
	// Log
	CString sLog;
	CString sPortNo;
	if( nPortNo <= 0 ) sPortNo = "";
	else sPortNo.Format( "%d", nPortNo );

	CString sSlotNo;
	if( nSlotNo <= 0 ) sSlotNo = "";
	else sSlotNo.Format( "%d", nSlotNo );

	sLog.Format( "SEQRB\t%s\t%s\t%s\t%s", sSeqName, sPortNo, sSlotNo, sMessage );
	Log.TextOut( sLog );

	sLog.Format( "%s : %s", sSeqName, sMessage );
	Msg( WM_EQP_LOG, 0, (LPARAM)(LPCTSTR) sLog );
}

BOOL CDicsRbSeq::IsRbUse( int nRbNo )
{
	BOOL bUse = TRUE;
	bUse &= m_pDicsData->m_JobCond.bRbUse[nRbNo];
	bUse &= ( ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE]==0 ) ? FALSE : TRUE );
	bUse &= IsRbPumpUse( nRbNo );

	switch( m_nServoMode )
	{
	case servoUP_ONLY:
//#ifndef _NO_RB		// 20100719 ssm
		if( nRbNo%2 == 0 )
			bUse &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupSWR2_RB_SERVO1_USE+(nRbNo/2)];
//#endif
		break;
	case servoBOTH:
//#ifndef _NO_RB		// 20100719 ssm
		bUse &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupSWR2_RB_SERVO1_USE+nRbNo];
//#endif
		break;
	}

	return bUse;
}


BOOL CDicsRbSeq::IsRbPumpUse( int nRbNo )
{
	BOOL bUse = TRUE;

	bUse &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP1_USE + nRbNo / MAX_RB_PART ];	// hskwak RB 4  Ʈ   . 4 ƴ   ʿ.

	return bUse;
}

BOOL CDicsRbSeq::IsRbPumpAlarm( int nRbNo )
{
	BOOL bAlarm = FALSE;

#ifndef _MY_DEBUG
	bAlarm = m_pEqpAssy->m_DiTank[0].GetPumpAlarm( CDicsPumpSeq::noDIPUMP1 + nRbNo / MAX_RB_PART );	// hskwak RB 4  Ʈ   . 4 ƴ   ʿ.
#endif

	return bAlarm;
}

BOOL CDicsRbSeq::IsRbMotorRun( int nRbNo )
{
	return m_RbMotor.nRefRbVelocity[nRbNo] != 0 ? TRUE : FALSE;
}

#ifdef _RB_USE_SERVO
void CDicsRbSeq::SeqInit()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	static int nAlarmId[MAX_RB_GAP_SERVO_NO]	= { 0, };
	static BOOL bHomeOk[MAX_RB_GAP_SERVO_NO]	= { FALSE, };
	static BOOL bServoUse[MAX_RB_GAP_SERVO_NO]	= { FALSE, };
	static int nServoNo = 0;
	static DWORD dwStartTickCount;

	int i, nRv = -1;
	int tm = TM_SEQ_INIT_RB;
	int nInitTimeout = atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupEQP_INITIAL_TIME] );
	int nSeqNo = m_nSeqInitNo;

	CString sSeqName( "INIT    " );

	if( m_SeqFlag.bInitComp ) return;

	switch( nSeqNo )
	{
	case 0:
		if( m_SeqFlag.bInitReq )
		{
			m_SeqFlag.bInitReq	= FALSE;
			m_SeqFlag.bInitComp = FALSE;
			m_SeqFlag.bInitFail = FALSE;

			nSeqNo = 10;
		}
		break;

	case 10:
		{
			nServoNo = 0;
			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO, MSG_STATUS_START );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
				
			SetLog( sSeqName, 0, 0, "RB Servo E-Stop Req." );
			TIMER_ON( tm, 20 ); // 2 sec
			nSeqNo = 20;
		}
		break;	
	case 20:
		if( m_pMmc->EStopRbt( RBT_RB_UNIT_UP_RB1_UNIT+nServoNo ) )
		{
			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_SERVO_ESTOP );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			m_sMessage.Format( "R/B%d servo motor E-stop OK", nServoNo + 1 );
			SetLog( sSeqName, 0, 0, m_sMessage );

			nServoNo++;
			if( nServoNo < MAX_RB_GAP_SERVO_NO )
			{
				m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_START );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
	
				TIMER_ON( tm, 20 );
			}
			else if( nServoNo >= MAX_RB_GAP_SERVO_NO )
			{
				nServoNo = 0;
				m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_START );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

				for( int i=0; i<MAX_RB_GAP_SERVO_NO; i++)
				{
					bServoUse[i]  = m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupRB_SERVO1_USE+i] ? 1 : 0;
					bServoUse[i] &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] ? 1 : 0;
				}

				TIMER_ON( tm, 20 ); // 2 sec				

				nSeqNo = 30;
			}
		}
		else if( TIMER( tm ) )
		{
			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_NG );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			nAlarmId[nServoNo] = ALM_RB_UNIT_UP_RB1_SERVO_ALARM + nServoNo;
			m_sMessage.Format( "R/B%d servo motor E-stop Error", nServoNo + 1 );
			
			m_pEqpStateSeq->SetAlarm( nAlarmId[nServoNo] );

			SetLog( sSeqName, 0, 0, m_sMessage );

			m_SeqFlag.bInitFail = TRUE;

			nSeqNo = 1000;
		}
		break;

	case 30 :
		if( !bServoUse[nServoNo] || 
		  ( m_pMmc->ResetRbt( RBT_RB_UNIT_UP_RB1_UNIT+nServoNo ) > 0 && 
		    m_pMmc->m_bServoReady[RBT_RB_UNIT_UP_RB1_UNIT+nServoNo] ) ) 
		{
			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_SERVO_RESET );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
 
			if( bServoUse[nServoNo] ) m_sMessage.Format( "R/B%d servo motor reset OK", nServoNo + 1 );
			else	m_sMessage.Format( "R/B%d servo motor reset Skip", nServoNo + 1 );
			SetLog( sSeqName, 0, 0, m_sMessage );

			nServoNo++;

			if( nServoNo < MAX_RB_GAP_SERVO_NO )
			{
				m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_START );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
				
				TIMER_ON( tm, 20 );
			}
			else if( nServoNo >= MAX_RB_GAP_SERVO_NO )
			{
				nServoNo = 0;
				m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_START );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

				for( i=0; i<MAX_RB_GAP_SERVO_NO; i++ ) bHomeOk[i] = FALSE;
				dwStartTickCount = GetTickCount();
		
				nSeqNo = 40;
			}
		}
		else if( TIMER( tm ) )
		{
			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_NG );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
	
			nAlarmId[nServoNo] = ALM_RB_UNIT_UP_RB1_SERVO_ALARM + nServoNo;
			m_sMessage.Format( "R/B%d servo motor Reset Error", nServoNo + 1 );

			m_pEqpStateSeq->SetAlarm( nAlarmId[nServoNo] );

			SetLog( sSeqName, 0, 0, m_sMessage );

			m_SeqFlag.bInitFail = TRUE;
			
			nSeqNo = 1000;
		}
		break;

	case 40:
		if( GetTickCount() - dwStartTickCount > 500 )
		{
			TIMER_ON( tm, 100 );
			nSeqNo = 50;
		}
		break;

	case 50:
		if( !bHomeOk[nServoNo] && 
		  ( !bServoUse[nServoNo] || 
		  ( nRv = m_pMmc->MoveRbtHome( RBT_RB_UNIT_UP_RB1_UNIT+nServoNo ) ) == 0 ) ) 
		{
			bHomeOk[nServoNo] = TRUE;

			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_OK );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			if( bServoUse[nServoNo] ) m_sMessage.Format( "R/B servo motor%d Homing OK", nServoNo + 1 );
			else	m_sMessage.Format( "R/B servo motor%d Homing Skip", nServoNo + 1 );
			SetLog( sSeqName, 0, 0, m_sMessage );				

			nServoNo++;
			if( nServoNo < MAX_RB_GAP_SERVO_NO )
			{
				m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_START );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

				dwStartTickCount = GetTickCount();
				nSeqNo = 40;
			}
			else if( nServoNo >= MAX_RB_GAP_SERVO_NO )
			{
				BOOL bAllOk = TRUE;
				for( nServoNo=0; nServoNo<MAX_RB_GAP_SERVO_NO; nServoNo++ ) bAllOk &= bHomeOk[nServoNo];
			
				if( bAllOk )
				{					
					TIMER_ON( tm, 10 ); // 1 sec
					nSeqNo = 60;
				}
			}
		}
		else if( nRv > 0 && TIMER( tm ) )
		{
			m_sMessage.Format("%2d%d", CDicsData::initSWR2_RB_UP1_SERVO+nServoNo, MSG_STATUS_NG );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			nAlarmId[nServoNo] = ALM_RB_UNIT_UP_RB1_SERVO_HOME + nServoNo;
			m_sMessage.Format( "R/B Servo motor%d homing error", nServoNo + 1 );
			m_pEqpStateSeq->SetAlarm( nAlarmId[nServoNo] );

			SetLog( sSeqName, 0, 0, m_sMessage );

			m_SeqFlag.bInitFail = TRUE;

			nSeqNo = 1000;
		}
		break;

	case 60:
		if( TIMER( tm ) )
		{
			m_SeqFlag.bInitComp = TRUE;
			m_SeqFlag.bInitReq	= FALSE;

			SetLog( sSeqName, 0, 0, "Init Complete" );
			
			nSeqNo = 0;
		}
		break;
	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			for( i=0; i<MAX_RB_GAP_SERVO_NO; i++ )
			{
				if( nAlarmId[i] )
				{
					m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );
					
					nAlarmId[i] = 0;
				}
			}

			// Log
			SetLog( sSeqName, 0, 0, "Alarm reset" );

			nSeqNo = 0;
		}
		break;
	}

	m_nSeqInitNo = nSeqNo;
}
#endif

void CDicsRbSeq::SeqRbMotorCond( int nRbNo )
{
/////////////////////////////////////////////////////////////////////////////
// bInit : Wait for Initialize Device 
/////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
/////////////////////////////////////////////////////////////////////////////	
				
	if( m_pDicsData->m_GenInfo.bEqpInitComp )
	{
		BOOL bUse = IsRbUse( nRbNo );
#ifndef _MY_DEBUG
		if( m_pEqpAssy->m_RbMotor.GetMotorAlarm( nRbNo ) && bUse ) 
#else
		if( FALSE )
#endif
		{
			m_RbMotor.bRbMotorCond[nRbNo] = FALSE;
//#ifndef _NO_RB
			if( !m_pEqpStateSeq->IsAlarm( ALM_RB_FBL_MOTOR_DRIVER_ALARM1 + nRbNo ) )
			{
				m_pEqpStateSeq->SetAlarm( ALM_RB_FBL_MOTOR_DRIVER_ALARM1 + nRbNo );
			}
//#endif
		}
#ifndef _MY_DEBUG
		else if( m_pEqpAssy->m_RbMotor.GetMotorCpAlarm( nRbNo ) && bUse ) 
#else
		else if( FALSE )
#endif
		{
			m_RbMotor.bRbMotorCond[nRbNo] = FALSE;
//#ifndef _NO_RB
			if( !m_pEqpStateSeq->IsAlarm( ALM_RB_FBL_MOTOR_CP_OFF1 + nRbNo ) )
			{
				m_pEqpStateSeq->SetAlarm( ALM_RB_FBL_MOTOR_CP_OFF1 + nRbNo );
			}
//#endif
		}
		else if( m_RbMotor.bRbMotorCond[nRbNo] != TRUE )
		{
			m_RbMotor.bRbMotorCond[nRbNo] = TRUE;
//#ifndef _NO_RB
			if( m_pEqpStateSeq->IsAlarm( ALM_RB_FBL_MOTOR_DRIVER_ALARM1 + nRbNo ) )
			{
				m_pEqpStateSeq->ResetAlarm( ALM_RB_FBL_MOTOR_DRIVER_ALARM1 + nRbNo );
			}
			if( m_pEqpStateSeq->IsAlarm( ALM_RB_FBL_MOTOR_CP_OFF1 + nRbNo ) )
			{
				m_pEqpStateSeq->ResetAlarm( ALM_RB_FBL_MOTOR_CP_OFF1 + nRbNo );
			}
//#endif
		}
	}	
}

//---------------------------------------------------------------------------
// - SetRbSpeed
//---------------------------------------------------------------------------
int CDicsRbSeq::SetRbSpeed( int nRbNo )
{
	static BOOL bManualModeChangeComp[MAX_RB_NO] = { FALSE, };
	int nRbSpeed;

	BOOL bRun = TRUE;
	bRun &= !m_pDicsData->m_GenInfo.bEmo;
	bRun &= !m_pDicsData->m_GenInfo.bLeakSensor;
	bRun &= !m_pDicsData->m_GenInfo.bCoverOpen;
	bRun &= !m_pDicsData->m_GenInfo.bDoorOpen;
	
#ifndef _MY_DEBUG
	bRun &= !m_pEqpAssy->m_RbMotor.GetMotorAlarm( nRbNo );
	bRun &= !m_pEqpAssy->m_RbMotor.GetMotorCpAlarm(nRbNo);
#endif


	if( !m_pDicsData->m_GenInfo.bAuto )
	{
		if( !bRun || !bManualModeChangeComp[nRbNo] )
		{
			bManualModeChangeComp[nRbNo] = TRUE;
			nRbSpeed = 0;
		}
		else
		{
			nRbSpeed = m_RbMotor.nRefRbVelocity[nRbNo];		
		}
	}
	else
	{
		bManualModeChangeComp[nRbNo] = FALSE;

		bRun &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE];
		bRun &= !IsRbPumpAlarm( nRbNo );
		bRun &= IsRbPumpUse( nRbNo );
		bRun &= m_pDicsData->m_IfPump.bDiStart;
		bRun &=	!m_pDicsData->m_GenInfo.bPause;
//		bRun &= !m_pDicsData->m_IfFlag[IF_UNIT_RB-1].bSeqStop;
		bRun &=	!m_pDicsData->m_SeqFlag.bStopCount2;
//		bRun &= !m_pDicsView->m_pMainSeq->m_IfFlag[IF_UNIT_RB].bGageFault;

		// Check CvStop Condition
		BOOL bCvStop = FALSE;
//#ifndef _NO_RB
		for( int i = IF_UNIT_SWR2; i<= IF_UNIT_UL; i++ )
		{
			bCvStop |= m_pDicsData->m_IfFlag[i].bInError;
			bCvStop |= m_pDicsData->m_IfFlag[i].bOutError;
		}
		bRun &= !bCvStop;
//#endif
		if( !bRun )
		{
			nRbSpeed = 0;	
		}
		else if( m_pDicsData->m_GenInfo.bIdleRunning )
		{
			if( !m_pSetupItems->m_nValueIdleRun[CSetupItems::idleDIW_ZONE_USE] )
			{
				nRbSpeed = 0;
			}
			else
			{
				int nCW  = m_pSetupItems->m_nValueIdleRun[CSetupItems::idleSWR2_UPPER_RB1_DIR + nRbNo];
				nRbSpeed = ( nCW ? 1:-1 ) * m_pSetupItems->m_nValueIdleRun[CSetupItems::idleSWR2_UPPER_RB1_SPEED + nRbNo];
			}
		}
		else 
		{
			BOOL bUse = IsRbUse( nRbNo );
			if( !bUse )
			{
				nRbSpeed = 0;
			}
			else
			{
				int nCW  = m_pDicsData->m_JobCond.bRbDir[nRbNo];
				nRbSpeed =  ( nCW ? 1:-1 ) * ( m_pDicsData->m_JobCond.nRbSpeed[nRbNo]);
			}
		}
	}

	return nRbSpeed;
}

//---------------------------------------------------------------------------
// - SeqRbMotorControl
//---------------------------------------------------------------------------
void CDicsRbSeq::SeqRbMotorControl(int nRbNo)
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif


/////////////////////////////////////////////////////////////////////////////
// bInit : Wait for Initialize Device 
/////////////////////////////////////////////////////////////////////////////
	static BOOL bInit = FALSE;
	static DWORD dwStartTick = GetTickCount();
	if( !bInit )
	{
		if( GetTickCount() - dwStartTick > 2000 )	bInit = TRUE;
		if( !bInit ) return;
	}
/////////////////////////////////////////////////////////////////////////////	


	int i = nRbNo;	

#ifndef _MY_DEBUG
	double dGain = 0.2;
#else
	double dGain = 0.5;
#endif

	m_RbMotor.nRefRbVelocity[i] = SetRbSpeed( i );

	// Get Error Value
	m_RbMotor.dErrRbVelocity[i] = m_RbMotor.nRefRbVelocity[i] - m_RbMotor.dCurRbVelocity[i];
	
	if( ABS( m_RbMotor.dErrRbVelocity[i] ) < 1. && ABS( m_RbMotor.dCurRbVelocity[i] ) < 1. ) 
		m_RbMotor.dErrRbVelocity[i] = 0.;	

	if( m_RbMotor.dErrRbVelocity[i] != 0. ) // +/- 10 rpm
	{
		double dProperty;
		dProperty = dGain * m_RbMotor.dErrRbVelocity[i];
		
		if( ABS( dProperty ) > 0.5 ) 
		{
			if( dProperty > 0 ) dProperty = 0.5;
			else dProperty = -0.5;
		}
		
		m_RbMotor.dCurRbVelocity[i] = m_RbMotor.dCurRbVelocity[i] + dProperty;
		
		if( m_RbMotor.dCurRbVelocity[i] > 0 ) 
		{
			m_pEqpAssy->m_RbMotor.SetMotorFw( i, ON );
			m_pEqpAssy->m_RbMotor.SetMotorBw( i, OFF );
		}
		else
		{
			m_pEqpAssy->m_RbMotor.SetMotorFw( i, OFF );
			m_pEqpAssy->m_RbMotor.SetMotorBw( i, ON );
		}
		int nVal = ( int )ABS( m_RbMotor.dCurRbVelocity[i] );
		m_pEqpAssy->m_RbMotor.SetMotorRpm( i , nVal );
	}
	else if( m_RbMotor.nRefRbVelocity[i] == 0 )
	{
		m_pEqpAssy->m_RbMotor.SetMotorFw( i, OFF );
		m_pEqpAssy->m_RbMotor.SetMotorBw( i, OFF );
	}
}

//---------------------------------------------------------------------------
// - SetRbGap
//---------------------------------------------------------------------------
int CDicsRbSeq::SetRbGap( int nRbNo )
{
	static BOOL bManualModeChangeComp[MAX_RB_GAP_SERVO_NO] = { FALSE, };
	int nRbPosNo = rbNOOP;

	BOOL bRun = TRUE;
	bRun &= !m_pDicsData->m_GenInfo.bEmo;

	BOOL bGapRun = bRun;
	bGapRun &= m_pDicsData->m_GenInfo.bEqpInitComp;
	bGapRun &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE];
//#ifndef _NO_RB		// 20100719 ssm
	bGapRun &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupSWR2_RB_SERVO1_USE+nRbNo];
//#endif

	if( !m_pDicsData->m_GenInfo.bAuto )
	{
		if( !bRun || !bManualModeChangeComp[nRbNo] )
		{		
			bManualModeChangeComp[nRbNo] = TRUE;
			
			if( !bRun )			nRbPosNo = rbOFF;
			else if( bGapRun )	nRbPosNo = rbWAIT;
		}
		else
		{
			nRbPosNo = rbNOOP;
		}
	}
	else
	{
		bManualModeChangeComp[nRbNo] = FALSE;

		int nIndex = ( m_nServoMode == servoBOTH ) ? nRbNo : nRbNo*2;

		BOOL bSetPos = bGapRun;
		bSetPos &= !m_pDicsData->m_GenInfo.bIdleRunning;
		bSetPos &= ( IsRbMotorRun( nIndex ) != 0 ) ? TRUE : FALSE;
		bSetPos &= m_pDicsData->m_JobCond.bRbUse[nIndex];
	
		if( bSetPos )		nRbPosNo = rbPROCESS;	
		else if( bGapRun )	nRbPosNo = rbWAIT;
		else if( bRun )		nRbPosNo = rbNOOP;
		else				nRbPosNo = rbOFF;
	}

	return nRbPosNo;
}

#ifdef _RB_USE_SERVO
void CDicsRbSeq::SeqRbGapControl(int nRbNo)
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	int i = nRbNo;
	int nRv;
	int nSeqNo	 = m_nSeqRbGapControlNo[i];
	int nServoNo = RBT_RB_UNIT_UP_RB1_UNIT + i;

	static RBT_POS	rpRefPos[MAX_RB_GAP_SERVO_NO];
	static RBT_POS	rpOldRefPos[MAX_RB_GAP_SERVO_NO];
	static RBT_POS	rpCurPos[MAX_RB_GAP_SERVO_NO];
	static DWORD	dwTickStart[MAX_RB_GAP_SERVO_NO]		= { 0, };
	static int		nAlarmId[MAX_RB_GAP_SERVO_NO];

	m_sSeqName.Format( "RB GAP %d", i+1 );	

	m_RbGap.nRefRbPosition[i] = SetRbGap( i );

	switch( nSeqNo )
	{
	case 0 :
		if( m_RbGap.nRefRbPosition[i] == rbNOOP )
		{
			m_RbGap.bGapSettingComp[i] = TRUE;
		}
		else if( m_RbGap.nRefRbPosition[i] == rbOFF && !m_pDicsData->m_GenInfo.bEmo )
		{ // NoUse Case
			if( !m_pMmc->m_bServoReady[nServoNo] )
			{
				m_RbGap.bGapSettingComp[i] = TRUE;
			}
			else
			{
				m_RbGap.bGapSettingComp[i] = FALSE;
				m_sMessage.Format( "R/B%d Servo E-Stop Req", i+1 );
				SetLog( m_sSeqName, 0, 0, m_sMessage );
				nSeqNo = 200;
			}
		}
		else if( m_RbGap.nRefRbPosition[i] == rbPROCESS )
		{ // ProcessPosition Move
			rpRefPos[i].dPos[Z_AXIS] = m_pDicsData->m_JobCond.dRbGap[i] + 
										m_pMmc->m_RbtInfo[nServoNo].pos[RBT_RB_ZERO_POS].dPos[Z_AXIS];
			nSeqNo = 10;
		}
		else if( m_RbGap.nRefRbPosition[i] == rbWAIT )
		{ // Wait Position Move
			rpRefPos[i].dPos[Z_AXIS] = m_pMmc->m_RbtInfo[nServoNo].pos[RBT_RB_WAIT_POS].dPos[Z_AXIS];
			nSeqNo = 10;
		}
		break;
	case 10 :
		if( rpRefPos[i].dPos[Z_AXIS] == rpOldRefPos[i].dPos[Z_AXIS] )
		{
			if( m_RbGap.bGapSettingComp[i] )	nSeqNo = 0;
			else	nSeqNo = 100;
		}
		else if( rpRefPos[i].dPos[Z_AXIS] != rpOldRefPos[i].dPos[Z_AXIS] )
		{
			rpOldRefPos[i] = rpRefPos[i];
			m_sMessage.Format( "R/B%d Servo Pos Change Req", i+1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );

			if( m_RbGap.bGapSettingComp[i] )
			{ 
				m_RbGap.bGapSettingComp[i] = FALSE;
				nSeqNo = 100;
			}
			else
			{
				m_sMessage.Format( "R/B%d Servo Stop Req", i+1 );
				SetLog( m_sSeqName, 0, 0, m_sMessage );
				nSeqNo = 200;			
			}
		}
		break;
	case 100 :
		if( !m_pMmc->m_bServoReady[nServoNo] )
		{
			m_sMessage.Format( "R/B%d Servo Reset Req", i+1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
			nSeqNo = 300;
		}
		else if( !m_pMmc->m_bHomeComp[nServoNo] || !m_pMmc->m_bServoReady[nServoNo] )
		{
			nAlarmId[i] = ALM_RB_UNIT_UP_RB1_SERVO_MOVE_POS + i;
			m_pEqpStateSeq->SetAlarm( nAlarmId[i] );

			m_sMessage.Format( "R/B%d Servo Not Ready", i+1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
			
			nSeqNo = 1000;		
		}
#ifndef _MY_DEBUG
		else if( ( nRv = m_pMmc->MoveRbtPos( nServoNo, rpRefPos[i] ) ) == 0 )
#else	
		else if( TRUE )
#endif
		{
			rpCurPos[i] = rpRefPos[i];

			if( !m_RbGap.bGapSettingComp[i] )
			{
				m_RbGap.bGapSettingComp[i] = TRUE;
				
				if( m_RbGap.nRefRbPosition[i] == rbPROCESS )
				{
					m_sMessage.Format( "R/B%d Servo Gap Set Comp : %2.2f",  i + 1, rpRefPos[i].dPos[Z_AXIS] );
				}
				else if( m_RbGap.nRefRbPosition[i] == rbWAIT )
				{
					m_sMessage.Format( "R/B%d Servo Wait Set Comp : %2.2f",  i + 1, rpRefPos[i].dPos[Z_AXIS] );
				}
				
				SetLog( m_sSeqName, 0, 0, m_sMessage );
			}
			
			nSeqNo = 0;
		}
		else if( nRv > 0 )
		{
			nAlarmId[i] = ALM_RB_UNIT_UP_RB1_SERVO_MOVE_POS + i;
			m_pEqpStateSeq->SetAlarm( nAlarmId[i] );
			m_sMessage.Format( "R/B%d Servo Moving Error : %d", i+1, nRv );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
			
			nSeqNo = 1000;
		}
		else
		{
			nSeqNo = 0;
		}
		break;
	case 200 :
		if( m_pMmc->EStopRbt( nServoNo ) &&
			!m_pMmc->m_bServoReady[nServoNo] )
		{
			m_sMessage.Format( "R/B%d Servo E-Stop Comp", i+1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
			nSeqNo = 0;
		}
		break;
	case 300 :
		if( m_pMmc->ResetRbt( nServoNo ) &&
			m_pMmc->m_bServoReady[nServoNo] )
		{
			m_sMessage.Format( "R/B%d Servo Reset Comp", i+1 );
			SetLog( m_sSeqName, 0, 0, m_sMessage );
			nSeqNo = 0;
		}
		break;
	case 1000 :
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			SetLog( m_sSeqName, 0, 0, "Error Recovery" );
			m_pEqpStateSeq->ResetAlarm( nAlarmId[i] );

			dwTickStart[i] =  GetTickCount();
			nSeqNo = 1100;
		}
		break;
	case 1100 :
		if( ( GetTickCount() - dwTickStart[i] ) > 2000 )
		{
			nSeqNo = 0;
		}
		break;
	}

	m_nSeqRbGapControlNo[i] = nSeqNo;
}
#endif

BOOL CDicsRbSeq::GetRbCond()
{
	BOOL bCond = TRUE;

	for( int i = 0; i < MAX_RB_NO; i++ )
	{
		BOOL bUse = IsRbUse( i );
		if( !bUse )
		{
			bCond &= ( m_RbMotor.nRefRbVelocity[i] == 0 );
			bCond &= ( ABS( m_RbMotor.dErrRbVelocity[i] ) < 50 ) ;
		}
		else
		{
			bCond &= m_RbMotor.bRbMotorCond[i];
			bCond &= ( m_RbMotor.nRefRbVelocity[i] != 0 || m_pDicsData->m_JobCond.nRbSpeed[i] == 0 ) ;
			bCond &= ( ABS( m_RbMotor.dErrRbVelocity[i] ) < 50 ) ;
		}
	}

	for( int i = 0; i < MAX_RB_GAP_SERVO_NO; i++ )
	{
		bCond &= m_RbGap.bGapSettingComp[i];
	}
		
	return bCond;
}
