#include <stdafx.h>
#include "XSecs.h"
#include "..\X\XLog.h"

CMutex g_SecsMutex;
CMutex g_SecsMutexOut;
/*HSMS*/const int SECS_LengthBlockSize = 4;
const int SECS_HeaderSize = 10;
const int SECS_ChecksumSize = 2;
/*HSMS*/const int SECS_MaxHDBlockSize = 32766;
const int SECS_MaxDataBlockSize = SECS_MaxHDBlockSize - SECS_HeaderSize;
const int SECS_DataBlockOffset = SECS_LengthBlockSize + SECS_HeaderSize;
const int SECS_MaxMsgBlockBufSize = SECS_LengthBlockSize + SECS_HeaderSize + SECS_MaxHDBlockSize + SECS_ChecksumSize;

// -- XSecsLibrary -- //
//
XSecsTranxThread* XSecsLibrary::Find( LPCTSTR szName, int iDir )	// szName = SmFn or Description
{
	XSecsTranxThread* pTranx;
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
	{
		pTranx = m_TranxList.GetNext( pos );
		if( (pTranx->GetName() == szName || pTranx->GetDescription() == szName)
			&& ((iDir==-1)?TRUE:(pTranx->Primary.GetDirection()==iDir)) )
				return pTranx;
	}
	return NULL;	// Not Exist!
}

XSecsTranxThread* XSecsLibrary::AddNewTransaction( LPCTSTR SmFn, LPCTSTR Description )
{
	m_TranxList.AddTail( new XSecsTranxThread( SmFn, Description ) );
	return m_TranxList.GetTail();
}


// -- XSecsMessageBlock -- //

XSecsMessageBlock::XSecsMessageBlock()
{
	ClearAll();
}

XSecsMessageBlock::~XSecsMessageBlock()
{
	if( m_lpBuf ) delete m_lpBuf;
	ClearAll();
}

void XSecsMessage::SetItemData( LPCTSTR lpszName, LPCTSTR pBuf, int iLength, BOOL bDelete )
{
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
//	if( pBuf == NULL ) pBuf = "0";	// 100520 halesu
	if( strcmp( lpszName, "L" ) == 0 /* || strcmp( lpszName, "N" ) == 0*/ ) return;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( lpszName, 0, bDelete );
	if( pItem != NULL ) pItem->SetData( pBuf, iLength );
}

void XSecsMessage::SetItemData( LPCTSTR lpszName, unsigned char* pBuf, int iLength, BOOL bDelete )
{
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
	if( strcmp( lpszName, "L" ) == 0 /*|| strcmp( lpszName, "N" ) == 0*/ ) return;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( lpszName, 0, bDelete );
	if( pItem != NULL ) pItem->SetData( pBuf, iLength );
}

void XSecsMessage::SetHeader( LPCTSTR lpszName, LPCTSTR lpszDescription, int iDirectionFlag, int iWaitFlag )
{
	m_szName = lpszName;
	m_szDescription = lpszDescription;

	CString szSFCodedName( lpszName );				// -- ver 0.91 : lpszName SmFm format̾...
	m_iStreamCode = atoi(szSFCodedName.Mid( 1, szSFCodedName.Find('F')-1 ));
	m_iFunctionCode = atoi( szSFCodedName.Mid(szSFCodedName.Find('F')+1) );		// -- ver 0.9
	m_iDirectionFlag = iDirectionFlag;
	m_iWaitFlag = iWaitFlag;
}


void XSecsMessage::AddNewItem( LPCTSTR lpszName, LPCTSTR lpszDescription, BYTE byteFormat, int iLength )
{
	XSecsItem* pItem = m_Item.FindItem( "" );	// -- ver 0.9 : append only to tail
	if( pItem != NULL )
	{
		pItem->Format( m_iNItems, lpszName, lpszDescription, byteFormat, iLength );
		if( byteFormat != XSecsItem::wsFormatList /*&& byteFormat != XSecsItem::wsFormatVirtualList*/ ) m_iNItems++;
	}
}

void XSecsMessageBlock::Build(BYTE *lpDataBlock, int iDataBlockSize)
{
	if( m_lpBuf ) delete m_lpBuf;

	// allocate for each message
	m_iDataBlockSize = iDataBlockSize;
	m_lpBuf = new BYTE[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize];

/*HSMS*/	const int HSMSLengthOffset = 3;

	// LENGTH
//HSMS//	m_lpBuf[0]	= SECS_HeaderSize + iDataBlockSize;
	int len = SECS_HeaderSize + iDataBlockSize;
	m_lpBuf[0] = (len>>24)&0x0ff;
	m_lpBuf[1] = (len>>16)&0x0ff;
	m_lpBuf[2] = (len>>8)&0x0ff;
	m_lpBuf[3] = (len)&0x0ff;
/*HSMS*/

	// HEADER
	//m_lpBuf[1+HSMSLengthOffset]	= (BYTE)(m_iDeviceID>>8) | (BYTE)((m_iDirectionFlag==XSecsMessage::toHost)?0x80:0x00);
	m_lpBuf[1+HSMSLengthOffset]	= (BYTE)(m_iDeviceID>>8) | (BYTE)((m_iDirectionFlag==XSecsMessage::toHost)?0x00:0x00);
	m_lpBuf[2+HSMSLengthOffset]	= (BYTE)m_iDeviceID;
	m_lpBuf[3+HSMSLengthOffset]	= (BYTE)m_iStreamCode | (BYTE)((m_iWaitFlag==XSecsMessage::waitFlagOn)?0x80:0x00);
	m_lpBuf[4+HSMSLengthOffset]	= (BYTE)m_iFunctionCode;
	m_lpBuf[5+HSMSLengthOffset]	= (BYTE)(m_iBlockNo>>8) | (BYTE)(m_iEndBlockFlag)?0x80:0x00;
	m_lpBuf[6+HSMSLengthOffset]	= (BYTE)m_iBlockNo;
	m_lpBuf[7+HSMSLengthOffset]	= m_byteSystemBytes[0];
	m_lpBuf[8+HSMSLengthOffset]	= m_byteSystemBytes[1];
	m_lpBuf[9+HSMSLengthOffset]	= m_byteSystemBytes[2];
	m_lpBuf[10+HSMSLengthOffset]	= m_byteSystemBytes[3];

	for( int jj = 0; jj<10; jj++ )
	{
		m_cHead[jj] = m_lpBuf[jj + 1 + HSMSLengthOffset];
	}


	// DATA
	memcpy( m_lpBuf+SECS_DataBlockOffset, lpDataBlock, m_iDataBlockSize );

	// CHECKSUM
	unsigned int uChecksum = 0;
	for( int i=SECS_LengthBlockSize; i<SECS_LengthBlockSize+SECS_HeaderSize+iDataBlockSize; i++ ) uChecksum += m_lpBuf[i];
	m_lpBuf[SECS_LengthBlockSize+SECS_HeaderSize+iDataBlockSize] = (BYTE)((uChecksum>>8)&0x00ff);
	m_lpBuf[SECS_LengthBlockSize+SECS_HeaderSize+iDataBlockSize+1] = (BYTE)(uChecksum&0x00ff);
}

BOOL XSecsMessageBlock::Parse( BYTE* lpBuf )
{
	// -- ver 0.9 : only for reserved item data, fixed format
	//?if( m_lpData == NULL ) Build();		// -- ver 0.9
	//if( lpBuf[0] != SECS_HeaderSize + m_iDataBlockSize + SECS_ChecksumSize ) return FALSE;	// Error() : Size ġ

/*HSMS*/	const int HSMSLengthOffset = 3;

	// LENGTH???
//HSMS//	m_iDataBlockSize = lpBuf[0] - SECS_HeaderSize;
/*HSMS*/
	m_iDataBlockSize = ((int)lpBuf[0]<<24) + ((int)lpBuf[1]<<16) 
		+ ((int)lpBuf[2]<<8) + ((int)lpBuf[3]) - SECS_HeaderSize;

	// Check CHECKSUM
/*	unsigned int uChecksum = 0;
	for( int i=SECS_LengthBlockSize; i<SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize; i++ ) uChecksum += lpBuf[i];
	if( lpBuf[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize] != (BYTE)((uChecksum>>8)&0x00ff) 
		&& lpBuf[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+1] != (BYTE)(uChecksum&0x00ff) ) return FALSE;	// Error() : Checksum Error
*/	/*
		RESERVED CODE FOR VER 1.0

	// allocation when receive!!! ( with lpBuf[0] )
	lpBuf = new BYTE[ m_Item.GetSize() + HEADERSIZE+CHECKSUMSIZE ];
	// LENGTH
	lpBuf[0]	= m_iLength =  m_Item.GetSize() + HEADERSIZE;
	*/

	// HEADER - miss match 쿣  óϴ°?(!)
	m_iDeviceID		= ((int)(lpBuf[1+HSMSLengthOffset]&0x7f)<<8) | (int)lpBuf[2+HSMSLengthOffset];
	m_iStreamCode	= (int)lpBuf[3+HSMSLengthOffset]&0x7f;
	m_iFunctionCode	= (int)lpBuf[4+HSMSLengthOffset];
	m_iDirectionFlag= (lpBuf[1+HSMSLengthOffset]&0x80)?(XSecsMessage::toHost):(XSecsMessage::fromHost);
	m_iWaitFlag		= (lpBuf[3+HSMSLengthOffset]&0x80)?(XSecsMessage::waitFlagOn):(XSecsMessage::waitFlagOff);
	m_iBlockNo		= ((int)(lpBuf[5+HSMSLengthOffset]&0x7f)<<8) | (int)lpBuf[6+HSMSLengthOffset];
	m_iEndBlockFlag	= (lpBuf[5+HSMSLengthOffset]&0x80)?1:0;
	m_byteSystemBytes[0] = lpBuf[7+HSMSLengthOffset];
	m_byteSystemBytes[1] = lpBuf[8+HSMSLengthOffset];
	m_byteSystemBytes[2] = lpBuf[9+HSMSLengthOffset];
	m_byteSystemBytes[3] = lpBuf[10+HSMSLengthOffset];

	// ITEM DATA -- ver 0.9 : Check lpBuf & Set to m_lpData
	if( m_lpBuf ) delete m_lpBuf;
	m_lpBuf = new BYTE[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize];
	memcpy( m_lpBuf, lpBuf, SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize );
//	if( m_Item.ParseIHData( m_lpData+SECS_LengthBlockSize+SECS_HeaderSize, lpBuf+SECS_LengthBlockSize+SECS_HeaderSize ) == -1 )
//		return FALSE;	// Error 

	for( int jj = 0; jj<10; jj++ )
	{
		m_cHead[jj] = lpBuf[jj + 1 + HSMSLengthOffset];
	}

	return TRUE;
}

// -- XSecsItem -- //

XSecsItem::XSecsItem()
{
	m_iItemNo = -1;
	m_lpData = NULL;
	m_iLength = -1;
	m_iBackupLength = -1;
	m_iByteLength = -1;
	m_byteFormat = wsFormatNull;
	m_szName = "";
}

XSecsItem::~XSecsItem()
{
	if( m_lpData != NULL )
	{
		if( m_byteFormat == wsFormatList /*|| m_byteFormat == wsFormatVirtualList*/ )
		{
			for( int i=0; i<m_iLength; i++ )
				delete ((XSecsItem**)m_lpData)[i];
		}
		delete (XSecsItem**)m_lpData;
	}
}

void XSecsItem::Format( int iItemNo, LPCTSTR lpszName, LPCTSTR lpszDescription, BYTE byteFormat, int iLength )
{
	// ver 0.9 : only for new item, can not be modified
	int i;

	m_szName = lpszName;
	m_szDescription = lpszDescription;

	m_byteFormat = byteFormat;
	m_iBackupLength = m_iLength = iLength;

	if( m_szName == "L" /*|| m_szName == "N"*/ )
		m_iByteLength = 0;
	else
		m_iByteLength = iLength;	// -- ver 0.9 : equal to m_iLength in this version

	switch( byteFormat )
	{
		case wsFormatList:
//		case wsFormatVirtualList:
			if( iLength != 0 )		// ver 0.9 : å !
			{
				m_lpData = new XSecsItem*[iLength];
				for( i=0; i<iLength; i++ )
					((XSecsItem**)m_lpData)[i] = new XSecsItem;
			}
			break;
		case wsFormatBinary:
			m_lpData = (void*)new BYTE[iLength];
			memset( m_lpData, 0, iLength );
			m_iItemNo = iItemNo;
			break;
		case wsFormatBoolean://2008.11.26 kimgun	
			m_lpData = (void*)new BYTE[iLength];
			memset( m_lpData, 0, iLength );
			m_iItemNo = iItemNo;
			break;
		case wsFormatAscii:
		case wsFormatU1:
			m_lpData = (void*)new BYTE[iLength];
			memset( m_lpData, 0, iLength );
			m_iItemNo = iItemNo;
			break;
		case wsFormatI2:
		case wsFormatU2:
			iLength *= 2;
			m_iBackupLength = m_iLength = iLength;
			m_lpData = (void*)new BYTE[iLength];
			memset( m_lpData, 0, iLength );
			m_iItemNo = iItemNo;
			break;
		case wsFormatU4:
			iLength *= 4;
			m_iBackupLength = m_iLength = iLength;
			m_lpData = (void*)new BYTE[iLength];
			memset( m_lpData, 0, iLength );
			m_iItemNo = iItemNo;
			break;
	}

	if( m_szName == "L" /*|| m_szName == "N"*/ )
		m_iByteLength = 0;
	else
		m_iByteLength = iLength;	// -- ver 0.9 : equal to m_iLength in this version
}

void XSecsItem::SetData( LPCTSTR lpData, int iLength )
{
	//ASSERT m_lpData, m_iLength;
	switch( m_byteFormat )
	{
	case wsFormatAscii :
		memset( m_lpData, ' ', m_iLength );
		memcpy( m_lpData, (void*)lpData, min(m_iLength,(int)strlen((LPCTSTR)lpData)/*iLength*/) );
		break;
	case wsFormatBinary :
	case wsFormatBoolean://2008.11.26 kimgun	
	case wsFormatU1:
		//if( m_iLength != iLength ) return;	// -- ver 0.9 : Format Error !!
		memcpy( m_lpData, (void*)lpData, min(m_iLength,iLength) );
		break;
	case wsFormatI2 :
	case wsFormatU2 :
	case wsFormatU4 :
//		memcpy( m_lpData, (void*)lpData, min(m_iLength,iLength)*2 );
		memcpy( m_lpData, (void*)lpData, min(m_iLength,iLength) );
		break;
	default :
		// Not formatted !!
		break;
	}
}

void XSecsItem::SetData( unsigned char* lpData, int iLength )
{
	//ASSERT m_lpData, m_iLength;
	switch( m_byteFormat )
	{
	case wsFormatAscii :
		memset( m_lpData, ' ', m_iLength );
		memcpy( m_lpData, (void*)lpData, min(m_iLength,(int)strlen((LPCTSTR)lpData)/*iLength*/) );
		break;
	case wsFormatBinary :
	case wsFormatBoolean://2008.11.26 kimgun
	case wsFormatU1:
		//if( m_iLength != iLength ) return;	// -- ver 0.9 : Format Error !!
		memcpy( m_lpData, (void*)lpData, min(m_iLength,iLength) );
		break;
	case wsFormatI2 :
	case wsFormatU2 :
	case wsFormatU4 :
//		memcpy( m_lpData, (void*)lpData, min(m_iLength,iLength)*2 );
		memcpy( m_lpData, (void*)lpData, min(m_iLength,iLength) );
		break;
	default :
		// Not formatted !!
		break;
	}
}

XSecsItem& XSecsItem::operator [](int idx)
{
	if( idx < 0 || idx >= m_iLength ) return *this;
	return *((XSecsItem**)m_lpData)[idx];
}

XSecsItem* XSecsItem::FindItem(LPCTSTR lpszName, int iDepth, BOOL bDelete)
{
	XSecsItem* pItem;
	static BOOL bFlag = FALSE;

	if( m_szName == lpszName ) return this;

	if( iDepth == 0 ) bFlag = FALSE;
	if( ( m_szName == "L" /*|| m_szName == "N"*/ ) && m_iLength != 0 )
	{
		iDepth += 1;

		for( int i=0; i<m_iLength; i++ )
		{
			pItem = ((XSecsItem**)m_lpData)[i]->FindItem( lpszName, iDepth, bDelete );
			if( pItem != NULL ) 
			{
				if( bFlag ) return pItem;
				if( bDelete ) { bFlag = TRUE; m_iBackupLength = m_iLength; m_iLength = 0; }
				return pItem;
			}
		}
	}
	else if( iDepth == 0 && m_iLength != 0 ) return this;
	return NULL;
}

//  List Size ǵ.
void XSecsItem::SetListSize()
{
	if( m_szName == "") return;
	if( m_szName == "L" )
	{
		if( m_iLength != m_iBackupLength ) m_iLength = m_iBackupLength;
		for( int i=0; i<m_iLength; i++ )
		{
			((XSecsItem**)m_lpData)[i]->SetListSize();
		}
	}
}

// ϴ List Size ٲ۴.
void XSecsItem::SetListSize(int iDepth, int iLength, int iTargetLen, int iIncDepth, int iTargetList, int iIncList)
{
	static int iCurDepth;
	static int iCurList = 0;
	iCurDepth = iIncDepth;
	iCurList = iIncList;

	if( m_szName == "L" )
	{
		
		if( iTargetLen > 0 )
		{
			if( iCurDepth == iDepth && iTargetLen == m_iLength ) 
			{
				if( iTargetList == -1 )
				{
					m_iBackupLength = m_iLength;
					m_iLength = iLength;
					return;
				}
					
				iCurList++;
				if( iTargetList == iCurList )
				{
					m_iBackupLength = m_iLength;
					m_iLength = iLength;
					return;
				}
			}
		}
		else
		{
			if( iCurDepth == iDepth ) 
			{
				m_iBackupLength = m_iLength;
				m_iLength = iLength;
//				iCurDepth = 0; //LYS20080219
				return;
			}
		}

		iCurDepth += 1;
		for( int i=0; i<m_iLength; i++ )
		{
			((XSecsItem**)m_lpData)[i]->SetListSize( iDepth, iLength, iTargetLen, iCurDepth, iTargetList, iCurList );
		}
		if( iCurDepth > 0)  //LYS20080219
			iCurDepth -= 1;
	}
}

int XSecsItem::GetListSize(int iDepth, int iIncDepth )
{
	static int iCurDepth = 0;

	iCurDepth = iIncDepth ;

	if( m_szName == "L" )
	{
		if( iCurDepth == iDepth ) 
		{
			iCurDepth = 0;
			return m_iLength;
		}

		iCurDepth += 1;
		for( int i=0; i<m_iLength; i++ )
		{
			int iLength = ((XSecsItem**)m_lpData)[i]->GetListSize(iDepth, iCurDepth );
			if( iLength > 0 ) return iLength;
		}
	}

	return 0;
}

void XSecsItem::SaveAllItems(XLog* pLog, int iDepth)
{
	CString szBuf;
	static CString szTab;
	int i;
	if( !iDepth ) szTab = "";

	if( m_szName == "") return;
	if( m_szName == "L" && m_iLength != 0 )
	{
		szTab = "";
		for( i=0; i<iDepth; i++ ) szTab += "\t";
		iDepth += 1;

		szBuf.Format( "L%d", m_iLength );
		szBuf = szTab + szBuf;
		pLog->TextOut( szBuf, XLog::stampNoDateTime );

		for( i=0; i<m_iLength; i++ )
		{
			((XSecsItem**)m_lpData)[i]->SaveAllItems( pLog, iDepth );
		}
	}
/*	else if( m_szName == "N" && m_iLength != 0 )
	{
		szTab = "";
		for( int i=0; i<iDepth; i++ ) szTab += "\t";
		iDepth += 1;

		szBuf.Format( "N%d", m_iLength );
		szBuf = szTab + szBuf;
		SFLog.TextOut( szBuf, XLog::stampNoDateTime );

		for( i=0; i<m_iLength; i++ )
		{
			((XSecsItem**)m_lpData)[i]->SaveAllItems( iDepth );
		}
	}*/
	else
	{
		char buf[256];

		GetData( buf );
		if( m_byteFormat == wsFormatAscii ) szBuf.Format( "<A %s \'%s\'>", m_szName,  buf );
		else if( m_byteFormat == wsFormatBinary ) 
		{
			CString szBuf1;

			szBuf.Format( "<B %s \'%d", m_szName, (unsigned char)buf[0] );
			for( int i=1; i<m_iLength; i++ )
			{
				szBuf1.Format( " %d", (unsigned char)buf[i] );
				szBuf += szBuf1;
			}
			szBuf1.Format("\'>");
			szBuf += szBuf1;
		}
		else if( m_byteFormat == wsFormatBoolean ) //2008.11.26 kimgun
		{
			CString szBuf1;

			szBuf.Format( "<B %s \'%d", m_szName, (unsigned char)buf[0] );
			for( int i=1; i<m_iLength; i++ )
			{
				szBuf1.Format( " %d", (unsigned char)buf[i] );
				szBuf += szBuf1;
			}
			szBuf1.Format("\'>");
			szBuf += szBuf1;
		}
		else if( m_byteFormat == wsFormatU1 ) 
		{
			CString szBuf1;

			szBuf.Format( "<U1 %s \'%d", m_szName, (unsigned char)buf[0] );
			for( int i=1; i<m_iLength; i++ )
			{
				szBuf1.Format( " %d", (unsigned char)buf[i] );
				szBuf += szBuf1;
			}
			szBuf1.Format("\'>");
			szBuf += szBuf1;
		}
		else if( m_byteFormat == wsFormatI2 ) 
		{
			CString szBuf1;

			szBuf.Format( "<I2 %s \'%d", m_szName, (int)buf[0] );
			for( int i=1; i<m_iLength; i++ )
			{
				szBuf1.Format( " %d", (int)buf[i] );
				szBuf += szBuf1;
			}
			szBuf1.Format("\'>");
			szBuf += szBuf1;
		}
		else if( m_byteFormat == wsFormatU2 ) 
		{
			CString szBuf1;

			szBuf.Format( "<U2 %s \'%d", m_szName, (unsigned char)buf[0] );
			for( int i=1; i<m_iLength; i++ )
			{
				szBuf1.Format( " %d", (unsigned char)buf[i] );
				szBuf += szBuf1;
			}
			szBuf1.Format("\'>");
			szBuf += szBuf1;
		}
		else if( m_byteFormat == wsFormatU4 ) 
		{
			CString szBuf1;

			szBuf.Format( "<U4 %s \'%d", m_szName, (unsigned char)buf[0] );
			for( int i=1; i<m_iLength; i++ )
			{
				szBuf1.Format( " %d", (unsigned char)buf[i] );
				szBuf += szBuf1;
			}
			szBuf1.Format("\'>");
			szBuf += szBuf1;
		}
		szBuf = "\t" + szTab + szBuf;
		pLog->TextOut( szBuf, XLog::stampNoDateTime );
	}
}

int XSecsItem::GetSize()
{
	int i, childSize = 0, IHlength;

	IHlength = 2;			/* -- version 0.9 */

	if( m_szName == "" ) return 0;
	if( m_szName == "L" /*|| m_szName == "N"*/ )
	{
		if( m_iLength == 0 ) return IHlength;
		for( i=0; i<m_iLength; i++ )
			childSize += ((XSecsItem**)m_lpData)[i]->GetSize();

//		if( m_szName == "N" ) IHlength = 0;
	}
	return childSize + m_iByteLength + IHlength;
}
void XSecsItem::BuildIHData(BYTE* buf)
{
	XSecsItem* pItem;
	unsigned int offset = 0, IHlength;

	IHlength = 2;							/* -- version 0.9 */
	const int LengthOfIHLength = 1;			/* -- version 0.9 */

	if( m_szName == "" ) return;

//	if( m_szName != "N" )
	{
		buf[0] = m_byteFormat | LengthOfIHLength;	/* -- version 0.9 */
//		if( m_byteFormat == wsFormatI2 ) buf[1] = m_iLength*2;
//		else 
		buf[1] = m_iLength;
		offset = IHlength;
	}

	if( m_szName == "L" /*|| m_szName == "N"*/ )
	{
		if( m_iLength == 0 ) return;
		for( int i=0; i<m_iLength; i++ )
		{
			pItem = ((XSecsItem**)m_lpData)[i];
			pItem->BuildIHData( buf + offset );
			offset += pItem->GetSize();
		}
	}
	else
		memcpy( buf+IHlength, m_lpData, m_iByteLength );
}

BOOL XSecsItem::GetData(void *lpData)	// SHOULD BE TESTED!!! How about length management?
{
	if( m_lpData == NULL || m_iLength < 0 ) return FALSE;	// Error() : Not Formatted !!

	memcpy( lpData, m_lpData, m_iLength );
	if( m_byteFormat == wsFormatAscii ) ((LPTSTR)lpData)[m_iLength] = '\x0';

	return TRUE;
}

BOOL XSecsMessage::GetItemData(char *lpszName, void *pBuf)
{
	// -- ӽ!!!
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
	if( strcmp( lpszName, "L" ) == 0 /*|| strcmp( lpszName, "N" ) == 0*/ ) return FALSE;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( lpszName );
	if( pItem != NULL ) pItem->GetData( pBuf );
	return TRUE;
}

XSecsItem* XSecsItem::FindItem(int iItemNo)
{
	XSecsItem* pItem;

	if( m_iItemNo == iItemNo ) return this;
	if( m_szName == "L" && m_iLength != 0 )
	{
		for( int i=0; i<m_iLength; i++ )
		{
			pItem = ((XSecsItem**)m_lpData)[i]->FindItem( iItemNo );
			if( pItem != NULL ) return pItem;
		}
	}
	return NULL;
}

void XSecsMessage::SetItemData(int iItemNo, LPCTSTR pBuf, int iLength)
{
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
	//if( strcmp( lpszName, "L" ) == 0 ) return;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( iItemNo );
	if( pItem != NULL ) pItem->SetData( pBuf, iLength );
}

int XSecsMessageBlock::GetBuf(unsigned char *lpBuf)
{
//HSMS//	const int iLength = SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize;
/*HSMS*/	const int iLength = SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize;
	memcpy( lpBuf, m_lpBuf, iLength );
	return iLength;
}

XSecsLibrary::XSecsLibrary()
{
}

XSecsLibrary::~XSecsLibrary()
{
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
		delete m_TranxList.GetNext( pos );
	//m_TranxList.RemoveAll();
}

void XSecsLibrary::SetDefaults()
{
	XSecsTranxThread* t;

	// S1F1 - S1F2 (Host->Machine)
	t = AddNewTransaction( "S1F1", "Online Confirm" );
	if( t == NULL )
	{
		// Allocation fail
		return;//-1????
	}
	
	t->Primary.SetHeader( "S1F1", "", XSecsMessage::fromHost, XSecsMessage::waitFlagOn );
	t->Primary.Build();
	
	t->Secondary.SetHeader( "S1F2", "", XSecsMessage::fromHost, XSecsMessage::waitFlagOn );
	t->Secondary.AddNewItem( "L", "", XSecsItem::wsFormatList, 2 );
	t->Secondary.AddNewItem( "MDLN", "ġ", XSecsItem::wsFormatAscii, 6 );
	t->Secondary.AddNewItem( "SOFTREV", "Soft Revision", XSecsItem::wsFormatAscii, 6 );
	t->Secondary.Build();

	// S1F69 - S1F70 (Machine->Host)
	t = AddNewTransaction( "S1F69", "Batch Report" );
	if( t == NULL )
	{
		// Allocation Fail
		return;//-1????
	}
	
	t->Primary.SetHeader( "S1F69", "", XSecsMessage::fromHost, XSecsMessage::waitFlagOn );
	t->Primary.AddNewItem( "L", "", XSecsItem::wsFormatList, 2 );
	t->Primary.AddNewItem( "RMS", "", XSecsItem::wsFormatAscii, 3 );
	t->Primary.AddNewItem( "ALMS", "", XSecsItem::wsFormatAscii, 6 );
	t->Primary.Build();

	t->Secondary.SetHeader( "S1F70", "", XSecsMessage::fromHost, XSecsMessage::waitFlagOn );
	t->Secondary.Build();
}

int XSecsItem::ParseIHData(BYTE *lpFormatted, BYTE *lpInput)
{
	int i = 0, subLength = 0, length = 0;

	if( m_iLength < 0 ) return 0;

#define IH_FORMAT(X) ((X)&0xfc)
#define IH_DATA_LEN(X) ((X)&0x03) 
	// Hong Jaehee ? m_iLength *lpInput   ?
	int nIHLen = IH_DATA_LEN( *lpInput );

	if( nIHLen == 1 ) // 0XFF, 255
	{
		int nLen = *(lpInput+1);
		if( nLen != m_iLength ) 
		{
			m_iBackupLength = m_iLength;
		}
		m_iLength = nLen;
	}
	else if( nIHLen == 2 ) // 0XFFFF, 65535
	{
		int nLen = ((*(lpInput+1)&0xFF)<<8) + *(lpInput+2);
		if( nLen != m_iLength ) 
		{
			m_iBackupLength = m_iLength;
		}
		m_iLength = nLen;
		lpInput++;
	}
	else if( nIHLen == 3 ) // 0XFFFFFF, 16777215
	{
		int nLen = ((*(lpInput+1)&0xFF)<<16) + ((*(lpInput+2)&0xFF)<<8) + *(lpInput+3);
		if( nLen != m_iLength ) 
		{
			m_iBackupLength = m_iLength;
		}
		m_iLength = nLen;
		lpInput++;
		lpInput++;
	}


	switch( IH_FORMAT(/**lpFormatted*/ *lpInput) )
	{
		case XSecsItem::wsFormatList :
			if( m_szName != "L" ) return -1;						// Error() : Item List name, type ġ
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatList ) return -1;	// Error() : Formatted m_lpData format ġ
			if( m_iLength == 0 ) return 2/*ver 0.9*/;
			lpFormatted += 2;	// ver 0.9
			lpInput += 2;		// ver 0.9
			for( i=0, length=0; i<m_iLength; i++ )
			{
				subLength = ((XSecsItem**)m_lpData)[i]->ParseIHData( lpFormatted + length, lpInput + length );
				if( subLength == -1 ) return -1;	// Error 
				length += subLength;
			}
			break;
		case XSecsItem::wsFormatBinary :
			if( m_byteFormat != XSecsItem::wsFormatBinary ) return -1;// Error() : Item type mismatch with Item List format
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatBinary ) return -1;	// Error() : Item type mismatch with m_lpData
			length = m_iLength;		// ver 0.9
			lpFormatted += 2;		// ver 0.9
			lpInput += 2;			// ver 0.9
			memcpy( lpFormatted, lpInput, length );
			memcpy( m_lpData, lpInput, length );
			break;
		case XSecsItem::wsFormatU1 :
			if( m_byteFormat != XSecsItem::wsFormatU1 ) return -1;// Error() : Item type mismatch with Item List format
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatU1 ) return -1;	// Error() : Item type mismatch with m_lpData
			length = m_iLength;		// ver 0.9
			lpFormatted += 2;		// ver 0.9
			lpInput += 2;			// ver 0.9
			memcpy( lpFormatted, lpInput, length );
			memcpy( m_lpData, lpInput, length );
			break;
		case XSecsItem::wsFormatI2 :
			if( m_byteFormat != XSecsItem::wsFormatI2 ) return -1;// Error() : Item type mismatch with Item List format
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatI2 ) return -1;	// Error() : Item type mismatch with m_lpData
//			length = m_iLength*2;		// ver 0.9
			length = m_iLength;		// ver 0.9
			lpFormatted += 2;		// ver 0.9
			lpInput += 2;			// ver 0.9
			memcpy( lpFormatted, lpInput, length );
			memcpy( m_lpData, lpInput, length );
			break;
		case XSecsItem::wsFormatU2 :
			if( m_byteFormat != XSecsItem::wsFormatU2 ) return -1;// Error() : Item type mismatch with Item List format
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatU2 ) return -1;	// Error() : Item type mismatch with m_lpData
			length = m_iLength;		// ver 0.9
			lpFormatted += 2;		// ver 0.9
			lpInput += 2;			// ver 0.9
			memcpy( lpFormatted, lpInput, length );
			memcpy( m_lpData, lpInput, length );
			break;
		case XSecsItem::wsFormatU4 :
			if( m_byteFormat != XSecsItem::wsFormatU4 ) return -1;// Error() : Item type mismatch with Item List format
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatU4 ) return -1;	// Error() : Item type mismatch with m_lpData
			length = m_iLength;		// ver 0.9
			lpFormatted += 2;		// ver 0.9
			lpInput += 2;			// ver 0.9
			memcpy( lpFormatted, lpInput, length );
			memcpy( m_lpData, lpInput, length );
			break;
		case XSecsItem::wsFormatAscii :
			if( m_byteFormat != XSecsItem::wsFormatAscii ) 
				return -1;// Error() : Item type mismatch with Item List
			if( IH_FORMAT(*lpInput) != XSecsItem::wsFormatAscii ) 
				return -1;	// Error() : Item type mismatch with m_lpData

			//length = m_iLength;		// ver 0.9
			length = min(m_iLength, *(lpInput+1));		// ver 0.9
			memset( lpFormatted, 0, m_iLength );
			memset( m_lpData, 0, m_iLength );

			lpFormatted += 2;		// ver 0.9
			lpInput += 2;			// ver 0.9
			memcpy( lpFormatted, lpInput, length );
			memcpy( m_lpData, lpInput, length );
			break;
		default :
			return -1;	// Error() : Not a ver 0.9 format code
	}
	return 2/*ver 0.9*/ + length;
}

int XSecsItem::FindIHData(BYTE *lpFormatted, BYTE *lpInput)
{
	int i = 0, subLength = 0, length = 0;

	if( *(lpInput+1) != m_iLength ) return -1;				// Error() : Data length mismatch

#define IH_FORMAT(X) ((X)&0xfc)

	switch( IH_FORMAT(/**lpFormatted*/ *lpInput) )
	{
		case XSecsItem::wsFormatList :
			if( m_szName != "L" ) return -1;						// Error() : Item List name, type ġ
			if( m_iLength == 0 ) return 2/*ver 0.9*/;
			lpFormatted += 2;	// ver 0.9
			lpInput += 2;		// ver 0.9
			for( i=0, length=0; i<m_iLength; i++ )
			{
				subLength = ((XSecsItem**)m_lpData)[i]->FindIHData( lpFormatted + length, lpInput + length );
				if( subLength == -1 ) return -1;	// Error 
				length += subLength;
			}
			break;
		case XSecsItem::wsFormatBinary :
			if( m_byteFormat != XSecsItem::wsFormatBinary ) return -1;// Error() : Item type mismatch with Item List format
			length = m_iLength;		// ver 0.9
			break;
		case XSecsItem::wsFormatU1 :
			if( m_byteFormat != XSecsItem::wsFormatU1 ) return -1;// Error() : Item type mismatch with Item List format
			length = m_iLength;		// ver 0.9
			break;
		case XSecsItem::wsFormatI2 :
			if( m_byteFormat != XSecsItem::wsFormatI2 ) return -1;// Error() : Item type mismatch with Item List format
			length = m_iLength;		// ver 0.9
			break;
		case XSecsItem::wsFormatU2 :
			if( m_byteFormat != XSecsItem::wsFormatU2 ) return -1;// Error() : Item type mismatch with Item List format
			length = m_iLength;		// ver 0.9
			break;
		case XSecsItem::wsFormatU4 :
			if( m_byteFormat != XSecsItem::wsFormatU4 ) return -1;// Error() : Item type mismatch with Item List format
			length = m_iLength;		// ver 0.9
			break;
		case XSecsItem::wsFormatAscii :
			if( m_byteFormat != XSecsItem::wsFormatAscii ) return -1;// Error() : Item type mismatch with Item List
			length = min(m_iLength, *(lpInput+1));		// ver 0.9
			break;
		default :
			return -1;	// Error() : Not a ver 0.9 format code
	}
	return 2/*ver 0.9*/ + length;
}

int XSecsMessage::GetStreamCode()	// ӽ
{
	return m_iStreamCode;
}

int XSecsMessage::GetFunctionCode()	// ӽ
{
	return m_iFunctionCode;
}

int XSecsLibrary::LoadSDF(LPCTSTR lpszFileName)
{
	CString szBuf;
	CFile f;

	#ifdef __LOG__
	m_pLoadSDFLog->TimeStampOut();
	szBuf.Format( "<%s> Loading Start", lpszFileName );
	m_pLoadSDFLog->TextOut( szBuf );
	#endif

	if( !f.Open( lpszFileName, CFile::modeRead ) )
	{
		#ifdef __LOG__
		m_pLoadSDFLog->TextOut( "FileOpen Error" );
		#endif
		return -1;	// Error() : File Open Error
	}

	CArchive ar( &f, CArchive::load );

	XSDFTokenList xTokenList;

	// Lexical Analysis
	while( ar.ReadString( szBuf ) )		// How can I check EOS ?????????????
			xTokenList.LexicalAnalysis( szBuf, m_pLoadSDFLog );

	#ifdef __LOG__
	m_pLoadSDFLog->TextOut( "\r\nLexical Analysis Complete\r\n" );
	#endif

	// Parser
	CString szName, szDescr;
	XSDFToken xToken;
	XSecsTranxThread* t;	// You're always in my mind.

	xToken = xTokenList.GetHead();
	while( xToken != "" )	// until Empty or End-of-TokenList
	{
		// Transaction Declaration & Header
		if( xToken != "TRANSACTION:" ) 
		{
#ifdef __LOG__
			m_pLoadSDFLog->TextOut( "Parser Error : TRANSACTION" );
#endif
			return  -1; 			// [Transaction:]
		}
		xToken = xTokenList.GetNext();
		if( xToken.Type != XSDFToken::ttSFCode ) 
		{
#ifdef __LOG__
			m_pLoadSDFLog->TextOut( "Parser Error : SF Code Error" );
#endif
			return  -1;	// [SmFn]
		}
		szName = xToken;
		xToken = xTokenList.GetNext();
		if( xToken.Type == XSDFToken::ttDescription )
		{
			szDescr = xToken;							// ['Description']
			xToken = xTokenList.GetNext();
		}
		else szDescr = "";

		// Add to Transaction List
		#ifdef __LOG__
		m_pLoadSDFLog->TextOut( "\r\nAdd New Transaction : " + szName + szDescr );
		#endif

		if( (t = AddNewTransaction( (LPCTSTR)szName, (LPCTSTR)szDescr )) == NULL )
		{
#ifdef __LOG__
			m_pLoadSDFLog->TextOut( "Parser Error : Add New Transaction Error" );
#endif
			return -1;	// Allocation fail
		}

		// [{]
		if( xToken != "{" ) 
		{
#ifdef __LOG__
			m_pLoadSDFLog->TextOut( "Parser Error : Not \"{\"" );
#endif
			return  -1;
		}

		// Primary Message
		if( (xToken=xTokenList.GetNext()) != "PRIMARY:" ) 
		{
#ifdef __LOG__
			m_pLoadSDFLog->TextOut( "Parser Error : Not \"PRIMARY\"" );
#endif
			return  -1;				// [Primary:]
		}
		ParseMsgHeader( xTokenList, t->Primary );
		xToken = xTokenList.GetCur();
		if( xToken != "SECONDARY:" && xToken != "}" )
		{
			ParseItem( xTokenList, t->Primary );
		}
		xToken = xTokenList.GetNext();
		t->Primary.Build();

		// Secondary Message
		if( xToken == "SECONDARY:" )	// [Secondary:]
		{
			ParseMsgHeader( xTokenList, t->Secondary );
			if( (xToken=xTokenList.GetCur()) != "}" )
			{
				ParseItem( xTokenList, t->Secondary );
			}
			xToken = xTokenList.GetNext();
			t->Secondary.Build();
		}

		if( xToken != "}" && xToken != "TRANSACTION:" && xToken != "") 
		{
#ifdef __LOG__
			m_pLoadSDFLog->TextOut( "Parser Error : Not \"{\" and \"PRIMARY\"" );
#endif
			return  -1;
//			return  0;	//̰͵ ӽ
		}
		if( xToken == "}" ) xToken = xTokenList.GetNext();
	}

	#ifdef __LOG__
	m_pLoadSDFLog->TextOut( "SDF file Loading Complete." );
	#endif

	f.Close();
	
	return 0;	//ӽ???
}

int XSecsLibrary::ParseMsgHeader(XSDFTokenList& xTokenList, XSecsMessage& xMsg)
{
	XSDFToken xToken, xTOKEN;
	CString szName, szDescr;
	int iDirectionFlag, iWaitFlag;

	xToken = xTokenList.GetNext();
	if( xToken.Type != XSDFToken::ttSFCode ) return  -1;				// [SmFn]
	szName = xToken;
	xToken = xTokenList.GetNext();
	if( xToken.Type == XSDFToken::ttDescription )
	{
		szDescr = xToken;												// ['Description']
		xToken = xTokenList.GetNext();
	}
	else szDescr = "";
	if( xToken == "H->M" || xToken == "M<-H" )		iDirectionFlag = XSecsMessage::fromHost;		// [Direction:H->M/M->H]
	else if( xToken == "H<-M" || xToken == "M->H" ) iDirectionFlag = XSecsMessage::toHost;
	else return  -1;
	xToken = xTokenList.GetNext();
	xTOKEN = xToken;
	xTOKEN.MakeUpper();
	if( xTOKEN == "WAIT" )
	{
		iWaitFlag = XSecsMessage::waitFlagOn;							// [WaitFlag]
	}
	else
	{
		iWaitFlag = XSecsMessage::waitFlagOff;
		xToken = xTokenList.GetPrev();
	}

	// Set Message Header
	xMsg.SetHeader( szName, szDescr, iDirectionFlag, iWaitFlag );

	#ifdef __LOG__
	CString szBuf;
	szBuf.Format( "SetHeader( %s, %s, %d, %d )", (LPCTSTR)szName, (LPCTSTR)szDescr, iDirectionFlag, iWaitFlag );
	m_pLoadSDFLog->TextOut( szBuf );
	#endif

	return 0;
}

int XSecsLibrary::ParseItem(XSDFTokenList& xTokenList, XSecsMessage& xMsg)
{
	XSDFToken xToken, xTOKEN;
	CString szName, szDescr;
	int iSize = 0, iType = 0;

	// Item Data Declaration
	xToken = xTokenList.GetNext();
	while( xToken.Type == XSDFToken::ttItemName )							// <ItemName>
	{
		if( xToken.Left(1) == "L" && (xToken.Mid(1)==xToken.Mid(1).SpanIncluding("0123456789")) )
		{
			iSize = atoi( xToken.Mid(1) );
			xMsg.AddNewItem( "L", "", XSecsItem::wsFormatList, iSize );

			#ifdef __LOG__
			CString szBuf;
			szBuf.Format( "Add New Item( L, '', <list>, %d )", iSize );
			m_pLoadSDFLog->TextOut( szBuf );
			#endif
		}
		else
		{
			szName = xToken;
			xToken = xTokenList.GetNext();
			if( xToken.Type == XSDFToken::ttDescription )					// ['Description']
			{
				szDescr = xToken;
				xToken = xTokenList.GetNext();
			}
			else szDescr = "";
			xTOKEN = xToken;
			xTOKEN.MakeUpper();
			if( xTOKEN == "ASCII" ) iType = XSecsItem::wsFormatAscii;		// [Type]
			else if( xTOKEN == "BIN" || xTOKEN == "BINARY" ) iType = XSecsItem::wsFormatBinary;
			else if( xTOKEN == "BOOL" ) 
			{
				iType = XSecsItem::wsFormatBoolean;
			}
			else if( xTOKEN == "I2" ) iType = XSecsItem::wsFormatI2;
			else if( xTOKEN == "U1" ) iType = XSecsItem::wsFormatU1;
			else if( xTOKEN == "U2" ) iType = XSecsItem::wsFormatU2;
			else if( xTOKEN == "U4" ) iType = XSecsItem::wsFormatU4;
			else return  -1;
			xToken = xTokenList.GetNext();
			if( xToken != xToken.SpanIncluding("0123456789") ) return  -1;	// [Size]
			iSize = atoi( xToken );

			xMsg.AddNewItem( szName, szDescr, iType, iSize );

			#ifdef __LOG__
			CString szBuf;
			szBuf.Format( "Add New Item( %s, %s, %d, %d )", (LPCTSTR)szName, (LPCTSTR)szDescr, iType, iSize );
			m_pLoadSDFLog->TextOut( szBuf );
			#endif
		}
		xToken = xTokenList.GetNext();
	}
	xToken = xTokenList.GetPrev();

	return 0;
}


//////////////////////////////////////////////////////////////////////
// XSDFTokenList Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XSDFTokenList::XSDFTokenList()
{
	//m_xNull = "";
	m_xNull.Type = XSDFToken::ttOther;
}

XSDFTokenList::~XSDFTokenList()
{
	for( m_pos=m_xTokenList.GetHeadPosition(); m_pos!=NULL; )
	{
		if( m_pos == NULL ) break;
		delete /*(XSDFToken&)*/m_xTokenList.GetNext( m_pos );
	}
}

XSDFToken& XSDFTokenList::GetHead()
{
	m_pos = m_xTokenList.GetHeadPosition();
	return (XSDFToken&)*m_xTokenList.GetNext( m_pos );
}

XSDFToken& XSDFTokenList::GetNext()
{
	if( m_pos == NULL )
		return m_xNull;
	else
		return (XSDFToken&)*m_xTokenList.GetNext( m_pos );
}

XSDFToken& XSDFTokenList::GetPrev()
{
	if( m_pos == NULL )
		return m_xNull;
	else
		return (XSDFToken&)*m_xTokenList.GetPrev( m_pos );
}

XSDFToken& XSDFTokenList::GetCur()
{
	if( m_pos == NULL )
		return m_xNull;
	else
		return (XSDFToken&)*m_xTokenList.GetAt( m_pos );
}

void XSDFTokenList::LexicalAnalysis(CString &szBuf, XLog* pLog)
{
	XSDFToken xToken;
	xToken.Type = XSDFToken::ttOther;

	for(;;)
	{
		szBuf.TrimLeft( " \t," );
		if( szBuf.GetLength() == 0 ) break;
		if( szBuf.SpanExcluding(" \t") == ":" )		// Block Declaration; Transaction/Primary/Secondary
		{
			XSDFToken *t;
			t = m_xTokenList.GetTail();
			t->MakeUpper();
			*t += ":";
			t->Type = XSDFToken::ttBDeclaration;
			szBuf.TrimLeft( ": \t" );
			continue;
		}
		else if( szBuf.Left(1) == "<" )				// 'ItemName' token
		{
			xToken = (XSDFToken&)szBuf.Mid(1).SpanExcluding( ">" );	// (HOW)if fail return  -1
			xToken.Type = XSDFToken::ttItemName;
			// modified 990203 : ѱۿ TrimLeft ̴.
			//szBuf.TrimLeft( xToken + "<>" );
			szBuf = szBuf.Mid( szBuf.Find('>')+1 );
		}
		else if( szBuf.Left(1) == "'" )				// 'Description' token
		{
			xToken = (XSDFToken&)szBuf.Mid(1).SpanExcluding( "'" ); // (HOW)if fail return  -1
			xToken.Type = XSDFToken::ttDescription;
			// modified 990203 : ѱۿ TrimLeft ̴.
			//szBuf.TrimLeft( xToken + "'" );
			szBuf = szBuf.Mid( szBuf.Mid(1).Find("'")+2 );
		}
		else										// other tokens
		{
			xToken = (XSDFToken&)szBuf.SpanExcluding( " \t," );
			if( xToken.GetLength() >=4 )	// modified 990203 : ߿ ٽ Ұ!--"SF" ϱ  ߻ϴ...
			{
				if( xToken.SpanIncluding( "SF0123456789_" ) == xToken	// Check SF Code
					&& xToken.Find('S') == 0
					&& (xToken.Find('F') == 2 || xToken.Find('F') == 3) 
					&& xToken[1] >= '0' && xToken[1] <= '9' )
						xToken.Type = XSDFToken::ttSFCode;
				else	xToken.Type = XSDFToken::ttOther;
			}
			else	xToken.Type = XSDFToken::ttOther;

			szBuf.TrimLeft( xToken );
		}
		if( xToken.Left(1) == "!" ) return;			// ! = comment

		#ifdef __LOG__
		CString szBuf;
		switch( xToken.Type )
		{
		case XSDFToken::ttSFCode :
			szBuf.Format( "%s(SF Code)", (LPCTSTR)xToken );
			pLog->TextOut( szBuf );
			break;
		case XSDFToken::ttItemName :
			szBuf.Format( "%s(ItemName)", (LPCTSTR)xToken );
			pLog->TextOut( szBuf );
			break;
		case XSDFToken::ttDescription :
			szBuf.Format( "%s(Description)", (LPCTSTR)xToken );
			pLog->TextOut( szBuf );
			break;
		case XSDFToken::ttOther :
			szBuf.Format( "%s", (LPCTSTR)xToken );
			pLog->TextOut( szBuf );
			break;
		}
		#endif
		m_xTokenList.AddTail( new XSDFToken( xToken ) );
	}
}

//////////////////////////////////////////////////////////////////////
// XSDFToken Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XSDFToken::XSDFToken()
{
	Type = ttOther;
}

XSDFToken::~XSDFToken()
{

}

int XSecsMessageBlock::GetBufSize()
{
	return SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize;
}

int XSecsMessage::GetDirection()
{
	return m_iDirectionFlag;
}

XSecsTranxThread* XSecsLibrary::Find(int Sm, int Fn, int dir)
{
	//Primary    : dir == fromHost, pTranx->m_hThread == NULL
	//Secondary  : dir == -1	  , pTranx->m_hThread != NULL
	//Secondary (T3 TimeOut retry ) : dir == -1	  , pTranx->m_hThread == NULL => return NULL
	XSecsTranxThread* pTranx;
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
	{
		pTranx = m_TranxList.GetNext( pos );
		if( ( pTranx->Primary.GetStreamCode() == Sm && pTranx->Primary.GetFunctionCode() == Fn
		    || pTranx->Secondary.GetStreamCode() == Sm && pTranx->Secondary.GetFunctionCode() == Fn )
			&& ( (dir==-1)?TRUE:(pTranx->Primary.GetDirection()==dir) )
			/*&& ( dir == XSecsMessage::fromHost || pTranx->m_hThread != NULL )*/ ) /*KDS041223*/
				return pTranx;
	}
	return NULL;	// Not Exist!
}

XSecsTranxThread* XSecsLibrary::Find(XSecsMessage &msg, int Sm, int Fn, int dir)
{
	XSecsTranxThread* pTranx;
	XSecsTranxThread* pOldTranx = NULL;
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
	{
		pTranx = m_TranxList.GetNext( pos );

		if( ( pTranx->Primary.GetStreamCode() == Sm && pTranx->Primary.GetFunctionCode() == Fn
		    || pTranx->Secondary.GetStreamCode() == Sm && pTranx->Secondary.GetFunctionCode() == Fn )
			&& ( (dir==-1)?TRUE:(pTranx->Primary.GetDirection()==dir) ) 
			/*&& ( dir == XSecsMessage::fromHost || pTranx->m_hThread != NULL )*/ ) /*KDS041223*/
		{
			if( dir == XSecsMessage::fromHost )
			{
				pOldTranx = pTranx;
				if( pTranx->Primary.FindItemData( msg ) == 0 ) return pTranx;
			}
			else return pTranx;
		}
	}
	if( pOldTranx != NULL ) return pOldTranx;
	return NULL;	// Not Exist!
}

#define _SmNotFound		-1
#define _FnNotFound		0
#define _FoundOk		1
int XSecsLibrary::SmFnFind(int Sm, int Fn)
{
	int iRv = _SmNotFound;

	if( Sm == 0 ) return _SmNotFound; //LYS20070608

	XSecsTranxThread* pTranx;
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
	{
		pTranx = m_TranxList.GetNext( pos );
		if( pTranx->Primary.GetStreamCode() == Sm )
		{
			iRv = _FnNotFound;
			if( pTranx->Primary.GetFunctionCode() == Fn ) 
				return _FoundOk;
		}
		if( pTranx->Secondary.GetStreamCode() == Sm )
		{
			iRv = _FnNotFound;
			if( pTranx->Secondary.GetFunctionCode() == Fn ) 
				return _FoundOk;
		}
	}
	return iRv;
}

void XSecsLibrary::SetDeviceID(int iDeviceID)
{
	XSecsTranxThread* pTranx;
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
	{
		pTranx = m_TranxList.GetNext( pos );
		pTranx->Primary.SetDeviceID( iDeviceID );
		pTranx->Secondary.SetDeviceID( iDeviceID );
	}
}

//////////////////////////////////////////////////////////////////////
// XSecsTransaction Class
//////////////////////////////////////////////////////////////////////

// -- XSecsTransaction -- //

XSecsTransaction::XSecsTransaction()
{
	m_lIndex = 0L;
	m_szName = "";
	m_szDescription = "";
	m_lDeviceID = 0;
	m_bAutoSystemBytes = FALSE;
	m_bReplyExpected = FALSE;
	m_lSystemBytes = 0L;
}

XSecsTransaction::~XSecsTransaction()
{
}

XSecsTransaction::XSecsTransaction(LPCTSTR SmFn, LPCTSTR Description)
{
	m_lIndex = 0L;
	m_szName = "";
	m_szDescription = "";
	m_lDeviceID = 0;
	m_bAutoSystemBytes = FALSE;
	m_bReplyExpected = FALSE;
	m_lSystemBytes = 0L;

	SetHeader( SmFn, Description );
}

void XSecsTransaction::SetHeader( LPCTSTR SmFn, LPCTSTR Description)
{
	m_szName = SmFn;
	m_szDescription = Description;
}

CString& XSecsTransaction::GetName()
{
	return m_szName;
}

CString& XSecsTransaction::GetDescription()
{
	return m_szDescription;
}

void XSecsTransaction::operator =(XSecsTransaction &tranx)
{
	m_lIndex = tranx.m_lIndex;

	m_szName = tranx.m_szName;
	m_szDescription = tranx.m_szDescription;

	m_lDeviceID = tranx.m_lDeviceID;
	m_bAutoSystemBytes = tranx.m_bAutoSystemBytes;
	m_bReplyExpected = tranx.m_bReplyExpected;
	m_lSystemBytes = tranx.m_lSystemBytes;

	Primary = tranx.Primary;	// ̰  Ѱ..?
	Secondary = tranx.Secondary;
}

/////////////////////////////////////////////////////////////////////////////
// XSecsTranxThread

XSecsTranxThread::XSecsTranxThread() 
 : m_evSecondaryIn( FALSE ), 
   m_evSecondaryRcvd( FALSE, TRUE )	// Initially Own = FALSE, Manual Reset = TRUE
{
	m_pSECS = NULL;
	m_bSecondaryRcvd	= FALSE;
	T3_FirstTimeout = FALSE;
	ReConnetCount = 0;
}

XSecsTranxThread::~XSecsTranxThread()
{
}


/////////////////////////////////////////////////////////////////////////////
// XSecsTranxThread message handlers

int XSecsTranxThread::MsgLinkThread( int iType )
{
	CString szBuf;
	char cWait;
	int /*i, m,*/ len = 0;
	unsigned char buf[SECS_MaxMsgBlockBufSize];
	XSecsMessage msg;
	SYSTEMBYTES sysBytes;

	T3_Timeout = FALSE;

	if( m_pSECS == NULL ) 
	{
		#ifdef __LOG__
		szBuf.Format( "S%dF%d send Error", msg.GetStreamCode(), msg.GetFunctionCode());
		m_pSECS->m_pHexLog->TextOut( szBuf );
		#endif //__LOG__
		
		return -1;	// XSecsControl not initilized !!
	}

	switch( iType )
	{
	case XSecsHSMS::smPrimaryOut :

////////////LYS20051221////////////////////////////////////////////////////////
	if( m_ArrayPrimarySystemByte.GetSize() > 0 )
	{
		sysBytes = m_ArrayPrimarySystemByte.GetAt( 0 );
		m_ArrayPrimarySystemByte.RemoveAt( 0 );
		Primary.SetSystemBytes( sysBytes );
	}
//////////////////////////////////////////////////////////////////////////////

		Primary.Build();
		msg = Primary;
		msg.m_cHead = &(Primary.m_cHead[0]);

		if( msg.GetWaitFlag() == XSecsMessage::waitFlagOn ) cWait = 'W';
		else cWait = ' ';
		SYSTEMBYTES		sysBytes;
		msg.GetSystemBytes( &sysBytes );
		szBuf.Format( "1st Message SEND(H<-E) S%dF%d (SystemByte = %0X) %c    \"%s\"", msg.GetStreamCode(), msg.GetFunctionCode(), sysBytes.bytes[3], cWait, GetDescription() );
		m_pSECS->m_pSFLog->TextOut( szBuf );
		Primary.BuildSFLog( m_pSECS->m_pSFLog );

		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( szBuf );
		#endif //__LOG__
		break;

	case XSecsHSMS::smSecondaryOut :

////////////LYS20051221////////////////////////////////////////////////////////
	if( m_ArraySecondarySystemByte.GetSize() > 0 )
	{
		sysBytes = m_ArraySecondarySystemByte.GetAt( 0 );
		m_ArraySecondarySystemByte.RemoveAt( 0 );
		Secondary.SetSystemBytes( sysBytes );
	}
//////////////////////////////////////////////////////////////////////////////
		Secondary.Build();
		msg = Secondary;
		msg.m_cHead = &(Secondary.m_cHead[0]);

		msg.GetSystemBytes( &sysBytes );
		szBuf.Format( "2nd Message SEND (H<-E) S%dF%d (SystemByte = %0X)", msg.GetStreamCode(), msg.GetFunctionCode(), sysBytes.bytes[3] );
		m_pSECS->m_pSFLog->TextOut( szBuf );

		if( msg.GetFunctionCode() == 0 )
		{
			break;
		}
		else
		{
			Secondary.BuildSFLog( m_pSECS->m_pSFLog );
		}

		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( szBuf );
		#endif //__LOG__
		break;
	}

	CSingleLock sLock( &m_pSECS->m_mtxSecs );
	sLock.Lock();

	#ifdef __LOG__
	m_pSECS->m_pHexLog->TextOut( "Lock to Transaction Thread", XLog::stampNoDateTime );
	#endif //__LOG__

	// Send message to main view
	m_pSECS->Msg( WM_SECS_MSG, iType, (LPARAM)this );

	// Send Mesage Block
	len = msg[0].GetBuf( buf );
	// 2002.8.30 Thread Writing ̷ Ƿ View óϵ Ѵ.
	if( m_pSECS->Write( buf, len ))
	{
		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( "Message Block ۽", XLog::stampNoDateTime );
		m_pSECS->m_pHexLog->HexOut( buf, len );
		#endif //__LOG__
	}
	m_pSECS->m_bTransComp = TRUE;//2008.10.20 kimgun

	// List Size  .
	Primary.SetListSize();
	Secondary.SetListSize();

	sLock.Unlock();
	#ifdef __LOG__
	m_pSECS->m_pHexLog->TextOut( "Unlock from Transaction Thread", XLog::stampNoDateTime );
	#endif //__LOG__

	// if PrimaryOut, Wait SecondaryIn
	if( iType == XSecsHSMS::smPrimaryOut && Primary.GetWaitFlag() == XSecsMessage::waitFlagOn )
	{
		CSingleLock secondaryIn( &m_evSecondaryIn );
		T3_Timeout = FALSE;

		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( "SecondaryIn Wait"/*, XLog::stampNoDateTime*/ );
		#endif //__LOG__

		// 20200908 hskwak S6F66 T3 Timeout Error Skip.
		if( T3_FirstTimeout)
		{
			//Wait SecondaryIn Event On with m_pSECS->m_lT3 timeout
			if( secondaryIn.Lock( 3 * 1000 ) )
			{
				m_bSecondaryRcvd = TRUE;
				T3_FirstTimeout = FALSE;	// first Reset
				ReConnetCount = 0;
			}
			else
			{
				#ifdef __LOG__
				szBuf.Format( "S%dF%d SecondaryIn T3 Timeout", msg.GetrStreamCode(), msg.GetFunctionCode() );
				m_pSECS->m_pHexLog->TextOut( szBuf, XLog::stampNoDateTime );
				#endif //__LOG__
				T3_Timeout = TRUE;
				m_bSecondaryRcvd = TRUE;
				XSecsTranxThread* pTranx = m_pSECS->Transaction("S9F9");
				if( pTranx )
				{
					msg.GetSystemBytes( &sysBytes ); 
					pTranx->Primary.SetSystemBytes( sysBytes );
					pTranx->Primary.SetItemData( "SHEAD", msg.m_cHead );
					//pTranx->PrimaryOut( FALSE );
					pTranx->PrimaryOut(); // 101013 halesu test
					m_pSECS->Msg( WM_SECS_MSG, XSecsHSMS::smT3Error, (LPARAM)this );
					T3_FirstTimeout = TRUE; // 230607 ssm 

					ReConnetCount++;

					if(ReConnetCount > 2) // for S6F66 Msg loss and try reconnect  
					{
						m_pSECS->m_bRetryConnect = TRUE;
						ReConnetCount = 0;
					}

					return 0;
				}
				return -1;
			}
		}
		else // check time change 3sec after first T3 timeout 
		{
			//Wait SecondaryIn Event On with m_pSECS->m_lT3 timeout
			if( secondaryIn.Lock( m_pSECS->HSMST3 * 1000 ) )  // 230607 3sec during Check
			{
				m_bSecondaryRcvd = TRUE;
				T3_FirstTimeout = FALSE;	// first Reset
				ReConnetCount = 0;
			}
			else
			{
				#ifdef __LOG__
				szBuf.Format( "S%dF%d SecondaryIn T3 Timeout", msg.GetStreamCode(), msg.GetFunctionCode() );
				m_pSECS->m_pHexLog->TextOut( szBuf, XLog::stampNoDateTime );
			#endif //__LOG__
			T3_Timeout = TRUE;
			m_bSecondaryRcvd = TRUE;
			XSecsTranxThread* pTranx = m_pSECS->Transaction("S9F9");
				if( pTranx )
				{
					msg.GetSystemBytes( &sysBytes ); 
					pTranx->Primary.SetSystemBytes( sysBytes );
					pTranx->Primary.SetItemData( "SHEAD", msg.m_cHead );
					//pTranx->PrimaryOut( FALSE );
					pTranx->PrimaryOut(); // 101013 halesu test
					m_pSECS->Msg( WM_SECS_MSG, XSecsHSMS::smT3Error, (LPARAM)this );
					T3_FirstTimeout = TRUE; // 230607 ssm 

					ReConnetCount++;	// 230803 ssm

					return 0;
				}
				return -1;
			}
		}
		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( "SecondaryIn Complete"/*, XLog::stampNoDateTime*/ );
		#endif //__LOG__
	}

	#ifdef __LOG__
	m_pSECS->m_pHexLog->TextOut( "...Transaction End", XLog::stampNoDateTime );
	#endif //__LOG__

	return 0;
}

BOOL XSecsTranxThread::IsPrimaryOutOK()
{
	if( m_pSECS == NULL )
	{
		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( "Secs Control  " );
		#endif //__LOG__
		return FALSE;
	}
	return TRUE;
}

int XSecsTranxThread::PrimaryOut( BOOL bSetSysByte )
{
	if( m_pSECS == NULL )
	{
		#ifdef __LOG__
		m_pSECS->m_pHexLog->TextOut( "Secs Control  " );
		#endif //__LOG__
		return -1;
	}

	CSingleLock listRemove( &m_pSECS->m_evListRemove );
	listRemove.Lock();

	m_pSECS->m_evListAdd.ResetEvent();

	if( bSetSysByte )
	{
		// Assign New System Bytes
		SYSTEMBYTES sysBytes;
		m_pSECS->GetSystemBytes( &sysBytes );
//LYS20051221
///////////////////////////////////////////////////////////	
//		Primary.SetSystemBytes( sysBytes );
		m_ArrayPrimarySystemByte.Add( sysBytes );
///////////////////////////////////////////////////////////
	}

	m_evSecondaryRcvd.ResetEvent();
	m_iMessageType = XSecsHSMS::smPrimaryOut;
	#ifdef __LOG__
	m_pSECS->m_pHexLog->TextOut( "Transaction Start...", XLog::stampNoDateTime );
	#endif //__LOG__	

	m_pSECS->m_TranxList.AddTail( this );
	m_pSECS->m_evListAdd.SetEvent();

	m_bSecondaryRcvd = FALSE;
	
	return 0;
}

int XSecsTranxThread::SecondaryOut( BOOL bSetSysByte )
{
	#ifdef __LOG__
	m_pSECS->m_pHexLog->TextOut( "[SecondaryOut]" );
	#endif //__LOG__

//////////////LYS20051221 DELETE//////////////////////////////////
//	if( bSetSysByte )
//	{
		// Copy Primary's System Bytes to Secondary's
//		SYSTEMBYTES sysBytes;  
//		Primary.GetSystemBytes( &sysBytes );
//      Secondary.SetSystemBytes( sysBytes );
//	}
//////////////////////////////////////////////////////////////////

	// Secondary Out
	m_iMessageType = XSecsHSMS::smSecondaryOut;

//KDS041230
	CSingleLock listRemove( &m_pSECS->m_evListRemove );
	listRemove.Lock();

	m_pSECS->m_evListAdd.ResetEvent();
	m_pSECS->m_TranxList.AddTail( this );
	m_pSECS->m_evListAdd.SetEvent();
//KDS041230

	return 0;
}

int XSecsTranxThread::SecondaryIn( XSecsMessage& msg )
{
	//Secondary.Parse(  );
	//Secondary = msg;
	int err = 0;
	if( Secondary.SetItemData( msg ) < 0 ) err = -1;

	CString szBuf;
	SYSTEMBYTES		sysBytes;
	msg.GetSystemBytes( &sysBytes );
	szBuf.Format( "2nd Message RECV (H->E) S%dF%d (SystemByte = %0X)", msg.GetStreamCode(), msg.GetFunctionCode(), sysBytes.bytes[3] );
	m_pSECS->m_pSFLog->TextOut( szBuf );
	Secondary.BuildSFLog( m_pSECS->m_pSFLog );

	//m_evSecondaryIn.SetEvent();
	//m_evSecondaryRcvd.SetEvent();

	return err;
}

int XSecsTranxThread::PrimaryIn( XSecsMessage& msg )
{
	//Primary = msg;
	//׷, ״ ϸ ȵȴ.
	//Primary fully descript  ̰,
	//msg raw data ִ ̱ ...!!( ~path ʹ  򰥸.)
	//Primary.Parse( buf );
	if( Primary.SetItemData( msg ) < 0 ) return -1;
	
	CString szBuf;
	char cWait;
	if( msg.GetWaitFlag() == XSecsMessage::waitFlagOn ) cWait = 'W';
	else cWait = ' ';
	SYSTEMBYTES		sysBytes;
	msg.GetSystemBytes( &sysBytes );
//	szBuf.Format( "1 ޼  (H->E) S%dF%d %c \"%s\"", msg.GetStreamCode(), msg.GetFunctionCode(), cWait, GetDescription() );
	szBuf.Format( "1st Message RECV (H->E) S%dF%d (SystemByte = %0X) %c \"%s\"", msg.GetStreamCode(), msg.GetFunctionCode(), sysBytes.bytes[3], cWait, GetDescription() );
	m_pSECS->m_pSFLog->TextOut( szBuf );
	Primary.BuildSFLog( m_pSECS->m_pSFLog );

	return 0;
}

XSecsTranxThread::XSecsTranxThread(LPCTSTR SmFn, LPCTSTR Description)//HSMS, XSecsHSMS *pControl)
	: XSecsTransaction( SmFn, Description), m_evSecondaryIn( FALSE )
{
	m_pSECS = NULL;
	m_bSecondaryRcvd	= FALSE;
//???	m_pSECS = pControl;
}

void XSecsTranxThread::SetControl(XSecsHSMS *pHSMS)
{
	m_pSECS = pHSMS;
}


void XSecsTranxThread::ResetSystemByte(int iType)
{
////////////HJYOU20061110///////////////////
	int nIndex = 0;
	switch( iType )
	{
	case XSecsHSMS::smPrimaryOut :
		if( ( nIndex = m_ArrayPrimarySystemByte.GetSize() ) > 0 )
		{
			m_ArrayPrimarySystemByte.RemoveAt( nIndex - 1 );
		}
		break;

	case XSecsHSMS::smSecondaryOut :
		if( ( nIndex = m_ArraySecondarySystemByte.GetSize() ) > 0 )
		{
			m_ArraySecondarySystemByte.RemoveAt( nIndex - 1  );
		}
		break;
	}
//////////////////////////////////////////////////////////////////////////////
}

void XSecsLibrary::SetControl(XSecsHSMS *pControl)
{
	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
		m_TranxList.GetNext( pos )->SetControl( pControl );
}

/*inline*/ int XSecsMessage::GetWaitFlag()
{
	return m_iWaitFlag;
}

void XSecsMessage::SetSystemBytes( SYSTEMBYTES sysBytes )
{
	for( int i=0; i<4; i++ )
		m_byteSystemBytes[i] = sysBytes.bytes[i];
}

void XSecsMessage::GetSystemBytes( SYSTEMBYTES *sysBytes )
{
	for( int i=0; i<4; i++ )
		sysBytes->bytes[i] = m_byteSystemBytes[i];
}

XSecsTranxThread* XSecsLibrary::Find(SYSTEMBYTES tgtSysBytes)
{
	int i = 0;
	SYSTEMBYTES sysBytesBuf;
	XSecsTranxThread* pTranx;

	for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
	{
		pTranx = m_TranxList.GetNext( pos );
		pTranx->Primary.GetSystemBytes( &sysBytesBuf );
		for( i=0; i<4; i++ ) 
			if( sysBytesBuf.bytes[i] != tgtSysBytes.bytes[i] ) break;
		if( i >= 4 ) 
			return pTranx;
	}
	return NULL;	// Not Exist!
}

void XSecsMessageBlock::ClearAll()
{
//	m_iDeviceID = 1;
	m_iDeviceID = 0;
	m_iStreamCode = 0;
	m_iFunctionCode = 0;
	m_iDirectionFlag = 0;
	m_iWaitFlag = 0;
	m_iBlockNo = 0;
	m_iEndBlockFlag = 0;
	m_byteSystemBytes[0] = 0;
	m_byteSystemBytes[1] = 0;
	m_byteSystemBytes[2] = 0;
	m_byteSystemBytes[3] = 0;
	m_iDataBlockSize = 0;
	m_lpBuf = NULL;
}

BOOL XSecsMessageBlock::IsLastBlock()
{
	return (m_iEndBlockFlag)?TRUE:FALSE;
}

//////////////////////////////////////////////////////////////////////
// XSecsMessage Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XSecsMessage::XSecsMessage()
{
	ClearAll();
}

XSecsMessage::~XSecsMessage()
{
	if( m_lpBuf ) delete m_lpBuf;
	ClearAll();
}

int XSecsMessage::Build()
{
	CSingleLock sLock( &m_mutex );

	sLock.Lock();

	// Clear all buffer
	if( m_lpBuf ) delete m_lpBuf;

	// Allocate & build pre-divided long data block buffer
	m_iLDataBlockSize = m_Item.GetSize();
	m_lpBuf = new BYTE[(m_iLDataBlockSize>0)?m_iLDataBlockSize:1];	//2007.01.30 jemoon
	m_Item.BuildIHData( m_lpBuf );

	// Allocate for Divided Message Blocks
	m_iNumOfBlocks = (m_iLDataBlockSize<=1)?1:((m_iLDataBlockSize-1)/SECS_MaxDataBlockSize + 1);

	// Build Each Message Block
	if( !m_MessageBlockList.IsEmpty() )
	{
		for( POSITION pos=m_MessageBlockList.GetHeadPosition(); pos != NULL; )
			delete m_MessageBlockList.GetNext( pos );
		m_MessageBlockList.RemoveAll();
	}
	BYTE* buf = m_lpBuf;
	for( int i=0; i<m_iNumOfBlocks; i++ )
	{
		int iDataBlockSize = min( m_iLDataBlockSize-SECS_MaxDataBlockSize*i, SECS_MaxDataBlockSize );
		XSecsMessageBlock* pMsgBlock = new XSecsMessageBlock;
		pMsgBlock->SetHeader( m_iDeviceID, m_iStreamCode, m_iFunctionCode, m_iDirectionFlag, m_iWaitFlag, 0, 0, m_byteSystemBytes );
		pMsgBlock->Build( buf, iDataBlockSize );
		buf += iDataBlockSize;
		m_MessageBlockList.AddTail( pMsgBlock );

		m_cHead = &(pMsgBlock->m_cHead[0]);
	}

	sLock.Unlock();

	return m_iNumOfBlocks;
}

void XSecsMessage::BuildSFLog(XLog* pLog)
{
//	SFLog.TextOut( "{", XLog::stampNoDateTime );
	m_Item.SaveAllItems( pLog );
//	SFLog.TextOut( "}", XLog::stampNoDateTime );
}

void XSecsMessage::SetListSize()
{
	m_Item.SetListSize();
}

void XSecsMessage::SetListSize(int iDepth, int iLength, int iTargetLen, int iTargetList)
{
	m_Item.SetListSize(iDepth, iLength, iTargetLen, 0, iTargetList);
}

int XSecsMessage::GetListSize(int iDepth)
{
	return m_Item.GetListSize(iDepth);
}

void XSecsMessage::SetDeviceID(int iDevID)
{
	m_iDeviceID = iDevID;
}

void XSecsMessage::ClearAll()
{
	m_szName = "";
	m_szDescription = "";

	m_iStreamCode = 0;
	m_iFunctionCode = 0;
	m_iWaitFlag = 0;
	m_iDirectionFlag = 0;
	m_iDeviceID = 0;
	for( int i=0; i<4; i++ ) m_byteSystemBytes[i] = 0;

	m_iNItems = 0;
	m_lpBuf = NULL;
	m_iNumOfBlocks = 0;

	m_iLDataBlockSize = 0;

	if( !m_MessageBlockList.IsEmpty() )
	{
		for( POSITION pos=m_MessageBlockList.GetHeadPosition(); pos != NULL; )
			delete m_MessageBlockList.GetNext( pos );
		m_MessageBlockList.RemoveAll();
	}
}

void XSecsMessageBlock::SetHeader( int iDeviceID, int iStreamCode, int iFunctionCode, int iDirectionFlag, int iWaitFlag, int iBlockNo, int iEndBlockFlag, BYTE byteSystemBytes[] )
{
	m_iDeviceID = iDeviceID;
	m_iStreamCode = iStreamCode;
	m_iFunctionCode = iFunctionCode;
	m_iDirectionFlag = iDirectionFlag;
	m_iWaitFlag = iWaitFlag;
	m_iBlockNo = iBlockNo;
	m_iEndBlockFlag = iEndBlockFlag;
	for( int i=0; i<4; i++ ) m_byteSystemBytes[i] = byteSystemBytes[i];
}

int XSecsMessage::Parse(BYTE *lpBuf)
{
	CSingleLock sLock( &m_mutex );

	sLock.Lock();

	// -- ver 0.9 : only for reserved item data, fixed format
	if( m_lpBuf == NULL ) Build();		// -- ver 0.9
	if( lpBuf[0] != SECS_HeaderSize + m_iLDataBlockSize + SECS_ChecksumSize )
	{
		sLock.Unlock();
		return -1;	// Error() : Size ġ
	}

	// LENGTH
	int m_iDataBlockSize = lpBuf[0] - SECS_HeaderSize - SECS_ChecksumSize;

	// Check CHECKSUM
	unsigned int uChecksum = 0;
	for( int i=SECS_LengthBlockSize; i<SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize; i++ ) uChecksum += lpBuf[i];
	if( lpBuf[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize] != (BYTE)((uChecksum>>8)&0x00ff) 
		&& lpBuf[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+1] != (BYTE)(uChecksum&0x00ff) )
	{
		sLock.Unlock();
		return -1;	// Error() : Size ġ
	}
	/*
		RESERVED CODE FOR VER 1.0

	// allocation when receive!!! ( with lpBuf[0] )
	lpBuf = new BYTE[ m_Item.GetSize() + HEADERSIZE+CHECKSUMSIZE ];
	// LENGTH
	lpBuf[0]	= m_iLength =  m_Item.GetSize() + HEADERSIZE;
	*/
	////   MessageBlock  ?
/*HSMS*/	const int HSMSLengthOffset = 3;

	// HEADER - miss match 쿣  óϴ°?(!)
	m_iDeviceID		= ((int)(lpBuf[1+HSMSLengthOffset]&0x7f)<<8) | (int)lpBuf[2+HSMSLengthOffset];
	m_iStreamCode	= (int)lpBuf[3+HSMSLengthOffset]&0x7f;
	m_iFunctionCode	= (int)lpBuf[4+HSMSLengthOffset];
	m_iDirectionFlag= (lpBuf[1+HSMSLengthOffset]&0x80)?(XSecsMessage::toHost):(XSecsMessage::fromHost);
	m_iWaitFlag		= (lpBuf[3+HSMSLengthOffset]&0x80)?(XSecsMessage::waitFlagOn):(XSecsMessage::waitFlagOff);
//	m_iBlockNo		= ((int)(lpBuf[5]&0x7f)<<8) | (int)lpBuf[6];
//	m_iEndBlockFlag	= (lpBuf[5]&0x80)?1:0;
	m_byteSystemBytes[0] = lpBuf[7+HSMSLengthOffset];
	m_byteSystemBytes[1] = lpBuf[8+HSMSLengthOffset];
	m_byteSystemBytes[2] = lpBuf[9+HSMSLengthOffset];
	m_byteSystemBytes[3] = lpBuf[10+HSMSLengthOffset];

	// ITEM DATA -- ver 0.9 : Check lpBuf & Set to m_lpData
	if( m_Item.ParseIHData( m_lpBuf+SECS_DataBlockOffset, lpBuf+SECS_DataBlockOffset ) == -1 )
	{
		sLock.Unlock();
		return -1;	// Error() : Size ġ
	}

	return 0;
}

int XSecsMessageBlock::GetBlockNo()
{
	return m_iBlockNo;
}

// if last message block, return 0; else return 1;
int XSecsMessage::AddMsgBlock(XSecsMessageBlock& msgBlock)
{
	if( m_MessageBlockList.IsEmpty() )
	{
		m_iDeviceID			 = msgBlock.m_iDeviceID;
		m_iStreamCode		 = msgBlock.m_iStreamCode;
		m_iFunctionCode		 = msgBlock.m_iFunctionCode;
		m_iDirectionFlag	 = msgBlock.m_iDirectionFlag;
		m_iWaitFlag			 = msgBlock.m_iWaitFlag;
		m_byteSystemBytes[0] = msgBlock.m_byteSystemBytes[0];
		m_byteSystemBytes[1] = msgBlock.m_byteSystemBytes[1];
		m_byteSystemBytes[2] = msgBlock.m_byteSystemBytes[2];
		m_byteSystemBytes[3] = msgBlock.m_byteSystemBytes[3];
	}
	else
	{
		if( m_iDeviceID			 != msgBlock.m_iDeviceID ) return -1;
		if( m_iStreamCode		 != msgBlock.m_iStreamCode ) return -1;
		if( m_iFunctionCode		 != msgBlock.m_iFunctionCode ) return -1;
		if( m_iDirectionFlag	 != msgBlock.m_iDirectionFlag ) return -1;
		if( m_iWaitFlag			 != msgBlock.m_iWaitFlag ) return -1;
		if( m_byteSystemBytes[0] != msgBlock.m_byteSystemBytes[0] ) return -1;
		if( m_byteSystemBytes[1] != msgBlock.m_byteSystemBytes[1] ) return -1;
		if( m_byteSystemBytes[2] != msgBlock.m_byteSystemBytes[2] ) return -1;
		if( m_byteSystemBytes[3] != msgBlock.m_byteSystemBytes[3] ) return -1;
	}
	m_MessageBlockList.AddTail( new XSecsMessageBlock( msgBlock ) );

	return (msgBlock.IsLastBlock())?0:1;
}

int XSecsMessage::MergeMsgBlocks()
{
	POSITION pos;
	int i = 0, iLen = 0, iDataBlockSize = 0;
	unsigned char buf[SECS_MaxMsgBlockBufSize];

	if( m_lpBuf ) delete m_lpBuf;
	m_lpBuf = new BYTE[m_MessageBlockList.GetCount()*SECS_MaxHDBlockSize];

	// Test & Sort
	// ver0.9 -- only for normal order

	// Merge
	BYTE* pBuf = m_lpBuf;
	for( pos=m_MessageBlockList.GetHeadPosition(), i=0; pos != NULL; i++ )
	{
		iLen = m_MessageBlockList.GetNext( pos )->GetBuf( buf );
		iDataBlockSize = iLen - SECS_LengthBlockSize - SECS_HeaderSize /*- SECS_ChecksumSize*/;
		memcpy( pBuf, buf + SECS_DataBlockOffset, iDataBlockSize );
		pBuf += iDataBlockSize;
	}
	return 0;
}

XSecsMessage::XSecsMessage(XSecsMessage &msg)
{
	m_szName = msg.m_szName;
	m_szDescription = msg.m_szDescription;

	m_iStreamCode = msg.m_iStreamCode;
	m_iFunctionCode = msg.m_iFunctionCode;
	m_iWaitFlag = msg.m_iWaitFlag;
	m_iDirectionFlag = msg.m_iDirectionFlag;
	m_iDeviceID = msg.m_iDeviceID;
	for( int i=0; i<4; i++ ) m_byteSystemBytes[i] = msg.m_byteSystemBytes[i];

	m_iNItems = msg.m_iNItems;
	m_iNumOfBlocks = msg.m_iNumOfBlocks;

	m_iLDataBlockSize = msg.m_iLDataBlockSize;

	m_lpBuf = new BYTE[(m_iLDataBlockSize>0)?m_iLDataBlockSize:1];
	memcpy( m_lpBuf, msg.m_lpBuf, m_iLDataBlockSize );

	for( POSITION pos=msg.m_MessageBlockList.GetHeadPosition(); pos != NULL; )
	{
		XSecsMessageBlock *pMsgBlock = new XSecsMessageBlock( *msg.m_MessageBlockList.GetNext( pos ) );
		m_MessageBlockList.AddTail( pMsgBlock );
	}
}

int XSecsMessage::GetNumOfBlocks()
{
	return m_iNumOfBlocks;
}

XSecsMessageBlock& XSecsMessage::operator [](int idx)
{
	POSITION pos = m_MessageBlockList.FindIndex( idx );
	return *m_MessageBlockList.GetAt( pos );
}

XSecsMessage& XSecsMessage::operator =(XSecsMessage &msg)
{
	m_szName = msg.m_szName;
	m_szDescription = msg.m_szDescription;

	m_iStreamCode = msg.m_iStreamCode;
	m_iFunctionCode = msg.m_iFunctionCode;
	m_iWaitFlag = msg.m_iWaitFlag;
	m_iDirectionFlag = msg.m_iDirectionFlag;
	m_iDeviceID = msg.m_iDeviceID;
	for( int i=0; i<4; i++ ) m_byteSystemBytes[i] = msg.m_byteSystemBytes[i];

	m_iNItems = msg.m_iNItems;
	m_iNumOfBlocks = msg.m_iNumOfBlocks;

	m_iLDataBlockSize = msg.m_iLDataBlockSize;

	if( m_lpBuf ) delete m_lpBuf;
	m_lpBuf = new BYTE[(m_iLDataBlockSize>0)?m_iLDataBlockSize:1];
	memcpy( m_lpBuf, msg.m_lpBuf, m_iLDataBlockSize );

	POSITION pos;
	if( !m_MessageBlockList.IsEmpty() )
	{
		for( pos=m_MessageBlockList.GetHeadPosition(); pos != NULL; )
			delete m_MessageBlockList.GetNext( pos );
		m_MessageBlockList.RemoveAll();
	}
	for( pos=msg.m_MessageBlockList.GetHeadPosition(); pos != NULL; )
	{
		XSecsMessageBlock *pMsgBlock = new XSecsMessageBlock;
		m_MessageBlockList.AddTail( pMsgBlock );
		*pMsgBlock = *msg.m_MessageBlockList.GetNext( pos );
	}

	return *this;
}

XSecsMessageBlock& XSecsMessageBlock::operator =(XSecsMessageBlock &msg)
{
	m_iDeviceID = msg.m_iDeviceID;
	m_iStreamCode = msg.m_iStreamCode;
	m_iFunctionCode = msg.m_iFunctionCode;
	m_iDirectionFlag = msg.m_iDirectionFlag;
	m_iWaitFlag = msg.m_iWaitFlag;
	m_iBlockNo = msg.m_iBlockNo;
	m_iEndBlockFlag = msg.m_iEndBlockFlag;
	m_byteSystemBytes[0] = msg.m_byteSystemBytes[0];
	m_byteSystemBytes[1] = msg.m_byteSystemBytes[1];
	m_byteSystemBytes[2] = msg.m_byteSystemBytes[2];
	m_byteSystemBytes[3] = msg.m_byteSystemBytes[3];
	m_iDataBlockSize = msg.m_iDataBlockSize;
	
	if( m_lpBuf ) delete m_lpBuf;
	m_lpBuf = new BYTE[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize];
	memcpy( m_lpBuf, msg.m_lpBuf, SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize );

	return *this;
}

XSecsMessageBlock::XSecsMessageBlock(XSecsMessageBlock &msgBlock)
{
	m_iDeviceID = msgBlock.m_iDeviceID;
	m_iStreamCode = msgBlock.m_iStreamCode;
	m_iFunctionCode = msgBlock.m_iFunctionCode;
	m_iDirectionFlag = msgBlock.m_iDirectionFlag;
	m_iWaitFlag = msgBlock.m_iWaitFlag;
	m_iBlockNo = msgBlock.m_iBlockNo;
	m_iEndBlockFlag = msgBlock.m_iEndBlockFlag;
	m_byteSystemBytes[0] = msgBlock.m_byteSystemBytes[0];
	m_byteSystemBytes[1] = msgBlock.m_byteSystemBytes[1];
	m_byteSystemBytes[2] = msgBlock.m_byteSystemBytes[2];
	m_byteSystemBytes[3] = msgBlock.m_byteSystemBytes[3];
	m_iDataBlockSize = msgBlock.m_iDataBlockSize;
	
	m_lpBuf = new BYTE[SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize];
	memcpy( m_lpBuf, msgBlock.m_lpBuf, SECS_LengthBlockSize+SECS_HeaderSize+m_iDataBlockSize+SECS_ChecksumSize );
}

// int XSecsMessage::GetSFCode()
//		SF Code int յ ڷ mappingϿ switch ̿  ֵ ش.
//		 SmFn  [mmnn]ε  int  m*100+n   ֵ
//		׻ ڸ+ڸ ڷ ǥ  ֵ ؾѴ.
//		 S1F65 (int)0165
//		printf format string ǥڸ ( "%02d%02d", StreamCode, FunctionCode )̴.
//		׸ S1F65 (int)0165  "secs.h" defineϿ θ .
int XSecsMessage::GetSFCode()
{
	if( m_iFunctionCode > 100 )
	  return m_iStreamCode*1000 + m_iFunctionCode;  
	else 
	return m_iStreamCode*100 + m_iFunctionCode;
}

int XSecsMessage::FindItemData(XSecsMessage &msg)
{
	for( int i=0; i<4; i++ ) m_byteSystemBytes[i] = msg.m_byteSystemBytes[i];
//	if( m_iLDataBlockSize != msg.m_iLDataBlockSize ) return -1;	// ver 0.9
	if( m_Item.FindIHData( m_lpBuf, msg.m_lpBuf ) == -1 ) 
		return -1;

	return 0;
}

int XSecsMessage::SetItemData(XSecsMessage &msg)
{
	for( int i=0; i<4; i++ ) m_byteSystemBytes[i] = msg.m_byteSystemBytes[i];
//	if( m_iLDataBlockSize != msg.m_iLDataBlockSize ) return -1;	// ver 0.9

	if( m_Item.ParseIHData( m_lpBuf, msg.m_lpBuf ) == -1 ) return -1;

	return 0;
}

void XSecsMessage::SetItemData(LPCTSTR lpszName, int iValue, BOOL bDelete)
{
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
	if( strcmp( lpszName, "L" ) == 0 /*|| strcmp( lpszName, "N" ) == 0*/ ) return;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( lpszName, 0, bDelete );
	if( pItem != NULL )  pItem->SetData( iValue );
}

void XSecsMessage::SetItemData(LPCTSTR lpszName, UINT uValue, BOOL bDelete)
{
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
	if( strcmp( lpszName, "L" ) == 0 /*|| strcmp( lpszName, "N" ) == 0*/ ) return;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( lpszName, 0, bDelete );
	if( pItem != NULL ) pItem->SetData( uValue );
}

void XSecsItem::SetData(const int iValue)
{
	//ASSERT m_lpData, m_iLength;
	switch( m_byteFormat )
	{
	case wsFormatBinary :
	case wsFormatBoolean://2008.11.26 kimgun
	case wsFormatU1 :
		//if( m_iLength != iLength ) return;	// -- ver 0.9 : Format Error !!
		((char*)m_lpData)[0] = (char)iValue;		// -- ver 0.9 : 1 byte only
		break;
	case wsFormatI2 :
	case wsFormatU2 :
		((char*)m_lpData)[0] = (char)(iValue>>8);		// -- ver 0.9 : 2 byte only
		((char*)m_lpData)[1] = (char)iValue;		// -- ver 0.9 : 2 byte only
		break;
	case wsFormatU4 :
		((char*)m_lpData)[0] = (char)(iValue>>24);		// -- ver 0.9 : 2 byte only
		((char*)m_lpData)[1] = (char)(iValue>>16 );		// -- ver 0.9 : 2 byte only
		((char*)m_lpData)[2] = (char)(iValue>>8 );		// -- ver 0.9 : 2 byte only
		((char*)m_lpData)[3] = (char)iValue;		// -- ver 0.9 : 2 byte only
		break;
	case wsFormatAscii :
	default :
		// Not formatted !!
		break;
	}
}

void XSecsItem::SetData(const UINT uValue)
{
	//ASSERT m_lpData, m_iLength;
	switch( m_byteFormat )
	{
	case wsFormatI2 :
	case wsFormatBinary :
	case wsFormatBoolean://2008.11.26 kimgun	
	case wsFormatU1 :
	case wsFormatU2 :	
		//if( m_iLength != iLength ) return;	// -- ver 0.9 : Format Error !!
		((unsigned char*)m_lpData)[0] = (unsigned char)(uValue>>8);		// -- ver 0.9 : 2 byte only
		((unsigned char*)m_lpData)[1] = (unsigned char)uValue;		// -- ver 0.9 : 2 byte only
		break;
	case wsFormatU4 :
		//if( m_iLength != iLength ) return;	// -- ver 0.9 : Format Error !!
		((unsigned char*)m_lpData)[0] = (unsigned char)(uValue>>24);		// -- ver 0.9 : 2 byte only
		((unsigned char*)m_lpData)[1] = (unsigned char)(uValue>>16);		// -- ver 0.9 : 2 byte only
		((unsigned char*)m_lpData)[2] = (unsigned char)(uValue>>8);		// -- ver 0.9 : 2 byte only
		((unsigned char*)m_lpData)[3] = (unsigned char)uValue;		// -- ver 0.9 : 2 byte only
		break;
	case wsFormatAscii :
	default :
		// Not formatted !!
		break;
	}
}

void XSecsItem::SetData(const char chValue)
{
	//ASSERT m_lpData, m_iLength;
	switch( m_byteFormat )
	{
	case wsFormatBinary :
	case wsFormatBoolean:
		//if( m_iLength != iLength ) return;	// -- ver 0.9 : Format Error !!
		break;
	case wsFormatAscii :
		((char*)m_lpData)[0] = (char)chValue;		// -- ver 0.9 : 1 byte only
		break;
	default :
		// Not formatted !!
		break;
	}
}

void XSecsMessage::SetItemData(LPCTSTR lpszName, const char chValue, BOOL bDelete)
{
	// -- ver 0.9 : will be updated to access m_lpData directly in ver 1.0 !
	if( strcmp( lpszName, "L" ) == 0 /*|| strcmp( lpszName, "N" ) == 0*/ ) return;	// usage error !!!
	XSecsItem* pItem = m_Item.FindItem( lpszName, 0, bDelete );
	if( pItem != NULL ) pItem->SetData( chValue );
}

BOOL XSecsTranxThread::IsSecondaryRcvd(BOOL bLock)
{
	CSingleLock secondaryIn( &m_evSecondaryRcvd );

	BOOL bRcvd = FALSE;

	if( bLock )
	{
		bRcvd = secondaryIn.Lock( m_pSECS->HSMST3 * 1000 );
		// WaitForSingleObject( m_hThread, INFINITE ); 
	}
	else bRcvd = secondaryIn.IsLocked();
	
	return bRcvd; // Fail ̸ 0
}

BOOL XSecsTranxThread::IsSecondaryRcvd()
{
	if( m_bSecondaryRcvd )
	{
		m_bSecondaryRcvd = FALSE;
		return TRUE;
	}

	return FALSE;
}


UINT XSecsMessage::GetDeviceID()
{
	return (UINT)m_iDeviceID;
}

/////////////////////////////////////////////////////////////////////////////
// XSecsHSMS
IMPLEMENT_DYNCREATE(XSecsHSMS, CWinThread )

XSecsHSMS::XSecsHSMS()
 : m_evListRemove( TRUE, TRUE ),	
	m_evListAdd( TRUE, TRUE )	// Initially Own = FALSE, Manual Reset = TRUE
{
	m_bAutoDelete = FALSE;
	m_evDead = CreateEvent( NULL, TRUE, FALSE, NULL );	// MANUAL Reset, Initial state = non-signal
	m_evKill = CreateEvent( NULL, TRUE, FALSE, NULL );

	m_bLinkTestRespRecvd = FALSE;
	m_pListenSocket = NULL;
	m_pClientSocket = NULL;
	m_pOwner = NULL;

	m_bActiveMode = TRUE;

	m_lT1 = 300;
	m_lT2 = 500;
	m_lT3 = 45000;
	m_lT4 = 10000;
	m_lRTY = 3;

	HSMST3 = 45;
	HSMST5 = 10;
	HSMST6 = 50;
	HSMST7 = 10;
	HSMST8 = 5;
	HSMST9 = 5;

	IPAddressLocal = "0.0.0.0";
	IPAddressRemote = "0.0.0.0";
	IPPortLocal = 0;
	IPPortRemote = 0;

	m_bConnected = FALSE;
	m_bSelected = FALSE;
	m_bTransComp = TRUE;
	m_lDefaultDeviceID = 0;
	m_ulSystemBytes = 0;
	m_uDeviceID = 0;
	m_bRetryConnect = FALSE; // 230803 ssm
}

XSecsHSMS::~XSecsHSMS()
{
	if(m_pListenSocket)
		delete m_pListenSocket;
	if(m_pClientSocket)
		delete m_pClientSocket;
}

BOOL XSecsHSMS::InitInstance()
{
	while( WaitForSingleObject( m_evKill, 0 ) != WAIT_OBJECT_0 ) 
	{
		// hskwak Mutex add
		CSingleLock sLock( &g_SecsMutex );
		sLock.Lock();

		Sleep( 10 );

		for( POSITION pos=m_TranxList.GetHeadPosition(); pos!=NULL; )
		{
			XSecsTranxThread* pTranx = m_TranxList.GetNext( pos );
			pTranx->MsgLinkThread( pTranx->m_iMessageType );			

			CSingleLock listAdd( &m_evListAdd );
			listAdd.Lock();

			m_evListRemove.ResetEvent();
			m_TranxList.RemoveHead();
			m_evListRemove.SetEvent();
		}
		// hskwak Mutex add
		sLock.Unlock();
	}

	VERIFY( SetEvent( m_evDead ) );
	return FALSE;
}

int XSecsHSMS::ExitInstance()
{
	return CWinThread::ExitInstance();
}

BOOL XSecsHSMS::Start( HWND hWnd )
{
	if( hWnd == NULL ) return FALSE;

	// get the pointer of the parent window
	SetOwnerWnd( CWnd::FromHandle( hWnd ) );

	// create the thread
	if( !CWinThread::CreateThread() ) return FALSE;

	return TRUE;
}

void XSecsHSMS::End()
{
	// Exit the thread
	if( m_hThread )
	{
		VERIFY( SetEvent( m_evKill ) );

		// When ExitInstance function calls, it is signaled.
		VERIFY( WaitForSingleObject( m_evDead, INFINITE ) == WAIT_OBJECT_0 );
		VERIFY( WaitForSingleObject( m_hThread, INFINITE ) == WAIT_OBJECT_0 );
		Sleep(200);
	}

	DWORD exitCode;
	if( !GetExitCodeThread( m_hThread, &exitCode ) )
	{
		if( GetLastError() == STILL_ACTIVE )
		{
			TerminateThread( m_hThread, -1 );
			Sleep( 200 );
			if( !GetExitCodeThread( m_hThread, &exitCode ) )
				AfxMessageBox( "Windows ϰ ý  OFFϿ ֽʽÿ.", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
		}
	}
}

BEGIN_MESSAGE_MAP(XSecsHSMS, CWinThread)
	//{{AFX_MSG_MAP(XSecsTranxThread)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void XSecsHSMS::SetOwnerWnd(CWnd* pOwner)
{
	m_pOwner = pOwner;
}

/////////////////////////////////////////////////////////////////////////////
// XSecsHSMS member functions
int XSecsHSMS::Connect( LPCTSTR ipAddr, long port, BOOL bShowMsgBox )
{
	IPAddressRemote = (char*)ipAddr;
	IPPortRemote = port;

	if( m_bActiveMode )
	{
		m_pClientSocket = new XClientSocket( this );
		if( !m_pClientSocket->Init( ipAddr, port, bShowMsgBox ) ) 
		{
			delete m_pClientSocket;
			m_pClientSocket = NULL;	
			
			return 0;
		}

		m_bConnected = TRUE;
		Msg( WM_SECS_MSG, smConnected, 0L );
		m_Library.SetControl( this );

		Select();
	}
	else
	{
		if( m_pListenSocket )
		{
			delete m_pListenSocket;
			m_pListenSocket = NULL;
		}
		m_pListenSocket = new XListenSocket( this );
		m_pListenSocket->Create( port );

		// Wait for client
		m_pListenSocket->Listen();
	}

	return 1;
}

BOOL XSecsHSMS::ConnectEnable( LPCTSTR ipAddr, long port )
{
	BOOL bRv = FALSE;
	IPAddressRemote = (char*)ipAddr;
	IPPortRemote = port;

	if( m_bActiveMode )
	{
		m_pClientSocket = new XClientSocket( this );
		if( m_pClientSocket->Init( ipAddr, port, FALSE ) )
		{
			bRv = TRUE;
		}
		else
		{
			bRv = FALSE;
		}

		delete m_pClientSocket;
		m_pClientSocket = NULL;	
	}

	return bRv;
}

void XSecsHSMS::OnClose()
{
	m_bConnected = FALSE;
	m_bSelected = FALSE;
	Msg( WM_SECS_MSG, smDisconnected, 0L );

	delete m_pClientSocket;
	m_pClientSocket = NULL;
}

void XSecsHSMS::OnAccept() 
{
	if( m_pClientSocket ) 
	{
		delete m_pClientSocket;
		m_pClientSocket = NULL;
	}

	m_pClientSocket = new XClientSocket( this );	// if connected, create XClient Socket
	if( m_pListenSocket->Accept( *m_pClientSocket ) )
	{
		m_pClientSocket->Init();

		m_bConnected = TRUE;
		m_Library.SetControl( this );

		Msg( WM_SECS_MSG, smConnected, 0L );
	}
	else
	{
		AfxMessageBox( "Failed to connection of client" );
		delete m_pClientSocket;
		m_pClientSocket = NULL;
	}
}

void XSecsHSMS::OnReceive() 
{
	if( ReadMessage() < 0 )
	{
	}
	else
	{
	}
}

void XSecsHSMS::AbortTransaction()
{

}

void XSecsHSMS::GetSystemBytes(SYSTEMBYTES *sysBytes)
{
	m_ulSystemBytes++;

	for( int i=0; i<4; i++ )
		sysBytes->bytes[i] = (char)(m_ulSystemBytes>>((3-i)<<3));
}

int XSecsHSMS::LoadLibrary(LPCTSTR lpszFileName)
{
	m_Library.SetLog( m_pLoadSDFLog );

	if( !lpszFileName )
		m_Library.SetDefaults();
	else if( m_Library.LoadSDF( lpszFileName ) < 0 )
		m_Library.SetDefaults();

	return 0;
}

void XSecsHSMS::SetDeviceID(UINT uID)
{
	m_uDeviceID = uID;
	m_Library.SetDeviceID( uID );		// Host  ID
}

XSecsTranxThread* XSecsHSMS::Transaction(char *lpszName, int dir)
{
	return m_Library.Find( lpszName, dir );
}

int XSecsHSMS::MessageProc(XSecsMessage &msg)
{
	// Test SecondaryIn
	SYSTEMBYTES systemBytes;
	msg.GetSystemBytes( &systemBytes );
//KDS	XSecsTranxThread* pTranx = m_Library.Find( msg.GetStreamCode(), msg.GetFunctionCode() );
	
//	if( pTranx == NULL )  //T3 ߻ؼ retry   	
//		pTranx = m_Library.Find( systemBytes );
	XSecsTranxThread* pTranx = NULL;
	if( m_TranxList.GetCount() != 0 ) pTranx = m_TranxList.GetHead();

	// --- SecondaryIn
	if( pTranx != NULL && !(msg.GetFunctionCode()&(int)1) )
	{

		if( msg.GetFunctionCode() == 0 )	// Abort Transaction
		{
			/*m_szMessage = "Abort Transaction ";
			Msg( WM_SECS_MSG, smError, (LPARAM)m_szMessage.GetBuffer(m_szMessage.GetLength()) );
			m_szMessage.ReleaseBuffer();*/
			//ActiveTranxList.RemoveAll();	// & abort all
			Msg( WM_SECS_MSG, smAbortTransaction, (LPARAM) pTranx );
			m_szMessage.ReleaseBuffer();
	
			//2006.01.27 Youngsik => T3 ߻Ű ʱ ...
			pTranx->m_evSecondaryIn.SetEvent();
			//2006.01.27 Youngsik => Secondary   ó...
			//pTranx->m_evSecondaryRcvd.SetEvent();

			pTranx->Secondary.SetListSize();
			return 0;
		}
		else if( pTranx->SecondaryIn( msg ) < 0 )
		{
//			TRACE( "Secondary Message  " );

			#ifdef __LOG__
			m_pHexLog->TextOut( "Secondary Message  " );
			#endif

			m_pSFLog->TextOut( "Secondary Message  " );

				SYSTEMBYTES sysBytes;
			CString szBuf;

			szBuf.Format( "(H->E) S%dF%d ", msg.GetStreamCode(), msg.GetFunctionCode() );
			m_pSFLog->TextOut( szBuf );
//			Msg( WM_SECS_MSG, smPrimaryIn, (LPARAM)pTranx );
			
			pTranx = Transaction( "S9F7" );
			if( pTranx != NULL )
			{
				msg.GetSystemBytes( &sysBytes );
				pTranx->Primary.SetSystemBytes( sysBytes );
				pTranx->Primary.SetItemData( "MHEAD", msg.m_cHead );
				pTranx->PrimaryOut( FALSE );
				return 0;
			}
			return -2;

		}	
		
		//secondary ..
		Msg( WM_SECS_MSG, smSecondaryIn, (LPARAM)pTranx );
		OnSecondaryRcvd(pTranx);

		pTranx->m_evSecondaryIn.SetEvent();
		pTranx->m_evSecondaryRcvd.SetEvent();

		pTranx->Secondary.SetListSize();
		return 0;
	}

	if( msg.GetFunctionCode() == 0 )	// Abort Transaction
	{
		/*m_szMessage = "Abort Transaction ";
		Msg( WM_SECS_MSG, smError, (LPARAM)m_szMessage.GetBuffer(m_szMessage.GetLength()) );
		m_szMessage.ReleaseBuffer();*/
		//ActiveTranxList.RemoveAll();	// & abort all
		Msg( WM_SECS_MSG, smAbortTransaction, (LPARAM) pTranx );
		m_szMessage.ReleaseBuffer();
	}
	else if( msg.GetFunctionCode() & (int)1 )
	{
		// --- PrimaryIn : Ȧ FunctionCode
		XSecsTranxThread* pTranx = m_Library.Find( msg, msg.GetStreamCode(), msg.GetFunctionCode(), XSecsMessage::fromHost );
		if( pTranx != NULL )
		{
			if( pTranx->PrimaryIn( msg ) < 0 )
			{
				SYSTEMBYTES sysBytes;
				CString szBuf;

				szBuf.Format( "(H->E) S%dF%d ", msg.GetStreamCode(), msg.GetFunctionCode() );
				m_pSFLog->TextOut( szBuf );
//				Msg( WM_SECS_MSG, smPrimaryIn, (LPARAM)pTranx );
				
				pTranx = Transaction( "S9F7" );
				if( pTranx != NULL )
				{
					msg.GetSystemBytes( &sysBytes );
					pTranx->Primary.SetSystemBytes( sysBytes );
					pTranx->Primary.SetItemData( "MHEAD", msg.m_cHead );
					pTranx->PrimaryOut( FALSE );
					return 0;
				}
				return -2;
			}
			m_szReceivedData.ReleaseBuffer();
			Msg( WM_SECS_MSG, smPrimaryIn, (LPARAM)pTranx );

//LYS20051221
///////////////////////////////////////////////////////////	
			SYSTEMBYTES sysBytes;  
			pTranx->Primary.GetSystemBytes( &sysBytes );
			pTranx->m_ArraySecondarySystemByte.Add( sysBytes );
///////////////////////////////////////////////////////////

			OnPrimaryRcvd(pTranx);
		}
		else
		{
//ipj20060428 
/////////////// T3 ߻  systembyte   /////
			if( msg.GetStreamCode() == 9 && msg.GetFunctionCode() == 9 )	
			{
				XSecsTranxThread* pTranx = m_Library.Find( 9, 9 );
				if( pTranx->PrimaryIn( msg ) == 0 )		//SHEAD Data setting
				{
					char buf[256];
					pTranx->Primary.GetItemData( "SHEAD", buf );

					//S9F9 SHEAD data T3 ߻ SnFn
					int Sn = buf[2];
					int Fn = buf[3];

					//T3 ߻ system byte
					SYSTEMBYTES sysBytes;  
					sysBytes.bytes[0] = buf[6];
					sysBytes.bytes[1] = buf[7];
					sysBytes.bytes[2] = buf[8];
					sysBytes.bytes[3] = buf[9];

					//T3 ߻ pTranx
					pTranx = m_Library.Find( Sn, Fn );

					if( pTranx != NULL )
					{
						int nSize = pTranx->m_ArraySecondarySystemByte.GetSize();
						for( int i=0; i<nSize; i++ )
						{
							SYSTEMBYTES sysBt = pTranx->m_ArraySecondarySystemByte.GetAt( i );
							if( sysBt.bytes[0] == sysBytes.bytes[0] &&
								sysBt.bytes[1] == sysBytes.bytes[1] &&
								sysBt.bytes[2] == sysBytes.bytes[2] &&
								sysBt.bytes[3] == sysBytes.bytes[3] ) 
							{
								pTranx->m_ArraySecondarySystemByte.RemoveAt( i ); 
							}
						}
					}
				}
			}
//////////////////////////////////////////////////////////////////////////

			//S?F?.PrimaryOut();
			m_szMessage = "ϵ  Transaction Primary Message ";
			Msg( WM_SECS_MSG, smError, (LPARAM)m_szMessage.GetBuffer(m_szMessage.GetLength()) );
			m_szMessage.ReleaseBuffer();
		}
	}
	else
	{
		//S?F?.PrimaryOut();
		m_szMessage = "SecondaryIn ";
		Msg( WM_SECS_MSG, smError, (LPARAM)m_szMessage.GetBuffer(m_szMessage.GetLength()) );
		m_szMessage.ReleaseBuffer();

		#ifdef __LOG__
		m_szMessage.Format( "S%dF%d : SecondaryIn ", msg.GetStreamCode(), msg.GetFunctionCode() );
		m_pHexLog->TextOut( m_szMessage );

		if( pTranx == NULL )
		{
			m_szMessage.Format( "S%dF%d : pTranx == NULL", msg.GetStreamCode(), msg.GetFunctionCode() );
			m_pHexLog->TextOut( m_szMessage );
		}

		#endif //__LOG__
	}
	return 0;
}

int XSecsHSMS::OnPrimaryRcvd(XSecsTranxThread *pTranx)
{
	return 0;
}

int XSecsHSMS::OnSecondaryRcvd(XSecsTranxThread *pTranx)
{
	return 0;
}

int XSecsHSMS::ReadMessage()
{
	int ret = 0;
	XSecsMessage msg;
	XSecsMessageBlock msgBlock;
	XSecsTranxThread* pTranx;
	SYSTEMBYTES sysBytes;

	for( int i=0; i<1/*TEMP*/; i++ )
	{
		ret = ReadMsgBlock( &msgBlock );
		msg.m_cHead = &(msgBlock.m_cHead[0]);
		if( ret < 0 )
		{
			#ifdef __LOG__
			m_pHexLog->TextOut( "Illigal Format(1)", XLog::stampNoDateTime );
			#endif	//__LOG__

			m_pSFLog->TextOut( "Illigal Format(1)", XLog::stampNoDateTime );
//			TRACE( "Illigal Format(1)" );
			pTranx = Transaction( "S9F7" );
			if( pTranx != NULL )
			{
				msg.GetSystemBytes( &sysBytes );
				pTranx->Primary.SetSystemBytes( sysBytes );
				pTranx->Primary.SetItemData( "MHEAD", msgBlock.m_cHead );
				pTranx->PrimaryOut( FALSE );
				return 0;
			}
			return -2;
		}
		if( (ret=msg.AddMsgBlock( msgBlock )) < 0 )
		{
			#ifdef __LOG__
			m_pHexLog->TextOut( "Illigal Format(2)", XLog::stampNoDateTime );
			#endif // __LOG__

			m_pSFLog->TextOut( "Illigal Format(2)", XLog::stampNoDateTime );
////			TRACE("Illigal Format(2)");
			pTranx = Transaction( "S9F7" );
			if( pTranx != NULL )
			{
				msg.GetSystemBytes( &sysBytes );
				pTranx->Primary.SetSystemBytes( sysBytes );
				pTranx->Primary.SetItemData( "MHEAD", msgBlock.m_cHead );
				pTranx->PrimaryOut( FALSE );
				return 0;
			}
			return -2;
		}
		// if last message block, return 0; else return 1;
//HSMS//		if( ret == 0 ) break;
/*HSMS*/break;
	}

	if( msg.MergeMsgBlocks() < 0 )
	{
		#ifdef __LOG__
		m_pHexLog->TextOut( "Illigal Format(3)", XLog::stampNoDateTime );
		#endif // __LOG__

		m_pSFLog->TextOut( "Illigal Format(3)", XLog::stampNoDateTime );
//		TRACE("Illigal Format(3)");
		pTranx = Transaction( "S9F7" );
		if( pTranx != NULL )
		{
			msg.GetSystemBytes( &sysBytes );
			pTranx->Primary.SetSystemBytes( sysBytes );
			pTranx->Primary.SetItemData( "MHEAD", msgBlock.m_cHead );
			pTranx->PrimaryOut( FALSE );
			return 0;
		}
		return -2;
	}

//	if( i==0 )
//	{
		msg.GetSystemBytes( &sysBytes );
		if( msgBlock.GetBlockNo() == 1 )
		{
			//TRACE( "Select Request Received\n" );
			SelectResp( sysBytes );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 2 )
		{
			//TRACE( "Select Response Received\n" );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 3 )
		{
			//TRACE( "[03] Received\n" );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 4 )
		{
			//TRACE( "[04] Received\n" );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 5 )
		{
			//TRACE( "Link Test Request Received\n" );
			LinkTestResp( sysBytes );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 6 )
		{
			//TRACE( "Link Test Response Received\n" );
			m_bLinkTestRespRecvd = TRUE;
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 7 )
		{
			//TRACE( "Reject Request Received\n" );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 8 )
		{
			//TRACE( "[8] Received\n" );
			return 0;
		}
		else if( msgBlock.GetBlockNo() == 9 )
		{
			//TRACE( "Separate Request Received\n" );
			return 0;
		}
//	}

	CString szBuf;
	szBuf.Format( "(H->E) S%dF%d Received", msg.GetStreamCode(), msg.GetFunctionCode() );
	#ifdef __LOG__
	m_pHexLog->TextOut( szBuf/*, XLog::stampNoDateTime*/ );
	#endif // __LOG__

	// Unrecognized Device Id Check
	if( msg.GetDeviceID() != m_uDeviceID )
	{
		m_pSFLog->TextOut( szBuf );

		pTranx = Transaction( "S9F1" );
		if( pTranx != NULL )
		{
			msg.GetSystemBytes( &sysBytes );
			pTranx->Primary.SetSystemBytes( sysBytes );
			pTranx->Primary.SetItemData( "MHEAD", msgBlock.m_cHead );
			//pTranx->PrimaryOut( FALSE );
			pTranx->PrimaryOut(); // 101013 halesu test
			return 0;
		}
	}

	// Unrecognized Stream Type Check
//	int nStreamCode = msg.GetStreamCode();
//	if( (nStreamCode > 10) ||
//		(nStreamCode != 1) && (nStreamCode = 2) &&
//		(nStreamCode != 5) && (nStreamCode = 6) &&
//		(nStreamCode != 7) && (nStreamCode = 9) &&
//		(nStreamCode != 10) )
	int iResult = m_Library.SmFnFind( msg.GetStreamCode(), msg.GetFunctionCode() );
	if( iResult == _SmNotFound )
	{
		m_pSFLog->TextOut( szBuf );

		pTranx = Transaction( "S9F3" );
		if( pTranx != NULL )
		{
			msg.GetSystemBytes( &sysBytes );
			pTranx->Primary.SetSystemBytes( sysBytes );
			pTranx->Primary.SetItemData( "MHEAD", msgBlock.m_cHead );
			pTranx->PrimaryOut( );
			return 0;
		}
	}

	// Unrecognized Funtion Type Check
//	if( msg.GetFunctionCode() > 99 )
	if( iResult == _FnNotFound )
	{
		m_pSFLog->TextOut( szBuf );

		pTranx = Transaction( "S9F5" );
		if( pTranx != NULL )
		{
			msg.GetSystemBytes( &sysBytes );
			pTranx->Primary.SetSystemBytes( sysBytes );
			pTranx->Primary.SetItemData( "MHEAD", msgBlock.m_cHead );
			pTranx->PrimaryOut( );
			return 0;
		}
	}

	return MessageProc( msg );
}

int XSecsHSMS::ReadMsgBlock(XSecsMessageBlock *pMsgBlock)
{
	int iLen;
	unsigned char buf[SECS_MaxHDBlockSize+SECS_ChecksumSize];
	CString szBuf;

	// Read <Length>
	if( (iLen=m_pClientSocket->Read( buf, 4 )) != 4 )
	{
		#ifdef __LOG__
		m_pHexLog->TextOut( "Error : <Length> Read Error", XLog::stampNoDateTime );
		#endif // __LOG__

		m_pSFLog->TextOut( "Error : <Length> Read Error", XLog::stampNoDateTime );
		return -1;
	}
	int iBlockSize = (((int)buf[0])<<24)+(((int)buf[1])<<16)+(((int)buf[2])<<8)+(((int)buf[3]));
	if( iBlockSize > SECS_MaxHDBlockSize || iBlockSize < SECS_HeaderSize ) return -1;// Error : <Length> is abnormal

	Sleep( 100 );

	// Read <Data Block> + <Checksum>
	iLen = m_pClientSocket->Read( buf, iBlockSize );

	while( iLen < iBlockSize )
	{
		Sleep( 100 );
		int iCurLen;
//		iCurLen = m_pClientSocket->Read( &buf[iLen], iBlockSize );
		iCurLen = m_pClientSocket->Read( &buf[iLen], iBlockSize - iLen );
		iLen += iCurLen;
	}

	if( iLen != iBlockSize )
	{
		iLen = m_pClientSocket->Read( buf, iBlockSize );
		szBuf.Format( "Error : <Data Block> + <Checksum> Read Error" );
		m_pSFLog->TextOut( szBuf );

		#ifdef __LOG__
		m_pHexLog->TextOut( szBuf, XLog::stampNoDateTime );
		#endif // __LOG__

		szBuf.Format( "legnth byte = 0x%X, number of read bytes = 0x%X", iBlockSize+SECS_ChecksumSize, iLen );
		m_pSFLog->TextOut( szBuf );
		
		#ifdef __LOG__
		m_pHexLog->TextOut( szBuf, XLog::stampNoDateTime );
		#endif // __LOG__
		return -2;
	}

	const int HSMSLengthOffset = 3;

	// Merge to message block
	BYTE* lpReceivedData = (BYTE*)m_szReceivedData.GetBuffer( SECS_LengthBlockSize + iBlockSize );
	lpReceivedData[0] = (char)(iLen>>24);
	lpReceivedData[1] = (char)(iLen>>16);
	lpReceivedData[2] = (char)(iLen>>8);
	lpReceivedData[3] = (char)(iLen);
	memcpy( lpReceivedData+1+HSMSLengthOffset, buf, iBlockSize );

	// Test Checksum
	// ??? ver0.9
	
	#ifdef __LOG__
	szBuf.Format( "%d bytes Received", SECS_LengthBlockSize+iBlockSize );
	m_pHexLog->TextOut( szBuf, XLog::stampNoDateTime );
	m_pHexLog->HexOut( (char*)lpReceivedData, SECS_LengthBlockSize+iBlockSize );
	#endif // __LOG__

	// Parsing the message
	if( !pMsgBlock->Parse( lpReceivedData ) )
	{
		#ifdef __LOG__
		m_pHexLog->TextOut( "Received Message Format Error[Parsing Error]", XLog::stampNoDateTime );
		#endif // __LOG__

		m_pSFLog->TextOut( "Received Message Format Error[Parsing Error]", XLog::stampNoDateTime );
		return -3;
	}

	m_szReceivedData.ReleaseBuffer();

	return 0;
}

BOOL XSecsHSMS::Msg(UINT msg, WPARAM wParam, LPARAM lParam)
{
	if( m_pOwner )
	{
		// Get the IP address and port number
		CString sAddr;
		UINT nPort, nSetPort;
		if( m_bActiveMode ) nSetPort = IPPortRemote;
		else nSetPort = IPPortLocal;

		if( m_pClientSocket != NULL )
		{
			m_pClientSocket->GetSockName( sAddr, nPort );
			if( nPort != nSetPort )
				m_pClientSocket->GetPeerName( sAddr, nPort );
		}
		// 1011101111****
		//  Port No  wParam
		m_pOwner->SendMessage( msg, ( nPort << 4 ) + wParam, lParam );
		return TRUE;
	}
	return FALSE; 
//	if( m_pOwner ) //LYS20060315 S10F5 CIM/INTERFACE VIEW Message Display
//	{
//		m_pOwner->SendMessage( msg, wParam, lParam );
}

//DWORD XSecsHSMS::Read(LPSTR lpBuf, DWORD dwLength)
DWORD XSecsHSMS::Read(unsigned char* lpBuf, DWORD dwLength)
{
	if( m_pClientSocket == NULL )
		return FALSE;

	UINT uLen = m_pClientSocket->Read( lpBuf, dwLength );
	return (DWORD)uLen;

}

BOOL XSecsHSMS::Write(unsigned char* lpBuf, DWORD dwLength)
{
	BOOL bRv;

	if( m_pClientSocket == NULL )
		return FALSE;

	bRv = m_pClientSocket->Write( lpBuf, dwLength );
	return bRv;
}

int XSecsHSMS::Select()
{
	unsigned char buf[14] = { '\0', '\0', '\0', '\x0a', '\xff', '\xff', '\0', '\0', '\0', '\1', '\x80', '\0', '\0', '\1' };
	if( m_pClientSocket ) m_pClientSocket->Write( buf, 14 );	// KDS050512
	return 0;
}

int XSecsHSMS::SelectResp( SYSTEMBYTES sysBytes )
{
	unsigned char buf[14] = { '\0', '\0', '\0', '\x0a', '\xff', '\xff', '\0', '\0', '\0', '\2', '\x80', '\0', '\0', '\2' };
	buf[10] = sysBytes.bytes[0];
	buf[11] = sysBytes.bytes[1];
	buf[12] = sysBytes.bytes[2];
	buf[13] = sysBytes.bytes[3];
	if( m_pClientSocket ) m_pClientSocket->Write( buf, 14 );	// KDS050512

	m_bSelected = TRUE;
	return 0;
}

int XSecsHSMS::LinkTestReq()
{
	unsigned char buf[14] = { '\0', '\0', '\0', '\x0a', '\xff', '\xff', '\0', '\0', '\0', '\5', '\x80', '\0', '\0', '\1' };
	if( m_pClientSocket ) m_pClientSocket->Write( buf, 14 );	// KDS050512
	return 0;
}

int XSecsHSMS::LinkTestResp( SYSTEMBYTES sysBytes )
{
	unsigned char buf[14] = { '\0', '\0', '\0', '\x0a', '\xff', '\xff', '\0', '\0', '\0', '\6', '\x80', '\0', '\0', '\1' };
	buf[10] = sysBytes.bytes[0];
	buf[11] = sysBytes.bytes[1];
	buf[12] = sysBytes.bytes[2];
	buf[13] = sysBytes.bytes[3];
	if( m_pClientSocket ) m_pClientSocket->Write( buf, 14 );	// KDS050512
	return 0;
}
