// GsEuvSeq.h: interface for the CGsEuvSeq class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CGsEuvSeq_H__56EBBC19_02F9_4695_8575_266EBDC8DB7E__INCLUDED_)
#define AFX_CGsEuvSeq_H__56EBBC19_02F9_4695_8575_266EBDC8DB7E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

///////////////////////////////////////////////////////////////////
// property
// ----------------------------------------------------------------
// 1. Lamp State ( ON / OFF )
// 2. EUV Mode ( Auto / Manual )
// 3. EUV Alarm ( Alarm )  
// 4. UT State
// 5. House State
// 6. Lamp Intensity Value
// ?. Lamp Life Time

///////////////////////////////////////////////////////////////////
// Methode
// ----------------------------------------------------------------
// 1. Lamp On Select ( for Lamp On )
// 2. Lamp Control ( Lamp On / Off )
// 3. Lamp Monitor Select ( for Intensity Check )
// 4. Lamp Intensity Check
// 5. EUV UT Control ( Valve On / Off ) 
// 6. GetLampOnState
// 7. IsRemoteMode
// 8. GetEuvAlarm
// 10. GetHouseState
// 11. GetIntensityValue
// ?. Get Lamp Life Time

#include "..\MOTHER\DicsEuvSeq.h"

#include "..\FactoryLib\GenSensor.h"

class CGsEuvSeq : public CDicsEuvSeq  
{
public:
	CGsEuvSeq();
	virtual ~CGsEuvSeq();
	

	enum euvSTATUS
	{
		euvREMOTE_MODE		= 0x01,
		euvMONITOR_READY	= 0x02,
		euvDATA_SET_ENABLE	= 0x04,
	};

	enum euvALARMINDEX
	{	// GS 
		euvAlmOverHeat		 = 0x01, // Over heat
		euvAlmLampHouseOpen  = 0x02, // House Open
		euvAlmLampFault	     = 0x04, // Lamp Fault
		euvAlmInverterErr    = 0x08, // Inverter Error
		euvAlmUtilityErr     = 0x10, // Utility Error
		euvAlmN2Err		     = 0x20, // N2 Error
		euvAlmSumErr	     = 0x40, // Sum Error
		euvAlmManualMode	 = 0x80, // ManualMode Error
	};

	void InitParameter();

	int GetStatus();
	
	void SeqLampIntensityCheck();
	void SeqLampIntensityCheck( int nPos );
	void SeqEuvAlarm();
	
	void LampOnSelect( );
	void LampOnSelect( int nLampNo, BOOL bOn );
	void LampControl( BOOL bOn );
	void UtControl( BOOL bOn );

	int	SetLampControl();
	int	GetEuvAlarm();
	BOOL GetHouseCloseState();
	BOOL GetLampOnSelected( int nLampNo );
	BOOL IsGlassExist();
	BOOL GetOnTimeOver( BOOL bEuvRun );
	BOOL GetEuvCond();
	BOOL GetUtState();

private:
	CGenSensor		*m_pSensorEuvHouseClose;
};

#endif // !defined(AFX_CGsEuvSeq_H__56EBBC19_02F9_4695_8575_266EBDC8DB7E__INCLUDED_)
