// HpmjSequence.cpp: implementation of the CHpmjSequence class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "HpmjSequence.h"
#include "..\X\XLog.h" 

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifndef _NO_HPMJ
XLog HPMJLog( "HPMJ", XLog::stampNoDateTime, TRUE ); // TRUE : Append
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHpmjSequence::CHpmjSequence()
{

}

CHpmjSequence::~CHpmjSequence()
{
}

BOOL CHpmjSequence::Initialize()
{
	InitParameter();
	return TRUE;

}

void CHpmjSequence::InitParameter()
{
	int i;
	m_pEqpAssy = ( (CDicsView *) m_pOwnerWnd )->m_pEqpAssy;
	m_SeqFlag.bInitReq = FALSE;
	m_SeqFlag.bInitFail = FALSE;
	m_SeqFlag.bInitComp = FALSE;

	m_nSeqInitNo = 0;
	m_nSeqModeChangeNo = 0;
	m_nSeqAlarmNo = 0;
	m_nSeqPumpRunNo = 0;
	m_nSeqStateMonitorNo = 0;
	m_nSeqHpmjAiNo = 0;
	
	for( i= 0; i < MAX_HPMJ_AI_INTRO_NO; i++ )
	{
		m_InterlockHpmj.bGaugeAlarmSet[i*2 + 0]		= FALSE;
		m_InterlockHpmj.bGaugeAlarmSet[i*2 + 1]		= FALSE;
		m_InterlockHpmj.bGaugeWarningSet[i*2 + 0]	= FALSE;
		m_InterlockHpmj.bGaugeWarningSet[i*2 + 1]	= FALSE;

		m_nSeqCheckInterlockNo[i] = 0;
	}

	for( i=0; i<MAX_HPMJ_STATE_DATA; i++ )
	{
		m_nHpmjStateData[i] = 0;
	}
}


BOOL CHpmjSequence::Sequence()
{
	int i;
	Sleep( 10 );

	SeqInit();
	SeqModeChange();
	
	SeqStateMonitor();
	SeqPumpRun();
	SeqAlarm();
	SeqCheckWaterJetManual();
	SeqHpmjAi();
#ifndef _HPMJ_SFU_CTRL
	SeqHpmjState();		// dspcrassus - HPMJ State Data Update
#endif

	for( i = 0 ; i < MAX_HPMJ_AI_INTRO_NO ; i ++ )
	{
		SeqGaugeInterlock( i );
	}

	for( i = 0 ; i < MAX_WATERJET_AI_NO ; i++ )
	{
		SeqAISampling( i );
	}

	return TRUE;
}


// Call After XSequence::Start( HWND hWnd )
void CHpmjSequence::SetOwnerPointer(CWnd* pWnd)
{
	SetDicsView( (CDicsView *) pWnd );
	SetDicsData( m_pDicsView->m_pDicsData );
	SetDevice( m_pDicsView->m_pDevice );
	SetSetupItems( m_pDicsView->GetDocument()->m_pSetupItems );
	SetEqpStateSequence( m_pDicsView->m_pEqpStateSeq );
	SetSeqTimer( m_pDicsView->m_pTimer );
	SetEqpAssy( m_pDicsView->m_pEqpAssy );
}

void CHpmjSequence::SetLog(CString sUnitName, CString sMessage, BOOL bSetGlsInfo)
{
	// Log
	m_sMessage.Format( "HPMJ\t%15s\t\t\t\t\t%s", sUnitName, sMessage );
#ifndef _NO_HPMJ
	HPMJLog.TextOut( m_sMessage );
#endif
}

void CHpmjSequence::SeqInit()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

#ifdef _UNIT_HPMJ
	static int nAlarmId = 0;
	static BOOL bAlarmFlg[MAX_HPMJ_ALARM_NO] = {FALSE,};
	
	int nSeqNo = m_nSeqInitNo;
	static DWORD dwTickCount = 0; 

	m_sSeqName.Format( "%s", "HPMJINIT" );

	switch( nSeqNo )
	{
	case 0:
		if( m_SeqFlag.bInitReq )
		{
			m_SeqFlag.bInitReq = FALSE;
			m_SeqFlag.bInitComp	= FALSE;
			m_SeqFlag.bInitFail = FALSE;
			
			SetLog( m_sSeqName, "Initialize Start");

			m_sMessage.Format( "%2d%d", CDicsData::initHPMJ_STATUS, MSG_STATUS_START );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			int nUseHpmj = m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE];
			
			if( nUseHpmj == 1 )	nSeqNo = 10;
			else	nSeqNo = 30;
		}
		break;

	case 10:
		{
			BOOL bAlarm = FALSE;

			for( int i = 0; i < MAX_HPMJ_ALARM_NO; i++)
			{
#ifndef _MY_DEBUG
				if( m_pEqpAssy->m_Hpmj.GetHpmjInputState( i ) )
#else
				if( 0 )
#endif
				{
					bAlarmFlg[i] = TRUE;
					SetLog( m_sSeqName, "HPMJ Unit Alarm" );
					m_pEqpStateSeq->SetAlarm( ALM_HPMJ_UNIT_ESTOP + i );
					bAlarm = TRUE;
				}
			}

			if( bAlarm )
			{
				bAlarm = FALSE;
				m_SeqFlag.bInitFail = TRUE;

		
				m_sMessage.Format( "%2d%d", CDicsData::initHPMJ_STATUS, MSG_STATUS_NG );
				Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );


				nSeqNo = 1000;
			}
			else
			{
				nSeqNo = 30;
			}
		}
		break;
		
	case 30:
		{
			SetLog( m_sSeqName, "Initialize Finished");

//			m_sMessage.Format( "%2d%d", CDicsData::initHPMJ_READY, MSG_STATUS_OK );
//			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			m_sMessage.Format( "%2d%d", CDicsData::initHPMJ_STATUS, MSG_STATUS_OK );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
			
			m_SeqFlag.bInitComp = TRUE;
			m_SeqFlag.bInitReq = FALSE;

			m_pDicsData->m_IfHpmj.bUse = m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE];	
			//m_pDicsData->m_IfHpmj.nPressure = m_pDicsData->m_JobCond.nPressure;

			nSeqNo = 0;
		}
		break;

	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			m_SeqFlag.bInitReq = FALSE;

			// Reset Alarm
			for( int i=0; i<MAX_HPMJ_ALARM_NO; i++ )
			{
				if( bAlarmFlg[i] )
				{
					bAlarmFlg[i] = FALSE;
					m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_UNIT_ESTOP + i );
				}
			}

			SetLog( m_sSeqName, "Alarm Recovery !");
			dwTickCount = GetTickCount();
			nSeqNo = 1010;
		}
		break;
	case 1010:
		if( ( GetTickCount() - dwTickCount ) > 2000 )
		{
			nSeqNo = 10;
		}
		break;
	}
	m_nSeqInitNo = nSeqNo;
#endif
}

void CHpmjSequence::SeqModeChange()
{
#ifdef _UNIT_HPMJ
	int nSeqNo = m_nSeqModeChangeNo;
	static int nPressure = 0;

	m_sSeqName.Format( "%s", "HPMJMODE" );

	switch( nSeqNo ) 
	{
	case 0 :
		if( m_pDicsData->m_GenInfo.bAuto )
		{
			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjMANUAL, OFF );
			SetLog( m_sSeqName, "Auto Mode" );

			nSeqNo = 10;
		}
		else if( !m_pDicsData->m_GenInfo.bAuto )
		{
			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjMANUAL, ON );
			SetLog( m_sSeqName, "Manual Mode" );

//			nPressure = m_pDicsData->m_JobCond.nPressure;

			m_sMessage.Format("Save Pressure = %d", nPressure );
			SetLog( m_sSeqName, m_sMessage );

			nSeqNo = 20;
		}
		break;

	case 10 :
		if( !m_pDicsData->m_GenInfo.bAuto )
		{	
			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjMANUAL, ON );
			SetLog( m_sSeqName, "Manual Mode" );

//			nPressure = m_pDicsData->m_JobCond.nPressure;

			m_sMessage.Format("Save Pressure = %d", nPressure );
			SetLog( m_sSeqName, m_sMessage );
		
			nSeqNo = 20;
		}
		break;

	case 20 :
		if( m_pDicsData->m_GenInfo.bAuto )
		{	
			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjMANUAL, OFF );
			SetLog( m_sSeqName, "Auto Mode" );

			m_pDicsData->m_IfHpmj.nPressure = nPressure;

			m_sMessage.Format("Setting Pressure = %d", nPressure );
			SetLog( m_sSeqName, m_sMessage );

			if( m_pDicsData->m_IfHpmj.bPumpRun ) 
			{
				m_nSeqStateMonitorNo = 10;

				SetLog( m_sSeqName, "bPumpRun = TRUE -> Seq StateMonitorNo = 10" );

				if( !m_SeqFlag.bInitComp )
				{
					m_pDicsData->m_IfHpmj.bPumpRun = FALSE;

					SetLog( m_sSeqName, "No Initial bPumpRun = FALSE" );
				}
			}
			nSeqNo = 10;
		}
		break;
	}

	m_nSeqModeChangeNo = nSeqNo;
#endif
}


void CHpmjSequence::SeqStateMonitor()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	BOOL bAlarm = FALSE;

	for( int i = 0; i < MAX_HPMJ_AI_INTRO_NO ; i++ )
	{
		bAlarm |= m_InterlockHpmj.bGaugeAlarmSet[i*2+0];
		bAlarm |= m_InterlockHpmj.bGaugeAlarmSet[i*2+1];

		if( bAlarm )
			m_pDicsData->m_IfHpmj.bGaugeInterlockAlarm = TRUE;
		else
			m_pDicsData->m_IfHpmj.bGaugeInterlockAlarm = FALSE;
	}

// Manual  ϶ Interlock  //////////////////////////////////////////////////////////////////
	if( !m_pDicsData->m_GenInfo.bAuto )
	{
		if( m_pDicsData->m_IfHpmj.bAlarm ||  m_pDicsData->m_IfHpmj.bGaugeInterlockAlarm ||
			m_pDicsData->m_IfHpmj.bPidError )
		{
			m_pDicsData->m_IfHpmj.bPumpRun = FALSE;
		}
	}
//////////////////////////////////////////////////////////////////////////////////////////////////

	if( !m_SeqFlag.bInitComp ) return;

	int nSeqNo = m_nSeqStateMonitorNo;

	BOOL bAlarm2	= FALSE;

	// Check CvStop Condition
	for( int i = IF_UNIT_SWR1; i<= IF_UNIT_UL; i++ )
	{
		if( i == IF_UNIT_SWR1 ) bAlarm2 |= m_pDicsData->m_IfFlag[i].bInError;		
		bAlarm2 |= m_pDicsData->m_IfFlag[i].bOutError;
	}

	BOOL bRun = TRUE;
	bRun &= m_pDicsData->m_GenInfo.bAuto;
	bRun &= !m_pDicsData->m_GenInfo.bEmo;
	bRun &= !m_pDicsData->m_GenInfo.bCoverOpen;
	bRun &= !m_pDicsData->m_GenInfo.bLeakSensor;
	bRun &= !m_pDicsData->m_IfHpmj.bAlarm;
	bRun &= !m_pDicsData->m_IfHpmj.bGaugeInterlockAlarm;
	bRun &= !m_pDicsData->m_IfHpmj.bPidError; 
	bRun &= ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE] == 1 );
	bRun &= ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupDI_PUMP1_USE] == 1 );
	bRun &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE];
	bRun &=	m_pDicsData->m_IfPump.bDiStart;
	bRun &= m_pEqpAssy->m_DiTank[0].GetPumpOnState();

	switch( nSeqNo )
	{
	case 0:
		if( bRun )
		{
			 m_pDicsData->m_IfHpmj.bPumpRun = TRUE;
			 nSeqNo = 10;
		}
		else if( m_pDicsData->m_GenInfo.bAuto )
		{
			// ڵ϶ Run  ƴѵ bPunRun    δ.
			if( !m_pDicsData->m_IfHpmj.bPumpRun )
			{
				m_pDicsData->m_IfHpmj.bPumpRun = FALSE;
			}
		}
		break;
	case 10:
		if( !bRun )
		{
			 m_pDicsData->m_IfHpmj.bPumpRun = FALSE;
			 nSeqNo = 0;
		}
		break;
	}
	m_nSeqStateMonitorNo = nSeqNo;
}

void CHpmjSequence::SeqPumpRun()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

#ifdef _UNIT_HPMJ
	static DWORD dwCount=0;

	int tm1 = TM_HPMJ_PUMP_CONTROL;
	int tm2 = TM_HPMJ_PID_TIME_OVER;
		
	static int nTime1;
	static int nTime2;

	int nSeqNo = m_nSeqPumpRunNo;

	m_sSeqName.Format( "HPMJ_RUN" );
	
#ifdef _MY_DEBUG
	static double press = 0.0;
	static double OldPress = 0.0;
	
	if( m_pDicsData->m_IfHpmj.bPumpRun )
	{
//		if( ( m_pDicsData->m_IfHpmj.nRefPressure - OldPress) >= 0 ) press += 0.1;
//		else press -= 0.1;

		press += 0.1;

		if( press > ( m_pDicsData->m_IfHpmj.nRefPressure - 1 ) &&
			press < ( m_pDicsData->m_IfHpmj.nRefPressure + 1 ) ) 
		{
			press = m_pDicsData->m_IfHpmj.nRefPressure;
//			OldPress = press;
		}

		m_pDicsData->m_IfHpmj.dCurPressure = press;
	}
	else
	{
		m_pDicsData->m_IfHpmj.dCurPressure = 0.0;
		press = 0.0;
	}
#endif
	
	BOOL bRunCond = TRUE;
	if( m_pDicsData->m_GenInfo.bAuto )
	{
		bRunCond &= m_SeqFlag.bInitComp;
		bRunCond &=	( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE] == 1 );
		bRunCond &= ( m_pDicsData->m_IfPump.bDiStart && m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE]==1 );
		if( !bRunCond && m_pDicsData->m_IfHpmj.bPumpRun )
		{
			m_pDicsData->m_IfHpmj.bPumpRun = FALSE;
		}
	}
	else if( !m_pDicsData->m_GenInfo.bAuto )
	{
		bRunCond &= m_pEqpAssy->m_DiTank[0].GetPumpOnState(); // SBJ Pump  ̸ HPMJ  
		if( !bRunCond && m_pDicsData->m_IfHpmj.bPumpRun )
		{
			m_pDicsData->m_IfHpmj.bPumpRun = FALSE;
		}
	}

	switch( nSeqNo )
	{
	case 0:
		if( m_pDicsData->m_IfHpmj.bPumpRun && bRunCond )
		{
			SetLog( m_sSeqName, "Pump Start" );	
			m_pDicsData->m_IfHpmj.dCurHertz = 0.0;
			m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0;
			nSeqNo = 10;
		}
		break;

	case 10:
		{
			m_pDicsData->m_IfHpmj.bPPIDChangeReq = FALSE;
			m_pDicsData->m_IfHpmj.bReady = FALSE;

			if( m_pDicsData->m_GenInfo.bAuto )
			{
				m_pDicsData->m_IfHpmj.nRefPressure = m_pDicsData->m_IfHpmj.nPressure + 1;

				m_sMessage.Format("Auto = Press Mode : Pressure = %d ", m_pDicsData->m_IfHpmj.nRefPressure );
				SetLog( m_sSeqName, m_sMessage );

				nSeqNo = 20;
			}
			else if( !m_pDicsData->m_GenInfo.bAuto )
			{
				// HPMJ RUN MODE 1 : PRESS SET 
				//               2 : FREQUENCY SET
				if( ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_RUN_MODE] == 1 ) &&
					!m_pEqpAssy->m_Hpmj.GetHpmjInputState( hpmjPUMPRUN ) )
				{
					int nVal = atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupHPMJ_PRESS_SET] );
					m_pDicsData->m_IfHpmj.nPressure = nVal;
					m_pDicsData->m_IfHpmj.nRefPressure = m_pDicsData->m_IfHpmj.nPressure + 1;

					m_sMessage.Format("Manual = Press Mode : Pressure = %d ", m_pDicsData->m_IfHpmj.nRefPressure );
					SetLog( m_sSeqName, m_sMessage );

					nSeqNo = 20;
				}
				else if( ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_RUN_MODE] == 0 ) ||
						 m_pEqpAssy->m_Hpmj.GetHpmjInputState( hpmjPUMPRUN ) )
				{
					int nVal = atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupHPMJ_FRQ_SET ] );
					if( m_pEqpAssy->m_Hpmj.GetHpmjInputState( hpmjPUMPRUN ) )
						nVal = 30;		// dspcrassus - 080401 : HPMJ Unit (Local)۽ Inverter Hertz(30) Fix
					m_pDicsData->m_IfHpmj.nCurAnalogHertz = PUMP_HERZ_2_AO( nVal );

					if( m_pDicsData->m_IfHpmj.nCurAnalogHertz > 0x7FFF ) m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0x7FFF;		
					else if( m_pDicsData->m_IfHpmj.nCurAnalogHertz < 0 ) m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0x0000;
						m_pEqpAssy->m_Hpmj.SetInverterHertz( INVERTER_HERTZ, m_pDicsData->m_IfHpmj.nCurAnalogHertz );

					m_sMessage.Format("Manual = Frequency Mode : Frequency = %d ", nVal );
					SetLog( m_sSeqName, m_sMessage );

					//m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjPUMPMCON, ON );
					m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjINVTRUN,  ON );

					TIMER_ON( tm1, 100 );

					nSeqNo = 100;
				}
			}
		}
		break;

	case 20:
		{
			int nVal = 10;
			m_sMessage.Format("PreSetting Frequency Value : Frequency = %d ", nVal );
			SetLog( m_sSeqName, m_sMessage );

			m_pDicsData->m_IfHpmj.nCurAnalogHertz = PUMP_HERZ_2_AO( nVal );

			if( m_pDicsData->m_IfHpmj.nCurAnalogHertz > 0x7FFF ) m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0x7FFF;		
			else if( m_pDicsData->m_IfHpmj.nCurAnalogHertz < 0 ) m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0x0000;
			m_pEqpAssy->m_Hpmj.SetInverterHertz( INVERTER_HERTZ, m_pDicsData->m_IfHpmj.nCurAnalogHertz );

			//m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjPUMPMCON, ON );
			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjINVTRUN,  ON );

			TIMER_ON( tm1, 2 );
			TIMER_ON( tm2, 500 );

			nSeqNo = 50;
		}
		break;

	case 30:
		{
			m_pDicsData->m_IfHpmj.bPPIDChangeReq = FALSE;
			m_pDicsData->m_IfHpmj.bReady = FALSE;

			if( m_pDicsData->m_GenInfo.bAuto )
			{
				m_pDicsData->m_IfHpmj.nRefPressure = m_pDicsData->m_IfHpmj.nPressure + 1;

				m_sMessage.Format("Auto Mode Recipe Change = Press Mode : Pressure = %d ", m_pDicsData->m_IfHpmj.nRefPressure );
				SetLog( m_sSeqName, m_sMessage );
			}
			else if( !m_pDicsData->m_GenInfo.bAuto )
			{
				int nVal = atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupHPMJ_PRESS_SET] );
				m_pDicsData->m_IfHpmj.nPressure = nVal;
				m_pDicsData->m_IfHpmj.nRefPressure = m_pDicsData->m_IfHpmj.nPressure + 1;

				m_sMessage.Format("Manual Mode Recipe Change = Press Mode : Pressure = %d ", m_pDicsData->m_IfHpmj.nRefPressure );
				SetLog( m_sSeqName, m_sMessage );
			}

			TIMER_ON( tm1, 2 );
			TIMER_ON( tm2, 450 );

			nSeqNo = 50;
		}
		break;

	case 50:
		if( m_pDicsData->m_IfHpmj.bPumpRun && TIMER( tm1 ) ) 
		{
			double dError;

			dError = SetHpmjPumpControl();

			if( dError >= 20.0 )
			{
				TIMER_ON( tm1, 2 );
			}
			else
			{
				TIMER_ON( tm1, 5 );
			}

			if( m_pDicsData->m_IfHpmj.bPPIDChangeReq )
			{
#ifdef _MY_DEBUG
				m_pDicsData->m_IfHpmj.dCurPressure = 0.0;
				press = 0.0;

				m_pDicsData->m_IfHpmj.dCurHertz = 0.0;
				m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0;
#endif
				nSeqNo = 30;
			}
		}
		else if( !m_pDicsData->m_IfHpmj.bPumpRun )
		{
			nSeqNo = 200;
		}
		else if( !m_pDicsData->m_IfHpmj.bReady && TIMER( tm2 ) )
		{
			SetLog( m_sSeqName, "Pump PID Control Alarm" );
			m_pEqpStateSeq->SetAlarm( ALM_HPMJ_PID_CONTROL_ERROR );

			m_pDicsData->m_IfHpmj.dCurHertz = 0.0;
			m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0;
			
			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjINVTRUN,  OFF );
			
			SetLog( m_sSeqName, "Pump Stop");			

			m_pDicsData->m_IfHpmj.bReady = FALSE;
			m_pDicsData->m_IfHpmj.bPidError = TRUE;

#ifdef _MY_DEBUG
			m_pDicsData->m_IfHpmj.dCurPressure = 0.0;
			press = 0.0;
#endif
			nSeqNo = 1000;
		}
		else if( ( m_pDicsData->m_IfHpmj.nCurAnalogHertz < 0 ) )
		{
			SetLog( m_sSeqName, "Pump PID Control Alarm" );
			m_pEqpStateSeq->SetAlarm( ALM_HPMJ_PID_CONTROL_ERROR );

			m_pDicsData->m_IfHpmj.dCurHertz = 0.0;
			m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0;

			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjINVTRUN,  OFF );
			
			SetLog( m_sSeqName, "Pump Stop");			

			m_pDicsData->m_IfHpmj.bReady = FALSE;
			m_pDicsData->m_IfHpmj.bPidError = TRUE;

#ifdef _MY_DEBUG
			m_pDicsData->m_IfHpmj.dCurPressure = 0.0;
			press = 0.0;
#endif
			nSeqNo = 1000;
		}
		break;

	case 100:
		if( m_pDicsData->m_IfHpmj.bPumpRun )
		{
			m_pDicsData->m_IfHpmj.dCurHertz = PUMP_ANALOG_HERTZ(m_pDicsData->m_IfHpmj.nCurAnalogHertz);

			if( TIMER( tm1 ) )
			{
				SetLog( m_sSeqName, "Pump is Ready" );

				m_pDicsData->m_IfHpmj.bReady = TRUE;
			}

			if( m_pDicsData->m_IfHpmj.bPPIDChangeReq )
			{
#ifdef _MY_DEBUG
				m_pDicsData->m_IfHpmj.dCurPressure = 0.0;
				press = 0.0;

				m_pDicsData->m_IfHpmj.dCurHertz = 0.0;
				m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0;
#endif
				nSeqNo = 10;
			}
		}
		else if( !m_pDicsData->m_IfHpmj.bPumpRun )
		{
			nSeqNo = 200;
		}
		break;

		
	case 200:
		if( !m_pDicsData->m_IfHpmj.bPumpRun || !bRunCond )
		{
			m_pDicsData->m_IfHpmj.dCurHertz = 0.0;
			m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0;

			m_pEqpAssy->m_Hpmj.SetHpmjAct( hpmjINVTRUN,  OFF );
			
			SetLog( m_sSeqName, "Pump Stop");			

			m_pDicsData->m_IfHpmj.bReady = FALSE;

			nSeqNo = 0;
		}
		break;

	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_PID_CONTROL_ERROR);

			m_pDicsData->m_IfHpmj.bPidError = FALSE;

			SetLog( m_sSeqName, "Alarm Recovery !");

			nSeqNo = 0;
		}
		break;
	}
	m_nSeqPumpRunNo = nSeqNo;
#endif
}


double CHpmjSequence::SetHpmjPumpControl()
{
	double dError = 0.0;
	int nProperty = 0;
#ifdef _UNIT_HPMJ

#ifndef _MY_DEBUG
	m_pDicsData->m_IfHpmj.dCurPressure = m_pEqpAssy->m_Hpmj.GetHpmjAanlogInput( hpmjOUTPRESS );
#endif
	
	m_sSeqName.Format( "%s", "HPMJCTRL" );

	// Get Error Value
	m_pDicsData->m_IfHpmj.dErrPressure = m_pDicsData->m_IfHpmj.nRefPressure - m_pDicsData->m_IfHpmj.dCurPressure;

	dError  = ABS( m_pDicsData->m_IfHpmj.dErrPressure );

#ifndef _MY_DEBUG
	if( dError < 1. ) 
#else
	if( dError < 5. )
#endif
	{
		m_pDicsData->m_IfHpmj.dErrPressure = 0.;	
	    m_pDicsData->m_IfHpmj.bReady = TRUE;

		if( m_pDicsData->m_IfHpmj.nRefPressure != m_pDicsData->m_IfHpmj.nPressure )
		{
			m_pDicsData->m_IfHpmj.nRefPressure = m_pDicsData->m_IfHpmj.nPressure;

			m_sMessage.Format("Ref Change %d", m_pDicsData->m_IfHpmj.nRefPressure );
			SetLog( m_sSeqName, m_sMessage );

			SetLog( m_sSeqName, "Pump is Ready" );
		}
	}

	if( dError > 100 ) nProperty = 350;
	else if( dError > 95 ) nProperty = 5;
	else if( dError > 55 ) nProperty = 350;
	else if( dError > 50 ) nProperty = 10;
	else if( dError > 30 ) nProperty = 250;
	else if( dError > 25 ) nProperty = 200;
	else if( dError > 5 ) nProperty = 100;
	else if( dError > 3 ) nProperty = 50;
	else nProperty = 35;
	
	if( m_pDicsData->m_IfHpmj.bReady )
		nProperty = 5;

	if( m_pDicsData->m_IfHpmj.dErrPressure != 0. ) 
	{
		m_sMessage.Format( "%d, %.2f", nProperty, m_pDicsData->m_IfHpmj.dCurPressure );
		SetLog( m_sSeqName, m_sMessage );

		if( m_pDicsData->m_IfHpmj.dErrPressure > 0. )
		{
			m_pDicsData->m_IfHpmj.nCurAnalogHertz += nProperty ;
		}
		else
		{
			m_pDicsData->m_IfHpmj.nCurAnalogHertz -= nProperty ;
		}
		
		if( m_pDicsData->m_IfHpmj.nCurAnalogHertz > 0x7FFF ) m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0x7FFF;		
		else if( m_pDicsData->m_IfHpmj.nCurAnalogHertz < 0 ) m_pDicsData->m_IfHpmj.nCurAnalogHertz = 0x0000;
			m_pEqpAssy->m_Hpmj.SetInverterHertz(INVERTER_HERTZ, m_pDicsData->m_IfHpmj.nCurAnalogHertz );

		m_pDicsData->m_IfHpmj.dCurHertz = PUMP_ANALOG_HERTZ(m_pDicsData->m_IfHpmj.nCurAnalogHertz);

		m_sMessage.Format("AO = %d, %.2f", m_pDicsData->m_IfHpmj.nCurAnalogHertz, m_pDicsData->m_IfHpmj.dCurHertz );
		SetLog( m_sSeqName, m_sMessage );
	}

#endif
	return dError;
}

void CHpmjSequence::SeqAlarm()
{
//	if( !m_SeqFlag.bInitComp ) return;

	int nUse = -1;
	nUse &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE];

	static BOOL bAlarmFlg[MAX_HPMJ_ALARM_NO] = {FALSE,};
	static DWORD dwStartTick = 0;
	
	int nSeqNo = m_nSeqAlarmNo;

	m_sSeqName.Format( "%s", "HPMJ_ALM" );

	switch( nSeqNo )
	{
	case 0:
		{
			dwStartTick = GetTickCount();
			
			nSeqNo = 5;
		}
		break;

	case 5:
		if( GetTickCount() - dwStartTick > 2500 )
		{
			nSeqNo = 10;
		}
	/*
		else if( nUse == 1 && !dosWATERJET_UNIT_SHOWER_PUMP_MC_ON )  // MC  
			{
				doWATERJET_UNIT_SHOWER_PUMP_MC_ON( ON );
			}*/
		break;

	case 10:
		if( GetTickCount() - dwStartTick > 5000 )
		{
			nSeqNo = 20;
		}
		break;

	case 20:
		if( nUse )
		{
			for( int i = 0; i < MAX_HPMJ_ALARM_NO; i++ )
			{
#ifndef _MY_DEBUG
				if( m_pEqpAssy->m_Hpmj.GetHpmjInputState( i ) )
#else
				if( 0 )
#endif
				{
					bAlarmFlg[i] = TRUE;
				}
			}

			for( int i = 0; i < MAX_HPMJ_ALARM_NO; i++ )
			{
				if( bAlarmFlg[i] )
				{
					m_pDicsData->m_IfHpmj.bAlarm = TRUE;

					m_sMessage.Format("HPMJ Unit %d Alarm", i+1);
					SetLog( m_sSeqName, m_sMessage );
#ifndef _NO_HPMJ
					m_pEqpStateSeq->SetAlarm( ALM_HPMJ_UNIT_ESTOP + i );
#endif

					nSeqNo = 1000;
				}
			}
		}
		else if( m_pDicsData->m_IfHpmj.bAlarm )
		{
			m_pDicsData->m_IfHpmj.bAlarm = FALSE;
		}
		break;

	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			BOOL bAlarm = FALSE;

			for( int i = 0; i < MAX_HPMJ_ALARM_NO; i++)
			{
#ifndef _MY_DEBUG
				if( m_pEqpAssy->m_Hpmj.GetHpmjInputState( i ) )
#else
				if( 0 )
#endif
				{
					bAlarm = TRUE;
				}
			}

			if( !bAlarm )
			{
				// Reset Alarm
				for( int i=0; i<MAX_HPMJ_ALARM_NO; i++ )
				{
					if( bAlarmFlg[i] )
					{
						bAlarmFlg[i] = FALSE;

						m_sMessage.Format("HPMJ Unit %d Alarm Reset", i+1);
						SetLog( m_sSeqName, m_sMessage );
#ifndef _NO_HPMJ
						m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_UNIT_ESTOP + i );
#endif
					}
				}
				SetLog( m_sSeqName, "Alarm Recovery !");

				m_pDicsData->m_IfHpmj.bAlarm = FALSE;
				nSeqNo = 1010;

				dwStartTick = GetTickCount();
			}
		}
		break;

	case 1010:
		if( ( GetTickCount() - dwStartTick ) > 1000 )
		{
			nSeqNo = 20;
		}
		break;
	}

	m_nSeqAlarmNo = nSeqNo;
}

BOOL CHpmjSequence::GetCheckCond( int nIntrNo )
{
	BOOL bCheckCond = TRUE;
//	bCheckCond &= m_pDicsData->m_GenInfo.bAuto;
	bCheckCond &= !m_pDicsData->m_GenInfo.bEmo;	// HJYOU 2007.11.28
	bCheckCond &= !m_pDicsData->m_GenInfo.bLeakSensor;
	bCheckCond &= !m_pDicsData->m_GenInfo.bCoverOpen;
	bCheckCond &= !m_pDicsData->m_GenInfo.bDoorOpen;
//	bCheckCond &= !m_pDicsData->m_GenInfo.bPause;
	bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE];	
	bCheckCond &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE];	
//	bCheckCond &= m_pSetupItems->m_nValueGaugeLimit_Option[nIntrNo];
	bCheckCond &= m_pDicsData->m_IfHpmj.bReady;

	switch( nIntrNo ) 
	{
		case CSetupItems::gaugeHPMJ_UNIT_SHOWER_FLOW:
			break;
		case CSetupItems::gaugeHPMJ_UNIT_FILTER_IN_PRESS:
			break;
		case CSetupItems::gaugeHPMJ_UNIT_FILTER_OUT_PRESS:
			break;
		case CSetupItems::gaugeHPMJ_UNIT_MAIN_DI_PRESS:
			break;
		case CSetupItems::gaugeHPMJ_UNIT_RESISTIVITY:		
			break;
		case CSetupItems::gaugeHPMJ_UNIT_CO2_PRESS:
			break;
		case CSetupItems::gaugeHPMJ_UNIT_INVERTER_CURRENT:
			break;
		case CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS:
			break;
	}	
	return bCheckCond;
}

void CHpmjSequence::SeqGaugeInterlock( int nIntrNo )
{/*
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
	
#ifndef _UNIT_HPMJ
	// return if not initialized
	if( !m_pDicsData->m_GenInfo.bEqpInitComp ) return;
	
//	static BOOL	bCheck[MAX_HPMJ_AI_INTRO_NO]		= { FALSE, };
//	static int nAlarmId[MAX_HPMJ_AI_INTRO_NO]		= { 0, };
//	static int nWarningId[MAX_HPMJ_AI_INTRO_NO]		= { 0, };
//	static int nTime1[MAX_HPMJ_AI_INTRO_NO];
	static int nReturnSeqNo[MAX_HPMJ_AI_INTRO_NO];
	static DWORD dwTickStart[MAX_HPMJ_AI_INTRO_NO];
	DWORD dwDelayTime = 10000;

	int nSeqNo	= m_nSeqCheckInterlockNo[nIntrNo];
	int nHpmjAiStartNo = HPMJ_AI_START_NO;

	BOOL bCheckCond = TRUE;
	bCheckCond = GetCheckCond( nIntrNo );

	m_sSeqName.Format( "%s", "INTRLOCK" );

	switch( nSeqNo )
	{
	case 0 : // ó Check On̸ Time Delay ش.
		if( bCheckCond )
		{
			dwTickStart[nIntrNo] = GetTickCount();
			nSeqNo = 5;
		}
		else
		{
			int  nNo = nIntrNo;
			if( m_InterlockHpmj.bGaugeAlarmSet[nNo*2+0] )
			{
				m_InterlockHpmj.bGaugeAlarmSet[nNo*2+0] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_ERROR+ nNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", m_pSetupItems->m_infoHpmjPara[nNo].sName );
				SetLog( m_sSeqName, m_sMessage );
			}
			if( m_InterlockHpmj.bGaugeAlarmSet[nNo*2+1] )
			{
				m_InterlockHpmj.bGaugeAlarmSet[nNo*2+1] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_ERROR + nNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", m_pSetupItems->m_infoHpmjPara[nNo].sName );
				SetLog( m_sSeqName, m_sMessage );
			}
			if( m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] )
			{
				m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_WARNING + nNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", m_pSetupItems->m_infoHpmjPara[nNo].sName );
				SetLog( m_sSeqName, m_sMessage );
			}
			if( m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] )
			{
				m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_WARNING + nNo );

				m_sMessage.Format( "Interlock Recovery(No Check)[%s]", m_pSetupItems->m_infoHpmjPara[nNo].sName );
				SetLog( m_sSeqName, m_sMessage );
			}
		}
		break;

	case 5 :
		if( ( GetTickCount() - dwTickStart[nIntrNo] ) > dwDelayTime ) nSeqNo = 10;
		break;

	case 10 :	 
		if( !bCheckCond ) nSeqNo = 0;
		else
		{
			BOOL bAlarm		= FALSE;
			BOOL bWarning	= FALSE;

			// interlock θ CheckѴ.
			int  nNo		= nIntrNo;
			int  nAiNo;

			double dLowLimit	= atof(m_pSetupItems->m_sValueHpmjPara[0][nNo]);
			double dLoWarning	= atof(m_pSetupItems->m_sValueHpmjPara[1][nNo]);
			double dUpWarning	= atof(m_pSetupItems->m_sValueHpmjPara[2][nNo]);
			double dUpLimit	= atof(m_pSetupItems->m_sValueHpmjPara[3][nNo]);
			CString sName	= m_pSetupItems->m_infoHpmjPara[nNo].sName;

			double dCurVal = 0.0;
			double dInPress, dOutPress = 0.0;

			if( nIntrNo < CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS )
			{
				nAiNo = m_pAI->m_aiSet[nNo + nHpmjAiStartNo].nReal;
				dCurVal	= (double) m_pDevice->GetCalAI( nAiNo );			
			}
			else
			{

				dInPress =	(double) m_pDevice->GetCalAI( noWATERJET_UNIT_FILTER_IN_PRESS );
				dOutPress = (double) m_pDevice->GetCalAI( noWATERJET_UNIT_FILTER_OUT_PRESS );

				dCurVal	= ABS( dInPress - dOutPress );			
			}

			if( dCurVal < dLowLimit && nIntrNo < CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS )
			{
				bAlarm = TRUE;
//				m_sMessage.Format( "1st %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
//					dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
//				SetLog( m_sSeqName, m_sMessage );
			}
			else if( dCurVal > dUpLimit )
			{
				bAlarm = TRUE;
//				m_sMessage.Format( "1st %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
//					dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
//				SetLog( m_sSeqName, m_sMessage );
			}
			else if( dCurVal < dLoWarning && nIntrNo < CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS )
			{
				bWarning = TRUE;
//				m_sMessage.Format( "1st %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
//					dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
//				SetLog( m_sSeqName, m_sMessage );
			}		
			else if( dCurVal > dUpWarning  )
			{
				bWarning = TRUE;
//				m_sMessage.Format( "1st %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
//					dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
//				SetLog( m_sSeqName, m_sMessage );
			}
		
			if( bAlarm || bWarning )
			{	
				dwTickStart[nIntrNo] = GetTickCount();			

				nSeqNo = 15;
			}
		}
		break;

	case 15:
#ifndef _MY_DEBUG
		if( ( GetTickCount() - dwTickStart[nIntrNo] ) > 2000 )
#else
		if( ( GetTickCount() - dwTickStart[nIntrNo] ) > 5000 )
#endif
		{
			nSeqNo = 20;
		}
		break;

	case 20 :	 
		if( !bCheckCond ) nSeqNo = 0;
		else
		{
			BOOL bAlarm		= FALSE;
			BOOL bWarning	= FALSE;

			// interlock θ CheckѴ.
			int  nNo		= nIntrNo;
			int  nAiNo;
								
			double dLowLimit	= atof(m_pSetupItems->m_sValueHpmjPara[0][nNo]);
			double dLoWarning	= atof(m_pSetupItems->m_sValueHpmjPara[1][nNo]);
			double dUpWarning	= atof(m_pSetupItems->m_sValueHpmjPara[2][nNo]);
			double dUpLimit	= atof(m_pSetupItems->m_sValueHpmjPara[3][nNo]);
			CString sName	= m_pSetupItems->m_infoHpmjPara[nNo].sName;

			double dCurVal = 0.0;
			double dInPress, dOutPress = 0;

			if( nIntrNo < CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS )
			{
//				nAiNo = m_pAI->m_aiSet[nNo + nHpmjAiStartNo].nReal;
				dCurVal	= (double) m_pDevice->GetCalAI( nAiNo );			
			}
			else
			{

//				dInPress =	(double) m_pDevice->GetCalAI( noWATERJET_UNIT_FILTER_IN_PRESS );
//				dOutPress = (double) m_pDevice->GetCalAI( noWATERJET_UNIT_FILTER_OUT_PRESS );

				dCurVal	= ABS( dInPress - dOutPress );			
			}

			if( dCurVal < dLowLimit && nIntrNo < CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS )
			{
				bAlarm = TRUE;
				m_InterlockHpmj.bGaugeAlarmSet[nNo*2 + 0] = TRUE;
				m_pEqpStateSeq->SetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_ERROR + nNo );
				m_sMessage.Format( "Alarm : %s lower Alarm", sName );
				SetLog( m_sSeqName, m_sMessage );
				m_sMessage.Format( "2nd %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
					dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
				SetLog( m_sSeqName, m_sMessage );
			}
			else if( dCurVal > dUpLimit )
			{
				bAlarm = TRUE;
				m_InterlockHpmj.bGaugeAlarmSet[nNo*2 + 1] = TRUE;
				m_pEqpStateSeq->SetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_ERROR + nNo );
				m_sMessage.Format( "Alarm : %s upper Alarm", sName );
				SetLog( m_sSeqName, m_sMessage );
				m_sMessage.Format( "2nd %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
					dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
				SetLog( m_sSeqName, m_sMessage );

			}
			else if( dCurVal < dLoWarning && nIntrNo < CSetupItems::gaugeHPMJ_UNIT_FILTER_DIFFER_PRESS )
			{
				bWarning = TRUE;
				if( !m_InterlockHpmj.bGaugeWarningSet[nNo*2 + 0] )
				{
					m_InterlockHpmj.bGaugeWarningSet[nNo*2 + 0] = TRUE;
					m_pEqpStateSeq->SetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_WARNING + nNo );
					m_sMessage.Format( "Warning : %s lower Warning", sName );
					SetLog( m_sSeqName, m_sMessage );
					m_sMessage.Format( "2nd %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
						dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
					SetLog( m_sSeqName, m_sMessage );

				}
			}		
			else if( dCurVal > dUpWarning  )
			{
				bWarning = TRUE;
				if( !m_InterlockHpmj.bGaugeWarningSet[nNo*2 + 1] )
				{
					m_InterlockHpmj.bGaugeWarningSet[nNo*2 + 1] = TRUE;
					m_pEqpStateSeq->SetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_WARNING+ nNo );
					m_sMessage.Format( "Warning : %s upper Warning", sName );
					SetLog( m_sSeqName, m_sMessage );
					m_sMessage.Format( "2nd %s = %.3f ( %.1f, %.1f, %.1f, %.1f)", sName, 
						dCurVal, dLowLimit, dLoWarning, dUpWarning, dUpLimit );
					SetLog( m_sSeqName, m_sMessage );
				}
			}
		
			if( bAlarm )
			{	
//				m_pDicsData->m_GenInfo.bGaugeInterlock = TRUE;
				nReturnSeqNo[nNo] = 0;
				nSeqNo = 1000;
				break;
			}

			if( !bWarning && ( m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] || m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] ) )
			{	
				if( m_pEqpStateSeq->IsPushedAlarmReset() )
				{
					if( m_InterlockHpmj.bGaugeWarningSet[nNo*2+0])
					{
						m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] = FALSE;
						m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_WARNING+nNo);
					}
					if( m_InterlockHpmj.bGaugeWarningSet[nNo*2+1])
					{
						m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] = FALSE;
						m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_WARNING+nNo);
					}
				}
			}

			if( !bAlarm && !bWarning && 
				!m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] && !m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] )
			{
				m_sMessage.Format( "2nd Gauge Interlock Check OK");
				SetLog( m_sSeqName, m_sMessage );

				nSeqNo = 10;
			}
		}
		break;
	// Error ó
	case 1000 :
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			int  nNo = nIntrNo;	

			if( m_InterlockHpmj.bGaugeAlarmSet[nNo*2+0] )
			{
				m_InterlockHpmj.bGaugeAlarmSet[nNo*2+0] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_ERROR+nNo);
			}
			if( m_InterlockHpmj.bGaugeAlarmSet[nNo*2+1] )
			{
				m_InterlockHpmj.bGaugeAlarmSet[nNo*2+1] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_ERROR+nNo);
			}
			if( m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] )
			{
				m_InterlockHpmj.bGaugeWarningSet[nNo*2+0] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_LO_WARNING+nNo);
			}
			if( m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] )
			{
				m_InterlockHpmj.bGaugeWarningSet[nNo*2+1] = FALSE;
				m_pEqpStateSeq->ResetAlarm( ALM_HPMJ_SHOWER_FLOW_UP_WARNING+nNo);
			}

			m_sMessage.Format( "Interlock Recovery[%s]", m_pSetupItems->m_infoHpmjPara[nNo].sName );
			SetLog( m_sSeqName, m_sMessage );

			nSeqNo = nReturnSeqNo[nNo];
		}
		break;
	}
	m_nSeqCheckInterlockNo[nIntrNo] = nSeqNo;	
#endif */
}

void CHpmjSequence::SeqCheckWaterJetManual()
{
//	if( m_pDicsData->m_GenInfo.bAuto ) return;

/*
	static BOOL bPumpRun = FALSE;
	static BOOL bPumpStop = FALSE;
	if( diWATERJET_UNIT_PUMP_RUN && !bPumpRun && dosWATERJET_UNIT_MANUAL_ON &&
		m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupHPMJ_USE] )
	{
		bPumpRun = TRUE;
		m_pDicsData->m_IfHpmj.bPumpRun = TRUE;
		SetLog( "HPMJ_MAN", "HPMJ Unit Manual Test Run" );
	}
	else if( ( !diWATERJET_UNIT_PUMP_RUN || m_pDicsData->m_GenInfo.bAuto ) && bPumpRun ) 
	{
		bPumpRun = FALSE;
	}

	if( diWATERJET_UNIT_PUMP_STOP && !bPumpStop && dosWATERJET_UNIT_MANUAL_ON )
	{
		bPumpStop = TRUE;
		m_pDicsData->m_IfHpmj.bPumpRun = FALSE;
		SetLog( "HPMJ_MAN", "HPMJ Unit Manual Test Stop" );
	}
	else if( ( !diWATERJET_UNIT_PUMP_STOP || m_pDicsData->m_GenInfo.bAuto ) && bPumpStop )
	{
		bPumpStop = FALSE;
	}*/


}

void CHpmjSequence::SeqHpmjAi()
{
#ifdef _UNIT_HPMJ
	int nSeqNo = m_nSeqHpmjAiNo;
	static DWORD dwTickCount = 0; 

	m_sSeqName.Format( "%s", "HPMJFLOW" );
	switch( nSeqNo )
	{
	case 0:
		if( m_pDicsData->m_IfHpmj.bReady )
		{
			dwTickCount = GetTickCount();

			nSeqNo = 10;
		}
		break;

	case 10:
		if( GetTickCount() - dwTickCount > 1000 )
		{
			double dAi1, dAi2, dAi3, dAi4, dAi5;
			//2008.03.26 Youngsik
			dAi1 = m_pEqpAssy->m_Hpmj.GetHpmjAanlogInput( hpmjSHOWR );
			dAi2 = m_pEqpAssy->m_Hpmj.GetHpmjAanlogInput( hpmjINPRESS );
			dAi3 = m_pEqpAssy->m_Hpmj.GetHpmjAanlogInput( hpmjOUTPRESS );
			dAi4 = m_pEqpAssy->m_Hpmj.GetHpmjAanlogInput( hpmjMAINDIPRESS );
			dAi5 = m_pEqpAssy->m_Hpmj.GetHpmjAanlogInput( hpmjDIW_RESISTIVITY );

			m_sMessage.Format( "Shower Flow: %3.2f, Filter In Press: %3.2f, Filter Out Press : %3.2f, Main Di Press = %3.2f, DIW Res = %3.2f", 
				   dAi1, dAi2, dAi3, dAi4 , dAi5);
			SetLog( m_sSeqName, m_sMessage );

			nSeqNo  = 0;
		}
		break;
	}

	m_nSeqHpmjAiNo = nSeqNo;
#endif
}


//2008.03.26 Youngsik
void CHpmjSequence::SeqAISampling(int nAiNo)
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() )	return;
#endif
	
//	m_pDicsData->m_HpmjAI.fAiVal[nAiNo] = m_pAI->GetCalAI( nAiNo + HPMJ_AI_START_NO );
}

void CHpmjSequence::SeqHpmjState()
{

	CString sBuf = "";
	CDicsApp* pApp = (CDicsApp*) AfxGetApp();
	
	static DWORD dwTickStart = GetTickCount();
	static BOOL bUpdateFirst = TRUE;

	if( bUpdateFirst )
	{
		m_IniSetUpInfo.Open("HpmjInfo","SETUPINFO");
		bUpdateFirst = FALSE;
		for( int i=0; i<MAX_HPMJ_STATE_DATA; i++ )
		{
			if( i < 3 )
			{
				sBuf.Format( "VALUE_HPMJ_DATA_%d", i );
				m_nHpmjStateData[i] = m_IniSetUpInfo.GetIntValue("DICS HPMJ VIEW", sBuf, 0 ); 
				
			}
			else if( ( i == 3 ) || ( i == 4 ) )		// dspcrassus - SetupView HPMJ Tab  (Index: 4, 5)
			{
//				m_nHpmjStateData[i] = atoi( m_pSetupItems->m_sValueSetup[CSetupItems::partsFILTER4+i-3] ) * 60;
			}
			else									
			{
				m_nHpmjStateData[i] = 0;
			}
		}
	}
	
	if( m_pDicsData->m_IfHpmj.bReady )
	{
		m_IniSetUpInfo.Open("HpmjInfo","SETUPINFO");
		if( ( ( GetTickCount() - dwTickStart ) > ( 60 * 1000 ) ) )
		{
			dwTickStart = GetTickCount();
			
			for( int i=0; i<MAX_HPMJ_STATE_DATA; i++ )
			{
				if( i < 3 )								// dspcrassus - 0,1,2 : HPMJ 뿡  ð
				{
					sBuf.Format( "VALUE_HPMJ_DATA_%d", i );
					m_nHpmjStateData[i] = m_IniSetUpInfo.GetIntValue("DICS HPMJ VIEW", sBuf, 0 ); 
					
					m_nHpmjStateData[i] += 1;
					m_IniSetUpInfo.SetIntValue( "DICS HPMJ VIEW", sBuf, m_nHpmjStateData[i], TRUE );
				}
				else if( ( i == 3 ) || ( i == 4 ) )		// dspcrassus - SetupView HPMJ Tab  (Index: 4, 5)
				{
					//m_nHpmjStateData[i] = 
					//	atoi( m_pSetupItems->m_sValueSetup[CSetupItems::setupHPMJ_FILTER_EXCHANGE_TIME+i-3] ) * 60;
				}
				else									// dspcrassus - Alarm Code  
				{
					m_nHpmjStateData[i] = 0;
				}
			}
		}
	}
	else
	{
		dwTickStart = GetTickCount();
	}

	if( m_pDicsData->m_IfHpmj.bRunTimeCountReset )
	{
		m_IniSetUpInfo.Open("HpmjInfo","SETUPINFO");
		m_pDicsData->m_IfHpmj.bRunTimeCountReset = FALSE;
		sBuf.Format( "VALUE_HPMJ_DATA_0" );
		m_nHpmjStateData[0] = 0;
		m_IniSetUpInfo.SetIntValue( "DICS HPMJ VIEW", sBuf, m_nHpmjStateData[0], TRUE );
		
	}
	if( m_pDicsData->m_IfHpmj.bPumpChangeCountReset )
	{
		m_IniSetUpInfo.Open("HpmjInfo","SETUPINFO");
		m_pDicsData->m_IfHpmj.bPumpChangeCountReset = FALSE;
		sBuf.Format( "VALUE_HPMJ_DATA_1" );
		m_nHpmjStateData[1] = 0;
		m_IniSetUpInfo.SetIntValue( "DICS HPMJ VIEW", sBuf, m_nHpmjStateData[1], TRUE );
	}
	if( m_pDicsData->m_IfHpmj.bFilterChangeCountReset )
	{
		m_IniSetUpInfo.Open("HpmjInfo","SETUPINFO");
		m_pDicsData->m_IfHpmj.bFilterChangeCountReset = FALSE;
		sBuf.Format( "VALUE_HPMJ_DATA_2" );
		m_nHpmjStateData[2] = 0;
		m_IniSetUpInfo.SetIntValue( "DICS HPMJ VIEW", sBuf, m_nHpmjStateData[2], TRUE );
	}
}