// PlasmaSequence.cpp: implementation of the PlasmaSequence class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\dics.h"
#include "PlasmaSequence.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPlasmaSequence::CPlasmaSequence()
{
	m_fFlowRate =0;
	m_nSeqGasMointorNo   = 0;	
#ifdef _MY_DEBUG
	for( int i = 0; i<4; i++)
	{
		m_nPlasmaAlarmAddress[i] = 77+i;
	}
#endif
}

CPlasmaSequence::~CPlasmaSequence()
{
}
void CPlasmaSequence::SetOwnerPointer(CWnd *pWnd)
{
	SetDicsView( (CDicsView *) pWnd );
	SetDicsData( m_pDicsView->m_pDicsData );
	SetDevice( m_pDicsView->m_pDevice );
	SetSetupItems( m_pDicsView->GetDocument()->m_pSetupItems );
	SetEqpStateSequence( m_pDicsView->m_pEqpStateSeq );
	SetSeqTimer( m_pDicsView->m_pTimer );
	SetEqpAssy( m_pDicsView->m_pEqpAssy );
}
BOOL CPlasmaSequence::Initialize()
{
	InitParameter();
	return TRUE;

}

void CPlasmaSequence::InitParameter()
{

	m_pEqpAssy = ( (CDicsView *) m_pOwnerWnd )->m_pEqpAssy;
	m_nSeqInitNo		= 0;
	m_nSeqPlasmaNo		= 0;
	m_nSeqPlasmaAlarmNo = 0;
	m_nSeqPlasamProgressNo	= 0;

	m_SeqFlag.bInitReq = FALSE;
	m_SeqFlag.bInitComp = FALSE;
	m_SeqFlag.bInitFail = FALSE;

}

BOOL CPlasmaSequence::Sequence()
{
	Sleep(10);
	SeqInit();
	
	SeqPlasma();
	SeqPlasmaAlarm();

#ifdef _MY_DEBUG  
	SetDebugState();
#endif
	return TRUE;
}

void CPlasmaSequence::SeqInit()
{

	#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif

	int nSeqNo = m_nSeqInitNo;
	static DWORD dwTickStart;
	static int nAlarmId;
	static BOOL bEuvOk = FALSE;
	int nAlarmCode	= 0;

	switch( nSeqNo )
	{
	case 0:
		if( m_SeqFlag.bInitReq )
		{
			m_SeqFlag.bInitReq	= FALSE;
			m_SeqFlag.bInitComp	= FALSE;
			m_SeqFlag.bInitFail	= FALSE;

			m_sMessage.Format( "%2d%d", CDicsData::initAP_STATUS, MSG_STATUS_START );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			// Log
			SetLog( "INIT    ", 0, 0, "Initialize..." );

			dwTickStart = GetTickCount();
	
			if( !IsPlasmaUse() )	nSeqNo = 40;
			else
			{	
				SetLog( "INIT    ", 0, 0, "UT Turn ON" );
				nSeqNo = 10;
			}
		}
		break;
	case 10 :
		if ( ( GetTickCount() - dwTickStart ) > 3000 ) 
		{
			nSeqNo = 20;
		}
		break;
	case 20 :		
		if( m_pEqpAssy->m_Plasma.GetPlasmaStatus(3) )
		{
			// Set Log
			SetLog( "INIT    ", 0, 0, "Plasma Power Ready..." );
			nSeqNo = 30;
		}
		else
		{
#ifndef _NO_AP
			nAlarmId = ALM_AP_POWER_UNIT_NOT_READY;
#endif
			m_pEqpStateSeq->SetAlarm( nAlarmId );
					
			m_sMessage.Format( "%2d%d", CDicsData::initAP_STATUS, MSG_STATUS_NG );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );

			SetLog( "ALARM", 0, 0, "Alarm Set" );

			m_SeqFlag.bInitFail	= TRUE;

			nSeqNo = 1000;
		}	
		break;

	case 30 :
		if( ( nAlarmCode = GetPlasmaAlarm() ) == plasmaNoAlm )
		{
			SetLog( "INIT    ", 0, 0, "No Alarm" );	
			nSeqNo = 40;
		}
		else
		{
#ifndef _NO_AP
			nAlarmId = ALM_AP_FL_1_AP_FAIL + nAlarmCode;
#endif
			m_pEqpStateSeq->SetAlarm( nAlarmId );
			m_sMessage.Format( "%2d%d", CDicsData::initAP_STATUS, MSG_STATUS_NG );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
			SetLog( "ALARM", 0, 0, "Alarm Set" );
			m_SeqFlag.bInitFail	= TRUE;
			nSeqNo = 1000;
		}
		break;
		
	case 40 :
		if( GetHouseCloseState() )
		{
			SetLog( "INIT    ", 0, 0, "House Close Confirm" );	
			nSeqNo = 100;
		}
		else
		{
#ifndef _NO_AP
			nAlarmId = ALM_AP_PLASMA_HOUSE_OPEN;
#endif
			m_pEqpStateSeq->SetAlarm( nAlarmId );		
			m_sMessage.Format( "%2d%d", CDicsData::initAP_STATUS, MSG_STATUS_NG );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
			SetLog( "ALARM", 0, 0, "Alarm Set" );
			m_SeqFlag.bInitFail	= TRUE;
			nSeqNo = 1000;
		}
		break;
	case 100:
		{			
			// Flag Set
			m_SeqFlag.bInitComp = TRUE;
			m_sMessage.Format( "%2d%d", CDicsData::initAP_STATUS, MSG_STATUS_OK );
			Msg( WM_INIT_STATUS, MSG_STATUS_SET_MAIN, (LPARAM)(LPCTSTR)m_sMessage, TRUE );
			// Log
			SetLog( "INIT    ", 0, 0, "Ends Initialize" );
			Msg( WM_EQP_MESSAGE, MSG_TITLEBAR_MESSAGE, (LPARAM)(LPCTSTR)"READY", FALSE );
			nSeqNo = 0;
		}
		break;

	// Error
	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )
		{
			// Reset Alarm
			m_pEqpStateSeq->ResetAlarm( nAlarmId );

			nSeqNo = 0;
		}
		break;
	}

	m_nSeqInitNo = nSeqNo;
	
}

BOOL CPlasmaSequence::GetPlasmaStatus()
{
	BOOL bRv = FALSE;
	if( m_pEqpAssy->m_Plasma.GetPlasmaStatus( 3 ))
		bRv = TRUE;
	else bRv = FALSE;

	return bRv;
}

void CPlasmaSequence::SeqPlasma()
{
	
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
	if( !m_pDicsData->m_GenInfo.bEqpInitComp ) return;

	int nSeqNo = m_nSeqPlasmaNo;
    static int nReturnSeqNo = 0;
	static int nAlarmId = 0;
	//******************************************************************************
	// Get Run Condition
	//******************************************************************************
	BOOL bRunCond = FALSE;

	bRunCond |= m_pEqpAssy->m_CvMotor.GetMotorFwState( IF_UNIT_LD - IF_UNIT_LD, 0 );
//	bRunCond |= m_pEqpAssy->m_CvMotor.GetMotorFwState( IF_UNIT_AP - IF_UNIT_LD, 0 );
	bRunCond &= IsGlassExist();	
	bRunCond &= ( m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE] == 1 );

	BOOL bRunUse =  IsPlasmaUse();

	//******************************************************************************
	// Get Alarm Condition
	//******************************************************************************
	BOOL bAlarm = FALSE;
	
	for( int i = 0 ; i < 2 ; i ++ )
	{
		bAlarm |= ( m_pDicsData->m_IfFlag[i].bInError | m_pDicsData->m_IfFlag[i].bOutError );
	}

	//******************************************************************************
	// Get Recovery Condition
	//******************************************************************************
	BOOL bRecoveryReq = FALSE;
	
	bRecoveryReq |= m_pDicsData->m_IfFlag[IF_UNIT_LD].bRecoveryReq;
//	bRecoveryReq |= m_pDicsData->m_IfFlag[IF_UNIT_AP].bRecoveryReq;
	//******************************************************************************
	// Get Interlock Condition
	//******************************************************************************
	BOOL bInterlock = FALSE;
		
	bInterlock|= !GetHouseCloseState();
	bInterlock|= ( GetPlasmaAlarm() != plasmaNoAlm );
	bInterlock|= !( GetPlasmaStatus() );
//	bInterlock|= m_pEqpAssy->m_Plasma.GetPowerStatus();
	bInterlock|= !m_pDicsData->m_GenInfo.bAuto;
	bInterlock|= m_pDicsData->m_GenInfo.bPause;
	bInterlock|= m_pDicsData->m_GenInfo.bEmo;
	bInterlock|= m_pDicsData->m_GenInfo.bDoorOpen;
	
	BOOL bPlasmaPowerOn = FALSE;
	bPlasmaPowerOn = !bInterlock & bRunUse & bRunCond & ( bRecoveryReq || !bAlarm );
	SeqPlasamProgress( bPlasmaPowerOn );

	switch( nSeqNo )
	{
	case 0:
		if( bPlasmaPowerOn )
		{
			
			SetLog( "PLASMA POWER", 0, 0, "PLASMA INTERLOCK ON" );
			if( !m_pEqpAssy->m_Plasma.GetPlasmaInterlockActState() ) m_pEqpAssy->m_Plasma.SetPlasmaInterlock(ON);
			CString strPowerRate;
			strPowerRate = m_pSetupItems->m_sValueSetupTemp[CSetupItems::setupPLASMA_POWER_SET];
			m_fFlowRate = (float)atof(strPowerRate);
			SetPlasmaPowerSupply( (int)m_fFlowRate );
			m_sMessage.Format("PLASMA POWER %f kV Setting " , m_fFlowRate );
			SetLog( "PLASMA POWER", 0, 0, m_sMessage );			
			
			nSeqNo = 10;
		}
		break;

	case 10:
		{
			m_pEqpAssy->m_Plasma.SetPowerOn( ON );
			SetLog( "PLASMA POWER", 0, 0, "PLASMA POWER ON" );	
			nSeqNo = 20;
		}
		break;

	case 20:
		if( m_pEqpAssy->m_Plasma.GetPowerStatus() )
		{
			SetLog( "PLASMA POWER", 0, 0, "PLASMA POWER ON STATE CONFRIM" );
			nSeqNo = 30;
		}
		else
		{
#ifndef _NO_AP
			nAlarmId = ALM_AP_POWER_UNIT_NOT_READY;
#endif
			m_pEqpStateSeq->SetAlarm( nAlarmId);
			SetLog( "PLASMA POWER", 0, 0, "PLASMA POWER OFF Alarm Set" );
			nReturnSeqNo = nSeqNo;
			nSeqNo = 1000;		
		}
		break;
		
	case 30:
		if( !bPlasmaPowerOn )
		{
			m_pEqpAssy->m_Plasma.SetPowerOn( OFF );
			SetLog( "PLASMA POWER", 0, 0, "PLASMA POWER OFF" );
			nSeqNo = 0;
		}
		else if( !m_pEqpAssy->m_Plasma.GetPowerStatus() )
		{
	//		nAlarmId = ALM_AP_POWER_OFF;
			m_pEqpStateSeq->SetAlarm( nAlarmId);
			SetLog( "PLASMA POWER", 0, 0, "PLASMA POWER OFF Alarm Set" );
			nReturnSeqNo = nSeqNo;
			nSeqNo = 1000;		
		}
		break;

	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() )	
		{
			m_pEqpStateSeq->ResetAlarm( nAlarmId );
			SetLog( "PLASMA POWER", 0, 0, "Alarm Recovery" );
			nSeqNo = nReturnSeqNo;
		}
		break;	
	}
	m_nSeqPlasmaNo = nSeqNo;
}

void CPlasmaSequence::SeqPlasmaAlarm()
{
#ifndef _MY_DEBUG
	if( !m_pDevice->IsActive() ) return;
#endif
	if( !m_SeqFlag.bInitComp || !m_pDicsData->m_GenInfo.bAuto ) return;
	static int nAlarmId = 0;
	static DWORD dwStartTick = 0;

	int nAlarmCode	= 0;
	int nSeqNo = m_nSeqPlasmaAlarmNo;

	switch ( nSeqNo )
	{
	case 0:
		if( ( nAlarmCode = GetPlasmaAlarm() ) == plasmaNoAlm )
		{
			;	// No Operation
		}
		else if( IsPlasmaUse() )
		{
			dwStartTick = GetTickCount();
			nSeqNo = 10;
		}
		break;
	case 10:
		if( !IsPlasmaUse() ||
			( nAlarmCode = GetPlasmaAlarm() ) == plasmaNoAlm )
		{
			nSeqNo = 0;
		}
		else if( ( GetTickCount() - dwStartTick ) > 1500 )
		{	
			
//			nAlarmId = ALM_AP_STATUS0 + nAlarmCode;
			m_pEqpStateSeq->SetAlarm( nAlarmId);
				
			SetLog( "ALARM   ", 0, 0, "Alarm Set" );	
			dwStartTick = GetTickCount();
			nSeqNo = 1000;			
		}
		break;
	case 1000:
		if( m_pEqpStateSeq->IsPushedAlarmReset() || 
			( ( GetTickCount() - dwStartTick ) > 1000 && GetPlasmaAlarm() == plasmaNoAlm ) )
		{
			
			m_pEqpStateSeq->ResetAlarm( nAlarmId );
			nAlarmId = 0;
			SetLog( "ALARM", 0, 0, "Alarm Recovery" );
			nSeqNo = 0;
		}
		break;
	}
    
	m_nSeqPlasmaAlarmNo = nSeqNo;
}

void CPlasmaSequence::SeqPlasamProgress( BOOL bPlasmaPorgress )
{
	static DWORD dwStartTick = 0;
	static int nOldControl = -1;
	static int nOldTime = 0;
	int nTime;

	DWORD dwPlasmaOnTime = (DWORD) ( ( m_pSetupItems->m_fValueCvLength[( IF_UNIT_SWR1 - IF_UNIT_LD ) * 2 + 0] + GLASS_SIZE ) 
		                            / ( m_pDicsData->m_JobCond.nCvSpeed / 60 ) * 10 );

	int nSeqNo = m_nSeqPlasamProgressNo;
	
	switch ( nSeqNo )
	{
	case 0:
		if( bPlasmaPorgress )
		{	// Progress starts
			Msg( WM_AP_ON_PROGRESS, PROGRESS_START, dwPlasmaOnTime/10, FALSE );			

			TIMER_ON( TM_AP_PROGRESS, dwPlasmaOnTime );
			nOldTime = 0;
			dwStartTick = GetTickCount();
			nSeqNo = 10;
		}
		break;
	case 10 :
		if( TIMER( TM_AP_PROGRESS ) )
		{	
			nSeqNo = 20;
		}
		nTime = (int)( m_pTimer->m_pTimer[TM_AP_PROGRESS].dwNow / 100 );
		if( nTime - nOldTime > 2 )
		{
			nOldTime = nTime;
			Msg( WM_AP_ON_PROGRESS, PROGRESS_SET, (LPARAM)nTime, FALSE );
		}
		break;
	case 20:
		if( !bPlasmaPorgress )
		{	// Progress OFF
			Msg( WM_AP_ON_PROGRESS, PROGRESS_END, 0, FALSE );			
			nOldTime = 0;
			nSeqNo= 0;
		}
		break;

	case 30:
		if( bPlasmaPorgress )
		{
			m_pTimer->RePlayTimer( TM_AP_PROGRESS );
			nSeqNo = 10;
		}
		break;
	}
	m_nSeqPlasamProgressNo = nSeqNo;
}

void CPlasmaSequence::SetGasFlow(int nGaugeNo, float fFlowRate)
{
	m_pEqpAssy->m_Plasma.SetPlasmaValue( nGaugeNo, (WORD)fFlowRate );
}

void CPlasmaSequence::SetPlasmaPowerSupply(int nPower)
{
	m_pEqpAssy->m_Plasma.SetPlasmaValue( supKv, nPower );
}

BOOL CPlasmaSequence::GetHouseCloseState()
{
#ifndef _MY_DEBUG
	return m_pEqpAssy->m_Plasma.GetHouseCloseState();
#else
	return TRUE;
#endif
}

int CPlasmaSequence::GetPlasmaAlarm()
{
	int nRv = 0;
	for( int i = 0; i<3; i++ )
	{
		if( m_pEqpAssy->m_Plasma.GetPlasmaStatus( i ) )
		{	
			nRv |= (0x01<<i); 
		}
	}
	return nRv;
}

BOOL CPlasmaSequence::IsPlasmaUse()
{

	BOOL bUse = TRUE;

	bUse &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupJOB_MODE];
	bUse &= m_pSetupItems->m_nValueSetup_Option[CSetupItems::setupPLASMA_USED];

	return bUse;
}

BOOL CPlasmaSequence::IsGlassExist()
{
	BOOL bGlassExist = FALSE;
#ifndef _MY_DEBUG
	//bGlassExist |= m_pEqpAssy->m_GlassExistUnit.GetOr( ( IF_UNIT_LD*2 + 0 )  ); 
	bGlassExist |= m_pEqpAssy->m_GlassExistUnit.GetOr( IF_UNIT_LD*2 + 1 );
#else
	bGlassExist |= (( CDicsView* ) m_pOwnerWnd )->m_pDicsData->m_bDebugOutSensor[IF_UNIT_LD];
//	bGlassExist |= (( CDicsView* ) m_pOwnerWnd )->m_pDicsData->m_bDebugOutSensor[IF_UNIT_AP];
#endif
	return bGlassExist;
}

void CPlasmaSequence::SetDebugState()
{
	m_pDevice->SetDiState( m_nPlasmaAlarmAddress[0], ON );
	m_pDevice->SetDiState( m_nPlasmaAlarmAddress[1], ON );
	m_pDevice->SetDiState( m_nPlasmaAlarmAddress[2], ON );
	m_pDevice->SetDiState( m_nPlasmaAlarmAddress[3], ON );
}

//---------------------------------------------------------------------------
// Set the Log
//---------------------------------------------------------------------------
void CPlasmaSequence::SetLog( CString sSeqName, int nPortNo, int nSlotNo, CString sMessage )
{
	// Log
	CString sLog("");
	CString sPortNo;
	if( nPortNo <= 0 ) sPortNo = "";
	else sPortNo.Format( "%d", nPortNo );

	CString sSlotNo;
	if( nSlotNo <= 0 ) sSlotNo = "";
	else sSlotNo.Format( "%d", nSlotNo );

	sLog.Format( "CIM\t%s\t%s\t%s\t%s", sSeqName, sPortNo, sSlotNo, sMessage );
	Log.TextOut( sLog );

	sLog.Format( "%s : %s", sSeqName, sMessage );
	Msg( WM_HOST_LOG, 0, (LPARAM)(LPCTSTR) sLog );
}