// DatalogView.cpp : implementation file
//

#include "stdafx.h"
#include "..\dics.h"
#include "DatalogView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDatalogView

IMPLEMENT_DYNCREATE(CDatalogView, CFormView)

CDatalogView::CDatalogView()
	: CFormView(CDatalogView::IDD)
{
	//{{AFX_DATA_INIT(CDatalogView)
	m_sTextSearch = _T("");
	//}}AFX_DATA_INIT

	m_pTabDatalogTab1View	= NULL;
	m_pTabDatalogTab2View	= NULL;
}

CDatalogView::~CDatalogView()
{
	GetDocument()->m_pDatalogWnd = NULL;
}

void CDatalogView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDatalogView)
	DDX_Control(pDX, IDC_TAB_DATALOG, m_tabCtrl);
	DDX_Text(pDX, IDC_TEXT_SEARCH, m_sTextSearch);
	DDX_Control( pDX, IDC_SEARCH, m_macSearch );
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDatalogView, CFormView)
	//{{AFX_MSG_MAP(CDatalogView)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_SEARCH, OnSearch)
	//}}AFX_MSG_MAP
	ON_MESSAGE( WM_SEL_CHANGE_TAB, OnSelChangeTab )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDatalogView diagnostics

#ifdef _DEBUG
void CDatalogView::AssertValid() const
{
	CFormView::AssertValid();
}

void CDatalogView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CDicsDoc* CDatalogView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDicsDoc)));
	return (CDicsDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDatalogView message handlers

void CDatalogView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();

	// Set the current view to doc.
	GetDocument()->SetDatalogView( this );

	// Set the MAC button
	MAC_COLOR_SET color_set;
	m_macSearch.GetColorSetting( &color_set );
	color_set.crFace = LIGHT_GREEN_COLOR;
	color_set.crLiteShadow = MAGENTA_COLOR;
	m_macSearch.SetColorSetting( color_set );
	m_macSearch.SetBold();

	// Message Box
	CRect r;
	GetDlgItem( IDC_TITLE )->GetWindowRect( &r );
	ScreenToClient( &r );
	r -= CRect( 1, 1, 1, 1 );
	m_msgTitle.Create( 200, "0", WS_CHILD|WS_VISIBLE, r, this, IDC_TITLE,
                		  LIGHT_YELLOW_COLOR, BLUE_COLOR, 16, TRUE );
	m_msgTitle.SetOpMsg( "File Name" );

	GetDlgItem( IDC_FILE_NAME )->GetWindowRect( &r );
	ScreenToClient( &r );
	r -= CRect( 1, 1, 1, 1 );
	m_msgFileName.Create( 201, "0", WS_CHILD|WS_VISIBLE, r, this, IDC_FILE_NAME,
                	       WHITE_COLOR, BLUE_COLOR , 16, TRUE );
	m_msgFileName.SetOpMsg( "No file name" );
	
	// Create the TAB view 
	GetDlgItem( IDC_TAB_DATALOG )->GetWindowRect( &r );
	ScreenToClient( &r );
	r -= CRect( 0, 0, 6, 30 );

	CCreateContext context;
	m_pTabDatalogTab2View = new CDatalogTab2View( this );
	m_pTabDatalogTab2View->Create(NULL, NULL, WS_CHILD|WS_VISIBLE, r, &m_tabCtrl, 0, &context);
	m_tabCtrl.AddTab( (CWnd*) m_pTabDatalogTab2View, "    EDIT LOG VIEW    ", 0 );
	m_pTabDatalogTab2View->SetFocus();

	m_pTabDatalogTab1View = new CDatalogTab1View( this );
	m_pTabDatalogTab1View->Create(NULL, NULL, WS_CHILD|WS_VISIBLE, r, &m_tabCtrl, CDatalogTab1View::IDD, &context);
	m_tabCtrl.AddTab( (CWnd*) m_pTabDatalogTab1View, "    LIST LOG VIEW    ", 1 );

	m_tabCtrl.SetSelectedColor( RED_COLOR );
	m_tabCtrl.SetMouseOverColor( GREEN_COLOR );
	m_tabCtrl.SetNormalColor( DARK_GRAY_COLOR );
}

void CDatalogView::OnDestroy() 
{
	CFormView::OnDestroy();
	
	if( m_pTabDatalogTab1View ) 
	{
		m_pTabDatalogTab1View->SendMessage( WM_CLOSE );
		delete m_pTabDatalogTab1View;
	}

	// There is something that kill this view pointer, but I don't know what it is.
//	if( m_pTabDatalogTab2View ) 
//	{
//		m_pTabDatalogTab2View->SendMessage( WM_DESTROY );
//		delete m_pTabDatalogTab2View;
//	}
}

//--------------------------------------------------------------------
// OnOpenFile
//--------------------------------------------------------------------
void CDatalogView::OnOpenFile()
{
	int nTab = m_tabCtrl.GetCurSelTab();

	if( nTab == 1 ) m_pTabDatalogTab1View->OnOpenFile();
	else if( nTab == 0 ) m_pTabDatalogTab2View->OnOpenFile();
}

//--------------------------------------------------------------------
// OnSearch
//--------------------------------------------------------------------
void CDatalogView::OnSearch() 
{
	int nTab = m_tabCtrl.GetCurSelTab();

	if( nTab == 1 ) m_pTabDatalogTab1View->OnSearch();
	else if( nTab == 0 ) m_pTabDatalogTab2View->OnSearch();
}

//--------------------------------------------------------------------
// OnSelChangeTab
//--------------------------------------------------------------------
LRESULT CDatalogView::OnSelChangeTab( WPARAM wParam, LPARAM lParam )
{
	int nTab = m_tabCtrl.GetCurSelTab();

	if( nTab == 1 )
	{
		m_msgFileName.SetOpMsg( m_pTabDatalogTab1View->m_sPathName );
		m_pTabDatalogTab1View->SetFocus();
	}
	else if( nTab == 0 )
	{
		m_msgFileName.SetOpMsg( m_pTabDatalogTab2View->m_sPathName );
		m_pTabDatalogTab2View->SetFocus();
	}
	return 0;
}
