// XCCLink.cpp: implementation of the XCCLink class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "XCCLink.h"
#include "..\CCLink\mdfunc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
ADDR_INFO_CCLINK XCCLink::m_AddrInfo[] = 
{
	{ devX, CH1_X_START_ADD, 16 },
	{ devY, CH1_Y_START_ADD, 8 },
};

XCCLink::XCCLink()
{
	m_lPath = 0;
	m_nAlarmCode = 0;
	m_nBufSize = 0;
	m_nReadData = NULL;
	m_nState = linkNone;
	m_bIsOpen = FALSE;
}

XCCLink::~XCCLink()
{

}

BOOL XCCLink::Open(int nChannel, int nMode)
{
	int nRv = 0;
	int nLen = sizeof( m_AddrInfo ) / sizeof( ADDR_INFO_CCLINK );
	for( int i=0; i<nLen; i++ ) 
	{
		if( ( m_AddrInfo[i].nDevType == devX ) || ( m_AddrInfo[i].nDevType == devY) )
		{
			m_nBufSize += ( m_AddrInfo[i].nSize - 1 ) / 16 + 1;
		}
	}
	m_nReadData = new short[m_nBufSize];
	
	ReadErrorCode();
	
	//	ret = mdOpen(chan,mode,path);
	//	short	ret;	return value	OUT
	//	short	chan;	path of channel	IN
	//	short	mode;	dummy(select -1)	IN
	//	long	*path;	opened loop path pointer	OUT
	if( ( nRv = mdOpen( nChannel, nMode, &m_lPath ) ) != 0 )
	{
		CString sError = "";
		CString sMsg = "";
		if( !m_mapError.IsEmpty() ) m_mapError.Lookup( nRv, sMsg );

		sError.Format( "CC-LINK : mdOpen error [%d:%s]", nRv, sMsg );
		AfxMessageBox( sError );

		m_bIsOpen = FALSE;
		return FALSE;
	}
	m_bIsOpen = TRUE;
	return TRUE;
}

BOOL XCCLink::Close()
{
	int nRv = 0;

	// Release the memory
	delete []m_nReadData;

	//	ret = mdClose(path);
	//	short	ret;	return value	OUT
	//	long	path;	opened loop pointer	IN
	if( ( nRv = mdClose( m_lPath ) ) != 0 )
	{
		CString sError = "";
		CString sMsg = "";
		if( !m_mapError.IsEmpty() ) m_mapError.Lookup( nRv, sMsg );
		sError.Format( "CC-LINK : mdClose error [%d:%s]", nRv, sMsg );
		AfxMessageBox( sError );

		return FALSE;
	}
	return TRUE;
}

void XCCLink::ReadErrorCode()
{
	CStdioFile file;
	CString sbuffer = "";
	int nErrCode = 0;
	CString sErrDesc = "";
	
	if( file.Open( _T(".\\CCLink\\CCLinkErrorCode.txt"), CFile::modeRead ) == 0 ) return;
	if( file.ReadString( sbuffer ) == 0 )
	{
		file.Close();
		return;
	}

	try { file.SeekToBegin(); }
	catch( CFileException* pEX ) { pEX->Delete(); }

	while( file.ReadString( sbuffer ) )
	{
		int nFirst	= 0;
		int nNext	= 0;
		CString sbuf = sbuffer;
		nNext = sbuf.Find( '\t', nFirst );
		int nCount = nNext - nFirst;
		if( nNext > 0 )  nErrCode = atoi( sbuf.Mid( nFirst, nCount ) );
		sErrDesc = sbuf.Mid( nCount + 1 );
		m_mapError.SetAt( nErrCode, sErrDesc );
	}

	file.Close();
}

BOOL XCCLink::Monitor( short nNetNo, short nStNo )
{
	int nRv;
	short nStationNo = nStNo + ( nNetNo << 8 );

	int nLen = sizeof( m_AddrInfo ) / sizeof( ADDR_INFO_CCLINK );
	short* nDev = new short[3 * nLen + 1];
	nDev[0] = nLen;
	for( int i=0; i<nLen; i++ ) 
	{
		nDev[3 * i + 1] = m_AddrInfo[i].nDevType;
		nDev[3 * i + 2] = m_AddrInfo[i].nStartAddr;
		nDev[3 * i + 3] = m_AddrInfo[i].nSize;
	}
	
	//	ret = mdRandR(path,stno,dev,buf,bufsize);
	//	short	ret;	return value	OUT
	//	long	path;	path of channel	IN
	//	short	stno;	station number	IN
	//	short	dev[ ];	random selected device	IN
	//	short	buf[ ];	reading data(single precision)	OUT
	//	short	bufsize;	bit count of reading data store area	IN
	nRv = mdRandR( m_lPath, nStationNo, nDev, m_nReadData, 2 * m_nBufSize );
	delete nDev;

	if( nRv != 0 )
	{
		CString sError;
#ifdef _MEL_ALARM
		m_nAlarmCode = nRv;
		sError.Format( "CC-LINK : mdRandR error [%d]", nRv );
		AfxMessageBox( sError );
#endif
		return FALSE;
	}
	return TRUE;
}

BOOL XCCLink::SendBit( short nNetNo, short nStNo, short nDevNo, short nData )
{
	int nRv;

	int nStationNo = nStNo + ( nNetNo << 8 );
	if( nData )
	{
		//		ret = mdDevSet(path,stno,devtyp,devno);
		//		short	ret;	return value	OUT
		//		long	path;	path of channel	IN
		//		short	stno;	station number	IN
		//		short	devtyp;	device type	IN
		//		short	devno;	selected device number	IN
		if( ( nRv = mdDevSet( m_lPath, nStationNo, devY, nDevNo ) ) != 0 )
		{
			CString sError;
			if( !m_mapError.IsEmpty() && m_bIsOpen )
			{
				CString sMsg = "";
				m_mapError.Lookup( nRv, sMsg );
				sError.Format( "CC-LINK : mdDevSet error [%d:%s]", nRv, sMsg );
				//sError.Format( "CCLINK\t%d\t%s\t%s\t%s", nRv, "0", "0", sMsg );
				Log.TextOut( sError );
//				AfxMessageBox( sError );
			}
#ifdef _MEL_ALARM
			m_nAlarmCode = nRv;
			sError.Format( "CC-LINK : mdDevSet error [%d]", nRv );
			AfxMessageBox( sError );
#endif
			return FALSE;
		}
	}
	else
	{
		//		ret = mdDevRst(path,stno,devtyp,devno);
		//		short	ret;	return value	OUT
		//		long	path;	path of channel	IN
		//		short	stno;	station number	IN
		//		short	devtyp;	device type	IN
		//		short	devno;	selected device number	IN
		if( ( nRv = mdDevRst( m_lPath, nStationNo, devY, nDevNo ) ) != 0 )
		{
			CString sError;
#ifdef _MEL_ALARM
			m_nAlarmCode = nRv;
			sError.Format( "CC-LINK : mdDevRst error [%d]", nRv );
			AfxMessageBox( sError );
#endif
			return FALSE;
		}
	}

	return TRUE;
}

short XCCLink::ReceiveBit( short nNetNo, short nStNo, short nDevNo )
{
#ifdef _SIMULATE_CCLINK
	return TRUE;
#endif

	// Find the address
	int nLen = sizeof( m_AddrInfo ) / sizeof( ADDR_INFO_CCLINK );
	int nIndex = 0;
	for( int i=0; i<nLen; i++ )
	{
		if( ( ( m_AddrInfo[i].nDevType == devX ) || ( m_AddrInfo[i].nDevType == devY ) ) && 
			( nDevNo >= m_AddrInfo[i].nStartAddr && nDevNo < m_AddrInfo[i].nStartAddr + m_AddrInfo[i].nSize ) )
		{
			// Send the bit data
			return ( ( m_nReadData[nIndex] >> ( nDevNo - m_AddrInfo[i].nStartAddr ) ) & 0x0001 );
		}

		if( ( m_AddrInfo[i].nDevType == devX ) || ( m_AddrInfo[i].nDevType == devY ) )
		{
			nIndex += ( m_AddrInfo[i].nSize - 1 ) / 16 + 1;
		}
	}

	CString sError;
	sError.Format( "ReceiveBit Address %d is not registered", nDevNo );
	AfxMessageBox( sError );
	return 0;
}

BOOL XCCLink::RandR( short nNetNo, short nStNo, short* nDev, short* nData, short nSize )
{
	int nRv;
	short nStationNo = nStNo + ( nNetNo << 8 );
	
//	ret = mdRandR(path,stno,dev,buf,bufsize);
//	short	ret;	return value	OUT
//	long	path;	path of channel	IN
//	short	stno;	station number	IN
//	short	dev[ ];	random selected device	IN
//	short	buf[ ];	reading data(single precision)	OUT
//	short	bufsize;	bit count of reading data store area	IN
	nRv = mdRandR(m_lPath, nStationNo, nDev, nData, nSize );

 	if( nRv != 0 )
	{
		CString sError;
#ifdef _MEL_ALARM
		m_nAlarmCode = nRv;
		sError.Format( "CC-LINK : mdRandR error [%d]", nRv );
		AfxMessageBox( sError );
#endif
		return FALSE;
	}
	return TRUE;	
}

BOOL XCCLink::RandR32( short nNetNo, short nStNo, short nDevNo, short* nData, short nType )
{
	int nRv;
	short nStationNo = nStNo + ( nNetNo << 8 );
	short nDev[7];
	nDev[0] = 2;
	nDev[1] = nType;
	nDev[2] = nDevNo;
	nDev[3] = 16;
	nDev[4] = nType;
	nDev[5] = nDevNo + 0x10;
	nDev[6] = 16;
	
	// nData : array size = 32
	int nBufSize = ( ( nType == devX ) || ( nType == devY) ) ? 4 : 64;
//	ret = mdRandR(path,stno,dev,buf,bufsize);
//	short	ret;	return value	OUT
//	long	path;	path of channel	IN
//	short	stno;	station number	IN
//	short	dev[ ];	random selected device	IN
//	short	buf[ ];	reading data(single precision)	OUT
//	short	bufsize;	bit count of reading data store area	IN
	nRv = mdRandR(m_lPath, nStationNo, nDev, nData, nBufSize );

	if( nRv != 0 )
	{
		CString sError;
#ifdef _MEL_ALARM
		m_nAlarmCode = nRv;
		sError.Format( "CC-LINK : mdRandR error [%d]", nRv );
		AfxMessageBox( sError );
#endif
		return FALSE;
	}
	return TRUE;	
}

BOOL XCCLink::RandW( short nNetNo, short nStNo, short* nDev, short* nData, short nSize )
{
	int nRv;
	short nStationNo = nStNo + ( nNetNo << 8 );
	
//	ret = mdRandW(path,stno,dev,buf,bufsize);
//	short	ret;	return value	OUT
//	long	path;	path of channel	IN
//	short	stno;	station number	IN
//	short	dev[ ];	random selected device	IN
//	short	buf[ ];	written data(single precision)	OUT
//	short	bufsize;	dummy	IN
	nRv = mdRandW( m_lPath, nStationNo, nDev, nData, nSize );

	if( nRv != 0 )
	{
		CString sError;
#ifdef _MEL_ALARM
		m_nAlarmCode = nRv;
		sError.Format( "CC-LINK : mdRandW error [%d]", nRv );
		AfxMessageBox( sError );
#endif
		return FALSE;
	}
	return TRUE;
}

BOOL XCCLink::Send( short nNetNo, short nStNo, short nDevNo, short nData, short nSize )
{
	int nRv;
	//short nSize = 1;	// 0x0000~0x0008

	int nStationNo = nStNo + ( nNetNo << 8 );

	if( ( nRv = mdSend( m_lPath, nStationNo, devY, nDevNo, &nSize, &nData ) ) != 0 )
	{
		CString sError;
#ifdef _MEL_ALARM
		m_nAlarmCode = nRv;
		sError.Format( "mdSend error [%d]", nRv );
		AfxMessageBox( sError );
#endif
		return FALSE;
	}
	return TRUE;
}

BOOL XCCLink::Receive( short nNetNo, short nStNo, short nDevNo, short& nData, short nType, short nSize )
{
	int nRv;
	//short nSize = 1;	// devX, devY

	int nStationNo = nStNo + ( nNetNo << 8 );

	if( ( nRv = mdReceive(m_lPath, nStationNo, nType, nDevNo, &nSize, &nData) ) != 0 )
	{
		CString sError;

#ifdef _MEL_ALARM
		m_nAlarmCode = nRv;
		sError.Format( "mdReceive error [%d]", nRv );
		AfxMessageBox( sError );
#endif
		return FALSE;
	}
	return TRUE;
}
