// XChannel.cpp: implementation of the XChannel class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "XChannel.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

BOOL XChannel::m_bSimulation = FALSE;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
XChannel::XChannel()
{
	m_pDevice				= NULL;
	m_bInitialized			= FALSE;
	m_bDeviceLink			= FALSE;
	m_bIsMmcInput			= FALSE;
	m_arConnectedObjectNames.RemoveAll();
}

XChannel::~XChannel()
{

}

void XChannel::Create(XCtlDevice *pDevice, int nAdd, int nNode, int nNo, CString sName, CString sCpoint)
{
	if( !m_bInitialized )
	{
		m_pDevice	= pDevice;
		m_nAddress	= nAdd;
		m_nNodeNo	= nNode;
		m_nNo		= nNo;
		m_sName		= sName;
		m_sCpoint = sCpoint;
	
		m_bInitialized = TRUE;
	}
	else 
	{
		CString sMsg;
		sMsg.Format( "%d, %s is already initialized by %d, %s", nAdd, sName, m_nAddress, m_sName );
		AfxMessageBox( sMsg );
	}
}

void XChannel::Create()
{
	m_sName = "NULL";
	m_nAddress = -1;
	m_nNodeNo = -1;
	m_bInitialized = FALSE;
}

BOOL XChannel::Delete(int nAdd)
{
	if( !m_bInitialized )
	{
		CString sMsg;
		sMsg.Format( "It can't be deleted. Because %d was not occupied.", nAdd );
		AfxMessageBox( sMsg );
		return FALSE;
	}
	else
		m_bInitialized = FALSE;

	return TRUE;
}

CString XChannel::GetName()
{
	if( !m_bInitialized ) 
	{
		AfxMessageBox( "Uninitialized XChannel" );
		return "NULL";
	}
	return m_sName;
}

CString XChannel::GetConnectPoint()
{
	if( !m_bInitialized ) 
	{
		AfxMessageBox( "Uninitialized XChannel" );
		return "NULL";
	}
	return m_sCpoint;
}


int XChannel::GetAddress()
{
	if( !m_bInitialized ) AfxMessageBox( "Uninitialized Channel" );
	return m_nAddress;
}

void XChannel::SetName(CString sName)
{
	m_sName = sName;
}

void XChannel::SetAddress(int nNo)
{
	m_nAddress = nNo;
}

BOOL XChannel::IsActive()
{
	if( !m_bInitialized ) AfxMessageBox( "Uninitialized Channel" );
	if( m_pDevice->IsActive() == TRUE || m_bSimulation )
		return TRUE;
	else 
		return FALSE;
}

void XChannel::SetDevice(XCtlDevice *pDevice)
{
	m_pDevice = pDevice;
}

void XChannel::SetNodeNo(int nNode)
{
	m_nNodeNo = nNode;
}

int XChannel::GetCurrConnectedObjectNo()
{
	return m_arConnectedObjectNames.GetSize();
}

int XChannel::AddConnectingObjectName(CString sName)
{
	m_arConnectedObjectNames.Add( sName );
	return m_arConnectedObjectNames.GetSize();
}

BOOL XChannel::RemoveConnectedObjectName(CString sName)
{
	int i;
	for( i = 0 ; i < m_arConnectedObjectNames.GetSize() ; i ++ )
	{
		if( sName == m_arConnectedObjectNames.GetAt( i ) )
			m_arConnectedObjectNames.RemoveAt( i );
	}
	if( i == m_arConnectedObjectNames.GetSize() )
		return FALSE;
	return TRUE;
}


int XChannel::GetNodeNo()
{
	return m_nNodeNo;
}

int XChannel::GetNo()
{
	return m_nNo;
}

CString XChannel::GetWiringNo(CString sPrefix)
{
	CString sBuf;
	sBuf.Format( "%o%s%03o", m_nNodeNo, sPrefix, m_nNo );
	return sBuf;
}

void XChannel::SetSimulationMode(BOOL bOn)
{
	m_bSimulation = bOn;
}

BOOL XChannel::IsInitialized()
{
	return m_bInitialized;
}

void XChannel::SetDeviceLink(BOOL bLink)
{
	m_bDeviceLink = bLink;
}

BOOL XChannel::IsDeviceLink()
{
	return m_bDeviceLink;
}

void XChannel::SetMmcLink( XMmc* pMmc, int nAxisNo, DWORD dEventSource )
{
	m_pMmc = pMmc;
	m_nAxisNo = nAxisNo;
	m_dMmcEventSource = dEventSource;

	m_bIsMmcInput = TRUE;
	SetDeviceLink( TRUE );
	m_bInitialized = TRUE;
}

BOOL XChannel::IsMmcLink()
{
	return m_bIsMmcInput;
}