//---------------------------------------------------------------------------
// Copyright		: DMS Co., LTd
// File Name		: X\XCommThread.cpp
// Modified Date	: 2000.11.28, 
// Version			: 1.0
// Programmer		: kds
// XCommThread.cpp	: implementation of the XComm class.
// Description		:  ϴ XComm class . 
//					  ( Open Port Baud Rate, Stop Bit, ...  Ҽ ְѴ.)
//
// Usage :
//	1. XComm m_pComm[MAX_PORT]
//	2. m_pComm[PORT1].OpenPort( this, "COM1", CBR_9600, 8, ONESTOPBIT, NOPARITY, '\r' );
//		...
//	3. m_pComm[PORT1].ClosePort();
//---------------------------------------------------------------------------

// Include files
#include "stdafx.h"
#include "XCommThread.h"

// define 
#define MAX_BYTE	150	// receiver buffer MAX size

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// XCommThread

IMPLEMENT_DYNCREATE(XCommThread, CWinThread)		

// Constructor
XCommThread::XCommThread()
{
	// CWinThread::m_bAutoDelete : FALSE => not be automatically deleted at thread termination
	m_bAutoDelete	= FALSE;
	m_bConnected	= FALSE ;
	m_pOwner		= NULL;

	m_hDead = CreateEvent( NULL, TRUE, FALSE, NULL );	// MANUAL Reset, Initial state = non-signal
}

// Destructor
XCommThread::~XCommThread()
{
	CloseHandle( m_hDead );
}

BOOL XCommThread::InitInstance()
{
		WatchComm();
	return FALSE;
}

//Your implementation of ExitInstance should call the base classs version 
//after your code is executed.
int XCommThread::ExitInstance()
{
	SetEvent( m_hDead );
	int tmp = CWinThread::ExitInstance();
	return 0;
}

BEGIN_MESSAGE_MAP(XCommThread, CWinThread)
	//{{AFX_MSG_MAP(XCommThread)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// XCommThread Class Member functions

//---------------------------------------------------------------------------
// open the COMM port
//---------------------------------------------------------------------------
BOOL XCommThread::OpenPort(CWnd* pOwner,	// parent window pointer
				 LPCTSTR lpszPortName,		// "Emulate", "COM1", "COM2", ...
				 DWORD dwBaud,				// CBR_9600, CBR_4800, ...
				 BYTE byByte,				// 8, 7, ...
				 BYTE byStop,				// ONESTOPBIT, ONE5STOPBITS, TWOSTOPBITS
				 BYTE byParity,				// EVENPARITY, MARKPARITY, NOPARITY, ODDPARITY, SPACEPARITY
				 char cEvtChar)				// signal an event, (= 0 : EV_RXCHAR)
{
	if( !XComm::Open( lpszPortName, dwBaud, byByte, byStop, byParity, cEvtChar ) ) return FALSE;	// Error() : XComm Error

	m_pOwner		= pOwner;
	m_bConnected	= TRUE;

 // jemoon : 070205 add
	FlushTx();
	FlushRx();

	CreateThread();

	return TRUE;
}

//---------------------------------------------------------------------------
// close the COMM port
//---------------------------------------------------------------------------
void XCommThread::ClosePort()
{
	m_bConnected = FALSE;
	XComm::Close();	//ipj061121 Ʒ ̵.. WatchComm η  thread ȳ  ..

	if( m_hThread )
	{
		ResumeThread();		// ڴ Thread  (϶)

		// When ExitInstance function calls, it is signaled.
		WaitForSingleObject( m_hDead, INFINITE );
		WaitForSingleObject( m_hThread, INFINITE );
		Sleep(200);
	}

	DWORD dwExitCode;
	if( !GetExitCodeThread( m_hThread, &dwExitCode ) )
	{
		if( GetLastError() == STILL_ACTIVE )
		{
			TerminateThread( m_hThread, -1 );
			Sleep( 200 );
			if( !GetExitCodeThread( m_hThread, &dwExitCode ) )
				AfxMessageBox( "Windows ϰ ý  OFFϿֽʽÿ.", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
		}
	}

	//XComm::Close();
}

//---------------------------------------------------------------------------
// Polling function
// m_bConnected = False Ǳ ѹݺ
//---------------------------------------------------------------------------
BOOL XCommThread::WatchComm()
{
	DWORD		dwEvtMask;
	OVERLAPPED	os;

	memset( &os, 0, sizeof( OVERLAPPED ) ) ;
	os.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
	if( os.hEvent == NULL )
	{
		AfxMessageBox( "Can't create the event of WatchCommProc !", MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL );
		return FALSE;
	}

	switch( m_nEvtType )
	{
		case 0 :
			if( !SetCommMask( m_hComm, EV_RXCHAR ) ) return FALSE;
			break;
		case 1 :
			if( !SetCommMask( m_hComm, EV_RXFLAG ) ) return FALSE;
			break;
	}

	while( m_bConnected )
	{
		//TRACE0( "In WatchComm Thread--------------------\n" );
		dwEvtMask = 0 ;
		WaitCommEvent( m_hComm, &dwEvtMask, NULL );

		switch( m_nEvtType )
		{
			case 0 :
				if( (dwEvtMask & EV_RXCHAR) == EV_RXCHAR ) DataReceived();
				break;
			case 1 :
				if( (dwEvtMask & EV_RXFLAG) == EV_RXFLAG ) DataReceived();
				break;
		}
	}
	CloseHandle( os.hEvent );
	return TRUE;
}

//---------------------------------------------------------------------------
// Read the received data from COMM buffer
//---------------------------------------------------------------------------
void XCommThread::DataReceived()
{
	Read( m_sReceivedData.GetBuffer( MAX_BYTE ), MAX_BYTE );
	m_sReceivedData.ReleaseBuffer();
}

CString& XCommThread::GetReceivedData()
{
	return m_sReceivedData;
}

//---------------------------------------------------------------------------
// Set the parent window pointer
void XCommThread::SetOwner(CWnd *pOwner)
{
	m_pOwner = pOwner;
}

//---------------------------------------------------------------------------
// Send the message to parent window
// immediately return : PostMessage()
BOOL XCommThread::Msg(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if( m_pOwner )
	{
		m_pOwner->SendMessage( uMsg, wParam, lParam );
		return TRUE;
	}
	else
		return FALSE;
}
