//------------------------------------------------------------------------------------------------
// Copyright		: DMS Co., Ltd
// File Name		: XCtlDeviceNet.cpp
// Modified Date	: 2001.01.27, 2001.01.30, 2003.12.12
// Version			: Ver1.1
// Programmer		: Kim Dongsoo
// Description		: implementation of the XCtlDeviceNet class.
//		1. DIO : XDio   Ѵ.
//		2. AIO : 
// Modify			: 2003.12.16 Hong Jae hee
//		Ver1.1 :	ReadDataԼ ߰ -> Read Buffer Data  оδ. 
//					DioRead AioReadԼ ReadData о 
//					ش  Ѱִ   ϵ 
//------------------------------------------------------------------------------------------------

#include "stdafx.h"
#include "..\Dics.h"
#include "XCtlDeviceNet.h"
#include <MATH.H>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// XCtlDeviceNet

IMPLEMENT_DYNCREATE(XCtlDeviceNet, CWinThread)

//------------------------------------------------------
// Procedure   : XCtlDeviceNet
// Description : Construction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
XCtlDeviceNet::XCtlDeviceNet()
{
	m_bAutoDelete	= FALSE;
	m_pOwner		= NULL;
	m_pInBuf		= NULL;
	m_pOutBuf		= NULL;
	m_aiInfo		= NULL;
	m_aiInfoDefault	= NULL;

	m_nAiNo			= 0;

	m_hDead = CreateEvent( NULL, TRUE, FALSE, NULL );	// MANUAL Reset, Initial state = non-signal
	m_hKill = CreateEvent( NULL, TRUE, FALSE, NULL );
}

AI_PRESET XCtlDeviceNet::m_aiPreSet[enMAX_SENSOR_TYPE] =
{
	{ enPG35		, "PG35",				0.,		1000.,	0xCCC,	0x3000 },
	{ enDW_605		, "Dwyer Manometer",	0.,		750.,	6553,	32767 },
	{ enSENSYS		, "SENSYS DPLH0750A",	0.,		750.,	6553,	32767 },
	{ enJLK_25		, "JLK 25A",			0.,		130.,	6553,	32767 },	
	{ enULK_25		, "ULK 25A",			0.,		130.,	6553,	32767 },	
	{ enCKD_FLOW	, "CKD FLUEREX",		0.,		4000.,	6553,	32767 },	
	{ enPT100OHM	, "PT 100 OHM",			0.,		4000.,	6553,	32767 },	
	{ enGSEUV		, "GS EUV Lamp Intensity",			0.,		385.,	6553,	32767 },	
	{ enLEVEL_GAGE	, "Level SENSYS PSH(0.1kgf/cm2)",	0.,		1000.,	6553,	32767 }	
};

//------------------------------------------------------
// Procedure   : ~XCtlDeviceNet
// Description : Destruction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
XCtlDeviceNet::~XCtlDeviceNet()
{
	if( m_aiInfo ) delete [] m_aiInfo;
	if( m_aiInfoDefault ) delete [] m_aiInfoDefault;
	if( m_hDead ) CloseHandle( m_hDead );
	if( m_hKill ) CloseHandle( m_hKill );

	delete []m_pOldAdc;
	delete []m_pFirstVal;
	delete []m_pFilterGain;
}

//------------------------------------------------------
// Procedure   : InitInstance
// Description : Run the thread
// Arguments   : void
// Returns     : BOOL (if return false, the thread is exited)
//------------------------------------------------------
BOOL XCtlDeviceNet::InitInstance()
{
	while( WaitForSingleObject( m_hKill, 0 ) != WAIT_OBJECT_0 ) 
	{
		Sleep( 1 );
		ReadData();			
		Watch();
	}

	VERIFY( SetEvent( m_hDead ) );
	return FALSE;
}

int XCtlDeviceNet::ExitInstance()
{
	// TODO:  perform any per-thread cleanup here
	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(XCtlDeviceNet, CWinThread)
	//{{AFX_MSG_MAP(XCtlDeviceNet)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// XCtlDeviceNet message handlers


//--------------------------------------------------------------------
// Procedure   : Start
// Description : OpenCard => Online => AddDevice => Start Scan
// Arguments   : see the below
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlDeviceNet::Start(CWnd* pOwner, UINT uMaxNode, // 
						UINT* pPointAI,		// Analog Input point no. (ex : 4)
						UINT* pPointAO,		// Analog Output point no.
						UINT* pPointDI,		// Digital Input point no. (ex : 64)
						UINT* pPointDO,		// Digital Output point no.		
						UINT* pPointPC )	// Pulse Counter  KL1501 no.		
{
	m_pOwner = pOwner;
	SetMaxNode( uMaxNode );

	// Allocate the point no each mac id
	m_pPointAI = new UINT[uMaxNode];
	m_pPointAO = new UINT[uMaxNode];
	m_pPointDI = new UINT[uMaxNode];
	m_pPointDO = new UINT[uMaxNode];
	m_pPointPC = new UINT[uMaxNode];

	// We have to allocate the memory each mac id and buffer size.
	m_pInBuf = new BYTE*[uMaxNode];
	m_pOutBuf = new BYTE*[uMaxNode];

	CDicsApp* pApp = ( CDicsApp* ) AfxGetApp();
//	CString sName = pApp->GetProfileString( "DICS SET", "CARD NAME", "5136-DNP-PCI-1-15" );
//	CString sName = pApp->GetProfileString( "DICS SET TAB1", "VALUE_DeviceNet Card Name", "5136-DNP-PCI-1-15" );//kjd
	CDicsDoc* pDoc = ( (CDicsView*) pOwner )->GetDocument();
	CString sName = pDoc->m_pSetupItems->m_sValueSetup[CSetupItems::setupDEVICENET_CARD_NAME];
	
	char szName[50];
	lstrcpy( szName, sName );

	m_nAiNo = 0;
	if( !OpenCard(szName) ) 
	{
		for( WORD i=0; i < m_uMaxNode; i++ )
		{
			m_pPointAI[i] = pPointAI[i];
			m_pPointAO[i] = pPointAO[i];
			m_pPointDI[i] = pPointDI[i];
			m_pPointDO[i] = pPointDO[i];
			m_pPointPC[i] = pPointPC[i];

			WORD wInputSize, wOutputSize;

			// Assume that AI is 16 bits 
			// + 1 : internal (only input)
			wInputSize		= ( WORD )( ( pPointPC[i] / 2 )*5 + pPointAI[i]*2 + pPointDI[i]/8 + ( pPointDI[i] % 8 ? 1 : 0 ) + 1 );
			wOutputSize		= ( WORD )( ( pPointPC[i] / 2 )*5 + pPointAO[i]*2 + pPointDO[i]/8 + ( pPointDO[i] % 8 ? 1 : 0 ) );

			m_pInBuf[i]		= new BYTE[wInputSize];
			m_pOutBuf[i]	= new BYTE[wOutputSize];
			// 031230 HongJaehee
			m_nAiNo += pPointAI[i];
		}
		// 031230 HongJaehee
		m_aiInfo = new aiInfo[m_nAiNo];
		m_aiInfoDefault = new aiInfo[m_nAiNo];
		m_pOldAdc = new int[m_nAiNo];
		m_pFirstVal = new BOOL[m_nAiNo];
		m_pFilterGain = new float[m_nAiNo];


		for( i=0; i<m_nAiNo; i++ ) 
		{
			m_pFirstVal[i]	= FALSE;

			m_aiInfoDefault[i].fRealMax = 1000.0;
			m_aiInfoDefault[i].fRealMin = 0.0;
			m_aiInfoDefault[i].nAdcMax 	= 0x3333;
			m_aiInfoDefault[i].nAdcMin 	= 0xCCC;

			GetCalPara( i, 
				m_aiInfo[i].fRealMax,
				m_aiInfo[i].fRealMin, 
				m_aiInfo[i].nAdcMax, 
				m_aiInfo[i].nAdcMin );
			GetFilterGain( i, m_pFilterGain[i] );

			GetInterlockPara( i, m_aiInfo[i].bUseInterlock,
				m_aiInfo[i].nUpLimit, m_aiInfo[i].nLoLimit );
			GetWarningPara( i, m_aiInfo[i].bUseWarning,
				m_aiInfo[i].nUpWarning, m_aiInfo[i].nLoWarning );
			m_aiInfo[i].sName = GetAiName( i );
			m_aiInfo[i].sUnitName = GetAiUnitName( i );

			SetAiCalScale(i);
		}
		return FALSE;
	}

	// Online
	if( !Online( TRUE ) ) 
	{
		m_nAiNo = 0;
		for( WORD i=0; i<m_uMaxNode; i++ )
		{
			m_pPointAI[i] = pPointAI[i];
			m_pPointAO[i] = pPointAO[i];
			m_pPointDI[i] = pPointDI[i];
			m_pPointDO[i] = pPointDO[i];
			m_pPointPC[i] = pPointPC[i];

			WORD wInputSize, wOutputSize;

			// Assume that AI is 16 bits 
			// + 1 : internal (only input)
			wInputSize		= ( WORD )( ( pPointPC[i] / 2 )*5 + pPointAI[i]*2 + pPointDI[i]/8 + ( pPointDI[i] % 8 ? 1 : 0 ) + 1 );
			wOutputSize		= ( WORD )( ( pPointPC[i] / 2 )*5 + pPointAO[i]*2 + pPointDO[i]/8 + ( pPointDO[i] % 8 ? 1 : 0 ) );

			m_pInBuf[i]		= new BYTE[wInputSize];
			m_pOutBuf[i]	= new BYTE[wOutputSize];

			// 031230 HongJaehee
			m_nAiNo += pPointAI[i];
		}
		// 031230 HongJaehee
		m_aiInfo = new aiInfo[m_nAiNo];
		m_aiInfoDefault = new aiInfo[m_nAiNo];
		m_pOldAdc = new int[m_nAiNo];
		m_pFirstVal = new BOOL[m_nAiNo];
		m_pFilterGain = new float[m_nAiNo];

		for( i=0; i<m_nAiNo; i++ ) 
		{
			m_pFirstVal[i]	= FALSE;

			m_aiInfoDefault[i].fRealMax = 1000.0;
			m_aiInfoDefault[i].fRealMin = 0.0;
			m_aiInfoDefault[i].nAdcMax 	= 0x3333;
			m_aiInfoDefault[i].nAdcMin 	= 0xCCC;

			GetCalPara( i, 
				m_aiInfo[i].fRealMax,
				m_aiInfo[i].fRealMin, 
				m_aiInfo[i].nAdcMax, 
				m_aiInfo[i].nAdcMin );
			GetFilterGain( i, m_pFilterGain[i] );

			GetInterlockPara( i, m_aiInfo[i].bUseInterlock,
				m_aiInfo[i].nUpLimit, m_aiInfo[i].nLoLimit );
			GetWarningPara( i, m_aiInfo[i].bUseWarning,
				m_aiInfo[i].nUpWarning, m_aiInfo[i].nLoWarning );
			m_aiInfo[i].sName = GetAiName( i );
			m_aiInfo[i].sUnitName = GetAiUnitName( i );
			SetAiCalScale(i);
		}
		return FALSE;
	}

	m_nAiNo = 0;
	for( WORD i=0; i < m_uMaxNode; i++ )
	{
		m_pPointAI[i] = pPointAI[i];
		m_pPointAO[i] = pPointAO[i];
		m_pPointDI[i] = pPointDI[i];
		m_pPointDO[i] = pPointDO[i];
		m_pPointPC[i] = pPointPC[i];

		WORD wInputSize, wOutputSize;

		// Assume that AI is 16 bits 
		// + 1 : internal (only input)
		wInputSize		= ( WORD )( ( pPointPC[i] / 2 )*5 + pPointAI[i]*2 + pPointDI[i]/8 + ( pPointDI[i] % 8 ? 1 : 0 ) + 1 );
		wOutputSize		= ( WORD )( ( pPointPC[i] / 2 )*5 + pPointAO[i]*2 + pPointDO[i]/8 + ( pPointDO[i] % 8 ? 1 : 0 ) );

		m_pInBuf[i]		= new BYTE[wInputSize];
		m_pOutBuf[i]	= new BYTE[wOutputSize];


		// 031230 HongJaehee
		m_nAiNo += pPointAI[i];
		// Add Devices
		if( !AddDevice( ( WORD )( i + 1 ), wInputSize, wOutputSize ) ) return FALSE;
	}

	// 031230 HongJaehee
	m_aiInfo = new aiInfo[m_nAiNo];
	m_aiInfoDefault = new aiInfo[m_nAiNo];
	m_pOldAdc = new int[m_nAiNo];
	m_pFirstVal = new BOOL[m_nAiNo];
	m_pFilterGain = new float[m_nAiNo];

	for( i=0; i<m_nAiNo; i++ ) 
	{
		m_pFirstVal[i]	= FALSE;

		m_aiInfoDefault[i].fRealMax = 1000.0;
		m_aiInfoDefault[i].fRealMin = 0.0;
		m_aiInfoDefault[i].nAdcMax 	= 0x3333;
		m_aiInfoDefault[i].nAdcMin 	= 0xCCC;

		GetCalPara( i, 
			m_aiInfo[i].fRealMax,
			m_aiInfo[i].fRealMin, 
			m_aiInfo[i].nAdcMax, 
			m_aiInfo[i].nAdcMin );
		GetFilterGain( i, m_pFilterGain[i] );
		
		GetInterlockPara( i, m_aiInfo[i].bUseInterlock,
			m_aiInfo[i].nUpLimit, m_aiInfo[i].nLoLimit );
		GetWarningPara( i, m_aiInfo[i].bUseWarning,
			m_aiInfo[i].nUpWarning, m_aiInfo[i].nLoWarning );
		m_aiInfo[i].sName = GetAiName( i );
		m_aiInfo[i].sUnitName = GetAiUnitName( i );
		SetAiCalScale(i);
	}

	// If the card is opened, monitoring the device net board
	if( !CWinThread::CreateThread() ) return FALSE;

	// Start Scan
	if( !StartScan( TRUE ) ) return FALSE;

	// Clear the all I/O
	ResetOutNodeAll();

	return TRUE;
}

/*
// 010829 ...
BOOL XCtlDeviceNet::Start(CWnd* pOwner, UINT uMaxNode, // 
						UINT* uPointAI,		// Analog Input point no. (ex : 4)
						UINT* uPointAO,		// Analog Output point no.
						UINT* uPointDI,		// Digital Input point no. (ex : 64)
						UINT* uPointDO)		// Digital Output point no.
{
	m_pOwner = pOwner;
	SetMaxNode( uMaxNode );

	// Allocate the point no each mac id
	m_pPointAI = new UINT[uMaxNode];
	m_pPointAO = new UINT[uMaxNode];
	m_pPointDI = new UINT[uMaxNode];
	m_pPointDO = new UINT[uMaxNode];

	// We have to allocate the memory each mac id and buffer size.
	m_pInBuf = new BYTE*[uMaxNode];
	m_pOutBuf = new BYTE*[uMaxNode];

	WORD wInputSize, wOutputSize;
	for( WORD i=0; i<m_uMaxNode; i++ )
	{
		m_pPointAI[i] = uPointAI[i];
		m_pPointAO[i] = uPointAO[i];
		m_pPointDI[i] = uPointDI[i];
		m_pPointDO[i] = uPointDO[i];

		// Assume that AI is 16 bits 
		// + 1 : internal (only input)
		wInputSize		= uPointAI[i]*2 + uPointDI[i]/8 + ( uPointDI[i] % 8 ? 1 : 0 ) + 1;
		wOutputSize		= uPointAO[i]*2 + uPointDO[i]/8 + ( uPointDO[i] % 8 ? 1 : 0 );
		m_pInBuf[i]	= new BYTE[wInputSize];
		m_pOutBuf[i]	= new BYTE[wOutputSize];
	}

	if( !OpenCard() ) return FALSE;

	// Online
	if( !Online( TRUE ) ) return FALSE;

	// Add Devices
	for( i=0; i<m_uMaxNode; i++ )
	{
		if( !AddDevice( i + 1, wInputSize, wOutputSize ) ) return FALSE;
	}

	// If the card is opend, monitoring the device net board
	CreateThread();

	// Start Scan
	if( !StartScan( TRUE ) ) return FALSE;

	// Clear the all I/O
	ResetOutNodeAll();

	return TRUE;
}
*/
//--------------------------------------------------------------------
// Procedure   : End
// Description : Exit the thread => CloseCard
// Arguments   : see the below
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlDeviceNet::End()
{
	ResetOutNodeAll();      
	
	// Exit the thread
	if( m_hThread )
	{
		VERIFY( SetEvent( m_hKill ) );

		// When ExitInstance function calls, it is signaled.
		VERIFY( WaitForSingleObject( m_hDead, INFINITE ) == WAIT_OBJECT_0 );
		VERIFY( WaitForSingleObject( m_hThread, INFINITE ) == WAIT_OBJECT_0 );

		Sleep(200);
	}

	DWORD dwExitCode;

	if( !GetExitCodeThread( m_hThread, &dwExitCode ) )
	{
		if( GetLastError() == STILL_ACTIVE )
		{
			TerminateThread( m_hThread, ( DWORD )(-1) );
			Sleep( 200 );

			if( !GetExitCodeThread( m_hThread, &dwExitCode ) )
				AfxMessageBox( "Windows ϰ ý  OFFϿ ֽʽÿ.", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
		}
	}

	// delete the allocated momory
	delete m_pPointAI;
	delete m_pPointAO;
	delete m_pPointDI;
	delete m_pPointDO;
	delete m_pPointPC;

	// delete the allocated momory
	for( WORD i=0; i<m_uMaxNode; i++ )
	{
		delete m_pInBuf[i];
		delete m_pOutBuf[i];
	}

	delete m_pInBuf;
	delete m_pOutBuf;

	// Close the card
	CloseCard();

	return TRUE;
}

//--------------------------------------------------------------------
// Procedure   : IoNo2AddrBit
// Description : 0,1,2,....222...  Assign IO BD Mac ID, Address Bit ٲپ ش.
//		mac_id : 
//		addr   :
//		bit    :
// Arguments   : UINT no
// Returns     : IO_BUFFER
//--------------------------------------------------------------------
IO_BUFFER XCtlDeviceNet::IoNo2AddrBit(UINT uNo, int nKind)
{
	IO_BUFFER ret_io_buffer = { 0, 0, 0 };

	UINT uTotalPointNo = 0;

	UINT* pPoint;
	if( nKind == DI ) pPoint = m_pPointDI;
	else if( nKind == DO ) pPoint = m_pPointDO;
	else if( nKind == AI ) pPoint = m_pPointAI;
	else if( nKind == AO ) pPoint = m_pPointAO;
	else if( nKind == PC ) pPoint = m_pPointPC;

	// no MacId   0, 1, 2, ...  Ѵ.
	// ⼭ no Ͽ  Node ãƳ Ѵ.
	for( WORD i=0; i < m_uMaxNode; i++ )
	{
		uTotalPointNo += pPoint[i];

		if( uNo < uTotalPointNo ) 
		{
			ret_io_buffer.wMac_id = (unsigned short)i + 1;
			break;
		}
	}

	// ش Node  Array  ˾Ƴ
	//    Bit ˾Ƴ.
	UINT uStartNo = uNo - ( uTotalPointNo - pPoint[i] );

	if( nKind == DI ) 
	{
		ret_io_buffer.wAddr	= ( m_pPointPC[i] / 2 ) * 5 + m_pPointAI[i] * 2 + uStartNo / 8; 
		ret_io_buffer.wBit	= uStartNo % 8;
	}
	else if( nKind == DO ) 
	{
		ret_io_buffer.wAddr	= ( m_pPointPC[i] / 2 ) * 5 + m_pPointAO[i] * 2 + uStartNo / 8;
		ret_io_buffer.wBit	= uStartNo % 8;
	}
	else if( nKind == AI || nKind == AO ) 
	{
		ret_io_buffer.wAddr	= ( m_pPointPC[i] / 2 ) * 5 + uStartNo * 2;
		ret_io_buffer.wBit	= 0;
	}
	else if( nKind == PC )
	{
		ret_io_buffer.wAddr	= 1 + ( uStartNo / 2 * 5 ) + ( uStartNo & 0x01 ? 0 : 1 ) * 2;
		ret_io_buffer.wBit	= 0;
	}
	return ret_io_buffer;
}

//--------------------------------------------------------------------
// Procedure   : ReadData
// Description : 
// Arguments   : 
// Returns     : BOOL
//--------------------------------------------------------------------
//BOOL XCtlDeviceNet::DioRead()
BOOL XCtlDeviceNet::ReadData()
{
	for( WORD i = 0 ; i < m_uMaxNode ; i ++ )
	{
		if( !ReadIo( i + 1, m_pInBuf[i] ) ) return FALSE;
	}
	return TRUE;
}

//--------------------------------------------------------------------
// Procedure   : DioRead
// Description : 
// Arguments   : UINT no
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlDeviceNet::DioRead(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, DI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;
//	if( !ReadIo( io_buffer.wMac_id, m_pInBuf[io_buffer.wMac_id - 1] ) ) return FALSE;
	
	UINT uVal = m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr];
	return( ( uVal & ( 0x01 << io_buffer.wBit ) ) ? TRUE : FALSE );
}

//--------------------------------------------------------------------
// Procedure   : DioWrite
// Description : 
// Arguments   : UINT no : 1, 2, 3, ...
//				 BOOL bState : TRUE=>ON, FALSE=>OFF
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlDeviceNet::DioWrite(UINT uNo, BOOL bState)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, DO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	if( bState ) m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] |= 0x01 << io_buffer.wBit;
	else m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] &= ~( 0x01 << io_buffer.wBit );

	return WriteIo( io_buffer.wMac_id, m_pOutBuf[io_buffer.wMac_id - 1] );
}

//--------------------------------------------------------------------
// Procedure   : GetDoState
// Description : 
// Arguments   : UINT no : 1, 2, 3, ...
//				 BOOL bState : TRUE=>ON, FALSE=>OFF
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlDeviceNet::GetDoState(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, DO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	return ( m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] >> io_buffer.wBit ) & 0x01 ? TRUE : FALSE;
}

void XCtlDeviceNet::SetDiState(UINT no, BOOL bState)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( no, DI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return ;

	if( bState ) m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] |= 0x01 << io_buffer.wBit;
	else m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] &= ~( 0x01 << io_buffer.wBit );
}


int XCtlDeviceNet::AioRead(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

//	if( !ReadIo( io_buffer.wMac_id, m_pInBuf[io_buffer.wMac_id - 1] ) ) return 0;

	//  AIO 12 bit ( Beckhoff, KL3064 : 0 ~ 10V )
	WORD wResult = ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] & 0xF0 ) + 
		( ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] & 0xFF ) << 8 );

	short nResult = (short)wResult;	

	return nResult;
}

WORD XCtlDeviceNet::GetAoValue(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

//	if( !ReadIo( io_buffer.wMac_id, m_pInBuf[io_buffer.wMac_id - 1] ) ) return 0;

	//  AIO 12 bit ( Beckhoff, KL3064 : 0 ~ 10V )
	WORD wResult = ( m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] & 0xF0 ) + 
		( ( m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] & 0x7F ) << 8 );

	return wResult;
}


void XCtlDeviceNet::SetAioValue(UINT uNo, WORD wValue)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return;

	m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] = wValue & 0x00F7;
	m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] = ( wValue >> 8 ) & 0x007F;
}



BOOL XCtlDeviceNet::AioWrite(UINT uNo, WORD wValue)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] = wValue & 0x00F0;
	m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] = ( wValue >> 8 ) & 0x007F;

	return WriteIo( io_buffer.wMac_id, m_pOutBuf[io_buffer.wMac_id - 1] );
}

WORD XCtlDeviceNet::GetPulseCnt(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, PC );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	// Pulse Counter KL1501( [control][2nd low][2nd high][1st low][1st high] )
	// [control]	: Control Byte //   ... ߿  ...
	// [2nd low]	: 2° Channel  Byte
	// [2nd high]	: 2° Channel  Byte
	// [1st low]	: 1° Channel  Byte
	// [1st high]	: 1° Channel  Byte
	WORD wResult = ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] & 0xFF ) + 
		( ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] & 0x7F ) << 8 );

	return wResult;
}
//--------------------------------------------------------------------
// Procedure   : ResetOutNode
// Description : 
// Arguments   : void
// Returns     : void
//--------------------------------------------------------------------
void XCtlDeviceNet::ResetOutNodeAll()
{
	for( UINT i=0; i < m_uMaxNode; i++) ResetOutNode( i );
}

void XCtlDeviceNet::ResetOutNode(UINT uNode)
{
	if(uNode < 0 || uNode >= m_uMaxNode) return;

	WORD wInputSize		= ( m_pPointPC[uNode] / 2 )*5 + m_pPointAI[uNode]*2 + m_pPointDI[uNode]/8 + ( m_pPointDI[uNode] % 8 ? 1 : 0 ) + 1;
	WORD wOutputSize	= ( m_pPointPC[uNode] / 2 )*5 + m_pPointAO[uNode]*2 + m_pPointDO[uNode]/8 + ( m_pPointDO[uNode] % 8 ? 1 : 0 );

	for( UINT i=0; i < wInputSize; i++) m_pInBuf[uNode][i] = 0x00;
	for( i=0; i < wOutputSize; i++) m_pOutBuf[uNode][i] = 0x00;
	WriteIo( uNode + 1, m_pOutBuf[uNode] );
}

//--------------------------------------------------------------------
// Procedure   : Watch
// Description : 
// Arguments   : void
// Returns     : void
//--------------------------------------------------------------------
void XCtlDeviceNet::Watch()
{
	for( WORD i=0; i < m_uMaxNode; i++ )
	{
		if( Monitor( i + 1 ) ) // Device Mac ID = 1, 2, ...
		{
			Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) m_sCurDeviceStatus );
		}
	}
}

//---------------------------------------------------------------------------
// Send the message to parent window
// wait for the termination of message process : SendMessage()
void XCtlDeviceNet::Msg(UINT uMsg,  WPARAM wParam, LPARAM lParam)
{
	if( m_pOwner == NULL ) return;

	m_pOwner->PostMessage( uMsg, wParam, lParam );
}

void XCtlDeviceNet::SetOwner(CWnd *pOwner)
{
	m_pOwner = pOwner;	
}

//---------------------------------------------------------------------------
// Analog Input Calibration

void XCtlDeviceNet::SetCalPara(int nID, float fRealMax, float fRealMin, int nAdcMax, int nAdcMin)
{
	m_aiInfo[nID].fRealMax = fRealMax;
	m_aiInfo[nID].fRealMin = fRealMin;
	m_aiInfo[nID].nAdcMax = nAdcMax;
	m_aiInfo[nID].nAdcMin = nAdcMin;

	CString sEntry;
	CString sVal;

	sEntry.Format( "AI_REAL_MAX%d", nID );
	sVal.Format( "%.8f", m_aiInfo[nID].fRealMax );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );

	sEntry.Format( "AI_REAL_MIN%d", nID );
	sVal.Format( "%.8f", m_aiInfo[nID].fRealMin );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );

	sEntry.Format( "AI_ADC_MAX%d", nID );
	sVal.Format( "%d", m_aiInfo[nID].nAdcMax );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );

	sEntry.Format( "AI_ADC_MIN%d", nID );
	sVal.Format( "%d", m_aiInfo[nID].nAdcMin );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );
}

void XCtlDeviceNet::GetCalPara(int nID, float& fRealMax, float& fRealMin, int& nAdcMax, int& nAdcMin, BOOL bDefault, int nSensorType )
{
	CString sEntry;
	CString sVal;

	if( bDefault )
		//m_aiInfo[nID].fRealMax = fRealMax = m_aiInfoDefault[nID].fRealMax;
		m_aiInfo[nID].fRealMax = fRealMax = m_aiPreSet[nSensorType].fRealMax;
	else
	{
		sEntry.Format( "AI_REAL_MAX%d", nID );
//		sVal.Format( "%.8f", fRealMax );
		sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "0.0" );
		m_aiInfo[nID].fRealMax = fRealMax = (float) atof( sVal );
	}
	if( bDefault )
		//m_aiInfo[nID].fRealMin = fRealMin = m_aiInfoDefault[nID].fRealMin;
		m_aiInfo[nID].fRealMin = fRealMin = m_aiPreSet[nSensorType].fRealMin;
	else
	{
		sEntry.Format( "AI_REAL_MIN%d", nID );
//		sVal.Format( "%.8f", fRealMin );
		sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "0.0" );
		m_aiInfo[nID].fRealMin = fRealMin = (float) atof( sVal );
	}

	if( bDefault )
		//m_aiInfo[nID].nAdcMax = nAdcMax = m_aiInfoDefault[nID].nAdcMax;
		m_aiInfo[nID].nAdcMax = nAdcMax = m_aiPreSet[nSensorType].nAdcMax;
	else
	{
		sEntry.Format( "AI_ADC_MAX%d", nID );
//		sVal.Format( "%d", nAdcMax );
		sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "0" );
		m_aiInfo[nID].nAdcMax = nAdcMax = atoi( sVal );
	}

	if( bDefault )
		//m_aiInfo[nID].nAdcMin = nAdcMin = m_aiInfoDefault[nID].nAdcMin;
		m_aiInfo[nID].nAdcMin = nAdcMin = m_aiPreSet[nSensorType].nAdcMin;
	else
	{
		sEntry.Format( "AI_ADC_MIN%d", nID );
//		sVal.Format( "%d", nAdcMin );
		sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "0" );
		m_aiInfo[nID].nAdcMin = nAdcMin = atoi( sVal );
	}
}

//--------------------------------------------------------------------
// Get the calibrated value
// + Low Pass Filter
//--------------------------------------------------------------------
float XCtlDeviceNet::GetCalAI(int nID)
{	
//
#ifndef _MY_DEBUG
	if( !IsActive() ) return 0.;
#endif
	int nADC = AioRead(nID);
	float f = m_pFilterGain[nID];
	if( !m_pFirstVal[nID] )
	{
		m_pFirstVal[nID] = TRUE;
		m_pOldAdc[nID] = nADC;
	}
	nADC = (int)( (float)nADC * f + (float)m_pOldAdc[nID] * ( 1. - f ) );
	m_pOldAdc[nID] = nADC;

	float	fRealMax, fRealMin;
	int		nAdcMax, nAdcMin;
	nAdcMax = m_aiInfo[nID].nAdcMax;
	nAdcMin = m_aiInfo[nID].nAdcMin;
	if( m_aiInfo[nID].nScale == scaleCOMMON )
	{
		fRealMax = m_aiInfo[nID].fRealMax;
		fRealMin = m_aiInfo[nID].fRealMin;
	}
	else if( m_aiInfo[nID].nScale == scaleLOG )
	{
		fRealMax = (float)log( (double)m_aiInfo[nID].fRealMax );
		fRealMin = (float)log( (double)m_aiInfo[nID].fRealMin );
	}
	else if( m_aiInfo[nID].nScale == scaleLOG10 )
	{
		fRealMax = (float)log10( (double)m_aiInfo[nID].fRealMax );
		fRealMin = (float)log10( (double)m_aiInfo[nID].fRealMin );
	}

	int nDiffAdc = nAdcMax - nAdcMin;
	float fDiffReal = fRealMax - fRealMin;

	if( nDiffAdc == 0 ) return (float) nADC;

	float fVal = ( fDiffReal / (float) nDiffAdc ) *
		( nADC - m_aiInfo[nID].nAdcMin ) + fRealMin;

	if( m_aiInfo[nID].nScale == scaleCOMMON )
	{
		return fVal;
	}
	else if( m_aiInfo[nID].nScale == scaleLOG )
	{
		return (float)( exp( (double)fVal ) );
	}
	else if( m_aiInfo[nID].nScale == scaleLOG10 )
	{
		return (float)( pow( 10., (double)fVal ) );
	}
	return fVal;
}

void XCtlDeviceNet::SetRawAI(int nID, float fVal)
{
	WORD wADC;
	int nDiffAdc = m_aiInfo[nID].nAdcMax - m_aiInfo[nID].nAdcMin;
	float fDiffReal = m_aiInfo[nID].fRealMax - m_aiInfo[nID].fRealMin;
		
	wADC = (int)((fVal - m_aiInfo[nID].fRealMin)*(nDiffAdc / fDiffReal)) + m_aiInfo[nID].nAdcMin;

	SetAioValue( nID, wADC );
}


void XCtlDeviceNet::SetInterlockPara(int nID, BOOL bSet, int nUpLimit, int nLoLimit)
{
	m_aiInfo[nID].bUseInterlock = bSet;
	m_aiInfo[nID].nLoLimit = nLoLimit;
	m_aiInfo[nID].nUpLimit = nUpLimit;

	CString sEntry;
	CString sVal;

	sEntry.Format( "USE_INTERLOCK%d", nID );
	AfxGetApp()->WriteProfileInt( "DICS AI", sEntry, (int)bSet );

	sEntry.Format( "AI_ADC_UP_LIMIT%d", nID );
	sVal.Format( "%d", nUpLimit );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );

	sEntry.Format( "AI_ADC_LO_LIMIT%d", nID );
	sVal.Format( "%d", nLoLimit );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );
}

void XCtlDeviceNet::GetInterlockPara(int nID, BOOL &bSet, int &nUpLimit, int &nLoLimit)
{
	CString sEntry;
	CString sVal;

	sEntry.Format( "USE_INTERLOCK%d", nID );
	int nVal = AfxGetApp()->GetProfileInt( "DICS AI", sEntry, 0 );
	m_aiInfo[nID].bUseInterlock = bSet = (BOOL) nVal;

	sEntry.Format( "AI_ADC_UP_LIMIT%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "10000." );
	m_aiInfo[nID].nUpLimit = nUpLimit = atoi( sVal );

	sEntry.Format( "AI_ADC_LO_LIMIT%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "-10000." );
	m_aiInfo[nID].nLoLimit = nLoLimit = atoi( sVal );
}

void XCtlDeviceNet::SetWarningPara(int nID, BOOL bSet, int nUpWarning, int nLoWarning)
{
	m_aiInfo[nID].bUseInterlock = bSet;
	m_aiInfo[nID].nLoWarning = nLoWarning;
	m_aiInfo[nID].nUpWarning = nUpWarning;
	
	CString sEntry;
	CString sVal;
	
	sEntry.Format( "USE_WARNING%d", nID );
	AfxGetApp()->WriteProfileInt( "DICS AI", sEntry, (int)bSet );
	
	sEntry.Format( "AI_ADC_UP_WARNING%d", nID );
	sVal.Format( "%d", nUpWarning );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );
	
	sEntry.Format( "AI_ADC_LO_WARNING%d", nID );
	sVal.Format( "%d", nLoWarning );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );
}

void XCtlDeviceNet::GetWarningPara(int nID, BOOL &bSet, int &nUpWarning, int &nLoWarning)
{
	CString sEntry;
	CString sVal;

	sEntry.Format( "USE_WARNING%d", nID );
	int nVal = AfxGetApp()->GetProfileInt( "DICS AI", sEntry, 0 );
	m_aiInfo[nID].bUseInterlock = bSet = (BOOL) nVal;

	sEntry.Format( "AI_ADC_UP_WARNING%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "10000." );
	m_aiInfo[nID].nUpWarning = nUpWarning = atoi( sVal );

	sEntry.Format( "AI_ADC_LO_WARNING%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "-10000." );
	m_aiInfo[nID].nLoWarning = nLoWarning = atoi( sVal );
}

CString XCtlDeviceNet::GetAiName( int nID )
{
	CString sEntry;
	CString sVal;

	sEntry.Format( "AI_NAME%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "SPARE" );
	m_aiInfo[nID].sName = sVal;
	return sVal;
}

void XCtlDeviceNet::SetAiName( int nID, CString sName )
{
	CString sEntry;

	sEntry.Format( "AI_NAME%d", nID );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, (LPCTSTR)sName );
	m_aiInfo[nID].sName = sName;
}

CString XCtlDeviceNet::GetAiUnitName( int nID )
{
	CString sEntry;
	CString sVal;

	sEntry.Format( "AI_UNIT_NAME%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "NONE" );
	m_aiInfo[nID].sUnitName = sVal;
	return sVal;
}

void XCtlDeviceNet::SetAiUnitName( int nID, CString sName )
{
	CString sEntry;

	sEntry.Format( "AI_UNIT_NAME%d", nID );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, (LPCTSTR)sName );
	m_aiInfo[nID].sUnitName = sName;
}

void XCtlDeviceNet::ResetInNodeAll()
{
	for( UINT i = 0 ; i < m_uMaxNode ; i ++ )
	{
		WORD wInputSize		= ( m_pPointPC[i]/2 ) * 5 + m_pPointAI[i]*2 + m_pPointDI[i]/8 + ( m_pPointDI[i] % 8 ? 1 : 0 ) + 1;
		for( UINT j=0; j < wInputSize; j++) 
			m_pInBuf[i][j] = 0x00;
	}
}


void XCtlDeviceNet::GetFilterGain(int nID, float &f)
{
	CString sEntry, sVal;
	sEntry.Format( "AI_FILTER_GAIN%d", nID );
	sVal = AfxGetApp()->GetProfileString( "DICS AI", sEntry, "1.0" );

	f = (float)(atof( sVal ));
}

void XCtlDeviceNet::SetFilterGain(int nID, float f)
{
	if( f < (float)0.1 ) f = (float)0.1;
	else if( f > 1. ) f = 1.;

	m_pFilterGain[nID] = f;

	CString sEntry, sVal;
	sEntry.Format( "AI_FILTER_GAIN%d", nID );
	sVal.Format( "%f", f );
	AfxGetApp()->WriteProfileString( "DICS AI", sEntry, sVal );
}

void XCtlDeviceNet::SetAiCalScale(int nID, int nScale)
{
	m_aiInfo[nID].nScale = nScale;
}
