//------------------------------------------------------------------------------------------------
// Copyright		: DMS Co., Ltd
// File Name		: XCtlEtherCAT.cpp
// Modified Date	: 2005.05.20
// Version			: Ver1.0
// Programmer		: Lee yongseok
// Description		: implementation of the XCtlEtherCAT class.
//------------------------------------------------------------------------------------------------

#include "stdafx.h"
#include "..\Dics.h"
#include "XCtlEtherCAT.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define DELAY_IN_100NS(x)		(x * 10000)
#define TASKMS_DELAY			10

/////////////////////////////////////////////////////////////////////////////
// XCtlEtherCAT

IMPLEMENT_DYNCREATE(XCtlEtherCAT, CWinThread)

//------------------------------------------------------
// Procedure   : XCtlEtherCAT
// Description : Construction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
XCtlEtherCAT::XCtlEtherCAT()
{
	m_bAutoDelete			= FALSE;
	m_pOwner				= NULL;

	m_aiInfo				= NULL;
	m_aiInfoDefault			= NULL;
	m_nAiNo					= 0;
	m_bConnect				= NULL;

	m_uMaxNode				= 0;
	m_uStatusOffset			= 0;
	m_nStatusByteSize		= 0;
	m_nOldAdsState			= -1;

	m_pPointDI				= NULL;
	m_pPointDO				= NULL;
	m_pPointAI				= NULL;
	m_pPointAO				= NULL;
	m_pPointPC				= NULL;

	m_pInBuf				= NULL;
	m_pOutBuf				= NULL;
	m_pStatusBuf			= NULL;

	m_pOldAdc				= NULL;
	m_pFirstVal				= NULL;
	m_pFilterGain			= NULL;


	// jemoon : 070823
	m_pTotInBuf				= NULL;
	m_pTotOutBuf			= NULL;
	m_pNodeInputSize		= NULL;
	m_pNodeOutputSize		= NULL;


	m_hDead = CreateEvent( NULL, TRUE, FALSE, NULL );	// MANUAL Reset, Initial state = non-signal
	m_hKill = CreateEvent( NULL, TRUE, FALSE, NULL );
}

AI_PRESET XCtlEtherCAT::m_aiPreSet[enMAX_SENSOR_TYPE] =
{
	{ enPG35		, "PG35",				0.,		1000.,	0xCCC,	0x3000 },
	{ enDW_605		, "Dwyer Manometer",	0.,		750.,	6553,	32767 },
	{ enSENSYS		, "SENSYS DPLH0750A",	0.,		750.,	6553,	32767 },
	{ enJLK_25		, "JLK 25A",			0.,		130.,	6553,	32767 },	
	{ enULK_25		, "ULK 25A",			0.,		130.,	6553,	32767 },	
	{ enCKD_FLOW	, "CKD FLUEREX",		0.,		4000.,	6553,	32767 },	
	{ enPT100OHM	, "PT 100 OHM",			0.,		4000.,	6553,	32767 },	
	{ enLEVEL_GAGE	, "Level SENSYS PSH(0.1kgf/cm2)",	0.,		1000.,	6553,	32767 },
	{ enTHERMO_SENSOR	, "ThermoCoupler",	0.,		100.,	0,	1000 }	
};

//------------------------------------------------------
// Procedure   : ~XCtlEtherCAT
// Description : Destruction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
XCtlEtherCAT::~XCtlEtherCAT()
{
	if( m_aiInfo ) delete [] m_aiInfo;
	if( m_aiInfoDefault ) delete [] m_aiInfoDefault;
	if( m_hDead ) CloseHandle( m_hDead );
	if( m_hKill ) CloseHandle( m_hKill );

	if( m_pOldAdc )		delete []m_pOldAdc;
	if( m_pFirstVal )	delete []m_pFirstVal;
	if( m_pFilterGain ) delete []m_pFilterGain;
}

//------------------------------------------------------
// Procedure   : InitInstance
// Description : Run the thread
// Arguments   : void
// Returns     : BOOL (if return false, the thread is exited)
//------------------------------------------------------
BOOL XCtlEtherCAT::InitInstance()
{
	USES_CONVERSION;
	HRESULT Hr;
	Hr = ::CoInitializeEx( NULL, COINIT_MULTITHREADED );
	if( SUCCEEDED( Hr ) ) 
	{
		TRACE( "CoInitializedEX failed !!!\n" );
	}

	Hr = m_TcComServer.CreateInstance( "TcSystemSrv.TcSystemServer.1" );
	if( FAILED( Hr ) ) 
	{
		TRACE( "CreateInstance Failed.!!!\n" );
	}

	ITcSystemServer	*pTcManage;
	m_TcComServer.QueryInterface( m_TcComServer.GetIID(), (void**) &pTcManage );

	while( WaitForSingleObject( m_hKill, 0 ) != WAIT_OBJECT_0 ) 
	{
		Sleep( 1 );
		Watch();
	}

	VERIFY( SetEvent( m_hDead ) );
	return FALSE;
}

int XCtlEtherCAT::ExitInstance()
{
	// TODO:  perform any per-thread cleanup here
	return CWinThread::ExitInstance();
}

BEGIN_MESSAGE_MAP(XCtlEtherCAT, CWinThread)
	//{{AFX_MSG_MAP(XCtlEtherCAT)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// XCtlEtherCAT message handlers

//--------------------------------------------------------------------
// Procedure   : Start
// Description : 
// Arguments   : see the below
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::Start(CWnd* pOwner, UINT uMaxNode, // 
						UINT* pPointAI,		// Analog Input point no. (ex : 4)
						UINT* pPointAO,		// Analog Output point no.
						UINT* pPointDI,		// Digital Input point no. (ex : 64)
						UINT* pPointDO,		// Digital Output point no.		
						UINT* pPointPC )	// Pulse Counter  KL1501 no.		
{
	m_pOwner = pOwner;
	m_uMaxNode = uMaxNode;

#ifndef _MY_DEBUG
	// EtherCAT Link Confirm
	if( !TcAdsOpen( AMSPORT_IO1 ) ) 
	{
		AfxMessageBox( "EtherCAT open error : TcAdsOpen Error", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
		return FALSE;
	}
#endif
	// Allocate the point no each mac id
	m_pPointAI = new UINT[uMaxNode];
	m_pPointAO = new UINT[uMaxNode];
	m_pPointDI = new UINT[uMaxNode];
	m_pPointDO = new UINT[uMaxNode];
	m_pPointPC = new UINT[uMaxNode];

	// We have to allocate the memory each mac id and buffer size.
	m_pInBuf	= new BYTE*[uMaxNode];
	m_pOutBuf	= new BYTE*[uMaxNode];

	//2006.05.17 Youngsik
	//for Monitor Status bit
	m_nStatusByteSize = ( m_uMaxNode / 8 ) + ( m_uMaxNode % 8 ? 1 : 0 );
	m_pStatusBuf = new BYTE[m_nStatusByteSize];


	// 2007.08.23 jemoon
	m_nTotInputSize = 0;
	m_nTotOutputSize = 0;
	m_pNodeInputSize = new int[uMaxNode];
	m_pNodeOutputSize = new int[uMaxNode];

	WORD i;
	for( i=0; i < m_uMaxNode; i++ )
	{
		m_pPointAI[i] = pPointAI[i];
		m_pPointAO[i] = pPointAO[i];
		m_pPointDI[i] = pPointDI[i];
		m_pPointDO[i] = pPointDO[i];
		m_pPointPC[i] = pPointPC[i];

		WORD wInputSize, wOutputSize;

		// Assume that AI is 16 bits 
		// + 1 : internal (only input)
		wInputSize		= ( pPointPC[i] / 2 )*5 + pPointAI[i]*2 + pPointDI[i]/8 + ( pPointDI[i] % 8 ? 1 : 0 );
		wOutputSize		= ( pPointPC[i] / 2 )*5 + pPointAO[i]*2 + pPointDO[i]/8 + ( pPointDO[i] % 8 ? 1 : 0 );

		m_pInBuf[i]		= new BYTE[wInputSize];
		m_pOutBuf[i]	= new BYTE[wOutputSize];

		// 031230 HongJaehee
		m_nAiNo += pPointAI[i];

		//2006.05.17 Youngsik
		//for Monitor Status bit
		//WORD wInputSize	= ( m_pPointPC[i] / 2 )*5 + m_pPointAI[i]*2 + m_pPointDI[i]/8 + ( m_pPointDI[i] % 8 ? 1 : 0 );
		m_uStatusOffset += wInputSize;


		// 2007.08.23 jemoon
		m_pNodeInputSize[i] = wInputSize;
		m_pNodeOutputSize[i] = wOutputSize;
		m_nTotInputSize += wInputSize;
		m_nTotOutputSize += wOutputSize;
	}

	// 2007.08.23 jemoon
	m_pTotInBuf = new BYTE[m_nTotInputSize];
	m_pTotOutBuf = new BYTE[m_nTotOutputSize];
	for( i=0; i<m_nTotInputSize; i++)	m_pTotInBuf[i] = 0x00;
	for( i=0; i<m_nTotOutputSize; i++)	m_pTotOutBuf[i] = 0x00;

	// 031230 HongJaehee
	m_aiInfo = new aiInfo[m_nAiNo];
	m_aiInfoDefault = new aiInfo[m_nAiNo];
	m_pOldAdc = new int[m_nAiNo];
	m_pFirstVal = new BOOL[m_nAiNo];
	m_pFilterGain = new float[m_nAiNo];

	for( i=0; i<m_nAiNo; i++ ) 
	{
		m_pFirstVal[i]	= FALSE;
		m_pOldAdc[i] = 0;


		m_aiInfoDefault[i].fRealMax = 1000.0;
		m_aiInfoDefault[i].fRealMin = 0.0;
		m_aiInfoDefault[i].nAdcMax 	= 0x3333;
		m_aiInfoDefault[i].nAdcMin 	= 0xCCC;

		GetCalPara( i, 
			m_aiInfo[i].fRealMax,
			m_aiInfo[i].fRealMin, 
			m_aiInfo[i].nAdcMax, 
			m_aiInfo[i].nAdcMin );
		GetFilterGain( i, m_pFilterGain[i] );

		GetInterlockPara( i, m_aiInfo[i].bUseInterlock,
			m_aiInfo[i].nUpLimit, m_aiInfo[i].nLoLimit );
		GetWarningPara( i, m_aiInfo[i].bUseWarning,
			m_aiInfo[i].nUpWarning, m_aiInfo[i].nLoWarning );
		m_aiInfo[i].sName = GetAiName( i );
		m_aiInfo[i].sUnitName = GetAiUnitName( i );

		int nScale;
		GetAiCalScale(i, nScale);
	}

	// If the card is opend, monitoring the device net board
	if( !CWinThread::CreateThread() ) return FALSE;

	// Clear the all I/O
	ResetOutNodeAll();

	return TRUE;
}

//--------------------------------------------------------------------
// Procedure   : End
// Description : Exit the thread => CloseCard
// Arguments   : see the below
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::End()
{

	ResetOutNodeAll();

	// Exit the thread
	if( m_hThread )
	{
		VERIFY( SetEvent( m_hKill ) );

		// When ExitInstance function calls, it is signaled.
		VERIFY( WaitForSingleObject( m_hDead, INFINITE ) == WAIT_OBJECT_0 );
		VERIFY( WaitForSingleObject( m_hThread, INFINITE ) == WAIT_OBJECT_0 );

		Sleep(200);
	}

	DWORD dwExitCode;

	if( !GetExitCodeThread( m_hThread, &dwExitCode ) )
	{
		if( GetLastError() == STILL_ACTIVE )
		{
			TerminateThread( m_hThread, -1 );
			Sleep( 200 );

			if( !GetExitCodeThread( m_hThread, &dwExitCode ) )
				AfxMessageBox( "Windows ϰ ý  OFFϿ ֽʽÿ.", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
		}
	}

	// delete the allocated momory
	if( m_pPointAI ) delete m_pPointAI;
	if( m_pPointAO ) delete m_pPointAO;
	if( m_pPointDI ) delete m_pPointDI;
	if( m_pPointDO ) delete m_pPointDO;
	if( m_pPointPC ) delete m_pPointPC;

	// delete the allocated momory
	for( WORD i=0; i<m_uMaxNode; i++ )
	{
		delete m_pInBuf[i];
		delete m_pOutBuf[i];
	}

	if( m_pInBuf )		delete m_pInBuf;
	if( m_pOutBuf )		delete m_pOutBuf;
	if( m_pStatusBuf )	delete m_pStatusBuf;

	// jemoon : 070823
	if( m_pTotInBuf )		delete []m_pTotInBuf;
	if( m_pTotOutBuf )		delete []m_pTotOutBuf;
	if( m_pNodeInputSize )	delete []m_pNodeInputSize;
	if( m_pNodeOutputSize ) delete []m_pNodeOutputSize;

	// Close the card
	TcAdsClose();

	return TRUE;
}

//--------------------------------------------------------------------
// Procedure   : GetInNodeSize
// Description : 
//		mac_id : 
//		addr   :
//		bit    :
// Arguments   : UINT uNode
// Returns     : InputSize
//--------------------------------------------------------------------
WORD XCtlEtherCAT::GetInNodeSize( UINT uNode )
{
	WORD wInputSize = 0;
	for( UINT i=0; i<uNode - 1; i++ )
	{
		wInputSize += ( m_pPointPC[i] / 2 )*5 + m_pPointAI[i]*2 + m_pPointDI[i]/8 + ( m_pPointDI[i] % 8 ? 1 : 0 );
	}
	return wInputSize;
}

//--------------------------------------------------------------------
// Procedure   : GetOutNodeSize
// Description : 
//		mac_id : 
//		addr   :
//		bit    :
// Arguments   : UINT uNode
// Returns     : InputSize
//--------------------------------------------------------------------
WORD XCtlEtherCAT::GetOutNodeSize( UINT uNode )
{
	WORD wOutputSize = 0;
	for( UINT i=0; i<uNode - 1; i++ )
	{
		wOutputSize	+= ( m_pPointPC[i] / 2 )*5 + m_pPointAO[i]*2 + m_pPointDO[i]/8 + ( m_pPointDO[i] % 8 ? 1 : 0 );
	}
	return wOutputSize;
}

//--------------------------------------------------------------------
// Procedure   : IoNo2AddrBit
// Description : 0,1,2,....222...  Assign IO BD Mac ID, Address Bit ٲپ ش.
//		mac_id : 
//		addr   :
//		bit    :
// Arguments   : UINT no
// Returns     : IO_BUFFER
//--------------------------------------------------------------------
IO_BUFFER XCtlEtherCAT::IoNo2AddrBit(UINT uNo, int nKind)
{
	IO_BUFFER ret_io_buffer = { 0, 0, 0 };

	UINT uTotalPointNo = 0;
	UINT uPreNodeSize = 0;

	UINT* pPoint;
	if( nKind == DI ) pPoint = m_pPointDI;
	else if( nKind == DO ) pPoint = m_pPointDO;
	else if( nKind == AI ) pPoint = m_pPointAI;
	else if( nKind == AO ) pPoint = m_pPointAO;
	else if( nKind == PC ) pPoint = m_pPointPC;

	// no MacId   0, 1, 2, ...  Ѵ.
	// ⼭ no Ͽ  Node ãƳ Ѵ.
	WORD i;
	for( i=0; i < m_uMaxNode; i++ )
	{ 
		uTotalPointNo += pPoint[i];

		if( uNo < uTotalPointNo ) 
		{
			ret_io_buffer.wMac_id = i + 1;
			break;
		}
	}

	// ش Node  Array  ˾Ƴ
	//    Bit ˾Ƴ.
	UINT uStartNo = uNo - ( uTotalPointNo - pPoint[i] );
	
	if( nKind == DI ) 
	{
		ret_io_buffer.wAddr	= ( m_pPointPC[i] / 2 ) * 5 + m_pPointAI[i] * 2 + uStartNo / 8; 
		//ret_io_buffer.wAddr = ( m_pPointPC[i] / 2 ) * 5 + uTotalAiPointNo * 2 + uStartNo / 8;
		ret_io_buffer.wBit	= uStartNo % 8;
	}
	else if( nKind == DO ) 
	{
		ret_io_buffer.wAddr	= ( m_pPointPC[i] / 2 ) * 5 + m_pPointAO[i] * 2 + uStartNo / 8;
		//ret_io_buffer.wAddr = ( m_pPointPC[i] / 2 ) * 5 + uTotalAoPointNo * 2 + uStartNo / 8;
		ret_io_buffer.wBit	= uStartNo % 8;
	}
	else if( nKind == AI || nKind == AO ) 
	{
		/*if( nKind == AI )
			uPreNodeSize = GetInNodeSize( ret_io_buffer.wMac_id );
		else
			uPreNodeSize = GetOutNodeSize( ret_io_buffer.wMac_id );*/

		ret_io_buffer.wAddr	= /*( m_pPointPC[i] / 2 ) * 5 + uPreNodeSize +*/ uStartNo * 2;
		ret_io_buffer.wBit	= 0;
	}
	else if( nKind == PC )
	{
		ret_io_buffer.wAddr	= 1 + ( uStartNo / 2 * 5 ) + ( uStartNo & 0x01 ? 0 : 1 ) * 2;
		ret_io_buffer.wBit	= 0;
	}
	return ret_io_buffer;
}

//--------------------------------------------------------------------
// Procedure   : Watch
// Description : 
// Arguments   : void
// Returns     : void
//--------------------------------------------------------------------
void XCtlEtherCAT::Watch()
{
	//if( Monitor() ) // Device Mac ID = 1, 2, ...
	//{
	//	Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) m_sCurAdsState );
	//}
	
	// Read IO
	ReadData();
	Sleep(2);
	
	// Write IO
	WriteData();
	Sleep(2);
	
	// Monitor ADS State
	Monitor();
	Sleep(2);
	
	// Update ADS State, fire message(WM_EQP_MESSAGE)
	UpdateState();
}


void XCtlEtherCAT::UpdateState()
{
	static CString sMsg = _T("");
	if( m_nAdsState == ADSSTATE_START || m_nAdsState == ADSSTATE_RUN )
	{
		int nRv = IsConnected();

		if( ( nRv == TRUE ) && m_bConnect && ( sMsg != "Active" ) )
		{
			sMsg.Format( "Active" );
			Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) sMsg );
		}
		else if( ( nRv == FALSE ) && !m_bConnect && ( sMsg != "DisConnected" ) )
		{
			sMsg.Format( "DisConnected" );
			Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) sMsg );
		}
		else if( ( nRv == FALSE ) && m_bConnect && ( sMsg != "Node Closed" ) )
		{
			sMsg.Format( "Node Closed" );
			Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) sMsg );
		}
	}
	else if( m_nAdsState == ADSSTATE_INVALID && m_nAdsState != m_nOldAdsState )
	{
		m_nOldAdsState = ADSSTATE_INVALID;
//		m_bConnect = FALSE;
		Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) "INVALID" );
		TcStart();
	}
	else if( m_nAdsState == ADSSTATE_SHUTDOWN && m_nAdsState != m_nOldAdsState ) //LYS20080825
	{
		m_nOldAdsState = ADSSTATE_SHUTDOWN;
		m_bConnect = FALSE;
		Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) "SHUTDOWN" );

		AfxMessageBox( "Please!!.Execute again Dics program after TwinCAT manager start", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
	}
	else if( m_nAdsState != m_nOldAdsState )
	{
		m_nOldAdsState = m_nAdsState;
//		m_bConnect = FALSE;

		if( m_nAdsState == ADSSTATE_STOP ) TcStart(); //LYS20070213 TwinCAT System Start
		Msg( WM_EQP_MESSAGE, (WPARAM) MSG_DEVICENET_STATE, (LPARAM)(LPCTSTR) m_sCurAdsState );
	}
}


//--------------------------------------------------------------------
// Procedure   : ReadData
// Description : 
// Arguments   : 
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::ReadData()
{
	// jemoon : 070823 -> Node read ѹ readϵ 
	if( !TcAdsRead( ADSIGRP_IOIMAGE_RWIB, 0, m_nTotInputSize, m_pTotInBuf ) )
	{
		return FALSE;
	}

	WORD wOffset = 0;
	for( WORD i=0; i<m_uMaxNode; i++ )
	{
		for( WORD j=0; j<m_pNodeInputSize[i]; j++)
		{
			m_pInBuf[i][j] = m_pTotInBuf[wOffset+j];
		}
		wOffset += m_pNodeInputSize[i];
	}

	return TRUE;
}

//--------------------------------------------------------------------
// Procedure   : WriteData
// Description : 
// Arguments   : 
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::WriteData()
{
	static BOOL bChanged = FALSE;
	WORD wOffset = 0;
	for( WORD i=0; i<m_uMaxNode; i++ )
	{
		for( WORD j=0; j<m_pNodeOutputSize[i]; j++)
		{
			if( m_pTotOutBuf[wOffset+j] != m_pOutBuf[i][j] )
			{
				m_pTotOutBuf[wOffset+j] = m_pOutBuf[i][j];
				bChanged = TRUE;
			}
		}
		wOffset += m_pNodeOutputSize[i];
	}

	if( !bChanged ) return TRUE;

	// jemoon : 070823 -> Seq   ϴ  OutBuffer ̿Ͽ ѹ 
	if( !TcAdsWrite( ADSIGRP_IOIMAGE_RWOB, 0, m_nTotOutputSize, m_pTotOutBuf ) )
	{
		return FALSE;
	}
	bChanged = FALSE;
	return TRUE;
}


//--------------------------------------------------------------------
// Procedure   : DioRead
// Description : 
// Arguments   : UINT no
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::DioRead(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, DI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;
	
	UINT uVal = m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr];
	return( ( uVal & ( 0x01 << io_buffer.wBit ) ) ? TRUE : FALSE );
}

//--------------------------------------------------------------------
// Procedure   : DioWrite
// Description : 
// Arguments   : UINT no : 1, 2, 3, ...
//				 BOOL bState : TRUE=>ON, FALSE=>OFF
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::DioWrite(UINT uNo, BOOL bState)
{
	CSingleLock sLock(&m_mutex ); // HJYOU :    [2/23/2010]
	sLock.Lock();

	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, DO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	if( bState ) 
	{
		m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] |= 0x01 << io_buffer.wBit;
	}
	else m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] &= ~( 0x01 << io_buffer.wBit );

	sLock.Unlock();

	return true;

	// jemoon : 070823 -> use outbuffer
	// Port: 301, IGrp: 0xF030, IOffs: ret_io_buffer.wAddr, Len: 1
	//return TcAdsWrite( ADSIGRP_IOIMAGE_RWOB, GetOutNodeSize( io_buffer.wMac_id ) + io_buffer.wAddr, 1, &m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] );
}

//--------------------------------------------------------------------
// Procedure   : GetDoState
// Description : 
// Arguments   : UINT no : 1, 2, 3, ...
//				 BOOL bState : TRUE=>ON, FALSE=>OFF
// Returns     : BOOL
//--------------------------------------------------------------------
BOOL XCtlEtherCAT::GetDoState(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, DO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	return ( m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] >> io_buffer.wBit ) & 0x01 ? TRUE : FALSE;
}

void XCtlEtherCAT::SetDiState(UINT no, BOOL bState)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( no, DI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return ;

	if( bState ) m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] |= 0x01 << io_buffer.wBit;
	else m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] &= ~( 0x01 << io_buffer.wBit );
}

WORD XCtlEtherCAT::AioRead(UINT uNo)
{
	CSingleLock sLock(&m_mutex );	// HJYOU :   [  ʴ  ] [2/23/2010]
	sLock.Lock();

	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

//	if( !ReadIo( io_buffer.wMac_id, m_pInBuf[io_buffer.wMac_id - 1] ) ) return 0;

	//  AIO 12 bit ( Beckhoff, KL3064 : 0 ~ 10V )
	WORD wResult = ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] & 0xF0 ) + 
		( ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] & 0x7F ) << 8 );

	sLock.Unlock();

	return wResult;
}

BOOL XCtlEtherCAT::AioWrite(UINT uNo, WORD wValue)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] = wValue & 0x00F0;
	m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] = ( wValue >> 8 ) & 0x007F;

	return true;

	// jemoon : 070823 -> use outbuffer
	// Port: 301, IGrp: 0xF030, IOffs: ret_io_buffer.wAddr, Len: 1
	//return TcAdsWrite( ADSIGRP_IOIMAGE_RWOB, GetOutNodeSize( io_buffer.wMac_id ) + io_buffer.wAddr, 2, &m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] );
}

WORD XCtlEtherCAT::GetAoValue(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AO );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	//  AIO 12 bit ( Beckhoff, KL3064 : 0 ~ 10V )
	WORD wResult = ( m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] & 0xF0 ) + 
		( ( m_pOutBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] & 0x7F ) << 8 );

	return wResult;
}

void XCtlEtherCAT::SetAiValue(UINT uNo, WORD wValue)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, AI );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return;

	m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] = wValue & 0x00F7;
	m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] = ( wValue >> 8 ) & 0x007F;
}

WORD XCtlEtherCAT::GetPulseCnt(UINT uNo)
{
	IO_BUFFER io_buffer = IoNo2AddrBit( uNo, PC );

	// Node  No Ҵϸ Return FALSE
	if( io_buffer.wMac_id == 0 ) return FALSE;

	// Pulse Counter KL1501( [control][2nd low][2nd high][1st low][1st high] )
	// [control]	: Control Byte //   ... ߿  ...
	// [2nd low]	: 2° Channel  Byte
	// [2nd high]	: 2° Channel  Byte
	// [1st low]	: 1° Channel  Byte
	// [1st high]	: 1° Channel  Byte
	WORD wResult = ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr] & 0xFF ) + 
		( ( m_pInBuf[io_buffer.wMac_id - 1][io_buffer.wAddr + 1] & 0x7F ) << 8 );

	return wResult;
}

//--------------------------------------------------------------------
// Procedure   : ResetOutNode
// Description : 
// Arguments   : void
// Returns     : void
//--------------------------------------------------------------------
void XCtlEtherCAT::ResetOutNodeAll()
{
	for( UINT i=0; i < m_uMaxNode; i++) ResetOutNode( i );
}

void XCtlEtherCAT::ResetOutNode(UINT uNode)
{
	if(uNode < 0 || uNode >= m_uMaxNode) return;

	WORD wInputSize		= m_pNodeInputSize[uNode];
	WORD wOutputSize	= m_pNodeOutputSize[uNode];
	UINT i;
	for( i=0; i < wInputSize; i++) m_pInBuf[uNode][i] = 0x00;
	for( i=0; i < wOutputSize; i++) m_pOutBuf[uNode][i] = 0x00;

	// Port: 301, IGrp: 0xF030, IOffs: ret_io_buffer.wAddr, Len: 1
//	TcAdsWrite( ADSIGRP_IOIMAGE_RWOB, GetOutNodeSize( uNode + 1 ), wOutputSize, m_pOutBuf[uNode] );
}

//---------------------------------------------------------------------------
// Send the message to parent window
// wait for the termination of message process : SendMessage()
void XCtlEtherCAT::Msg(UINT uMsg,  WPARAM wParam, LPARAM lParam)
{
	if( m_pOwner == NULL ) return;

	m_pOwner->PostMessage( uMsg, wParam, lParam );
}

void XCtlEtherCAT::SetOwner(CWnd *pOwner)
{
	m_pOwner = pOwner;	
}

//---------------------------------------------------------------------------
// Analog Input Calibration
void XCtlEtherCAT::SetCalPara(int nID, float fRealMax, float fRealMin, int nAdcMax, int nAdcMin)
{
	m_IniInfo.Open("AiControl", "AiControl");

	m_aiInfo[nID].fRealMax = fRealMax;
	m_aiInfo[nID].fRealMin = fRealMin;
	m_aiInfo[nID].nAdcMax = nAdcMax;
	m_aiInfo[nID].nAdcMin = nAdcMin;

	CString sEntry;
	CString sVal;

	sEntry.Format( "AI_REAL_MAX%d", nID );
	sVal.Format( "%.8f", m_aiInfo[nID].fRealMax );
	m_IniInfo.SetStringValue( "REAL_MAX", sEntry, sVal , TRUE);

	sEntry.Format( "AI_REAL_MIN%d", nID );
	sVal.Format( "%.8f", m_aiInfo[nID].fRealMin );
	m_IniInfo.SetStringValue( "REAL_MIN", sEntry, sVal , TRUE);

	sEntry.Format( "AI_ADC_MAX%d", nID );
	sVal.Format( "%f", (float)m_aiInfo[nID].nAdcMax );
	m_IniInfo.SetStringValue( "ADC_MAX", sEntry, sVal , TRUE);

	sEntry.Format( "AI_ADC_MIN%d", nID );
	sVal.Format( "%f", (float)m_aiInfo[nID].nAdcMin );
	m_IniInfo.SetStringValue( "ADC_MIN", sEntry, sVal , TRUE);
}

void XCtlEtherCAT::GetCalPara(int nID, float& fRealMax, float& fRealMin, int& nAdcMax, int& nAdcMin, BOOL bDefault, int nSensorType )
{
	m_IniInfo.Open("AiControl", "AiControl");
	CString sEntry;
	CString sVal;

	if( bDefault )
		//m_aiInfo[nID].fRealMax = fRealMax = m_aiInfoDefault[nID].fRealMax;
		m_aiInfo[nID].fRealMax = fRealMax = m_aiPreSet[nSensorType].fRealMax;
	else
	{
		sEntry.Format( "AI_REAL_MAX%d", nID );
//		sVal.Format( "%.8f", fRealMax );
		sVal = m_IniInfo.GetStringValue( "REAL_MAX", sEntry, "0.0");
		m_aiInfo[nID].fRealMax = fRealMax = (float) atof( sVal );
	}
	if( bDefault )
		//m_aiInfo[nID].fRealMin = fRealMin = m_aiInfoDefault[nID].fRealMin;
		m_aiInfo[nID].fRealMin = fRealMin = m_aiPreSet[nSensorType].fRealMin;
	else
	{
		sEntry.Format( "AI_REAL_MIN%d", nID );
//		sVal.Format( "%.8f", fRealMin );
		sVal = m_IniInfo.GetStringValue( "REAL_MIN", sEntry, "0.0");
		m_aiInfo[nID].fRealMin = fRealMin = (float) atof( sVal );
	}

	if( bDefault )
		//m_aiInfo[nID].nAdcMax = nAdcMax = m_aiInfoDefault[nID].nAdcMax;
		m_aiInfo[nID].nAdcMax = nAdcMax = m_aiPreSet[nSensorType].nAdcMax;
	else
	{
		sEntry.Format( "AI_ADC_MAX%d", nID );
//		sVal.Format( "%d", nAdcMax );
		sVal = m_IniInfo.GetStringValue( "ADC_MAX", sEntry, "0.0");
		m_aiInfo[nID].nAdcMax = nAdcMax = atoi( sVal );
	}

	if( bDefault )
		//m_aiInfo[nID].nAdcMin = nAdcMin = m_aiInfoDefault[nID].nAdcMin;
		m_aiInfo[nID].nAdcMin = nAdcMin = m_aiPreSet[nSensorType].nAdcMin;
	else
	{
		sEntry.Format( "AI_ADC_MIN%d", nID );
//		sVal.Format( "%d", nAdcMin );
		sVal = m_IniInfo.GetStringValue( "ADC_MIN", sEntry, "0.0");
		m_aiInfo[nID].nAdcMin = nAdcMin = atoi( sVal );
	}
}

//--------------------------------------------------------------------
// Get the calibrated value
// + Low Pass Filter
//--------------------------------------------------------------------
float XCtlEtherCAT::GetCalAI(int nID)
{	
	//return 0;
	
	int nADC = AioRead(nID);


	float f = m_pFilterGain[nID];
	if( !m_pFirstVal[nID] )
	{
		m_pFirstVal[nID] = TRUE;
		m_pOldAdc[nID] = nADC;
	}
	nADC = (int)( (float)nADC * f + (float)m_pOldAdc[nID] * ( 1. - f ) );
	m_pOldAdc[nID] = nADC;

	float	fRealMax, fRealMin;
	int		nAdcMax, nAdcMin;
	nAdcMax = m_aiInfo[nID].nAdcMax;
	nAdcMin = m_aiInfo[nID].nAdcMin;
	if( m_aiInfo[nID].nScale == scaleCOMMON )
	{
		fRealMax = m_aiInfo[nID].fRealMax;
		fRealMin = m_aiInfo[nID].fRealMin;
	}
	else if( m_aiInfo[nID].nScale == scaleLOG )
	{
		fRealMax = (float)log( (double)m_aiInfo[nID].fRealMax );
		fRealMin = (float)log( (double)m_aiInfo[nID].fRealMin );
	}
	else if( m_aiInfo[nID].nScale == scaleLOG10 )
	{
		fRealMax = (float)log10( (double)m_aiInfo[nID].fRealMax );
		fRealMin = (float)log10( (double)m_aiInfo[nID].fRealMin );
	}

	int nDiffAdc = nAdcMax - nAdcMin;
	float fDiffReal = fRealMax - fRealMin;

	if( nDiffAdc == 0 ) return (float) nADC;

	float fVal = ( fDiffReal / (float) nDiffAdc ) *
		( nADC - m_aiInfo[nID].nAdcMin ) + fRealMin;

	if( m_aiInfo[nID].nScale == scaleCOMMON )
	{
		return fVal;
	}
	else if( m_aiInfo[nID].nScale == scaleLOG )
	{
		return (float)( exp( (double)fVal ) );
	}
	else if( m_aiInfo[nID].nScale == scaleLOG10 )
	{
		return (float)( pow( 10., (double)fVal ) );
	}
	return fVal;
}

void XCtlEtherCAT::SetRawAI(int nID, float fVal)
{
	WORD wADC;
	int nDiffAdc = m_aiInfo[nID].nAdcMax - m_aiInfo[nID].nAdcMin;
	float fDiffReal = m_aiInfo[nID].fRealMax - m_aiInfo[nID].fRealMin;
		
	wADC = (int)((fVal - m_aiInfo[nID].fRealMin)*(nDiffAdc / fDiffReal)) + m_aiInfo[nID].nAdcMin;

	SetAiValue( nID, wADC );
}


void XCtlEtherCAT::SetInterlockPara(int nID, BOOL bSet, int nUpLimit, int nLoLimit)
{
	m_IniInfo.Open("AiControl", "AiControl");

	m_aiInfo[nID].bUseInterlock = bSet;
	m_aiInfo[nID].nLoLimit = nLoLimit;
	m_aiInfo[nID].nUpLimit = nUpLimit;

	CString sEntry;
	CString sVal;

	sEntry.Format( "USE_INTERLOCK%d", nID );
	m_IniInfo.SetIntValue( "LIMIT_INTERLOCK", sEntry, (int)bSet , TRUE);

	sEntry.Format( "AI_UP_LIMIT%d", nID );
	sVal.Format("%d", m_aiInfo[nID].nUpLimit);
	m_IniInfo.SetStringValue( "UP_LIMIT", sEntry, sVal , TRUE);

	sEntry.Format( "AI_LO_LIMIT%d", nID );
	sVal.Format("%d", m_aiInfo[nID].nLoLimit);
	m_IniInfo.SetStringValue( "LO_LIMIT", sEntry, sVal , TRUE);
}

void XCtlEtherCAT::GetInterlockPara(int nID, BOOL &bSet, int &nUpLimit, int &nLoLimit)
{
	m_IniInfo.Open("AiControl", "AiControl");
	CString sEntry;
	CString sVal;
	
	sEntry.Format( "USE_INTERLOCK%d", nID );
	int nVal = m_IniInfo.GetIntValue( "LIMIT_INTERLOCK", sEntry, 0);
	m_aiInfo[nID].bUseInterlock = bSet = (BOOL) nVal;

	sEntry.Format( "AI_UP_LIMIT%d", nID );
	sVal = m_IniInfo.GetStringValue( "UP_LIMIT", sEntry, "0");
	nUpLimit = m_aiInfo[nID].nUpLimit = atoi(sVal);

	sEntry.Format( "AI_LO_LIMIT%d", nID );
	sVal = m_IniInfo.GetStringValue( "LO_LIMIT", sEntry, "0");
	nLoLimit = m_aiInfo[nID].nLoLimit = atoi(sVal);
}

void XCtlEtherCAT::SetWarningPara(int nID, BOOL bSet, int nUpWarning, int nLoWarning)
{
	m_IniInfo.Open("AiControl", "AiControl");
	m_aiInfo[nID].bUseInterlock = bSet;
	m_aiInfo[nID].nLoWarning = nLoWarning;
	m_aiInfo[nID].nUpWarning = nUpWarning;
	
	CString sEntry;
	CString sVal;
	
	sEntry.Format( "USE_WARNING%d", nID );
	m_IniInfo.SetIntValue( "WARNING_INTERLOCK", sEntry, (int)bSet , TRUE);

	sEntry.Format( "AI_UP_WARN%d", nID );
	sVal.Format("%d", m_aiInfo[nID].nUpWarning);
	m_IniInfo.SetStringValue( "UP_WARN", sEntry, sVal, TRUE);

	sEntry.Format( "AI_LO_WARN%d", nID );
	sVal.Format("%d", m_aiInfo[nID].nLoWarning);
	m_IniInfo.SetStringValue( "LO_WARN", sEntry, sVal, TRUE);
}

void XCtlEtherCAT::GetWarningPara(int nID, BOOL &bSet, int &nUpWarning, int &nLoWarning)
{
	m_IniInfo.Open("AiControl", "AiControl");
	CString sEntry;
	CString sVal;

	sEntry.Format( "USE_WARNING%d", nID );
	int nVal = m_IniInfo.GetIntValue( "WARNING_INTERLOCK", sEntry, 0);
	m_aiInfo[nID].bUseInterlock = bSet = (BOOL) nVal;

	sEntry.Format( "AI_UP_WARN%d", nID );
	sVal = m_IniInfo.GetStringValue( "UP_WARN", sEntry, "0");
	nUpWarning = m_aiInfo[nID].nUpWarning = atoi(sVal);

	sEntry.Format( "AI_LO_WARN%d", nID );
	sVal = m_IniInfo.GetStringValue( "LO_WARN", sEntry, "0");
	nLoWarning = m_aiInfo[nID].nLoWarning = atoi(sVal);
}

CString XCtlEtherCAT::GetAiName( int nID )
{
	m_IniInfo.Open("AiOcxPraSet", "AiControl");
	CString sEntry;
	CString sVal;

	sEntry.Format( "AI_ITEM_NAME%d", nID );
	sVal = m_IniInfo.GetStringValue( "ITEM_NAME", sEntry, "SPARE");
	m_aiInfo[nID].sName = sVal;

	return sVal;
}

void XCtlEtherCAT::SetAiName( int nID, CString sName )
{
	m_IniInfo.Open("AiOcxPraSet", "AiControl");
	CString sEntry;
	
	sEntry.Format( "AI_ITEM_NAME%d", nID );
	m_IniInfo.SetStringValue( "ITEM_NAME", sEntry, (LPCTSTR)sName);
	m_aiInfo[nID].sName = sName;
}

CString XCtlEtherCAT::GetAiUnitName( int nID )
{
	m_IniInfo.Open("AiOcxPraSet", "AiControl");
	CString sEntry;
	CString sVal;

	sEntry.Format( "AI_UNIT_NO%d", nID );
	sVal.Format("%d", m_IniInfo.GetIntValue( "UNIT_NO", sEntry, 0) );
	m_aiInfo[nID].sUnitName = sVal;

	return sVal;
}

void XCtlEtherCAT::SetAiUnitName( int nID, CString sName )
{
	m_IniInfo.Open("AiOcxPraSet", "AiControl");
	CString sEntry;

	sEntry.Format( "AI_UNIT_NO%d", nID );
	m_IniInfo.SetIntValue( "UNIT_NO", sEntry, atoi(sName) );
	m_aiInfo[nID].sUnitName = sName;
}

void XCtlEtherCAT::ResetInNodeAll()
{
	for( UINT i = 0 ; i < m_uMaxNode ; i ++ )
	{
		WORD wInputSize	= m_pNodeInputSize[i];
		for( UINT j=0; j < wInputSize; j++) 
			m_pInBuf[i][j] = 0x00;
	}
}

void XCtlEtherCAT::GetFilterGain(int nID, float &f)
{
	m_IniInfo.Open("AiControl", "AiControl");
	CString sEntry, sVal;
	sEntry.Format( "AI_FILTER_GAIN%d", nID );
	sVal = m_IniInfo.GetStringValue( "FILTER_GAIN", sEntry, "1.0");
	f = (float)(atof( sVal ));
}

void XCtlEtherCAT::SetFilterGain(int nID, float f)
{
	m_IniInfo.Open("AiControl", "AiControl");

	if( f < (float)0.1 ) f = (float)0.1;
	else if( f > 1. ) f = 1.;

	m_pFilterGain[nID] = f;

	CString sEntry, sVal;
	sEntry.Format( "AI_FILTER_GAIN%d", nID );
	sVal.Format( "%f", f );
	m_IniInfo.SetStringValue( "FILTER_GAIN", sEntry, (LPCTSTR)sVal);
}

void XCtlEtherCAT::SetAiCalScale(int nID, int nScale)
{
	m_IniInfo.Open("AiControl", "AiControl");
	CString sEntry, sVal;
	sEntry.Format( "SCALE%d", nID );
	sVal.Format("%d", nScale);
	m_IniInfo.SetStringValue( "CALIBRATION_SCALE", sEntry, sVal);
	m_aiInfo[nID].nScale = nScale;
}

void XCtlEtherCAT::GetAiCalScale(int nID, int &nScale)
{
	m_IniInfo.Open("AiControl", "AiControl");
	CString sEntry, sVal;
	sEntry.Format( "SCALE%d", nID );
	sVal = m_IniInfo.GetStringValue( "CALIBRATION_SCALE", sEntry, "0");
	m_aiInfo[nID].nScale = nScale = atoi(sVal);
}

//Status Bit Monitor...
int XCtlEtherCAT::IsActive()
{
	return m_bConnect;
}

BOOL XCtlEtherCAT::IsConnected()
{
	if( !TcAdsRead( ADSIGRP_IOIMAGE_RWIB, m_uStatusOffset, m_nStatusByteSize, m_pStatusBuf ) )
	{
//		return -100;
		m_bConnect = FALSE;
		return FALSE;
	}

	for( WORD i = 0 ; i < m_uMaxNode ; i++ )
	{
		if( ( m_pStatusBuf[i / 8] & ( 0x01 << ( i % 8 ) ) ) )
		{
//			return i;
			m_bConnect = TRUE;
			return FALSE;
		}
	}

//	return -1;
	m_bConnect = TRUE;
	return TRUE;
}
