//---------------------------------------------------------------------
// Copyright		: DMS Co., Ltd
// File Name		: XCtlEtherCAT.h
// Modified Date	: 2005.05.20
// Version			: Ver1.0
// Programmer		: Lee yongseok
// Description		: Interface for the XCtlEtherCAT class.
//					XDio   IO Accessϱ Ͽ 
//---------------------------------------------------------------------
// Import 
#import "..\TwinCAT\TCATSysSrv.tlb"  no_namespace
#import "..\TwinCAT\TCatSysSrv.exe" //LYS20070213 

#if !defined(AFX_XCTLEtherCAT_H__F5DFBB53_F343_11D4_9245_000086500CF9__INCLUDED_)
#define AFX_XCTLEtherCAT_H__F5DFBB53_F343_11D4_9245_000086500CF9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// XCtlEtherCAT.h : header file
//

#include "Mmsystem.h"				// timeSetEvent Header file
#include "XEtherCAT.h"
#include "..\FactoryLib\IniData.h"	// Added by ClassView

// Windows messages for event notification
#define WM_DEVICENET_MONITOR	 WM_USER + 1

// status structure
typedef struct
{
	unsigned short wMac_id;
	unsigned short wAddr;
	unsigned short wBit;
} IO_BUFFER;

/////////////////////////////////////////////////////////////////////////////
// XCtlEtherCAT thread

class XCtlEtherCAT : public CWinThread, public XEtherCAT
{
	DECLARE_DYNCREATE(XCtlEtherCAT)
public:
	XCtlEtherCAT();           // protected constructor used by dynamic creation
	virtual ~XCtlEtherCAT();

// Attributes
public:
	BYTE** m_pInBuf;
	BYTE** m_pOutBuf;
	

	//jemoon : 070823	// add variable for output buffering
	BYTE* m_pTotInBuf;
	BYTE* m_pTotOutBuf;
	int m_nTotInputSize;
	int m_nTotOutputSize;
	int* m_pNodeInputSize;
	int* m_pNodeOutputSize;

	UINT* m_pPointDI;  
	UINT* m_pPointDO;
	UINT* m_pPointAI;
	UINT* m_pPointAO;
	UINT* m_pPointPC;

	int * m_pOldAdc;
	BOOL* m_pFirstVal;

	BYTE*	m_pStatusBuf;		// Status Bit ¸  Byte
	WORD	m_uStatusOffset;	// Status Bit  Address
	int		m_nStatusByteSize;	// Status Byte Array ũ...
	BOOL	m_bConnect;			// Box Status  Flag...
	int		m_nOldAdsState;

	float * m_pFilterGain;
	CIniData	m_IniInfo;

	struct aiInfo
	{
		int nAdcMax;
		int nAdcMin;
		float fRealMax;
		float fRealMin;
		int nLoLimit;
		int nUpLimit;
		int nLoWarning;
		int nUpWarning;
		BOOL bUseInterlock;
		BOOL bUseWarning;
		CString sName;
		CString	sUnitName;
		int nScale;
	} * m_aiInfo;

	struct aiInfo * m_aiInfoDefault;
	enum _aiSensorType
	{
		enPG35,		
		enDW_605,		
		enSENSYS,		
		enJLK_25,		
		enULK_25,		
		enCKD_FLOW,	
		enPT100OHM,	
		enLEVEL_GAGE,
		enTHERMO_SENSOR,
		enMAX_SENSOR_TYPE
	};

	enum _aiScale{
		scaleCOMMON,
		scaleLOG,
		scaleLOG10,
	};
	
	static AI_PRESET m_aiPreSet[enMAX_SENSOR_TYPE];
	int m_nAiNo;

// Operations
protected:
	BOOL IsConnected();
public:
	int  IsActive();
	WORD GetPulseCnt(UINT uNo);
	void GetAiCalScale(int nID, int& nScale);
	void SetAiCalScale(int nID, int nScale = scaleCOMMON );
	void SetFilterGain(int nID, float f);
	void GetFilterGain(int nID, float& f);
	WORD GetAoValue(UINT uNo);
	void ResetInNodeAll();
	void SetRawAI(int nID, float fVal);
	void SetDiState(UINT no, BOOL bState);
	void SetAiValue(UINT uNo, WORD wValue);
	void GetWarningPara(int nID, BOOL &bSet, int &nUpWarning, int &nLoWarning);
	void SetWarningPara(int nID, BOOL bSet, int nUpWarning, int nLoWarning);
	void GetInterlockPara(int nID, BOOL &bSet, int &nUpLimit, int &nLoLimit);
	void SetInterlockPara(int nID, BOOL bSet, int nUpLimit, int nLoLimit);
	CString GetAiName( int nID );
	void SetAiName( int nID, CString sName );
	CString GetAiUnitName( int nID );
	void SetAiUnitName( int nID, CString sName );
	void Watch();
	void SetOwner(CWnd *pOwner);
	void Msg(UINT uMsg,  WPARAM wParam, LPARAM lParam);
	void ResetOutNodeAll();
	void ResetOutNode(UINT uNode);
	void SetCalPara(int nID, float fRealMax, float fRealMin, int nAdcMax, int nAdcMin);
	void GetCalPara(int nID, float& fRealMax, float& fRealMin, int& nAdcMax, int& nAdcMin, BOOL bDefault = FALSE, int nSensorType = 0 );
	float GetCalAI(int nID);

	BOOL Start(CWnd* pOwner, UINT uMaxNode, UINT* pPointAI, UINT* pPointAO, 
		UINT* pPointDI, UINT* pPointDO, UINT* pPointPC = NULL );
	BOOL End();
	BOOL GetDoState(UINT uNo);
	BOOL DioRead(UINT uNo);
	BOOL DioWrite(UINT uNo, BOOL bState);
	WORD AioRead(UINT uNo);
	BOOL AioWrite(UINT uNo, WORD wValue);

	BOOL ReadData();
	BOOL WriteData();
	void UpdateState();

	IO_BUFFER IoNo2AddrBit(UINT uNo, int nKind);
	WORD GetInNodeSize( UINT uNode );
	WORD GetOutNodeSize( UINT uNode );
	UINT GetMaxNode() { return m_uMaxNode; }

	enum deviceIO
	{
		PC,
		DI,
		DO,
		AI,
		AO
	};

	CMutex	m_mutex;	// HJYOU  [2/23/2010]

private:
// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(XCtlEtherCAT)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation
protected:
	CWnd*	m_pOwner;
	HANDLE	m_hDead;
	HANDLE	m_hKill;

	UINT	m_uMaxNode;
	int		m_uMaxBit;

	// Generated message map functions
	//{{AFX_MSG(XCtlEtherCAT)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_XCTLETHERCAT_H__F5DFBB53_F343_11D4_9245_000086500CF9__INCLUDED_)
