// XEtherCAT.cpp: implementation of the XEtherCAT class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "XEtherCAT.h"
#include "..\x\xlog.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

XLog TcLog( "TwinCATLog", XLog::stampNoDateTime, TRUE ); // TRUE : Append
/////////////////////////////////////////////////////////////////////////////
// XDeviceNet
//------------------------------------------------------
// Procedure   : m_sDeviceStatus
// Description : Initialize the static variables
// Arguments   : static variable
// Returns     : 
//------------------------------------------------------
CString XEtherCAT::m_sAdsState[] =
{ 
	"INVALID",
	"IDLE",
	"RESET",
	"INIT",
	"START",
	"RUN",
	"STOP",
	"SAVECFG",
	"LOADCFG",
	"POWERFAILURE",
	"POWERGOOD",
	"ERROR",
	"SHUTDOWN",
	"SUSPEND",
	"RESUME",
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XEtherCAT::XEtherCAT()
{
	m_bActive       = FALSE;
	m_bOpen			= FALSE;
	m_nAdsState		= 0;
	m_nErr			= 0;
	
	int nLength = sizeof(m_Addr.netId.b) / sizeof(char);
	
	for(int i=0; i <nLength; i++)
	{
		m_Addr.netId.b[i] = ' ';
	}

	m_Addr.port = 0;
	m_pAddr = &m_Addr;
}

XEtherCAT::~XEtherCAT()
{

}

//------------------------------------------------------
// Procedure   : TcAdsOpen
// Description : Destruction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
BOOL XEtherCAT::TcAdsOpen(int AMSPORT)
{
	m_pAddr = &m_Addr;
	m_nPort = AdsPortOpen();
	m_nErr = AdsGetLocalAddress( m_pAddr );

	CString	sLog;
	if ( m_nErr ) 
	{
		TRACE( "Error: AdsGetLocalAddress : %02X\n", m_nErr );
		sLog.Format( "EtherCAT \t\t\t\tAdsGetLocalAddress Error(%02X).", m_nErr );
		TcLog.TextOut( sLog );
		return FALSE;
	}

	m_pAddr->port = AMSPORT;
	m_bOpen = TRUE;

	return TRUE;
}

//------------------------------------------------------
// Procedure   : TcAdsClose
// Description : Destruction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
void XEtherCAT::TcAdsClose()
{
	m_nPort = AdsPortClose();
	TcStop();  //LYS20070213 TwinCAT System Stop

	m_bOpen = FALSE;
}


//------------------------------------------------------
// Procedure   : TcAdsRead
// Description : Destruction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
BOOL XEtherCAT::TcAdsRead(unsigned long indexGroup, unsigned long indexOffset, unsigned long length, BYTE *Data)
{
	CString	sLog;
	static	int	nErrCode = 0;
	m_nErr = AdsSyncReadReq( m_pAddr, indexGroup, indexOffset, length, Data );

	if ( m_nErr ) 
	{
#ifndef _MY_DEBUG
		TRACE( "Error: AdsSyncReadReq : %02X\n", m_nErr );
		if( nErrCode != m_nErr )
		{
			nErrCode = m_nErr;
			sLog.Format( "EtherCAT \t\t\t\tAdsSyncReadReq Error(%02X).", m_nErr );
			TcLog.TextOut( sLog );
		}
#endif
		return FALSE;
	}

	return TRUE;
}

//------------------------------------------------------
// Procedure   : TcAdsWrite
// Description : Destruction
// Arguments   : void
// Returns     : 
//------------------------------------------------------
BOOL XEtherCAT::TcAdsWrite(unsigned long indexGroup, unsigned long indexOffset, unsigned long length, BYTE Data[])
{
	CString	sLog;
	static	int	nErrCode = 0;
	m_nErr = AdsSyncWriteReq( m_pAddr, indexGroup, indexOffset, length, Data );

	if ( m_nErr ) 
	{
#ifndef _MY_DEBUG
		TRACE( "Error: AdsSyncWriteReq : %02X\n", m_nErr );
		if( nErrCode != m_nErr )
		{
			nErrCode = m_nErr;
			sLog.Format( "EtherCAT \t\t\t\tAdsSyncWriteReq Error(%02X).", m_nErr );
			TcLog.TextOut( sLog );
		}
#endif
		return FALSE;
	}

	return TRUE;
}

//--------------------------------------------------------------------
// Procedure   : Monitor
// Description : AdsSyncReadStateReq()
//				 Monitoring the current ADS status
// Arguments   : WORD wMacID
// Return      : BOOL 
//				FALSE	: Status is not changed
//				TRUE	: Status is changed
//--------------------------------------------------------------------
BOOL XEtherCAT::Monitor()
{
	if( !m_bOpen ) return FALSE;

	BOOL bRv = FALSE;
	//long nAdsState = m_TcComServer->GetSystemState();
	static long nPreviousAdsState = -1;

	CString	sLog;
	CString strBuffer;
//LYS20070801
	try
	{
		m_nAdsState = m_TcComServer->GetSystemState();
	}
	catch (_com_error &e)
	{
		strBuffer.Format("Code = %08lX:nCode meaning = %snSource = %snDescription = %sn", 
									e.Error(), e.ErrorMessage(), e.Source(), e.Description());

		m_bOpen = FALSE;
		m_nAdsState = ADSSTATE_SHUTDOWN;

//      TRACE(strBuffer);
	}

	// update screen only if status changes
	if( nPreviousAdsState != m_nAdsState )
	{
		// save the current status
		nPreviousAdsState = m_nAdsState;

		// display device status
		switch( m_nAdsState )
		{
		case ADSSTATE_INVALID:
		case ADSSTATE_IDLE:
		case ADSSTATE_RESET:
		case ADSSTATE_INIT:
		case ADSSTATE_START:
		case ADSSTATE_RUN:
		case ADSSTATE_STOP:
		case ADSSTATE_SAVECFG:
		case ADSSTATE_LOADCFG:
		case ADSSTATE_POWERFAILURE:
		case ADSSTATE_POWERGOOD:
		case ADSSTATE_ERROR:
		case ADSSTATE_SUSPEND:
		case ADSSTATE_RESUME:
			m_sCurAdsState = m_sAdsState[m_nAdsState];
			sLog.Format( "EtherCAT \t\t\t\tAdsState (%s):", m_sCurAdsState );
			TcLog.TextOut( sLog );
			break;
		case ADSSTATE_SHUTDOWN:  //LYS20070801
			m_sCurAdsState = m_sAdsState[m_nAdsState];
			sLog.Format( "EtherCAT \t\t\t\tAdsState (%s):", m_sCurAdsState );
			TcLog.TextOut( sLog );
			sLog.Format( "EtherCAT \t\t\t\tCode(%s):", strBuffer );
			TcLog.TextOut( sLog );

//			AfxMessageBox( "Please!!.Execute again Dics program after TwinCAT manager start", MB_ICONSTOP | MB_SYSTEMMODAL | MB_OK );
			break;
		default:
			m_sCurAdsState = "Unknown status";
			break;
		}
		bRv = TRUE;
	}

	return bRv;
}

BOOL XEtherCAT::TcStart()
{
#ifdef _MY_DEBUG
	return TRUE;
#endif

	try //LYS20070825
	{
		HRESULT hr = m_TcComServer.CreateInstance("TcSystemSrv.TcSystemServer.1");	
		// Creates the SystemServer COM Object

		_variant_t par;
		m_TcComServer->StartSystem(&par);
	}
	catch ( _com_error ) { return FALSE;
	}
	return TRUE;
}

BOOL XEtherCAT::TcStop()
{
#ifdef _MY_DEBUG
	return TRUE;
#endif

	try //LYS20070825
	{
		HRESULT hr = m_TcComServer.CreateInstance("TcSystemSrv.TcSystemServer.1");	
		// Creates the SystemServer COM Object

		_variant_t par;
		m_TcComServer->StopSystem();
	}
	catch ( _com_error ) { return FALSE;
	}
	return TRUE;
}
