// XLT300.cpp: implementation of the XLT300 class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "..\Dics.h"
#include "XLT300.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
XLT300::XLT300()
{
	m_fCurSvValue = new float[MAX_LT300_NO];
	m_fCurPvValue = new float[MAX_LT300_NO];
}

XLT300::~XLT300()
{
	if( m_fCurSvValue != NULL ) delete [] m_fCurSvValue; m_fCurSvValue = NULL;
	if( m_fCurPvValue != NULL ) delete [] m_fCurPvValue; m_fCurPvValue = NULL;
}

//////////////////////////////////////////////////////////////////////
// SetSvValue(int nSlaveAddress, int nSv)
// return value 
//	1 : Command Result is not Received
//	0 : Command Result Received
//////////////////////////////////////////////////////////////////////
int XLT300::SetSvValue(int nSlaveAddress, int nSv)
{
	UINT nFnCode;
	nFnCode = fncdSET_ANALOG_PARA;
	UINT uRefNo	= paraREMOTE_SV1 - refANALOG_PARA;
	
	if( nSlaveAddress < 1 || nSlaveAddress > MAX_LT300_NO ) return 0;
	m_fCurSvValue[nSlaveAddress-1] = (float)nSv;

	m_sSendData.Format( "%02X%02X%04X%04X", nSlaveAddress, nFnCode, uRefNo, nSv );
	
	int nTemp;
	nTemp = nSlaveAddress + nFnCode + ( ( uRefNo & 0xFF00 ) >> 8 ) + ( uRefNo & 0xFF )
		+ ( ( nSv & 0xFF00 ) >> 8 ) + ( nSv & 0xFF );
	int nLRC = GetLRC( nTemp );

	CString sLRC;
	sLRC.Format( "%02X", nLRC );

	HexaString2Integer( sLRC );

	m_sSendData += sLRC;

	m_sSendData.Insert( 0, ':' );
	m_sSendData += "\r\n";

	for( int i = 0 ; i < m_arSendData.GetSize() ; i ++ )
	{
		if( m_arSendData.GetAt( i ) == m_sSendData ) break;
	}
	if( i == m_arSendData.GetSize() ) m_arSendData.Add( m_sSendData );
	
	BOOL bRv = Write( m_sSendData, m_sSendData.GetLength() );
	return 0;
}

//////////////////////////////////////////////////////////////////////
// GetLRC(CString sValue)
//////////////////////////////////////////////////////////////////////
int XLT300::GetLRC(int nValue)
{
	nValue = nValue & 0xFF;

	int nRv = ( 0xFF ^ nValue ) + 1;	
	return nRv;
}

//////////////////////////////////////////////////////////////////////
// GetCurSvValue(int nSlaveAddress)
// return value 
//	1 : Command Result is not Received
//	0 : Command Result Received
//////////////////////////////////////////////////////////////////////
int XLT300::GetCurSvValue(int nSlaveAddress)
{
	int nFnCode		= fncdGET_ANALOG_PARA;
	int nRefNo		= paraREMOTE_SV1 - refANALOG_PARA;

	m_sSendData.Format( "%02X%02X%04X%04X", nSlaveAddress, nFnCode, nRefNo, 0x04 );

	int nTemp;
	nTemp = nSlaveAddress + nFnCode + ( ( nRefNo & 0xFF00 ) >> 8 ) + ( nRefNo & 0xFF )
		+ ( ( 0x04 & 0xFF00 ) >> 8 ) + ( 0x04 & 0xFF );
	int nLRC = GetLRC( nTemp );

	CString sLRC;
	sLRC.Format( "%02X", nLRC );

	m_sSendData += sLRC;

	m_sSendData.Insert( 0, ':' );
	m_sSendData += "\r\n";
	
	BOOL bRv = Write( m_sSendData, m_sSendData.GetLength() );
	return 0;
}

//////////////////////////////////////////////////////////////////////
// GetCurPvValue(int nSlaveAddress)
//////////////////////////////////////////////////////////////////////
int XLT300::GetCurPvValue( int nSlaveAddress )
{
	int nRv = 0;
	return nRv;
}

void XLT300::DataReceived()
{
	CString sCommand;
	if( m_sReceivedData.GetLength() < 9 ) return;

	sCommand = m_sReceivedData.Mid( 1, 8 );

	for( int i = 0 ; i < m_arSendData.GetSize() ; i ++ )
	{
		if( m_arSendData.GetAt( i ).Find( sCommand ) != -1 )
		{
			m_arSendData.RemoveAt( i );
		}
	}
}

CString XLT300::GetCommandResult(CString sCommand)
{
	CString sRv;
	
	return sRv;
}

#include <math.h>
int XLT300::HexaString2Integer(CString sHex)
{
	int nRv = 0, nV;
	CString sTemp;
	for( int i = 0 ; i < sHex.GetLength() ; i ++ )
	{
		nV = sHex.GetAt( i );
		if( nV >= 'A' && nV <= 'F' )
		{
			nRv += (int) pow( 0x10, ( sHex.GetLength() - ( i + 1 ) ) ) * ( nV - 'A' + 0xA );
		}
		else if( nV >= 'a' && nV <= 'f' )
		{
			nRv += (int) pow( 0x10, ( sHex.GetLength() - ( i + 1 ) ) ) * ( nV - 'a' + 0xA );
		}
		else if( nV >= '0' && nV <= '9' )
		{
			nRv += (int) pow( 0x10, ( sHex.GetLength() - ( i + 1 ) ) ) * ( nV - '0' );
		}
	}
	return nRv;
}

