//---------------------------------------------------------------------
// Copyright		: DMS Co., Ltd
// File Name		: XMmc.cpp
// Modified Date	: 2001.03.27, 2001.04.23, 2003.01.06
// Version			: Ver1.0
// Programmer		: Kim Dongsoo
// Description		: Interface for the XMmc class.
//					MEI Modify
// 1. 01/04/23 : Library  ϰ Hard Fail     .
// 2. 03/01/06 : HomeOnly Լ Ͽ Limit sensor ɷ Homing ̷ 
// 3. 03/05/19 : Point Moving  EStop InitParameter()  ׳ Ǿ IsMoveOk( nRbtType ) SeqNo . 
//               Homing   ġ ̵϶ ϸ ̵ ʰ Ϸ Signal .
//               E-Stop s_move SeqNo ʱȭ ʿ  
//---------------------------------------------------------------------

#include "stdafx.h"
#include "..\Dics.h"
#include "XMmc.h"
#include <MATH.H>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//--------------------------------------------------------------------
// Teaching point ҷų Ǿ File Name 
//--------------------------------------------------------------------
CString XMmc::m_sRbtFileName[MAX_RBT_NO] =
{
//	"RbtLDUpdnUnit.pos",
//	"RbtSpareUnit.pos",
	"SWR2RbtUpRb1Unit.pos",
	"SWR2RbtLoRb1Unit.pos",
	"SWR2RbtUpRb2Unit.pos",
	"SWR2RbtLoRb2Unit.pos",
	"SWR3RbtUpRb1Unit.pos",
	"SWR3RbtLoRb1Unit.pos",
	"SWR3RbtUpRb2Unit.pos",
	"SWR3RbtLoRb2Unit.pos",

};

//--------------------------------------------------------------------
// κ => Robot view 
//--------------------------------------------------------------------
CString XMmc::m_sRbtName[MAX_RBT_NO] =
{
	" 1. SWR2 UPPER RB1 UNIT ",
	" 2. SWR2 LOWER RB1 UNIT ",
	" 3. SWR2 UPPER RB2 UNIT ",
	" 4. SWR2 LOWER RB2 UNIT ",
	" 5. SWR3 UPPER RB1 UNIT ",
	" 6. SWR3 LOWER RB1 UNIT ",
	" 7. SWR3 UPPER RB2 UNIT ",
	" 8. SWR3 LOWER RB2 UNIT ",
};

//--------------------------------------------------------------------
//  ࿡  Event => Robot view 
//--------------------------------------------------------------------
CString	XMmc::m_sAxisState[] =
{
	"NO_EVENT",
	"STOP_EVENT",
	"E_STOP_EVENT",
	"ABORT_EVENT"
};

//--------------------------------------------------------------------
//  ࿡   => Robot view 
//--------------------------------------------------------------------
CString	XMmc::m_sAxisSource[] =
{
	"ST_NONE",
	"ST_HOME_SWITCH",
	"ST_POS_LIMIT",
	"ST_NEG_LIMIT",
	"ST_AMP_FAULT",
	"ST_ACC_LIMIT",
	"ST_VEL_LIMIT",
	"ST_SOFT_NEG_LIMIT",
	"ST_SOFT_POS_LIMIT",
	"ST_ERROR_LIMIT",
	"ST_PC_COMMAND",
	"ST_OUT_OF_FRAMES",
	"ST_AMP_POWER_ONOFF",
	"ST_ABS_COMM_ERROR",
};

//--------------------------------------------------------------------
//  Robot Unit  Teaching Point Name=> Robot view 
//--------------------------------------------------------------------
CString XMmc::m_sRbtPosName[MAX_RBT_NO][10] = 
{
//	{ "HOME POS", "RECV POS", "SEND POS", },
	//{ "SPARE POS", "SPARE1 POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
	{ "ZERO POS", "WAIT POS",},
};

//--------------------------------------------------------------------
//  Robot Unit   ̸
//--------------------------------------------------------------------
CString XMmc::m_sRbtAxisName[MAX_RBT_NO][MAX_RBT_AXES] = 
{
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", },
	{ "1-AXIS (Z)", }, // RB  Z .
};

//--------------------------------------------------------------------
// Set the homing information about each axes.
// vel : ̵Ҷ ӵ
// acc : ̵Ҷ ӵ (Unit : 4msec, Range : 1 ~ 25000)
// f_dist :  ̵ Home Sensor  ݴ ̴ Ÿ(mm)
// home_type :  ̵ ( HOME_SENSOR_ONLY , HOME_SENSOR_AND_INDEX )
//--------------------------------------------------------------------
HOME_INFO XMmc::m_HomeInfo[MAX_AXES] =
{//		dVel   nAcc	dDist	nHome_type
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 3	SWR2 UP RB1  	Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 4	SWR2 UP RB2		Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 3	SWR2 LO RB1  	Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 4	SWR2 LO RB2		Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 3	SWR3 UP RB1  	Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 4	SWR3 UP RB2		Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 3	SWR3 LO RB1  	Z-Axis 50->100
	{ 5000.0,  100, 0.3,		HOME_SENSOR_ONLY },	// 4	SWR3 LO RB2		Z-Axis 50->100
};

//--------------------------------------------------------------------
// Set the moving information about each axes
// vel : ̵Ҷ ӵ
// acc : ̵Ҷ ӵ
// ratio : Motor 1 ȸ ̴ Ÿ mm
// encoder : Motor 1 ȸ Encoder Pulse
//--------------------------------------------------------------------
AXIS_PARA XMmc::m_AxisPara[MAX_AXES] = // kjd edit
{ // dVel       nAcc dRatio uEncoder, type, dTheta_H, Ratio    
//	{ 100000.0, 300, 4.17,	65535/*8192*/,	typeNORMAL, 0.0 },	// 10mm / 1rev : LD FISH Z-Axis
//	{ 10000.0, 400, 40, 32512/*8192*/,	typeNORMAL, 0.0 },	// 40mm / 1rev : LD UPDN Z-AXIS
//	{ 10000.0, 400, 40, 32767,	typeNORMAL, 0.0 },	// 40mm / 1rev : LD UPDN Z-AXIS
//	{ 10000.0, 400, 40, 24260,	typeNORMAL, 0.0 },	// 40mm / 1rev : LD UPDN Z-AXIS
//	{ 50000.0, 100, 2*PI,	819200,	typeRB_GAP, PI*(63.0/180.0) },	// R/B1	Upper
//	{ 50000.0, 100, 2*PI,	819200,	typeRB_GAP, PI*(63.0/180.0) },	// R/B1	Lower
//	{ 50000.0, 100, 2*PI,	819200,	typeRB_GAP, PI*(63.0/180.0) },	// R/B2	Upper
//	{ 50000.0, 100, 2*PI,	819200,	typeRB_GAP, PI*(63.0/180.0) },	// R/B2	Lower
//	{ 10000.0, 400, 40, 24260,	typeNORMAL, 0.0 },					// SPARE
/*
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE2 R/B1	Lower
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE2 R/B1	Upper
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE2 R/B2	Lower
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE2 R/B2	Upper
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE3 R/B1	Lower
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE3 R/B1	Upper
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE3 R/B2	Lower
	{ 5000.0, 100, 1.82*PI,	819200,	typeRB_GAP, PI*(50.0/180.0) },	// SWE3 R/B2	Upper
*/
	{ 5000.0, 100, 0.26,	8192,	typeNORMAL, 0.0, TRUE },	// SWE2 R/B1	Lower
	{ 5000.0, 100, 0.27,	8192,	typeNORMAL, 0.0, TRUE },	// SWE2 R/B1	Upper
	{ 5000.0, 100, 0.28,	8192,	typeNORMAL, 0.0, TRUE },	// SWE2 R/B2	Lower
	{ 5000.0, 100, 0.28,	8192,	typeNORMAL, 0.0, TRUE },	// SWE2 R/B2	Upper
	{ 5000.0, 100, 0.27,	8192,	typeNORMAL, 0.0, TRUE },	// SWE3 R/B1	Lower
	{ 5000.0, 100, 0.27,	8192,	typeNORMAL, 0.0, TRUE },	// SWE3 R/B1	Upper
	{ 5000.0, 100, 0.26,	8192,	typeNORMAL, 0.0, TRUE },	// SWE3 R/B2	Lower
	{ 5000.0, 100, 0.27,	8192,	typeNORMAL, 0.0, TRUE },	// SWE3 R/B2	Upper

};

//--------------------------------------------------------------------
// Information of each robot
// robot 1 : x, y ( linked )
// robot 2 : y, z
// robot 3 : z
// robot 4 : y, z
// robot 5 : z
//
// 1. start_axis : Robot ۵Ǿ  ȣ ex) Transfer Unit : 0, 1 -> start_axis = 0
// 2. max_axes : Robot    ex) transfer Unit : 0, 1 -> max_axes = 2;
// 3. mask : ϴ  
//													  (0,z,y,x)
//    0x03 => x,y; 0x04 => z;  0x06 => y,z; x, y, z => 0 1 1 1 => mask = 0x07
// 4. max_pos_no : teaching point number
// 5. move_type : robot moving type
// 6. home_order :   x  y  z  o
//                  {1, 1, 0, 2}̸  : z -> x, y  -> 0  ̵ 
//                  {0, 0, 0, 0} =>  ̵, {0, 1, 2, 3} => x=>y=>z=>0  ̵
// 7. xyz0_to_axis : maskǾ ִ ࿡   
// 8. link :  master slave  is_linked
//           {   0 ,   1 ,    TRUE   }
//                              
//    ex)  Transfer Unit : start_axis = 0 , master axis = start_axis + master(0) = 0
//         slave axis = start_axis + slave(1) = 1
// 9. *pos : Teaching Point ȭϿ о *pos . ׷Ƿ NULL
// 10. vel : Registry ӵ о vel . ׷Ƿ { 0.0, 0.0, 0.0, 0.0}
//--------------------------------------------------------------------
RBT_INFO XMmc::m_RbtInfo[MAX_RBT_NO] = 
{
	/*
	{	// RBT_1 (RBT_LD_UPND_UNIT)
		RBT_LD_UPDN_UNIT_S_AXIS,// start axis
		1,						// max_axes
		0x04,					// mask (z axis)
		MAX_RBT_LD_UPDN_UNIT_POS_NO,	// max. teaching point no
		RBT_S1_MOVE,			// robot moving type
		{ 0, 0, 0, 0 },			// home order
		{ 0, 0, 0, 0 },			// xyz0_to_axis
		{ 0, 0, FALSE },		// link information (not linked)
		NULL,					// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }	// velocity
	},
	{	// RBT_2 (SPARE_UNIT)
		RBT_SPARE_UNIT_S_AXIS,		// start axis
		1,								// max_axes
		0x04,							// mask (z axis)
		2,	// max. teaching point no
		RBT_S1_MOVE,					// robot moving type
		{ 0, 0, 0, 0 },					// home order
		{ 0, 0, 0, 0 },					// xyz0_to_axis
		{ 0, 0, FALSE },				// link information (not linked)
		NULL,							// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }			// velocity
	},
	*/
	{	// RBT_1 (RBT_UP_RB1_UNIT)
		RBT_SWR2_UP_RB1_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_2 (RBT_LO_RB1_UNIT)
		RBT_SWR2_LO_RB1_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_3 (RBT_UP_RB2_UNIT)
		RBT_SWR2_UP_RB2_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_4 (RBT_LO_RB2_UNIT)
		RBT_SWR2_LO_RB2_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_5 (RBT_UP_RB1_UNIT)
		RBT_SWR3_UP_RB1_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_6 (RBT_LO_RB1_UNIT)
		RBT_SWR3_LO_RB1_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_7 (RBT_UP_RB2_UNIT)
		RBT_SWR3_UP_RB2_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
	{	// RBT_8 (RBT_LO_RB2_UNIT)
		RBT_SWR3_LO_RB2_UNIT_S_AXIS,		// start axis
		1,									// max_axes
		0x04,								// mask (x, y axis)
		MAX_RBT_RB_UNIT_POS_NO,				// max. teaching point no
		RBT_S1_MOVE,						// robot moving type
		{ 0, 0, 0, 0 },						// home order
		{ 0, 0, 0, 0 },						// xyz0_to_axis
		{ 0, 0, FALSE },					// link information (linked, master : 0, slave : 1)
		NULL,								// position : NULL 
		{ 0.0, 0.0, 0.0, 0.0 }				// velocity
	},
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
XMmc::XMmc()
{
	int i;
	for( i = 0; i < MAX_RBT_NO; i++ )
	{
		m_dRatio[i] = 1.0;
		m_nCurPoint[i] = 0;
		
		m_bHomeComp[i] = FALSE;		// α׷ ó ϸ鼭 ʱȭ Ų.( 020911 )
		m_bServoReady[i] = FALSE;
	}

	for( i = 0;	i < MAX_RBT_NO; i++ )
	{
		for( int j=0; j<MAX_RBT_AXES; j++ )
		{
			m_RbtState[i].nAxis_state[j] = 3;
			m_RbtState[i].nAxis_source[j] = 10;
			m_RbtState[i].axis_sensor[j].Home		= FALSE;
			m_RbtState[i].axis_sensor[j].PosLimit	= FALSE;
			m_RbtState[i].axis_sensor[j].NegLimit	= FALSE;
		}
	}
}

XMmc::~XMmc()
{
	// Free the memory
	for( int i = 0; i < MAX_RBT_NO; i++ )
	{
		if( m_RbtInfo[i].link.bIs_linked )
		{
#ifdef _MY_DEBUG
			set_sync_control( FALSE );
#else
			set_sync_control_ax( m_RbtInfo[i].nStart_axis+m_RbtInfo[i].link.nSlave, FALSE, m_RbtInfo[i].nStart_axis, 0 );
#endif
		}

		if( m_RbtInfo[i].pos ) delete []m_RbtInfo[i].pos;
	}
}

//--------------------------------------------------------------------
//	distance unit = mm 
//  Ÿ ޽ ϴ Լ
//--------------------------------------------------------------------
double XMmc::Len2Pulse(INT nAxis, double dLen)
{
	double dPulse;

	if( m_AxisPara[nAxis].nType == typeNORMAL )
	{
		dPulse = (double)m_AxisPara[nAxis].uEncoder * dLen / 
			//m_AxisPara[nAxis].dRatio;
			GetAxisRatio(nAxis);	// hskwak 190806
		
	}
	else if( m_AxisPara[nAxis].nType == typeRB_GAP )
	{
	 	if( dLen > 10 ) dLen = 10;
//		else if( dLen < 0. ) dLen = 0.0;
//		double fRadian = acos( 1 - dLen / 4.0 );

		double dTheta_H = m_AxisPara[nAxis].dTheta_H;
	 			double fRadian = acos( cos( dTheta_H ) - ( dLen / 5 ) ) - dTheta_H;
//		double fTheta = 360.0 / ( 2.0 * PI ) * fRadian;
		dPulse = (double)m_AxisPara[nAxis].uEncoder * fRadian / 
				//m_AxisPara[nAxis].dRatio;
				GetAxisRatio(nAxis);	// hskwak 190806
	}
	return dPulse;
}

//--------------------------------------------------------------------
//  ޽ Ÿ ϴ Լ
//--------------------------------------------------------------------
double XMmc::Pulse2Len(INT nAxis, double dPulse)
{
	double dLen;

	if( m_AxisPara[nAxis].nType == typeNORMAL )
	{
		//dLen = m_AxisPara[nAxis].dRatio * dPulse  /
		dLen = GetAxisRatio(nAxis) * dPulse  /	// hskwak 190806
			(double)m_AxisPara[nAxis].uEncoder;
	}
	else if( m_AxisPara[nAxis].nType == typeRB_GAP )
	{
		double dTheta_H = m_AxisPara[nAxis].dTheta_H;
		//double fRadian = m_AxisPara[nAxis].dRatio * dPulse  / (double)m_AxisPara[nAxis].uEncoder;
		double fRadian = GetAxisRatio(nAxis) * dPulse  / (double)m_AxisPara[nAxis].uEncoder;	// hskwak 190806
//		double fRadian = ( 2.0 * PI ) / 360.0 * fTheta;
//		dLen = 4.0 * ( 1.0 - cos( fRadian ) );
		dLen = 5 * ( cos( dTheta_H ) - cos( dTheta_H + fRadian ) );
	}
	return dLen;	
}

//--------------------------------------------------------------------
// Initialize the parameter
//--------------------------------------------------------------------
void XMmc::InitParameter()
{
	int i;
	for( i = 0; i < MAX_RBT_NO; i++ )
	{
		m_nServoOnRbtNo[i]		= 0;
		m_nEStopRbtNo[i]		= 0;
		m_nMoveRbtHomeNo[i]	= 0;

		m_nMoveRbtPtp1No[i]	= 0;
		m_nMoveRbtPtp2No[i]	= 0;
		m_nMoveRbtS1No[i]	= 0;
		m_nMoveRbtS2No[i]	= 0;

		m_nMoveRbtPosNo[i]	= 0;
		m_nRelativeMoveNo[i]  = 0;
		m_nSetSyncMasterSlaveNo[i] = 0;
	}

	for( i=0; i<MAX_AXES; i++ )
	{
		m_nHomeOnlyNo[i]			= 0;
		m_nHomeAndIndexNo[i]		= 0;//[0]		= 0;
		m_nClearAxisErrNo[i]		= 0;
		m_nSetCalThetaNo[i]			= 0;
	}
}

//--------------------------------------------------------------------
// Initialize the MMC Board
//--------------------------------------------------------------------
void XMmc::Init()
{
	// Allocate the memory (position)
	int i;
	for( i = 0; i < MAX_RBT_NO; i++ )
	{
		m_RbtInfo[i].pos = new RBT_POS[m_RbtInfo[i].nMax_pos_no];

		for( int j = 0; j < m_RbtInfo[i].nMax_pos_no; j++ )
		{
			for( int k = 0; k < 4; k++ ) m_RbtInfo[i].pos[j].dPos[k] = 0.0;
		}
	}

	// Initialize the parameter;
	InitParameter();

	// Set the point and velocity
	for( i=0; i<MAX_RBT_NO; i++ )
	{
		//if( i == RBT_SPARE_UNIT) continue;

		// Set the velocity
		SetVelFromRegistry( i );
		// Set the theta
		SetThetaFromRegistry( i );
		// Set The Ratio
		SetAxisRatioFromRegistry( i );
		// Set The Ratio
		// Set the point
		PosFile2Mem( i );
	}

	// Initialize the board
	// Initialize the board
	//long addr[2] = { 0xd8000000, 0xd8400000 };
//	long addr[2] = { 0xb8000000, 0xb8400000 };	// ljy_050317 : IRQ address change
	long addr[1];// = { 0xb8000000, 0xb8400000 };	// ljy_050317 : IRQ address change
	if( mmc_initx( 1, addr ) )
	{
		char szErr[80];
		int nCode = get_mmc_error(); 
		error_message( nCode, szErr );

		CString sBuf;
		sBuf.Format( "Failed to Initialize the MMC board : %s (%d)", szErr, nCode );
		AfxMessageBox( sBuf );
		return;
	}

	// set the synchronous control
	for( i=0; i<MAX_RBT_NO; i++ )
	{
		if( m_RbtInfo[i].link.bIs_linked ) 
		{
			set_sync_map_axes( m_RbtInfo[i].nStart_axis + m_RbtInfo[i].link.nMaster, 
							   m_RbtInfo[i].nStart_axis + m_RbtInfo[i].link.nSlave );
#ifdef _MY_DEBUG
			set_sync_control( TRUE );
#else
			set_sync_control_ax( m_RbtInfo[i].nStart_axis+m_RbtInfo[i].link.nSlave, TRUE, m_RbtInfo[i].nStart_axis, 0 );
#endif
		}
	}
}

//--------------------------------------------------------------------
//	step : clear EVENT(STOP & E_STOP & ABORT)
//		   ABORT_EVENT Clear Ŀ Servo ON  .
//--------------------------------------------------------------------
BOOL XMmc::ClearAxisErr(INT nAxis)
{
#ifdef _MY_DEBUG
	return TRUE;
#endif

	BOOL bReturn = FALSE;
	static int nTryNo[MAX_AXES] = { 0, };

	int nSeqNo = m_nClearAxisErrNo[nAxis];

	switch( nSeqNo ) 
	{
	case 0:
		// clear STOP_EVENT, E_STOP_EVENT, ABORT_EVENT
		if( !clear_status( nAxis ) ) 
		{
//			controller_idle( axis ); // PID Control disable
			nTryNo[nAxis] = 0; 
			nSeqNo = 10; 
		}
		else
		{
			if( nTryNo[nAxis] == RETRY_NO ) 
			{ 
				bReturn = TRUE; 
				nTryNo[nAxis] = 0; 
			}
			else nTryNo[nAxis] += 1;
		}
		break;
	case 10:
		if( axis_done( nAxis ) ) 
		{
//			controller_run( axis );	// clear ABORT_EVENT
			frames_clear( nAxis );
			clear_status( nAxis );	// clear STOP_EVENT, E_STOP_EVENT
			bReturn = TRUE;
			nSeqNo = 0;
		}
		break;
	}

	m_nClearAxisErrNo[nAxis] = nSeqNo;
	return bReturn;
}

//--------------------------------------------------------------------
// Servo On robot (clear robot error => servo on)
// HJYOU 2007.10.15 
// 1. Master, Slave  Sync.   Servo ON ϸ EMO or E-Stop  ġ Ʋ . case 0: Sync.  FALSE Ѵ.
// 2. ϶ case 15:  100 msec Time delay ְ Ǹ EMO or E-Stop  ġ Ʋ .
//    Master, Slave  Servo ON Ǵ  ޶ ߻ϴ  Ǵܵ.
// 3. Servo Ready Flag TRUE  .  ϶ Master, Slave   Servo ON Ǵ  TRUE
//--------------------------------------------------------------------
BOOL XMmc::ServoOnRbt(INT nRbtType)
{
#ifdef _MY_DEBUG
	return TRUE;
#endif 
	CDicsApp* pApp = (CDicsApp*)AfxGetApp();// GetAppRegistryKey()
// 	if( nRbtType == RBT_LD_UPDN_UNIT && pApp->m_pMainView->m_pDicsData->m_GenInfo.bLdDoorOpen ) return FALSE;
// #ifndef _NO_UL_HAND
// 	if( nRbtType == RBT_UL_HAND_UNIT && pApp->m_pMainView->m_pDicsData->m_GenInfo.bUlDoorOpen ) return FALSE;
// #endif
	BOOL bReturn = FALSE;
	static int nAxis[MAX_RBT_NO] = { 0, };
	static DWORD dwTickStart[MAX_RBT_NO];
	int nSeqNo = m_nServoOnRbtNo[nRbtType];
	switch( nSeqNo ) 
	{
	case 0:
		// link Ǿ    
		if( m_RbtInfo[nRbtType].link.bIs_linked )
		{
#ifdef _MY_DEBUG
			set_sync_control( FALSE );
#else
			set_sync_control_ax( m_RbtInfo[nRbtType].nStart_axis+m_RbtInfo[nRbtType].link.nSlave, FALSE, m_RbtInfo[nRbtType].nStart_axis, 0 );
#endif
		}
		nAxis[nRbtType] = m_RbtInfo[nRbtType].nStart_axis;
		nSeqNo = 10;
		break;
	case 10:
		if( ClearAxisErr( nAxis[nRbtType] ) ) 
		{
			set_amp_enable( nAxis[nRbtType], TRUE );
			nAxis[nRbtType] += 1;
//			nSeqNo = 20;
			dwTickStart[nRbtType] = GetTickCount();
			nSeqNo = 15;
		}
		break;
	case 15:
		{
			if( GetTickCount() - dwTickStart[nRbtType] > 100 ) nSeqNo = 20;
		}
		break;
	case 20:
		if( nAxis[nRbtType] == m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nMax_axes ) 
		{
			m_bServoReady[nRbtType] = TRUE;
			// link Ǿ    
			if( m_RbtInfo[nRbtType].link.bIs_linked )
			{
#ifdef _MY_DEBUG
				set_sync_control( TRUE );
#else
				set_sync_control_ax( m_RbtInfo[nRbtType].nStart_axis+m_RbtInfo[nRbtType].link.nSlave, TRUE, m_RbtInfo[nRbtType].nStart_axis, 0 );
#endif
			}
			nSeqNo = 0;
			bReturn = TRUE;
		}
		else nSeqNo = 10;
		break;
	}
	m_nServoOnRbtNo[nRbtType] = nSeqNo;
	return bReturn;
}

//--------------------------------------------------------------------
// Reset the robot
//--------------------------------------------------------------------
BOOL XMmc::ResetRbt(INT nRbtType)
{
//	if( nRbtType == RBT_SPARE_UNIT ) return TRUE;
#ifdef _MY_DEBUG
	m_bServoReady[nRbtType] = TRUE;
	return TRUE;
#endif

	return ServoOnRbt( nRbtType );
}

//--------------------------------------------------------------------
// Servo Off the robot
//--------------------------------------------------------------------
void XMmc::ServoOffRbt(INT nRbtType)
{
#ifdef _MY_DEBUG
	m_bServoReady[nRbtType] = FALSE;
	return;
#endif

	for( int i = m_RbtInfo[nRbtType].nStart_axis;
		 i < m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nMax_axes; i++ )
	{
		set_amp_enable( i, FALSE );
		m_bServoReady[nRbtType] = FALSE;
	}
}

//--------------------------------------------------------------------
// when pressed e-stop button (e_stop => Servo off)
//  Robot Unit  Ŵ
//--------------------------------------------------------------------
BOOL XMmc::EStopRbt(INT nRbtType)
{
//	if( nRbtType == RBT_SPARE_UNIT ) return TRUE;
#ifdef _MY_DEBUG
//	m_bHomeComp[nRbtType] = FALSE;
	m_bServoReady[nRbtType] = FALSE;
	return TRUE;
#endif

	BOOL bReturn = FALSE;
	int i;
	int nSeqNo = m_nEStopRbtNo[nRbtType];

	switch( nSeqNo ) 
	{
	case 0:
		for( i = 0; i < m_RbtInfo[nRbtType].nMax_axes; i++ )
		{
			int nAxis = m_RbtInfo[nRbtType].nStart_axis + i;
			
			set_e_stop_rate( nAxis, 300 );  //hjyou 2007.10.16
			set_e_stop( nAxis );
			set_amp_enable( nAxis, FALSE );	// Servo off
//			set_amp_enable( nAxis, FALSE );	// hjyou 2007.10.16
			m_nHomeOnlyNo[nAxis] = 0;		// 2003.1.7 Hong Jaehee
			m_bServoReady[nRbtType] = FALSE;
		}
		nSeqNo = 10;
		break;
	case 10:
		if( IsMoveOk( nRbtType ) ) 
		{
			for( i = 0; i < m_RbtInfo[nRbtType].nMax_axes; i++ )
			{
				int nAxis = m_RbtInfo[nRbtType].nStart_axis + i;
				set_amp_enable( nAxis, FALSE );	// Servo off
			}
			// SeqNo => Initialize Update	///////////////////
			// E-Stop  ʿ.
//			m_bHomeComp[nRbtType] = FALSE;
			m_nMoveRbtHomeNo[nRbtType]	= 0;
			m_nMoveRbtPosNo[nRbtType]	= 0;
			m_nServoOnRbtNo[nRbtType]	= 0;//2007.04.30 kimgun
			m_nMoveRbtPtp1No[nRbtType]	= 0;
			m_nMoveRbtPtp2No[nRbtType]	= 0;
			m_nMoveRbtS1No[nRbtType]	= 0;
			m_nMoveRbtS2No[nRbtType]	= 0;
			m_nSetSyncMasterSlaveNo[nRbtType] = 0;
			///////////////////////////////////////////////////
			if( m_RbtInfo[nRbtType].link.bIs_linked )
			{
//				set_sync_control( FALSE );
				set_sync_control_ax( m_RbtInfo[nRbtType].nStart_axis+m_RbtInfo[nRbtType].link.nSlave, FALSE, m_RbtInfo[nRbtType].nStart_axis, 0 );
			}
			bReturn = TRUE;
			nSeqNo = 0;
		}
		break;
	}

	m_nEStopRbtNo[nRbtType] = nSeqNo;
	return bReturn;
}

//--------------------------------------------------------------------
// Pause Operation (MMC  ȵ)
//--------------------------------------------------------------------
void XMmc::PauseRbt(INT nRbtType, BOOL bPause)
{
#ifdef _MY_DEBUG
	return;
#endif
}

//--------------------------------------------------------------------
// Send Robot status
// axis_state : 0, 1, 2, 3
// axis_source : 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, ...
//--------------------------------------------------------------------
RBT_STATE XMmc::GetRbtStatus(INT nRbtType)
{
#ifdef _MY_DEBUG
	m_RbtState[nRbtType].nAxis_state[0] = m_RbtState[nRbtType].nAxis_state[1] = 
	m_RbtState[nRbtType].nAxis_state[2] = m_RbtState[nRbtType].nAxis_state[3] = 3;
	m_RbtState[nRbtType].nAxis_source[0] = m_RbtState[nRbtType].nAxis_source[1] = 
	m_RbtState[nRbtType].nAxis_source[2] = m_RbtState[nRbtType].nAxis_source[3] = 10;

	return m_RbtState[nRbtType];
#endif

	int nNo = 0;
	for( int i = 0; i < MAX_RBT_AXES; i++ ) 
	{
		int nVal = 0;

		if( m_RbtInfo[nRbtType].nMask & ( 1 << i ) ) 
		{
			int nAxis = m_RbtInfo[nRbtType].nStart_axis + nNo++;
			m_RbtState[nRbtType].nAxis_state[i] = axis_state( nAxis );
			m_RbtState[nRbtType].nAxis_source[i] = axis_source( nAxis );

			if( home_switch( nAxis ) )	m_RbtState[nRbtType].axis_sensor[i].Home = TRUE;
			else m_RbtState[nRbtType].axis_sensor[i].Home = FALSE;

			if( pos_switch( nAxis ) )	m_RbtState[nRbtType].axis_sensor[i].PosLimit = TRUE;
			else m_RbtState[nRbtType].axis_sensor[i].PosLimit = FALSE;

			if( neg_switch( nAxis ) )	m_RbtState[nRbtType].axis_sensor[i].NegLimit = TRUE;
			else m_RbtState[nRbtType].axis_sensor[i].NegLimit = FALSE;

			//int nSource = axis_source( nAxis );
			//nSource &= 0x1fff;
			//if( nSource )
			//{
			//	while( 1 ) { if( ( nSource >> nVal++ ) & 0x01 ) break; }
			//}
			//m_RbtState[nRbtType].nAxis_source[i] = nVal;
		}
		//else m_RbtState[nRbtType].nAxis_source[i] = nVal;
	}
	return m_RbtState[nRbtType];
}

//--------------------------------------------------------------------
// Check the robot status
//--------------------------------------------------------------------
BOOL XMmc::IsRbtStatusOk(INT nRbtType)
{
#ifdef _MY_DEBUG
	return TRUE;
#endif

	BOOL bRv = TRUE;

	for( int i = 0; i < m_RbtInfo[nRbtType].nMax_axes; i++ ) 
	{
		int nAxis = m_RbtInfo[nRbtType].nStart_axis + i;
		int nEvent = axis_state( nAxis );

		if( ( nEvent == STOP_EVENT ) || ( nEvent == E_STOP_EVENT ) || ( nEvent == ABORT_EVENT ) ) 
		{
			return FALSE;
		}

		int nSource = axis_source( nAxis );
		nSource &= 0x1FFF;
//		if( nSource != ST_NONE && nSource != ST_HOME_SWITCH && nSource != ST_PC_COMMAND ) return FALSE;

		if( nSource & ST_POS_LIMIT ) bRv = FALSE;
		if( nSource & ST_NEG_LIMIT ) bRv = FALSE;
		if( nSource & ST_AMP_FAULT ) bRv = FALSE;
		if( nSource & ST_AMP_POWER_ONOFF ) bRv = FALSE;
		if( nSource & ST_OUT_OF_FRAMES ) bRv = FALSE;
//		if( nSource & ST_A_LIMIT ) bRv = FALSE;
//		if( nSource & ST_V_LIMIT ) bRv = FALSE;
//		if( nSource & ST_X_NEG_LIMIT ) bRv = FALSE;
//		if( nSource & ST_X_POS_LIMIT ) bRv = FALSE;
//		if( nSource & ST_ERROR_LIMIT ) bRv = FALSE;
	}

	return bRv;
}

//--------------------------------------------------------------------
// Check the robot status
//--------------------------------------------------------------------
BOOL XMmc::IsRbtSensorOk(INT nRbtType)
{
#ifdef _MY_DEBUG
	return TRUE;
#endif

	BOOL bRv = TRUE;

	for( int i = 0; i < m_RbtInfo[nRbtType].nMax_axes; i++ ) 
	{
		int nAxis = m_RbtInfo[nRbtType].nStart_axis + i;
		int nEvent = axis_state( nAxis );

		if( ( nEvent == E_STOP_EVENT ) || ( nEvent == ABORT_EVENT ) ) 
		{
			return FALSE;
		}

		int nSource = axis_source( nAxis );
		nSource &= 0x1FFF;
//		if( nSource != ST_NONE && nSource != ST_HOME_SWITCH && nSource != ST_PC_COMMAND ) return FALSE;

		if( nSource & ST_POS_LIMIT ) bRv = FALSE;
		if( nSource & ST_NEG_LIMIT ) bRv = FALSE;
		if( nSource & ST_AMP_FAULT ) bRv = FALSE;
		if( nSource & ST_AMP_POWER_ONOFF ) bRv = FALSE;
		if( nSource & ST_OUT_OF_FRAMES ) bRv = FALSE;
//		if( nSource & ST_A_LIMIT ) bRv = FALSE;
//		if( nSource & ST_V_LIMIT ) bRv = FALSE;
//		if( nSource & ST_X_NEG_LIMIT ) bRv = FALSE;
//		if( nSource & ST_X_POS_LIMIT ) bRv = FALSE;
//		if( nSource & ST_ERROR_LIMIT ) bRv = FALSE;
	}

	return bRv;
}
//--------------------------------------------------------------------
//	Home sequence
//  Home Sensor ϴ ̵
//  (You should edit about link motion.)
//--------------------------------------------------------------------
int XMmc::HomeOnly(INT nAxis)
{
#ifdef _MY_DEBUG
	return 0;
#endif

	int nReturn = -1;

	static double dLimitPosNeg[MAX_AXES];
	static INT nActionNeg[MAX_AXES];

	static double dLimitPosPos[MAX_AXES];
	static INT nActionPos[MAX_AXES];

	int nSeqNo = m_nHomeOnlyNo[nAxis];
	// hskwak 190624
	static int nCount;
	static short nHomeLevel[MAX_AXES];

	switch( nSeqNo ) 
	{
	case 0:
		// set no event for software limit
		get_negative_sw_limit( nAxis, &dLimitPosNeg[nAxis], &nActionNeg[nAxis] );
		set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], NO_EVENT );
		get_positive_sw_limit( nAxis, &dLimitPosPos[nAxis], &nActionPos[nAxis] );
		set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], NO_EVENT );
		set_home( nAxis, NO_EVENT );
		set_negative_limit( nAxis, NO_EVENT ); // 2003.1.2 Hong jaehee
		set_positive_limit( nAxis, NO_EVENT ); // 2003.1.2 Hong jaehee

		clear_status( nAxis );

		set_stop_rate( nAxis, m_HomeInfo[nAxis].nAcc / 2 );
		set_index_required( nAxis, FALSE );
		set_home( nAxis, STOP_EVENT );
		set_negative_limit( nAxis, STOP_EVENT ); // 2003.1.2 Hong jaehee

		frames_clear( nAxis );
		v_move( nAxis, -m_HomeInfo[nAxis].dVel*4.0, m_HomeInfo[nAxis].nAcc / 2 );
		nSeqNo = 10;
		break;
	case 10:
		if( axis_done( nAxis ) ) 
		{
			set_home( nAxis, NO_EVENT );
			set_negative_limit( nAxis, NO_EVENT ); // 2003.1.2 Hong jaehee
			set_positive_limit( nAxis, E_STOP_EVENT ); // 2003.1.2 Hong jaehee
			clear_status( nAxis );
			frames_clear( nAxis );

			// 2003.1.2 Hong jaehee
			if( neg_switch( nAxis ) )
			{
				set_home( nAxis, STOP_EVENT );
				v_move( nAxis, m_HomeInfo[nAxis].dVel*4.0, m_HomeInfo[nAxis].nAcc / 2 );
				nSeqNo = 13;
			}
			else if( home_switch( nAxis ) )
			{
				nCount = 0;
				start_r_move( nAxis, GetDistanceHomeOut(nAxis), m_HomeInfo[nAxis].dVel*2.0, m_HomeInfo[nAxis].nAcc / 2 );
				nSeqNo = 20;
			}

		}
		break;
	case 13 :
		if( axis_done( nAxis ) )
		{
			set_home( nAxis, NO_EVENT );
			clear_status( nAxis );
			frames_clear( nAxis );

			start_r_move( nAxis, GetDistanceHomeOut(nAxis)*3., m_HomeInfo[nAxis].dVel*2.0, m_HomeInfo[nAxis].nAcc / 2 );
			nSeqNo = 15;
		}
		break;
	case 15 :
		if( axis_done( nAxis ) )
		{
			set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
			set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
			nSeqNo = 0;
		}
		break;
	case 20:
		if( axis_done( nAxis ) ) 
		{
			set_stop_rate( nAxis, m_HomeInfo[nAxis].nAcc / 2 );
			set_home( nAxis, STOP_EVENT );
			set_negative_limit( nAxis, E_STOP_EVENT ); // 2003.1.2 Hong jaehee

			if( home_switch( nAxis ) )
			{
#ifdef _MOT_REPEAT
				if(nCount < 10)	// hskwak 190624  װ Ŭ  ѹ Home ġ  ϹǷ 10 ݺ Ѵ. ʿ   ʿ.
				{
					frames_clear( nAxis );
					set_home( nAxis, NO_EVENT );
					start_r_move( nAxis, GetDistanceHomeOut(nAxis), m_HomeInfo[nAxis].dVel*2.0, m_HomeInfo[nAxis].nAcc / 2 );
					nCount++;
				}
#else
#ifdef _MOT_HOMEOUT
				// hskwak 190624  װ Ŭ  ѹ Home ġ  ϹǷ Home Level   Home Check + ( hoem   ) ٽ  .
				get_home_level( nAxis, &nHomeLevel[nAxis] );	//  Level ¸ .
				set_home_level( nAxis, !nHomeLevel[nAxis] );	//  Level ¸ ݴ .
				set_home( nAxis, STOP_EVENT );
				set_positive_limit( nAxis, E_STOP_EVENT );

				frames_clear( nAxis );
				v_move( nAxis, m_HomeInfo[nAxis].dVel*4.0, m_HomeInfo[nAxis].nAcc / 2 );
				nSeqNo = 22;
#else
				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				nReturn = 100; // 2006.03.31 HJYOU [dDistanceŭ ̵  Home Sensor OFF  ʴ ]
				nSeqNo = 0;
#endif
#endif
			}
			else
			{
				frames_clear( nAxis );
				v_move( nAxis, -m_HomeInfo[nAxis].dVel / 3.0, m_HomeInfo[nAxis].nAcc / 3 );
				nSeqNo = 30;
			}
		}
		break;

		// hskwak 190624 Home Sensor  ȵɶ ̵ Ѵ. Home Level ݴ . <<<<
	case 22:
		if( axis_done( nAxis ) ) 
		{
			set_home_level( nAxis, nHomeLevel[nAxis] );	// Home Level .
			set_home (nAxis, STOP_EVENT );

			frames_clear( nAxis );
			clear_status( nAxis );
			set_position( nAxis, 0.0 );
			start_r_move( nAxis, GetDistanceHomeOut(nAxis), m_HomeInfo[nAxis].dVel*2.0, m_HomeInfo[nAxis].nAcc / 2 );
			nSeqNo=20;
		}
		break;
		// >>>>>>>
	case 30:
		if( axis_done( nAxis ) ) 
		{
			set_home( nAxis, NO_EVENT );
			clear_status( nAxis );
			nSeqNo = 40;
		}
		break;
	case 40:
		if( axis_done( nAxis ) ) 
		{
			if( axis_state( nAxis ) != NO_EVENT )
			{
				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				nReturn = 1;
			}
			else
			{
				set_position( nAxis, 0.0 );
				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				nReturn = 0;
			}
			nSeqNo = 0;
		}
		break;
	}
	m_nHomeOnlyNo[nAxis] = nSeqNo;
	return nReturn;
}

//--------------------------------------------------------------------
//	HomeAndIndex(Int nAxis) 
//	Home Sensor Encoder Index Pulse ϴ ̵
//--------------------------------------------------------------------
int XMmc::HomeAndIndex(INT nAxis)
{
#ifdef _MY_DEBUG
	return 0;
#endif

	int nReturn = -1;

	static double dLimitPosNeg[MAX_AXES];
	static INT nActionNeg[MAX_AXES];

	static double dLimitPosPos[MAX_AXES];
	static INT nActionPos[MAX_AXES];

	int nSeqNo = m_nHomeAndIndexNo[nAxis];

	switch( nSeqNo ) 
	{
	case 0:
		// set no event for software limit
		get_negative_sw_limit( nAxis, &dLimitPosNeg[nAxis], &nActionNeg[nAxis] );
		set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], NO_EVENT );
		get_positive_sw_limit( nAxis, &dLimitPosPos[nAxis], &nActionPos[nAxis] );
		set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], NO_EVENT );
		set_home( nAxis, NO_EVENT );
//		controller_run( axis );
		frames_clear( nAxis );
		clear_status( nAxis );

		set_stop_rate( nAxis, m_HomeInfo[nAxis].nAcc / 2 );
		set_index_required( nAxis, FALSE );
		set_home( nAxis, STOP_EVENT );

		frames_clear( nAxis );
		v_move( nAxis, -m_HomeInfo[nAxis].dVel*4.0, m_HomeInfo[nAxis].nAcc / 2 );
		nSeqNo = 10;
		break;
	case 10:
		if( axis_done( nAxis ) ) 
		{
			set_home( nAxis, NO_EVENT );
//			controller_run( axis );
			frames_clear( nAxis );
			clear_status( nAxis );

			frames_clear( nAxis );
			start_r_move( nAxis, GetDistanceHomeOut(nAxis), m_HomeInfo[nAxis].dVel*2.0, m_HomeInfo[nAxis].nAcc / 2 );
			nSeqNo = 20;
		}
		break;
	case 20:
		if( axis_done( nAxis ) ) 
		{
			set_home( nAxis, NO_EVENT );
//			controller_run( axis );
			frames_clear( nAxis );
			clear_status( nAxis );

			// Set the Index
			set_index_required( nAxis, TRUE );
			set_stop_rate( nAxis, m_HomeInfo[nAxis].nAcc / 2 );
			set_home( nAxis, STOP_EVENT );

			frames_clear( nAxis );
			v_move( nAxis, -m_HomeInfo[nAxis].dVel / 5.0, m_HomeInfo[nAxis].nAcc / 3 );
			nSeqNo = 30;
		}
		break;
	case 30:
		if( axis_done( nAxis ) ) 
		{
			// Reset the Index
			set_index_required( nAxis, FALSE );

			set_home( nAxis, NO_EVENT );
//			controller_run( axis );
			frames_clear( nAxis );
			clear_status( nAxis );
			nSeqNo = 40;
		}
		break;
	case 40:
		if( axis_done( nAxis ) ) 
		{
			if( axis_state( nAxis ) != NO_EVENT )
			{
				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				nReturn = 1;
			}
			else
			{
				set_position( nAxis, 0.0 );
				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				nReturn = 0;
			}
			nSeqNo = 0;
		}
		break;
	}
	m_nHomeAndIndexNo[nAxis] = nSeqNo;
	return nReturn;
}

//--------------------------------------------------------------------
// Set the homing type
// Home Sensor or ( Home Sensor + Index Pulse of Encoder )
//--------------------------------------------------------------------
int XMmc::Home(INT nAxis)
{
	int nRv = 0;

	switch( m_HomeInfo[nAxis].nHome_type ) 
	{
	case HOME_SENSOR_AND_INDEX:
		nRv = HomeAndIndex( nAxis );
		break;
	case HOME_SENSOR_ONLY:
		nRv = HomeOnly( nAxis );
		break;
	}

	return nRv;
}

//--------------------------------------------------------------------
//  Robot  ̵  m_HomeAxis 
//                 x  y  z  o
// pAxes_order[] = {1, 1, 0, 2}̸
// z -> x, y  -> o  ̵ 
//--------------------------------------------------------------------
BOOL XMmc::GetHomeInfo(INT nRbtType, INT nMax_axes, INT *pAxes_order)
{
	INT nMin_axis = nMax_axes;
	INT nNo = 0;
	INT i;
	for( i = 0; i < MAX_RBT_AXES; i++ ) 
	{
		if( m_RbtInfo[nRbtType].nMask & ( 1 << i ) )
		{
			if( !m_HomeAxis[nRbtType].bFlag_home_end[nNo] )
			{
				if( nMin_axis > pAxes_order[i] ) nMin_axis = pAxes_order[i];
			}
			nNo++;
		}
	}

	// homing  .
	if( nMin_axis == nMax_axes ) return FALSE;

	// set the homing order
	INT order = nMin_axis;

	//  ϴ   
	nNo = 0;
	for( i = 0; i < MAX_RBT_AXES; i++ ) 
	{
		if( m_RbtInfo[nRbtType].nMask & ( 1 << i ) )
		{
			if( pAxes_order[i] == order ) 
			{
				// Set the homing axis
				m_HomeAxis[nRbtType].nFound[nNo] = i;
				nNo++;
			}
		}
	}

	m_HomeAxis[nRbtType].nNo = nNo;

	return TRUE;
}

//--------------------------------------------------------------------
// Move Robot Home
// m_HomeAxis[nRbtType].nNo : ÿ ̵ϴ  ִ  (2)
// m_HomeAxis[nRbtType].nFound : ̵   ã ( found[0]=1, found[1]=2 )
//		=> y, z ÿ ̵
// m_HomeAxis[nRbtType].order :  ̵   found no ã
// m_HomeAxis[nRbtType].nFlag_home_end[i] : TRUE => homing end
//										   FALSE => before homing
//--------------------------------------------------------------------
int XMmc::MoveRbtHome(INT nRbtType)
{
//	if( nRbtType == RBT_SPARE_UNIT) return 0;
#ifdef _MY_DEBUG
	//  Position 0
	m_nCurPoint[nRbtType] = 0;
	m_bHomeComp[nRbtType] = TRUE;
	return 0;
#endif

	static int nMax_axes[MAX_RBT_NO] = {0,};
	int nReturn = -1;

	int nSeqNo = m_nMoveRbtHomeNo[nRbtType];
	int i;

	switch( nSeqNo ) 
	{
	case 0:
		if( IsRbtSensorOk(nRbtType) )		// limit sensor  ҷ check   ֵ ߰ LYS20070109
		{									// limit sensor On ¸ Initial stop Ŵ
			if( m_RbtInfo[nRbtType].link.bIs_linked )
			{
				int nSlave = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].link.nSlave;
				nMax_axes[nRbtType] = m_RbtInfo[nRbtType].nMax_axes - 1;
			}
			else nMax_axes[nRbtType] = m_RbtInfo[nRbtType].nMax_axes;

			for( i = 0; i < MAX_RBT_AXES; i++ ) m_HomeAxis[nRbtType].bFlag_home_end[i] = TRUE;
			for( i = 0; i < nMax_axes[nRbtType]; i++ ) m_HomeAxis[nRbtType].bFlag_home_end[i] = FALSE;

			nSeqNo = 10;
		}
		else
		{
			return 1000; // Servo Sensor Status Error 
		}
		break;
	case 10:
		if( !GetHomeInfo( nRbtType, nMax_axes[nRbtType], m_RbtInfo[nRbtType].nHome_order ) )
		{	// Homing ends
			if( m_RbtInfo[nRbtType].link.bIs_linked )
			{
				int nSlave = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].link.nSlave;
				set_position( nSlave, 0.0 );	// set position to zero
			}
			for( i = 0; i < nMax_axes[nRbtType]; i++ ) m_HomeAxis[nRbtType].bFlag_home_end[i] = FALSE;
			
			//  Position 0
			m_nCurPoint[nRbtType] = 0;

			m_bHomeComp[nRbtType] = TRUE;
			nSeqNo = 0;
			nReturn = 0;
		}
		else nSeqNo = 20;
		break;
	case 20:
		for( i = 0; i < m_HomeAxis[nRbtType].nNo; i++ ) 
		{
			int nFound = m_RbtInfo[nRbtType].nXyz0_to_axis[m_HomeAxis[nRbtType].nFound[i]];

			if( !m_HomeAxis[nRbtType].bFlag_home_end[nFound] )	// ã axis  Homing  ʾҴٸ
			{
				int nResult = -1;
				int nAxis = m_RbtInfo[nRbtType].nStart_axis + nFound;
				if( ( nResult = Home( nAxis ) ) == 0 )
				{
					m_HomeAxis[nRbtType].bFlag_home_end[nFound] = TRUE;
					nSeqNo = 10;
				}
				else if( nResult > 0 ) // Homing error
				{
					nSeqNo = 0;
					return nResult;
				}
			}
		}
		break;
	}
	m_nMoveRbtHomeNo[nRbtType] = nSeqNo;
	return nReturn;
}

//--------------------------------------------------------------------
// MoveRbtPos()
// ش κ ۿϷḦ Ͽ ˻
//--------------------------------------------------------------------
BOOL XMmc::IsMoveOk(INT nRbtType)
{
	for( int i=0; i < m_RbtInfo[nRbtType].nMax_axes; i++ ) 
	{
		int nAxis = m_RbtInfo[nRbtType].nStart_axis + i;
		if( !axis_done( nAxis ) ) return FALSE;	// on moving
	}
	return TRUE;	// moving ends
}

///////////////////////////////////////////////////////////////////////////////////////////
// [2007.10.16]
// HJYOU 
// m_bServoReady[nRbtType] FALSE ϶  Լ ϸ ʵȴ.
///////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------
// Multi-Axis Point to Point Motion :   ȵ (trapezoidal move)
//--------------------------------------------------------------------
int XMmc::MoveRbtPtp1(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask)
{
#ifdef _MY_DEBUG
	return 0;
#endif
	if( !m_bServoReady[nRbtType] ) return -1;

	int nRv = -1;
	INT nNo = 0;
	int i;

	INT nAxes[MAX_RBT_AXES];
	double dPos_pulse[MAX_RBT_AXES];
	double dVel_pulse[MAX_RBT_AXES];
	INT nAcc_pulse[MAX_RBT_AXES];

	int nSeqNo = m_nMoveRbtPtp1No[nRbtType];

	switch( nSeqNo ) 
	{
	case 0:
		for( i = 0; i < MAX_RBT_AXES; i++ ) 
		{
			if( nMask & (1 << i) ) 
			{
				int nAxis = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nXyz0_to_axis[i];
				if( ( m_RbtInfo[nRbtType].link.bIs_linked ) && 
					( nAxis == m_RbtInfo[nRbtType].link.nSlave ) ) continue;

				nAxes[nNo] = nAxis;
				dPos_pulse[nNo] = Len2Pulse( nAxis, pos.dPos[i] );
				dVel_pulse[nNo] = Len2Pulse( nAxis, vel.dVel[i] ) * m_dRatio[nRbtType];
				nAcc_pulse[nNo] = m_AxisPara[nAxis].nAcc;
				nNo += 1;
			}
		}
		start_move_all( nNo, nAxes, dPos_pulse, dVel_pulse, nAcc_pulse );
		nSeqNo = 10;
		break;
	case 10:
		if( IsMoveOk( nRbtType ) ) 
		{
			nRv = get_mmc_error();
			nSeqNo = 0;
		}
		break;
	}
	m_nMoveRbtPtp1No[nRbtType] = nSeqNo;
	return nRv;
}

///////////////////////////////////////////////////////////////////////////////////////////
// [2007.10.16]
// HJYOU 
// m_bServoReady[nRbtType] FALSE ϶  Լ ϸ ʵȴ.
///////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------
// Multi-Axis Point to Point Motion :    (trapezoidal move)
//--------------------------------------------------------------------
int XMmc::MoveRbtPtp2(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask)
{
#ifdef _MY_DEBUG
	return 0;
#endif
	if( !m_bServoReady[nRbtType] ) return -1;

	int nRv = -1;
	int nNo = 0;
	int i;

	INT nAxes[MAX_RBT_AXES];
	double dPos_pulse[MAX_RBT_AXES];
	double dVel_pulse[MAX_RBT_AXES];
	int nAcc_pulse[MAX_RBT_AXES];

	double dMax_vel = 0;
	int nMax_acc = 0;

	int nSeqNo = m_nMoveRbtPtp2No[nRbtType];

	switch( nSeqNo ) 
	{
	case 0:
		for( i = 0; i < MAX_RBT_AXES; i++ ) 
		{
			if( nMask & (1 << i) ) 
			{
				int nAxis = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nXyz0_to_axis[i];
				if( ( m_RbtInfo[nRbtType].link.bIs_linked ) && 
					( nAxis == m_RbtInfo[nRbtType].link.nSlave ) ) continue;

				nAxes[nNo] = nAxis;
				dPos_pulse[nNo] = Len2Pulse( nAxis, pos.dPos[i] );
				dVel_pulse[nNo] = Len2Pulse( nAxis, vel.dVel[i] );
				nAcc_pulse[nNo] = m_AxisPara[nAxis].nAcc;

				// ϴ ִ밪  ̵Ѵ.
				if( dMax_vel < dVel_pulse[nNo] ) dMax_vel = dVel_pulse[nNo];
				if( nMax_acc < nAcc_pulse[nNo] ) nMax_acc = nAcc_pulse[nNo];
				nNo += 1;
			}
		}
		map_axes( nNo, nAxes );
		set_move_speed( dMax_vel * m_dRatio[nRbtType] );
		set_move_accel( nMax_acc );
		move_4( dPos_pulse[0], dPos_pulse[1], dPos_pulse[2], dPos_pulse[3] );
		nSeqNo = 10;
		break;
	case 10:
		if( IsMoveOk( nRbtType ) ) 
		{
			nRv = get_mmc_error();
			nSeqNo = 0;
		}
		break;
	}

	m_nMoveRbtPtp2No[nRbtType] = nSeqNo;
	return nRv;
}

//--------------------------------------------------------------------
// s_move :   ȵ (s-curve move)
//--------------------------------------------------------------------
int XMmc::MoveRbtS1(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask)
{
#ifdef _MY_DEBUG
	return 0;
#endif

	int nRv = -1;
	int nNo = 0;
	int i;

	int nAxes[MAX_RBT_AXES];
	double dPos_pulse[MAX_RBT_AXES];
	double dVel_pulse[MAX_RBT_AXES];
	int nAcc_pulse[MAX_RBT_AXES];

	int nSeqNo = m_nMoveRbtS1No[nRbtType];

	switch( nSeqNo ) 
	{
	case 0:
		for(i = 0; i < MAX_RBT_AXES; i++) 
		{
			if( nMask & (1 << i) ) 
			{
				int nAxis = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nXyz0_to_axis[i];
				if( ( m_RbtInfo[nRbtType].link.bIs_linked ) && 
					( nAxis == m_RbtInfo[nRbtType].link.nSlave ) ) continue;

				nAxes[nNo] = nAxis;
				dPos_pulse[nNo] = Len2Pulse( nAxis, pos.dPos[i] );
				dVel_pulse[nNo] = Len2Pulse( nAxis, vel.dVel[i] ) * m_dRatio[nRbtType];
				nAcc_pulse[nNo] = m_AxisPara[nAxis].nAcc;

				// S-Curve ӵ Profile, velӵ posġ ŭ ̵.
				start_s_move( nAxes[nNo], dPos_pulse[nNo], dVel_pulse[nNo], nAcc_pulse[nNo] );
				nNo += 1;
			}
		}
//		start_s_move_all( no, axes, pos_pulse, vel_pulse, acc_pulse );
		nSeqNo = 10;
		break;
	case 10:
		if( IsMoveOk( nRbtType ) ) 
		{
			nRv = get_mmc_error();

			char szErr[200];
			error_message( nRv, szErr );
			nSeqNo = 0;
		}
		break;
	}
	m_nMoveRbtS1No[nRbtType] = nSeqNo;
	return nRv;
}

//--------------------------------------------------------------------
// Multi-Axis S Motion :    (s-curve move)
//--------------------------------------------------------------------
int XMmc::MoveRbtS2(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask)
{
#ifdef _MY_DEBUG
	return 0;
#endif

	int nNo = 0;
	int i;
	int nRv = -1;

	INT nAxes[MAX_RBT_AXES];
	double dPos_pulse[MAX_RBT_AXES];
	double dVel_pulse[MAX_RBT_AXES];
	int nAcc_pulse[MAX_RBT_AXES];

	double dMax_vel = 0;
	int nMax_acc = 0;

	int nSeqNo = m_nMoveRbtS2No[nRbtType];

	switch( nSeqNo ) 
	{
	case 0:
		for( i=0; i<MAX_RBT_AXES; i++ ) 
		{
			if( nMask & (1 << i) ) 
			{
				int nAxis = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nXyz0_to_axis[i];
				if( ( m_RbtInfo[nRbtType].link.bIs_linked ) && 
					( nAxis == m_RbtInfo[nRbtType].link.nSlave ) ) continue;

				nAxes[nNo] = nAxis;
				dPos_pulse[nNo] = Len2Pulse( nAxis, pos.dPos[i] );
				dVel_pulse[nNo] = Len2Pulse( nAxis, vel.dVel[i] );
				nAcc_pulse[nNo] = m_AxisPara[nAxis].nAcc;

				// ϴ ִ밪  ̵Ѵ.
				if( dMax_vel < dVel_pulse[nNo] ) dMax_vel = dVel_pulse[nNo];
				if( nMax_acc < nAcc_pulse[nNo] ) nMax_acc = nAcc_pulse[nNo];
				nNo += 1;
			}
		}
		map_axes( nNo, nAxes );
		set_move_speed( dMax_vel * m_dRatio[nRbtType] );
		set_move_accel( nMax_acc );
		smove_4( dPos_pulse[0], dPos_pulse[1], dPos_pulse[2], dPos_pulse[3] );
		nSeqNo = 10;
		break;
	case 10:
		if( IsMoveOk( nRbtType ) ) 
		{
			nRv = get_mmc_error();
			nSeqNo = 0;
		}
		break;
	}
	m_nMoveRbtS2No[nRbtType] = nSeqNo;
	return nRv;
}

//--------------------------------------------------------------------
// Set the velocity of MMC information
//--------------------------------------------------------------------
void XMmc::SetVel(INT nRbtType, RBT_VEL vel)
{
	m_RbtInfo[nRbtType].vel = vel;
}

//--------------------------------------------------------------------
// Set the velocity value (From Registry)
//--------------------------------------------------------------------
void XMmc::SetVelFromRegistry(INT nRbtType)
{
//	if( nRbtType == RBT_SPARE_UNIT) return;

	m_IniVelInfo.Open("ServoVel", "ServoInfo");
	// Get the velocity
	INT	nMask = m_RbtInfo[nRbtType].nMask;
	BOOL bLink = m_RbtInfo[nRbtType].link.bIs_linked;
	INT nSlave = m_RbtInfo[nRbtType].link.nSlave;
	RBT_VEL rvVel;
	for( int j=0; j<MAX_RBT_AXES; j++) 
	{
		if( ( nMask & ( 1 << j ) ) && !( bLink && ( j == nSlave ) ) )
		{
			CString sEntry;
			sEntry.Format( "%s %2d", m_sRbtName[nRbtType], j );
			CDicsApp* pApp = (CDicsApp*) AfxGetApp();

			CString sVel = m_IniVelInfo.GetStringValue("DICS VEL",sEntry,"100.00");
			rvVel.dVel[j] = atof( sVel );
		}
		else rvVel.dVel[j] = 0.0;
	}

	SetVel( nRbtType, rvVel );
}

//--------------------------------------------------------------------
// Set the velocity value (To Registry)
//--------------------------------------------------------------------
void XMmc::SetVelToRegistry(INT nRbtType, RBT_VEL vel)
{
//	if( nRbtType == RBT_SPARE_UNIT) return;

	// Get the velocity
	m_IniVelInfo.Open("ServoVel", "ServoInfo");
	INT	nMask = m_RbtInfo[nRbtType].nMask;
	BOOL bLink = m_RbtInfo[nRbtType].link.bIs_linked;
	INT nSlave = m_RbtInfo[nRbtType].link.nSlave;
	for( int j=0; j<MAX_RBT_AXES; j++) 
	{
		if( ( nMask & ( 1 << j ) ) && !( bLink && ( j == nSlave ) ) )
		{
			CString sVel;
			sVel.Format( "%.2f", vel.dVel[j] );

			CString sEntry;
			sEntry.Format( "%s %2d", m_sRbtName[nRbtType], j );
			m_IniVelInfo.SetStringValue("DICS VEL",sEntry,sVel, TRUE);
		}
	}

	SetVel( nRbtType, vel );
}
//--------------------------------------------------------------------
// Set the Theta value (To Registry)
//--------------------------------------------------------------------
void XMmc::SetThetaToRegistry(INT nRbtType, double dtheta )
{
//	if( nRbtType == RBT_SPARE_UNIT ) return;

	// Get the velocity
	m_IniThetaInfo.Open("ServoTheta", "ServoInfo");
	INT	nMask = m_RbtInfo[nRbtType].nMask;
	BOOL bLink = m_RbtInfo[nRbtType].link.bIs_linked;
	INT nSlave = m_RbtInfo[nRbtType].link.nSlave;

	CString sTheta;
	sTheta.Format( "%.4f", dtheta );

	CString sEntry;
	sEntry.Format( "%s", m_sRbtName[nRbtType] );
	m_IniThetaInfo.SetStringValue("DICS THETA",sEntry,sTheta, TRUE);

	SetTheta( nRbtType, dtheta );
}

//--------------------------------------------------------------------
// Set the velocity value (From Registry)
//--------------------------------------------------------------------
void XMmc::SetThetaFromRegistry(INT nRbtType)
{
//	if( nRbtType == RBT_SPARE_UNIT ) return;

	m_IniThetaInfo.Open("ServoTheta", "ServoInfo");
	// Get the velocity
	INT	nMask = m_RbtInfo[nRbtType].nMask;
	BOOL bLink = m_RbtInfo[nRbtType].link.bIs_linked;
	INT nSlave = m_RbtInfo[nRbtType].link.nSlave;
	double dtheta = 0.0;

	CString sEntry;
	sEntry.Format( "%s", m_sRbtName[nRbtType]);
	CDicsApp* pApp = (CDicsApp*) AfxGetApp();

	CString sTheta = m_IniThetaInfo.GetStringValue("DICS THETA",sEntry,"0.8726");
	dtheta = atof( sTheta );

	SetTheta( nRbtType, dtheta );
}

//--------------------------------------------------------------------
// Set the Axis Ratio value (To Registry)
//--------------------------------------------------------------------
void XMmc::SetAxisRatioToRegistry(INT nRbtType, double dRatio)	// hskwak 190806
{
	// Get the velocity
	m_IniThetaInfo.Open("AxisRatio", "ServoInfo");
	INT	nMask = m_RbtInfo[nRbtType].nMask;
	BOOL bLink = m_RbtInfo[nRbtType].link.bIs_linked;
	INT nSlave = m_RbtInfo[nRbtType].link.nSlave;

	CString sTheta;
	sTheta.Format( "%.4f", dRatio );

	CString sEntry;
	sEntry.Format( "%s", m_sRbtName[nRbtType] );
	m_IniThetaInfo.SetStringValue("DICS RATIO",sEntry,sTheta, TRUE);

	SetAxisRatio( nRbtType, dRatio );
}

//--------------------------------------------------------------------
// Set the Axis Ratio value (From Registry)
//--------------------------------------------------------------------
void XMmc::SetAxisRatioFromRegistry(INT nRbtType)	// hskwak 190806
{
	m_IniThetaInfo.Open("AxisRatio", "ServoInfo");
	// Get the velocity
	INT	nMask = m_RbtInfo[nRbtType].nMask;
	BOOL bLink = m_RbtInfo[nRbtType].link.bIs_linked;
	INT nSlave = m_RbtInfo[nRbtType].link.nSlave;
	double dRatio = 0.0;

	CString sEntry;
	sEntry.Format( "%s", m_sRbtName[nRbtType]);
	CDicsApp* pApp = (CDicsApp*) AfxGetApp();

	CString sTheta = m_IniThetaInfo.GetStringValue("DICS RATIO",sEntry,"0.26");
	dRatio = atof( sTheta );

	// hskwak 20191007 Ѱ ߰.
	if( dRatio < 0.2 )		dRatio = 2.0;	
	else if( dRatio > 0.3 )	dRatio = 3.0;

	SetAxisRatio( nRbtType, dRatio );
}

//--------------------------------------------------------------------
// Set the Axis Ratio.
//--------------------------------------------------------------------
void XMmc::SetAxisRatio(INT nRbtType, double dTheta)
{
	m_dAxisRatio[nRbtType] = dTheta;
}
//--------------------------------------------------------------------
// Get the Axis Ratio.
//--------------------------------------------------------------------
double XMmc::GetAxisRatio(INT nRbtType)
{
	if( m_AxisPara[nRbtType].bRatioFile == FALSE )
		return m_AxisPara[nRbtType].dRatio;	// Para Ratio   ʿ.
	else
		return m_dAxisRatio[nRbtType];
}

//--------------------------------------------------------------------
// Set the velocity of MMC information
//--------------------------------------------------------------------
void XMmc::SetTheta(INT nRbtType, double dTheta)
{
	m_AxisPara[nRbtType].dTheta_H = dTheta;
}

//--------------------------------------------------------------------
// Get the velocity of MMC information
//--------------------------------------------------------------------
double XMmc::GetTheta(INT nRbtType)
{
	return m_AxisPara[nRbtType].dTheta_H;
}

//--------------------------------------------------------------------
// Get the velocity of MMC information
//--------------------------------------------------------------------
RBT_VEL XMmc::GetVel(INT nRbtType)
{
	return m_RbtInfo[nRbtType].vel;
}

///////////////////////////////////////////////////////////////////////////////////////////
// [2007.10.16]
// HJYOU 
// m_bServoReady[nRbtType] FALSE ϶  Լ ϸ ʵȴ.
///////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------
// MoveRbtPos() : moving by teaching point
// return : 0		=> Moving OK
//          1 ~ 14	=> MMC board error
//			100		=> position set error
//			200		=> event error
//			-1		=> now moving
//--------------------------------------------------------------------
int XMmc::MoveRbtPos(INT nRbtType, INT nPosNo)
{
#ifdef _MY_DEBUG
	//  ġ Set
	m_nCurPoint[nRbtType] = nPosNo;
	return 0;
#endif
	if( !m_bServoReady[nRbtType] ) return -1;

	int nRv = -1;
	int nResult;
	static int nRtyNo[MAX_RBT_NO] = { 0, };

	static RBT_POS pos[MAX_RBT_NO] = {0.0,};
	static RBT_VEL vel[MAX_RBT_NO] = {0.0,};
	static RBT_POS curPos[MAX_RBT_NO] = {0.0,};

	int nSeqNo = m_nMoveRbtPosNo[nRbtType];

	m_nCurPoint[nRbtType] = nPosNo;		// ġ ⼭ Ѵ.

	switch( nSeqNo ) 
	{
	case 0:
		if( (nPosNo > m_RbtInfo[nRbtType].nMax_pos_no - 1) || (nPosNo < 0) ) return 100;	// error
		switch( m_RbtInfo[nRbtType].nMove_type ) 
		{
		case RBT_S1_MOVE:
			if( ( nResult = MoveRbtS1( nRbtType, m_RbtInfo[nRbtType].pos[nPosNo], m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		case RBT_S2_MOVE:
			if( ( nResult = MoveRbtS2( nRbtType, m_RbtInfo[nRbtType].pos[nPosNo], m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		case RBT_T1_MOVE:
			if( ( nResult = MoveRbtPtp1( nRbtType, m_RbtInfo[nRbtType].pos[nPosNo], m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		case RBT_T2_MOVE:
			if( ( nResult = MoveRbtPtp2( nRbtType, m_RbtInfo[nRbtType].pos[nPosNo], m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		}
		break;
	case 10:
		if( IsRbtStatusOk( nRbtType ) )
		{
			nSeqNo = 20;
		}
		else
		{
			nRtyNo[nRbtType] = 0;
			nRv = 200;	// error
			nSeqNo = 0;
		}
		break;
		
	case 20:
		if ( GetCurPosition( nRbtType, curPos[nRbtType] ) )
		{
			
			if( (ABS_VAL( m_RbtInfo[nRbtType].pos[nPosNo].dPos[0] - curPos[nRbtType].dPos[0] ) < IN_POSITION_VALUE) &&
				(ABS_VAL( m_RbtInfo[nRbtType].pos[nPosNo].dPos[1] - curPos[nRbtType].dPos[1] ) < IN_POSITION_VALUE) &&
				(ABS_VAL( m_RbtInfo[nRbtType].pos[nPosNo].dPos[2] - curPos[nRbtType].dPos[2] ) < IN_POSITION_VALUE) &&
				(ABS_VAL( m_RbtInfo[nRbtType].pos[nPosNo].dPos[3] - curPos[nRbtType].dPos[3] ) < IN_POSITION_VALUE) )
			{
				//  ġ Set
	//			m_nCurPoint[nRbtType] = nPosNo;
				nRv = 0;
				nRtyNo[nRbtType] = 0;
				nSeqNo = 0;
			}
			else
			{
				if( nRtyNo[nRbtType] < RETRY_NO ) 
				{
					nSeqNo = 0;
					nRtyNo[nRbtType] += 1;
				}
				else
				{
					nRtyNo[nRbtType] = 0;
					nRv = 300;	// error
					nSeqNo = 0;
				}
			}
		}
		else
		{
			nRtyNo[nRbtType] = 0;
			nRv = 400;	// error
			nSeqNo = 0;			
		}
		break;  
	}

	m_nMoveRbtPosNo[nRbtType] = nSeqNo;

	return nRv;
}

///////////////////////////////////////////////////////////////////////////////////////////
// [2007.10.16]
// HJYOU 
// m_bServoReady[nRbtType] FALSE ϶  Լ ϸ ʵȴ.
///////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------
// MoveRbtPos() : moving by position data
// return : 0		=> Moving OK
//          1 ~ 14	=> MMC board error
//			100		=> position set error
//			200		=> event error
//			-1		=> now moving
//--------------------------------------------------------------------
int XMmc::MoveRbtPos(INT nRbtType, RBT_POS pos)
{
#ifdef _MY_DEBUG
	return 0;
#endif
	if( !m_bServoReady[nRbtType] ) return -1;
//	m_nCurPoint[nRbtType] = nPosNo;
	int nRv = -1;
	int nResult;
	static int nRtyNo[MAX_RBT_NO] = { 0, };
	
	static RBT_POS posCheck[MAX_RBT_NO] = {0.0,};
	static RBT_VEL vel[MAX_RBT_NO] = {0.0,};
	static RBT_POS curPos[MAX_RBT_NO] = {0,};
	posCheck[nRbtType].dPos[0] = pos.dPos[0];
	posCheck[nRbtType].dPos[1] = pos.dPos[1];
	posCheck[nRbtType].dPos[2] = pos.dPos[2];
	posCheck[nRbtType].dPos[3] = pos.dPos[3];
	int nSeqNo = m_nMoveRbtPosNo[nRbtType];

	switch( nSeqNo ) 
	{
	case 0:
		switch( m_RbtInfo[nRbtType].nMove_type ) 
		{
		case RBT_S1_MOVE:
			if( ( nResult = MoveRbtS1( nRbtType, pos, m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		case RBT_S2_MOVE:
			if( ( nResult = MoveRbtS2( nRbtType, pos, m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		case RBT_T1_MOVE:
			if( ( nResult = MoveRbtPtp1( nRbtType, pos, m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		case RBT_T2_MOVE:
			if( ( nResult = MoveRbtPtp2( nRbtType, pos, m_RbtInfo[nRbtType].vel, m_RbtInfo[nRbtType].nMask ) ) == 0 ) 
			{
				nSeqNo = 10;
			}
			else if( nResult > 0 ) return nResult;	// error
			break;
		}
		break;
	case 10:
		if( IsRbtStatusOk( nRbtType ) )
		{
			nSeqNo = 20;
		}
		else
		{
			nRtyNo[nRbtType] = 0;
			nRv = 200;	// error
			nSeqNo = 0;
		}
		break;
	case 20:
		if( GetCurPosition( nRbtType, curPos[nRbtType] ) )
		{
			if( ABS_VAL( posCheck[nRbtType].dPos[0] - curPos[nRbtType].dPos[0] ) < IN_POSITION_VALUE &&
				ABS_VAL( posCheck[nRbtType].dPos[1] - curPos[nRbtType].dPos[1] ) < IN_POSITION_VALUE &&
				ABS_VAL( posCheck[nRbtType].dPos[2] - curPos[nRbtType].dPos[2] ) < IN_POSITION_VALUE &&
				ABS_VAL( posCheck[nRbtType].dPos[3] - curPos[nRbtType].dPos[3] ) < IN_POSITION_VALUE )
			{
				nRv = 0;
				nRtyNo[nRbtType] = 0;
				nSeqNo = 0;
			}
			else
			{
				if( nRtyNo[nRbtType] < RETRY_NO ) 
				{
					nSeqNo = 0;
					nRtyNo[nRbtType] += 1;
				}
				else
				{
					nRtyNo[nRbtType] = 0;
					nRv = 300;	// error
					nSeqNo = 0;
				}
			}
		}
		else
		{
			nRtyNo[nRbtType] = 0;
			nRv = 400;	// error
			nSeqNo = 0;			
		}
		break;  
	}

	m_nMoveRbtPosNo[nRbtType] = nSeqNo;
	return nRv;
}

///////////////////////////////////////////////////////////////////////////////////////////
// [2007.10.16]
// HJYOU 
// m_bServoReady[nRbtType] FALSE ϶  Լ ϸ ʵȴ.
///////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------
// Constant velocity relative move
//	if Robot : x, z		mask : 0x05
//	nRbt_axis : v_move(z) => nRbt_axis = 2
//	but start_rs_move( 1, ...)
//--------------------------------------------------------------------
int XMmc::RelativeMove(INT nRbtType, INT nRbt_axis, INT nSign, double dDist, double dVel)
{
#ifdef _MY_DEBUG
	return 0;
#endif
	if( !m_bServoReady[nRbtType] ) return -1;

	int nRv = -1;
	static int nAxis[MAX_RBT_NO] = { 0, };
	int i;
	double dVel_pulse;
	double dDist_pulse;

	int nSeqNo = m_nRelativeMoveNo[nRbtType];

	switch( nSeqNo )
	{
	case 0:
		for( i=0; i < nRbt_axis; i++ ) 
		{
			if( m_RbtInfo[nRbtType].nMask & ( 1 << i ) ) nAxis[nRbtType]++;
		}

		nAxis[nRbtType] += m_RbtInfo[nRbtType].nStart_axis;
		dVel_pulse = Len2Pulse( nAxis[nRbtType], dVel );
		dDist_pulse = Len2Pulse( nAxis[nRbtType], dDist ) * nSign;
	
		frames_clear( nAxis[nRbtType] );
		start_rs_move( nAxis[nRbtType], dDist_pulse, dVel_pulse, m_AxisPara[nAxis[nRbtType]].nAcc );
		nSeqNo = 10;
		break;
	case 10:
		if( axis_done( nAxis[nRbtType] ) ) 
		{
			nAxis[nRbtType] = 0;
			nSeqNo = 0;
			nRv = 0;
		}
		break;
	}

	m_nRelativeMoveNo[nRbtType] = nSeqNo;
	return nRv;
}

//--------------------------------------------------------------------
// Constant velocity move
//	if Robot : x, z		mask : 0x05
//	nRbt_axis : v_move(z) => nRbt_axis = 2
//	but v_move( 1, ...)
// 
//	vel unit : mm/sec or rev/sec
//--------------------------------------------------------------------
void XMmc::StartVelMove(INT nRbtType, INT nRbt_axis, INT nSign, double dVel)
{
#ifdef _MY_DEBUG
	m_bVelMove[nRbtType] = TRUE;
	return;
#endif

	int nAxis = 0;
	for( int i=0; i < nRbt_axis; i++ ) 
	{
		if( m_RbtInfo[nRbtType].nMask & ( 1 << i ) ) nAxis++;
	}

	nAxis += m_RbtInfo[nRbtType].nStart_axis;
	double dVel_pulse = Len2Pulse( nAxis, dVel ) * nSign;



	frames_clear( nAxis );
	v_move( nAxis, dVel_pulse, m_AxisPara[nAxis].nAcc );

	m_bVelMove[nRbtType] = TRUE;
}

//--------------------------------------------------------------------
// Stop the constant velocity move
//--------------------------------------------------------------------
void XMmc::StopVelMove(INT nRbtType, INT nRbt_axis)
{
#ifdef _MY_DEBUG
	m_bVelMove[nRbtType] = FALSE;
	return;
#endif

	int nAxis = 0;
	for(int i=0; i < nRbt_axis; i++) 
	{
		if(m_RbtInfo[nRbtType].nMask & (1 << i)) nAxis++;
	}
	nAxis += m_RbtInfo[nRbtType].nStart_axis;
	v_move_stop( nAxis );

	m_bVelMove[nRbtType] = FALSE;
}

//--------------------------------------------------------------------
// Get current position
//--------------------------------------------------------------------
BOOL XMmc::GetCurPosition(INT nRbtType, RBT_POS& rpPos)
{
#ifdef _MY_DEBUG
	static double dTemp = 2.7;
	dTemp += 5.6;
	rpPos.dPos[0] = dTemp;
	rpPos.dPos[1] = dTemp;
	rpPos.dPos[2] = dTemp;
	rpPos.dPos[3] = dTemp;
	return TRUE;
#endif

	for(int i = 0; i < MAX_RBT_AXES; i++) 
	{
		if( m_RbtInfo[nRbtType].nMask & (1 << i) ) 
		{
			int nAxis = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nXyz0_to_axis[i];
			if( !get_position( nAxis, &rpPos.dPos[i] ) )
				rpPos.dPos[i] = Pulse2Len( nAxis, rpPos.dPos[i] );
			else return FALSE;
		}
		else rpPos.dPos[i] = 0.0;
	}

	return TRUE;
}

//--------------------------------------------------------------------
// Teaching PointȭϿ ޸𸮷 Position 
//--------------------------------------------------------------------
BOOL XMmc::PosFile2Mem(INT nRbtType)
{
	FILE* pFile;
	int nNo;
	double dPos[4];
	char szfile_name[80];

	wsprintf( szfile_name, "D:\\DICS\\ServoInfo\\Point\\%s", (LPCSTR) m_sRbtFileName[nRbtType] );
	pFile = fopen( szfile_name, "rt" );

	if( pFile == NULL ) 
	{
		char szpath_name[80];
		wsprintf( szpath_name, "D:\\DICS\\ServoInfo\\Point" );
		if( !CreateDirectory( (LPCSTR) szpath_name, NULL ) ) return FALSE;
		if( ( pFile = fopen( szfile_name, "rt" ) ) == NULL ) return FALSE;
	}

	int nMaxPosNo = m_RbtInfo[nRbtType].nMax_pos_no;
	RBT_POS* pPos = m_RbtInfo[nRbtType].pos;

	for( int i = 0; i < nMaxPosNo; i++ ) 
	{
		if( fscanf( pFile, "%d %lf %lf %lf %lf\n", &nNo, &dPos[0], &dPos[1], &dPos[2], &dPos[3] ) == EOF ) break;
		//for( int j = 0; j < 4; j++ ) pPos[i].dPos[j] = dPos[j];
		for( int j = 0; j < 4; j++ )	// hskwak 191005 Pulse Ͽ  ϹǷ о mm ٲپ ش
		{
			pPos[i].dPulse[j] = dPos[j];
			pPos[i].dPos[j] = Pulse2Len(nRbtType, dPos[j]);			
		}
	}

	fclose( pFile );

	return TRUE;
}

//--------------------------------------------------------------------
// ޸𸮿 ִ Position ȭϷ 
//--------------------------------------------------------------------
BOOL XMmc::PosMem2File(INT nRbtType)
{
	FILE* pFile;
	double dPos[4];
	char szfile_name[80];

	wsprintf( szfile_name, "D:\\DICS\\ServoInfo\\Point\\%s", (LPCSTR) m_sRbtFileName[nRbtType] );
	pFile = fopen( szfile_name, "wt" );

	if( pFile == NULL ) 
	{
		char szpath_name[80];
		wsprintf( szpath_name, "D:\\DICS\\ServoInfo\\Point" );
		if( !CreateDirectory( (LPCSTR) szpath_name, NULL ) ) return FALSE;
		if( ( pFile = fopen( szfile_name, "wt" ) ) == NULL ) return FALSE;
	}

	int nMaxPosNo = m_RbtInfo[nRbtType].nMax_pos_no;
	RBT_POS* pPos = m_RbtInfo[nRbtType].pos;

	for( int i = 0; i < nMaxPosNo; i++ ) 
	{
		//for( int j=0; j<4; j++ ) dPos[j] = pPos[i].dPos[j];
		for( int j = 0; j < 4; j++ )	// hskwak 191005 Pulse Ͽ  ϹǷ о mm ٲپ ش
		{
			dPos[j] = Len2Pulse(nRbtType, pPos[i].dPos[j]);			
		}
		fprintf( pFile, "%3d %.03f %.03f %.03f %.03f\n", i, dPos[0], dPos[1], dPos[2], dPos[3] );
	}
	fclose( pFile );

	return TRUE;
}

//--------------------------------------------------------------------
// return the MMC board error message
//--------------------------------------------------------------------
INT XMmc::GetErrorMessage( CString& sMessage ) 
{ 
	char szErr[80];

	INT nCode = get_mmc_error(); 
	error_message( nCode, szErr );
	sMessage.Format( "%s", szErr );

	return nCode;
}

//--------------------------------------------------------------------
// Homing sequence ResetѴ.
//--------------------------------------------------------------------
void XMmc::ResetRbtHomeSeq(INT nRbtType)
{
	m_nMoveRbtHomeNo[nRbtType] = 0;
	for( int i = 0; i < m_RbtInfo[nRbtType].nMax_axes; i++ )
	{
		int nAxis = m_RbtInfo[nRbtType].nStart_axis + i;
		m_nHomeOnlyNo[nAxis] = 0;
	}
}

//--------------------------------------------------------------------
//  Teaching Point ȣ ´.
//--------------------------------------------------------------------
int	XMmc::GetCurPoint( INT nRbtType )
{
#ifdef	_MY_DEBUG
	return m_nCurPoint[nRbtType];
#endif
	double dPos[4] = {0.0,};
	double dAbsPos[4] = {0.0,};	// absolute different position

	RBT_POS curPos = {0.0,};
	GetCurPosition( nRbtType, curPos );	// Servo  ġ ´.

	int nMaxPosNo = m_RbtInfo[nRbtType].nMax_pos_no;
	RBT_POS* pPos = m_RbtInfo[nRbtType].pos;

	for( int i = 0; i < nMaxPosNo; i++ ) 
	{
		for( int j = 0; j < 4; j++ )
		{
			if( m_RbtInfo[nRbtType].nMask & ( 1 << j ) ) 
			{
				dPos[j] = pPos[i].dPos[j];
			}
			else dPos[j] = 0.0;
			dAbsPos[j] = ABS_VAL( dPos[j] - curPos.dPos[j] );
		}
		if( ( dAbsPos[X_AXIS] < IN_POSITION_VALUE ) &&
			( dAbsPos[Y_AXIS] < IN_POSITION_VALUE ) &&
			( dAbsPos[Z_AXIS] < IN_POSITION_VALUE ) &&
			( dAbsPos[T_AXIS] < IN_POSITION_VALUE ) )
		{
			m_nCurPoint[nRbtType] = i;
			return m_nCurPoint[nRbtType];
		}
	}
	m_nCurPoint[nRbtType] = -1;
	return m_nCurPoint[nRbtType]; // ġϴ Teching ġ .
}

void XMmc::GetAmpEnable( int nAxis, short * nLevel )
{
	get_amp_enable( nAxis, nLevel );
}

BOOL XMmc::GetHomeSwitch(INT nAxis)
{
#ifndef _MY_DEBUG
	return home_switch( nAxis );
#else
	return TRUE;
#endif
}
//  [2/3/2006]//////////////////////////////////////////////////////////////////////////
//  JHHONG
//	nRbtType : ش ROBOT
//	nPos : Ϸ Teaching positionȣ
//  [2/3/2006]//////////////////////////////////////////////////////////////////////////
void XMmc::SetCurPosition( INT nRbtType, INT nPos )
{
	//if( nRbtType == RBT_SPARE_UNIT ) return;

	int nMask = m_RbtInfo[nRbtType].nMask;
	int nNo = 0;
	INT nAmpEnable;
	INT nEncoderDir;
	double dPosition;
	
	for( int i = 0; i < MAX_RBT_AXES; i++ ) 
	{
		if( nMask & (1 << i) ) 
		{
			int nAxis = m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].nXyz0_to_axis[i];
//			if( ( m_RbtInfo[nRbtType].link.bIs_linked ) && 
//				( nAxis == m_RbtInfo[nRbtType].link.nSlave ) ) continue;
			set_amp_enable( nAxis, 0 );
			get_amp_enable( nAxis, &nAmpEnable );
			if( !nAmpEnable )
			{
				get_encoder_direction( nAxis, &nEncoderDir );
				if( nEncoderDir ) 
					dPosition = m_RbtInfo[nRbtType].pos[nPos].dPos[i] * -1.;
				else dPosition = m_RbtInfo[nRbtType].pos[nPos].dPos[i];
				dPosition = Len2Pulse( nAxis, dPosition );
				set_position( nAxis, dPosition );
			}
		}
	}
}
// [2007.10.16] ////////////////////////////////////////////////////////////////////////////
// HJYOU 
// nRbtType : ش ROBOT
// Master&Slave Motor Position Sync.
int XMmc::SetSyncMasterSlave(int nRbtType)
{
	int nResult = -1;
	static int nAxis[MAX_RBT_NO] = { 0, };
	static RBT_POS curPos[MAX_RBT_NO] = {0,};
	static double dDistance[MAX_RBT_NO] = {0.0,};
	
	int nSeqNo = m_nSetSyncMasterSlaveNo[nRbtType];

	switch( nSeqNo )
	{
	case 0:
		if( m_RbtInfo[nRbtType].link.bIs_linked )
		{
			GetCurPosition( nRbtType, curPos[nRbtType] );
			dDistance[nRbtType] = curPos[nRbtType].dPos[0] - curPos[nRbtType].dPos[1];
			if( ABS_VAL(dDistance[nRbtType]) < 5.0 )
			{
				set_sync_control_ax( m_RbtInfo[nRbtType].nStart_axis+m_RbtInfo[nRbtType].link.nSlave, FALSE, m_RbtInfo[nRbtType].nStart_axis, 0 );
				nAxis[nRbtType] = m_RbtInfo[nRbtType].nStart_axis;
				nSeqNo = 10;
			}
			else nResult = 1;	// Master, Slave  ġ  5.0mm  ũ  ִ ɷ Ѵ.
		}
		else
		{
			nResult = 0;
		}
		break;
	case 10:
		if( ClearAxisErr( nAxis[nRbtType] ) )
		{
			set_amp_enable( nAxis[nRbtType], TRUE );
			if( nAxis[nRbtType] < m_RbtInfo[nRbtType].nStart_axis + m_RbtInfo[nRbtType].link.nSlave ) nAxis[nRbtType] += 1;
			else
			{
				clear_status( nAxis[nRbtType] );
				double dPulse = Len2Pulse( nAxis[nRbtType], dDistance[nRbtType] );
				frames_clear( nAxis[nRbtType] );
				Sleep(10);	// 1 sec Delay Ű  amp_enable  ʹ  start_r_move   servo ̵ Ѵ. 
				start_r_move( nAxis[nRbtType], dPulse, m_HomeInfo[nAxis[nRbtType]].dVel / 3.0, m_HomeInfo[nAxis[nRbtType]].nAcc / 2 );
				nSeqNo = 20;
			}
		}
		break;
	case 20:
		if( axis_done( nAxis[nRbtType] ) )
		{
			nResult = 0;
			nSeqNo = 0;
		}
		break;
	}

	m_nSetSyncMasterSlaveNo[nRbtType] = nSeqNo;
	return nResult;
}

double XMmc::SetCalTheta( int nRbtType ) 
{
	double dTheta = -1;
#ifdef _MY_DEBUG
	dTheta = 0.568;
	return dTheta;
#endif
	int nAxis = m_RbtInfo[nRbtType].nStart_axis;
	int nNo = nAxis - RBT_SWR2_UP_RB1_UNIT_S_AXIS;
	static DWORD dwOverTime = 0;
	int nSeqNo = m_nSetCalThetaNo[nNo]; // minhan 
	switch( nSeqNo ) 
	{
	case 0:
		// set no event for software limit
//		get_negative_sw_limit( nAxis, &dLimitPosNeg[nAxis], &nActionNeg[nAxis] );
//		set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], NO_EVENT );
//		get_positive_sw_limit( nAxis, &dLimitPosPos[nAxis], &nActionPos[nAxis] );
//		set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], NO_EVENT );
		set_home( nAxis, NO_EVENT );
		set_negative_limit( nAxis, NO_EVENT ); // 2003.1.2 Hong jaehee
		set_positive_limit( nAxis, NO_EVENT ); // 2003.1.2 Hong jaehee

//		frames_clear( axis );
		clear_status( nAxis );

		set_stop_rate( nAxis, m_HomeInfo[nAxis].nAcc / 2 );
		set_index_required( nAxis, FALSE );
		set_home( nAxis, STOP_EVENT );
		set_negative_limit( nAxis, STOP_EVENT ); // 2003.1.2 Hong jaehee

		frames_clear( nAxis );
		set_position( nAxis, 0.0 );
		v_move( nAxis, -m_HomeInfo[nAxis].dVel*4.0, m_HomeInfo[nAxis].nAcc / 2 );
		nSeqNo = 10;
		break;
	case 10:
		if( axis_done( nAxis ) ) 
		{
			set_home( nAxis, NO_EVENT );
			set_negative_limit( nAxis, NO_EVENT ); // 2003.1.2 Hong jaehee
			set_positive_limit( nAxis, E_STOP_EVENT ); // 2003.1.2 Hong jaehee
		//	frames_clear( axis );
			clear_status( nAxis );
			frames_clear( nAxis );

			// 2003.1.2 Hong jaehee
			if( neg_switch( nAxis ) )
			{
				nSeqNo = 100;
			}
			else if( home_switch( nAxis ) )
			{
				start_r_move( nAxis, GetDistanceHomeOut(nAxis), m_HomeInfo[nAxis].dVel*2.0, m_HomeInfo[nAxis].nAcc / 2 );
				dwOverTime = GetTickCount();
				nSeqNo = 20;
			}

		}
		break;

	case 20:
		if( axis_done( nAxis ) ) 
		{
			set_stop_rate( nAxis, m_HomeInfo[nAxis].nAcc / 2 );
			set_home( nAxis, STOP_EVENT );
			set_negative_limit( nAxis, E_STOP_EVENT ); // 2003.1.2 Hong jaehee

			if( home_switch( nAxis ) )
			{
				nSeqNo = 100;
			}
			else
			{
				frames_clear( nAxis );
				v_move( nAxis, -m_HomeInfo[nAxis].dVel / 3.0, m_HomeInfo[nAxis].nAcc / 3 );
				nSeqNo = 30;
			}
		}
		else if( GetTickCount() - dwOverTime > 5000) nSeqNo = 100;
		break;
	case 30:
		if( axis_done( nAxis ) ) 
		{
			set_home( nAxis, NO_EVENT );
			clear_status( nAxis );
			nSeqNo = 40;
		}
		break;
	case 40:
		if( axis_done( nAxis ) ) 
		{
			if( axis_state( nAxis ) != NO_EVENT )
			{
//				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
//				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				nSeqNo = 100;
			}
			else
			{
//				set_position( nAxis, 0.0 );
//				set_negative_sw_limit( nAxis, dLimitPosNeg[nAxis], nActionNeg[nAxis] );
//				set_positive_sw_limit( nAxis, dLimitPosPos[nAxis], nActionPos[nAxis] );
				RBT_POS	rpPos;
				get_position( nAxis, &rpPos.dPos[2] );

				dTheta = 90 - ( 720* (- rpPos.dPos[2]) / 819200.0   );//degree			
				dTheta = ( dTheta / 180.0 ) * PI;//radian
				set_position( nAxis, 0.0 );
				nSeqNo = 0;
			}
			
		}
		break;

	//MMC State Return
	case 100:
		{
			set_home( nAxis, NO_EVENT );
			set_negative_limit( nAxis, E_STOP_EVENT ); // 2003.1.2 Hong jaehee
			set_positive_limit( nAxis, E_STOP_EVENT ); // 2003.1.2 Hong jaehee
		//	frames_clear( axis );
			frames_clear( nAxis );
			clear_status( nAxis );
			nSeqNo = 0;
			dTheta = -100;//ng value
		}
		break;
	}
		m_nSetCalThetaNo[nNo] = nSeqNo;

		return dTheta;
}

double XMmc::GetDistanceHomeOut( int nAxis ) // hskwak 20190624
{
	return Len2Pulse( nAxis, m_HomeInfo[nAxis].dDist );
}