//---------------------------------------------------------------------
// Copyright		: DMS Co., Ltd
// File Name		: XMmc.h
// Modified Date	: 2001.03.27
// Version			: Ver1.0
// Programmer		: Kim Dongsoo
// Description		: Interface for the XMmc class.
//					MEI Modify
//  
//---------------------------------------------------------------------

#if !defined(AFX_XMMC_H__0F3460B8_B12C_4521_B84F_35988CF163C1__INCLUDED_)
#define AFX_XMMC_H__0F3460B8_B12C_4521_B84F_35988CF163C1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//#include ".\\..\mmc\mmcwfi40.h"
//#include ".\\..\mmc\mmcwhp13.h"
#include ".\\..\mmc\mmcwhp201.h"	//MMC PCI Type
#include "..\FactoryLib\IniData.h"	// Added by ClassView

// define the struct related to robot home information
typedef struct tagHOME_INFO 
{
	double	dVel;			//  : Pulse
	INT		nAcc;			//  : 4 msec
	double	dDist;			// Ƣ Ÿ (mm)
	INT		nHome_type;      // ̵ 
} HOME_INFO;

// axis parameter
typedef struct tagAXIS_PARA 
{
	double	dVel;		    //  : Pulse
	INT		nAcc;		    //  : 4 msec
	double	dRatio;	        //  : mm (The distance of 1 rev)
	unsigned int uEncoder;	// The pulse number of 1 rev
	INT		nType;			//    : R/B Gap(CAM), Ϲ
	double	dTheta_H;
	BOOL	bRatioFile;		// Ratio   . FALSE=̻, TRUE=.
} AXIS_PARA;

// robot position (x, y, z, theta)
typedef struct tagRBT_POS
{
	double dPos[4];
	double dPulse[4];	// Pulse
} RBT_POS;

// robot velocity (x, y, z, theta)
typedef struct tagRBT_VEL
{
	double dVel[4];
} RBT_VEL;

// link information( )
typedef struct tagRBT_LINK
{
	INT nMaster;
	INT nSlave;
	BOOL bIs_linked;
} RBT_LINK;

//  robot information
typedef struct tagRBT_INFO
{
	INT nStart_axis; 
	INT nMax_axes;
	INT nMask;
	INT nMax_pos_no;
	INT nMove_type;
	INT nHome_order[MAX_RBT_AXES];	//  κ ִ  4 
	INT nXyz0_to_axis[MAX_RBT_AXES];
	RBT_LINK	link;
	RBT_POS		*pos;
	RBT_VEL		vel;
} RBT_INFO;

typedef struct tagRbtSensor
{
	BOOL PosLimit;
	BOOL Home;
	BOOL NegLimit;
} RBT_SENSOR;

// Robot Status
typedef struct tagRbtState
{
	INT	nAxis_state[MAX_RBT_AXES];  // ࿡  Event
	INT	nAxis_source[MAX_RBT_AXES]; // ࿡  
	RBT_SENSOR axis_sensor[MAX_RBT_AXES];
} RBT_STATE;

class XMmc : public CObject
{
public:
	XMmc();
	virtual ~XMmc();
public:
	// robot information (static value)

	static HOME_INFO	m_HomeInfo[MAX_AXES];
	static AXIS_PARA	m_AxisPara[MAX_AXES];
	static RBT_INFO		m_RbtInfo[MAX_RBT_NO];

	static CString		m_sRbtFileName[MAX_RBT_NO];  
	static CString		m_sRbtName[MAX_RBT_NO];
	static CString		m_sRbtPosName[MAX_RBT_NO][10];
	static CString		m_sRbtAxisName[MAX_RBT_NO][MAX_RBT_AXES];
	static CString		m_sAxisState[];  // ࿡  Event 
	static CString		m_sAxisSource[]; // ࿡  

	RBT_STATE		m_RbtState[MAX_RBT_NO];  // Robot status
	BOOL			m_bVelMove[MAX_RBT_NO];
	BOOL			m_bHomeComp[MAX_RBT_NO];  // ̵ ϷǾ Ÿ(TRUE : ̵ Ϸ)
	BOOL				m_bServoReady[MAX_RBT_NO];	// Servo Ready   Ÿ(TRUE : Servo ON)

	enum actROBOT  
	{
		actNONE,
		actE_STOP,
		actRESET,
		actHOME,
		actMOVE,
		actR_MOVE,
		actMOVE_REPEAT
	};

	enum typeAXIS
	{
		typeNORMAL,
		typeRB_GAP,
	};

private:
	struct tagHomeAxis
	{
		BOOL	bFlag_home_end[MAX_RBT_AXES];
		INT		nFound[MAX_AXES];
		INT		nNo;
	} m_HomeAxis[MAX_RBT_NO];  // ̵Ҷ  ϴ   

	double	m_dRatio[MAX_RBT_NO]; // Robot ̵Ҷ ӵ (  1϶ 100 %)

	double	m_dAxisRatio[MAX_RBT_NO];	// hskwak 190806 : PPM  Ǵ Ratio.

	int	m_nCurPoint[MAX_RBT_NO];  //  ̵ Teaching Point ȣ

	// Sequence no
	int m_nSetSyncMasterSlaveNo[MAX_RBT_NO];
	int m_nClearAxisErrNo[MAX_AXES];
	int m_nRelativeMoveNo[MAX_RBT_NO];
	int m_nServoOnRbtNo[MAX_RBT_NO];
	int m_nEStopRbtNo[MAX_RBT_NO];
	int m_nMoveRbtHomeNo[MAX_RBT_NO];
	int m_nMoveRbtPtp1No[MAX_RBT_NO];
	int m_nMoveRbtPtp2No[MAX_RBT_NO];
	int m_nMoveRbtS1No[MAX_RBT_NO];
	int m_nMoveRbtS2No[MAX_RBT_NO];  
	int m_nMoveRbtPosNo[MAX_RBT_NO];
	int m_nHomeOnlyNo[MAX_AXES];
	int m_nHomeAndIndexNo[MAX_AXES];
	int m_nSetCalThetaNo[MAX_AXES];//2008.09.04 kimgun
	CIniData  m_IniVelInfo;
	CIniData  m_IniThetaInfo;

public:
	int		SetSyncMasterSlave(int nRbtType);	// 2007.10.16 HJYOU Master&Slave Motor Position Sync.
	BOOL	GetHomeSwitch(INT nAxis);
	void	GetAmpEnable(int nAxis, short* nLevel);
	BOOL	ServoOnRbt(INT nRbtType);
	BOOL	ResetRbt(INT nRbtType);
	BOOL	EStopRbt(INT nRbtType);
	BOOL	GetCurPosition(INT nRbtType, RBT_POS& rpPos);
	BOOL	PosFile2Mem(INT nRbtType);
	BOOL	PosMem2File(INT nRbtType);
	BOOL	IsRbtStatusOk(INT nRbtType);
	BOOL	IsRbtSensorOk(INT nRbtType);

	void	ResetRbtHomeSeq( INT nRbtType ); 
	void	SetCurPoint( INT nRbtType, INT nPoint ) { m_nCurPoint[nRbtType] = nPoint; } //  Robot Teaching Point 
	void	SetCurPosition( INT nRbtType, INT nPos );
	void	InitParameter();
	void	Init();
	void	ServoOffRbt(INT nRbtType);
	void	PauseRbt(INT nRbtType, BOOL bPause);
	void	StartVelMove(INT nRbtType, INT nRbt_axis, INT nSign, double dVel);
	void	StopVelMove(INT nRbtType, INT nRbt_axis);
	void	SetVel(INT nRbtType, RBT_VEL vel);
	void	SetTheta(INT nRbtType, double dTheta);
	double	GetTheta(INT nRbtType);
	void	SetVelFromRegistry(INT nRbtType);
	void	SetVelToRegistry(INT nRbtType, RBT_VEL vel);
	void	SetThetaToRegistry(INT nRbtType, double dtheta );//jun
	void	SetThetaFromRegistry(INT nRbtType);//jun
	void	SetAxisRatioToRegistry(INT nRbtType, double dRatio);	// hskwak 190806
	void	SetAxisRatioFromRegistry(INT nRbtType);	// hskwak 190806
	void	SetAxisRatio(INT nRbtType, double dTheta);
	double	GetAxisRatio(INT nRbtType);
	void	SetVelRatio(int nRbtType, double dRatio) { m_dRatio[nRbtType] = dRatio; } // Robot ӵ  

	int		GetSensorState(INT nRbtType, INT nRbt_axis);

	int		GetCurPoint( INT nRbtType );    
	int		MoveRbtHome(INT nRbtType);
	int		MoveRbtPos(INT nRbtType, INT nPosNo);
	int		MoveRbtPos(INT nRbtType, RBT_POS pos);
	int		RelativeMove(INT nRbtType, INT nRbt_axis, INT nSign, double dDist, double dVel);

	INT		GetErrorMessage( CString& sMessage );

	RBT_VEL	GetVel(INT nRbtType);

	RBT_STATE GetRbtStatus(INT nRbtType);
	double SetCalTheta( int nAxis );//2008.09.04 kimgun
	double GetDistanceHomeOut( int nAxis ); // hskwak 20190624

private:
	BOOL	ClearAxisErr(INT nAxis);
	BOOL	GetHomeInfo(INT nRbtType, INT nMax_axes, INT *pAxes_order);
	BOOL	IsMoveOk(INT nRbtType);

	int		HomeOnly(INT nAxis);
	int		HomeAndIndex(INT nAxis);
	int		Home(INT nAxis);
	int		MoveRbtPtp1(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask);
	int		MoveRbtPtp2(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask);
	int		MoveRbtS1(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask);
	int		MoveRbtS2(INT nRbtType, RBT_POS pos, RBT_VEL vel, INT nMask);

	double	Len2Pulse(INT nAxis, double dLen);
	double	Pulse2Len(INT nAxis, double dPulse);

};

#endif // !defined(AFX_XMMC_H__0F3460B8_B12C_4521_B84F_35988CF163C1__INCLUDED_)
