//---------------------------------------------------------------------------
// Copyright		: DMS Co., LTd
// File Name		: X\XComm.cpp
// Modified Date	: 2000.11.28, 
// Version			: 1.0
// Programmer		: kds
// XComm.cpp		: implementation of the XComm class.
// Description		:  ϴ XComm class . 
//---------------------------------------------------------------------------

#include "stdafx.h"
#include "..\Dics.h"	// It will be changed if this file is used by any other application.
#include "XComm.h"

#ifdef _DEBUG

#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XComm::XComm()
{
	m_hComm = NULL;
	m_nEvtType = 0;			// Default : EV_RXCHAR event
	m_bEmulation = FALSE;

	m_osRead.hEvent = NULL;
	m_osWrite.hEvent = NULL;
}

XComm::~XComm()
{
	if( m_osRead.hEvent ) CloseHandle( m_osRead.hEvent );
	if( m_osWrite.hEvent ) CloseHandle( m_osWrite.hEvent );
}

//---------------------------------------------------------------------------
// Open the COMM port
//---------------------------------------------------------------------------
BOOL XComm::Open(LPCTSTR lpszPortName,		// "Emulate", "COM1", "COM2", ...
				 DWORD dwBaud,				// CBR_9600, CBR_4800, ...
				 BYTE byByte,				// 8, 7, ...
				 BYTE byStop,				// ONESTOPBIT, ONE5STOPBITS, TWOSTOPBITS
				 BYTE byParity,				// EVENPARITY, MARKPARITY, NOPARITY, ODDPARITY, SPACEPARITY
				 char cEvtChar)				// signal an event, (= 0 : EV_RXCHAR)
{
	CString sBuf;

	// If Emulation Mode, return TRUE
	if( strcmp( lpszPortName, "Emulate" ) == 0 )
	{
		m_bEmulation = TRUE;
		return TRUE;
	}

	m_nPortNo = lpszPortName[3]-'0';	// COM1, COM2, COM3, COM4 ==> 1,2,3,4

	// Create File
	sBuf.Format( "\\\\.\\%s", lpszPortName );	// NT \\.\COM1 
	m_hComm = CreateFile( sBuf, GENERIC_READ | GENERIC_WRITE, 0, NULL,
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, NULL );
	
	if( m_hComm == INVALID_HANDLE_VALUE )
	{
		m_hComm = NULL;
		sBuf.Format( "Error : Can't open the communication port [%s]", lpszPortName );
		AfxMessageBox( sBuf, MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL );
		return FALSE;
	}

	m_osWrite.Offset = 0;
	m_osWrite.OffsetHigh = 0;
	m_osRead.Offset = 0;
	m_osRead.OffsetHigh = 0;

	// Create Event
	m_osRead.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );	// MANUAL Reset, Initial state = non-signal

	if( m_osRead.hEvent == NULL )
	{
		m_hComm = NULL;
		sBuf.Format( "Error : Cannot create osRead event! [COM%d]", m_nPortNo );
		AfxMessageBox( sBuf, MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL );
		CloseHandle( m_osRead.hEvent );
		return FALSE;
	}

	m_osWrite.hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );	// MANUAL Reset, Initial state = non-signal

	if( m_osWrite.hEvent == NULL )
	{
		m_hComm = NULL;
		sBuf.Format( "Error : Cannot create osWrite event! [COM%d]", m_nPortNo );
		AfxMessageBox( sBuf, MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL );
		CloseHandle( m_osWrite.hEvent );
		return FALSE;
	}

	SetCommMask( m_hComm, EV_RXFLAG );	// event character Ž event ߻
	SetupComm( m_hComm, 4096, 4096 );	// buffer queue size
	PurgeComm( m_hComm, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );	// clear all the buffer

	if( !SetDCB( dwBaud, byByte, byStop, byParity, cEvtChar ) ) return FALSE;
	SetTimeout( 2000, 1000 );

	//The EscapeCommFunction function directs a specified communications device 
	//to perform an extended function. 
	//extended function (SETDTR) => Sends the DTR (data-terminal-ready) signal.
	EscapeCommFunction( m_hComm, SETDTR );	// set DTR

	return TRUE;
}

//---------------------------------------------------------------------------
// Set the DCB
//---------------------------------------------------------------------------
BOOL XComm::SetDCB( DWORD dwBaud, BYTE byByteSize, BYTE byStopSize, BYTE byParity, char cEvtChar )
{
	//The DCB structure defines the control setting for a serial communications device
	DCB dcb;
	CString sBuf;

	if( !GetCommState( m_hComm, &dcb ) )
	{
		sBuf.Format( "Error : Get communication state [COM%d]", m_nPortNo );
		AfxMessageBox( sBuf, MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL );
		CloseHandle( m_hComm );
		return FALSE;
	}

	dcb.DCBlength	= sizeof( DCB );
	dcb.BaudRate	= dwBaud;				// CBR_9600	(Default)
	dcb.Parity		= byParity;				// NOPARITY
	dcb.ByteSize	= byByteSize;			// 8
	dcb.StopBits	= byStopSize;			// 1
	dcb.EvtChar		= cEvtChar;				// set event character
	
	//dcb.EofChar = m_cEofChar;
	//dcb.XonChar = 0x13;
	//dcb.XoffChar = 0x11;
	//dcb.XonChar = 0;              // Tx and Rx XON character 
    //dcb.XoffChar = 2; 

	if( !SetCommState( m_hComm, &dcb ) )
	{
		DWORD dwCode = GetLastError();
		TRACE( "SetCommState : %d\r\n", dwCode );

		sBuf.Format( "Error : Set communication state [COM%d]", m_nPortNo );
		AfxMessageBox( sBuf, MB_OK | MB_ICONSTOP | MB_SYSTEMMODAL );
		CloseHandle( m_hComm );
		return FALSE;
	}

	if( cEvtChar != 0 ) m_nEvtType = 1;
	return TRUE;
}

//---------------------------------------------------------------------------
// Close the COMM port
//---------------------------------------------------------------------------
BOOL XComm::Close()
{
	SetCommMask( m_hComm, 0 );
	EscapeCommFunction( m_hComm, CLRDTR );
	PurgeComm( m_hComm, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );
	CloseHandle( m_hComm );
	return TRUE;
}

//---------------------------------------------------------------------------
// Timeout Read
//---------------------------------------------------------------------------
DWORD XComm::TimeoutRead(LPSTR lpszBuf, DWORD dwLength, int nTimeout)
{
	SetTimeout( nTimeout );
	return Read( lpszBuf, dwLength );
}

//---------------------------------------------------------------------------
// Set the Timeout 
//---------------------------------------------------------------------------
void XComm::SetTimeout(int nReadTimeout, int nWriteTimeout)
{
	COMMTIMEOUTS CommTimeOuts;
	CommTimeOuts.ReadIntervalTimeout = 500;			// 0.5sec
	CommTimeOuts.ReadTotalTimeoutMultiplier = 2;	// 2msec for 9600bps
	CommTimeOuts.ReadTotalTimeoutConstant = nReadTimeout;
	CommTimeOuts.WriteTotalTimeoutMultiplier = 0;
	CommTimeOuts.WriteTotalTimeoutConstant = nWriteTimeout;
//	CommTimeOuts.ReadIntervalTimeout = 1000;	
//	CommTimeOuts.ReadTotalTimeoutMultiplier = 1000;
//	CommTimeOuts.ReadTotalTimeoutConstant = iReadTimeout;
//	CommTimeOuts.WriteTotalTimeoutMultiplier = 1000;
//	CommTimeOuts.WriteTotalTimeoutConstant = iWriteTimeout;
	SetCommTimeouts( m_hComm, &CommTimeOuts );
}

//---------------------------------------------------------------------------
// Added by KDS 041118
BOOL XComm::IsInQue()
{
 DWORD dwErrorFlags;
 COMSTAT ComStat;

 ClearCommError( m_hComm, &dwErrorFlags, &ComStat );
 if( !ComStat.cbInQue ) return FALSE;

 return TRUE;
}
// Added by KDS 041118
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// read the received data
//---------------------------------------------------------------------------
DWORD XComm::Read(LPSTR lpszBuf, DWORD dwLength)
{
	if( m_bEmulation ) return 0;

	if( !ReadFile( m_hComm, lpszBuf, dwLength, &dwLength, &m_osRead ) )
	{
		if( GetLastError() != ERROR_IO_PENDING )
			return dwLength+1;
		if( !GetOverlappedResult( m_hComm, &m_osRead, &dwLength, TRUE) )
			return dwLength+1;
	}

	lpszBuf[dwLength] = 0;
	return dwLength;
}

//---------------------------------------------------------------------------
// Write the given data
//---------------------------------------------------------------------------
BOOL XComm::Write(LPCTSTR lpszBuf, DWORD dwLength)
{
	if( m_bEmulation ) return 0;

	if( dwLength == -1 ) dwLength = strlen( lpszBuf );

	/* NT ڵ ߸ƴٴ Error ͼ ̰ɷ ...
	for( DWORD i=0; i<dwLength; i++ )
		TransmitCommChar( m_hComm, lpBuf[i] );
	return TRUE;
	*/

	// Flush(); jemoon : 070205 
	FlushTx();

	if( !WriteFile( m_hComm, lpszBuf, dwLength, &dwLength, &m_osWrite ) )
	{
		if( GetLastError() != ERROR_IO_PENDING )
			return FALSE;
		if( !GetOverlappedResult( m_hComm, &m_osWrite, &dwLength, TRUE) )
			return FALSE;
	}
	return TRUE;
}

//---------------------------------------------------------------------------
// Write one character
//---------------------------------------------------------------------------
void XComm::PutChar(char ch)
{
	if( m_bEmulation ) return;

	// Flush(); jemoon : 070205 
	FlushTx();

	TransmitCommChar( m_hComm, ch );
}

// jemoon : 070205 : Ʒ Լ  TX, RX δ Clearϰ ǹǷ
// InQue Data Ҷ CallǸ Data ߻Ѵ.
// . Flush()Լ Tx Rx иؾ .
//---------------------------------------------------------------------------
// ޸𸮿 Error ClearѴ.
//---------------------------------------------------------------------------
//void XComm::Flush()
//{
// DWORD dwErrorFlags;
// COMSTAT ComStat;
//
// PurgeComm( m_hComm, PURGE_TXCLEAR | PURGE_TXABORT | PURGE_RXCLEAR | PURGE_RXABORT );
// FlushFileBuffers( m_hComm );
// ClearCommError( m_hComm, &dwErrorFlags, &ComStat );
//}

void XComm::FlushTx()
{
	DWORD dwErrorFlags;
	COMSTAT ComStat;

//	PurgeComm( m_hComm, PURGE_TXCLEAR | PURGE_TXABORT | PURGE_RXCLEAR | PURGE_RXABORT );
	PurgeComm( m_hComm, PURGE_TXCLEAR | PURGE_TXABORT );
	FlushFileBuffers( m_hComm );
	ClearCommError( m_hComm, &dwErrorFlags, &ComStat );
}

void XComm::FlushRx()
{
	DWORD dwErrorFlags;
	COMSTAT ComStat;

//	PurgeComm( m_hComm, PURGE_TXCLEAR | PURGE_TXABORT | PURGE_RXCLEAR | PURGE_RXABORT );
	PurgeComm( m_hComm, PURGE_RXCLEAR | PURGE_RXABORT );
//	FlushFileBuffers( m_hComm );	// jemoon : 070205 - only use for Tx Case
	ClearCommError( m_hComm, &dwErrorFlags, &ComStat );
}



//---------------------------------------------------------------------------
// get the error message
//---------------------------------------------------------------------------
CString& XComm::GetErrorMsg()
{
	LPVOID lpMsgBuf;

	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR)&lpMsgBuf, 0, NULL );
	strcpy( m_sErrMsg.GetBuffer(strlen((char*)lpMsgBuf)), (char*)lpMsgBuf );
	m_sErrMsg.ReleaseBuffer();
	LocalFree( lpMsgBuf );
	return m_sErrMsg;
}

char XComm::GetCheckSum(LPCSTR lpszMsg, int nLen)
{
	char cRv = 0;
	
	for( int i=0; i < nLen; i++ ) cRv ^= lpszMsg[i];

	return cRv;
}